package org.exoplatform.faces.cms;

import javax.jcr.Node;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.xml.PortalContainerInfo;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;

public class CMSDelegate implements CMSViewComponent {

  private String portalName_;
  private WikiContext wikiContext;
  private WikiEngineService service_;
  private Node node;

  public CMSDelegate(Node node) {
    this.node = node;
    PortalContainer pcontainer = PortalContainer.getInstance();
    PortalContainerInfo info = (PortalContainerInfo) pcontainer
        .getComponentInstanceOfType(PortalContainerInfo.class);
    portalName_ = info.getContainerName();

    service_ = (WikiEngineService) pcontainer
        .getComponentInstanceOfType(WikiEngineService.class);
  }

  public String getPortalName() {
    return portalName_;
  }

  public String getWorkspaceName() throws Exception {
    return node.getSession().getWorkspace().getName();
  }

  public Node getNode() throws Exception {
    return node;
  }

  public void setNode(Node node) {
    this.node = node;
  }

  public boolean isNodeTypeSupported() {
    try {
      PortalContainer container = PortalContainer.getInstance();
      TemplateService templateService = (TemplateService) container
          .getComponentInstanceOfType(TemplateService.class);
      String nodeTypeName = node.getPrimaryNodeType().getName();
      return templateService.isNodeTypeConfigured(nodeTypeName);
    } catch (Exception e) {
      return false;
    }
  }

  public String getNodeType() throws Exception {
    return node.getPrimaryNodeType().getName();
  }

  public WikiContext getWikiContext() {
    if (wikiContext == null) {
      wikiContext = new WikiContext();
      wikiContext.put("portalHook", new PortalHook());
    }
    return wikiContext;
  }

  public WikiEngineService getWikiService() {
    return service_;
  }

  public String getTemplatePath() throws Exception {
    String nodeTypeName = node.getPrimaryNodeType().getName();
    PortalContainer container = PortalContainer.getInstance();
    TemplateService templateService = (TemplateService) container
        .getComponentInstanceOfType(TemplateService.class);
    return templateService.getTemplatePath(false, nodeTypeName);

  }

}
