package org.exoplatform.portlets.content.categories.component;

import java.util.List;

import javax.jcr.Node;

import org.exoplatform.faces.cms.component.JcrBrowser;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.categories.CategoriesService;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;

public class UICategoriesList extends UIExoCommand implements ChangeNodeListener, SelectorHandler{
  
  final static public String ADD_ACTION = "add".intern();  
  
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;

  protected CategoriesService categoriesService_;

  protected Node node_;

  protected CmsConfigurationService cmsConfig_;
  
  public UICategoriesList(CategoriesService categoriesService, CmsConfigurationService cmsConfig) {
    setId("UICategoriesList") ;
    setRendererType("VelocityRenderer") ;
    
    categoriesService_ = categoriesService;        
    cmsConfig_ = cmsConfig;
    
    addActionListener(AddActionListener.class, ADD_ACTION) ;    
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(CancelActionListener.class,   CANCEL_ACTION) ;
  }

  public void  onChange(UIJCRExplorer uiExplorer, Node node)  throws Exception {
    setRendered(false) ;
    node_ = node;
  }      
  
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }
   public Parameter[] getBackParams() { return CANCEL_PARAMS ; }
  
  public boolean hasCategories() throws Exception{
     return categoriesService_.hasCategories(node_);
  }
  
  public List<Node> getCategories() throws Exception{
    return categoriesService_.getCategories(node_);
  }
  
  public Node getNode() {
    return node_;
  }
  
  public void cancel(Selector selector) {
    selector.setRenderedSibling(UICategoriesList.class);
  }

  public void select(Selector selector) throws Exception {
    String selectedNodePath = selector.getInputValue();
    categoriesService_.addCategory(node_, selectedNodePath);
    selector.setRenderedSibling(UICategoriesList.class);
  }      

  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICategoriesList uicomp = (UICategoriesList) event.getComponent() ;
      UIJCRExplorer explorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      JcrBrowser jcrBrowser = (JcrBrowser) uicomp.getSibling(JcrBrowser.class);
      jcrBrowser.setHandler(uicomp) ;
      jcrBrowser.setSession(explorer.getSession());
      String workspace =  uicomp.cmsConfig_.getWorkspace();
      String categoriesPath = uicomp.cmsConfig_.getJcrPath(BasePath.CMS_CATEGORIES_PATH);
      jcrBrowser.setParameters(new String[] {workspace, categoriesPath});
      uicomp.setRenderedSibling(JcrBrowser.class);   
    }
  }       
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICategoriesList uicomp = (UICategoriesList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.categoriesService_.removeCategory(uicomp.node_, name);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }
    
}
