package org.exoplatform.portlets.content.categorybrowser.component;

import java.util.List;

import javax.jcr.query.Query;
import javax.jcr.query.QueryResult;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.queries.QueryService;

public class UIQueryList extends UIExoCommand {

  public static final String EXECUTE_ACTION = "execute".intern();

  public static Parameter[] EXECUTE_PARAMS = { new Parameter(ACTION,
      EXECUTE_ACTION) };
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION,
      DELETE_ACTION) };

  protected QueryService queryService_;

  public UIQueryList(QueryService queryService) {
    setId("UIQueryList");
    setRendererType("VelocityRenderer");

    queryService_ = queryService;

    addActionListener(ExecuteActionListener.class, EXECUTE_ACTION);
    addActionListener(DeleteActionListener.class, DELETE_ACTION);
  }

  public Parameter[] getExecuteParams() {
    return EXECUTE_PARAMS;
  }

  public Parameter[] getDeleteParams() {
    return DELETE_PARAMS;
  }

  public boolean hasQueries() throws Exception {
    List<Query> queries = queryService_.getQueries();
    if (queries == null || queries.isEmpty())
      return false;
    return true;
  }

  public List<Query> getQueries() throws Exception {
    return queryService_.getQueries();
  }

  static public class ExecuteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryList uicomp = (UIQueryList) event.getComponent();
      String queryPath = event.getParameter(OBJECTID);
      String queryName = queryPath.substring(queryPath.lastIndexOf("/") + 1);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Query query = uicomp.queryService_.getQuery(queryName);
        long beforeTime = System.currentTimeMillis();
        QueryResult queryResult = query.execute();
        long duration = System.currentTimeMillis() - beforeTime;
        UIResults results = (UIResults) uicomp.getSibling(UIResults.class);
        results.setQueryResults(duration, queryResult);
        uicomp.setRenderedSibling(UIResults.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryList uicomp = (UIQueryList) event.getComponent();
      String query = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.queryService_.removeQuery(query);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
}
