/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.display.dynamic.component;

import java.util.Map;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Session;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDialogComponent;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentUtil;
import org.exoplatform.portlets.content.display.model.ContentConfig;
import org.exoplatform.portlets.content.display.model.ContentConfigManipulationsUtils;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.cms.templates.TemplateService;

/**
 * Sat, Jan 03, 2004 @ 11:16
 * @author: Ove Ranheim
 * @email: oranheim@users.sourceforge.netx
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIContentHtmlEditorForm.java,v 1.2 2004/06/15 23:09:18
 *           oranheim Exp $
 */
public class UIContentEditor extends UISimpleMulltipartForm implements
    CMSDialogComponent {

  protected ContentConfig config_;

  protected TemplateService templateService_;

  protected boolean isNew_;

  public UIContentEditor(TemplateService templateService) throws Exception {
    super("editorForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    this.templateService_ = templateService;

    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
  }

  public ContentConfig getContentConfig() {
    return config_;
  }

  public void setContentConfig(ContentConfig config) {
    config_ = config;    
  }
  
  public void setIsNew(boolean b) {
    isNew_ = b;    
  }  

  public void clean() {
    clearInputs();
    getChildren().clear();
  }

  public String getTemplatePath() throws Exception {
    String documentType = config_.getDocumentType();
    return templateService_.getTemplatePath(true, documentType);
  }

  public Node getNode() throws Exception {
    String uri = ContentUtil.resolveLocalisedViewPath(config_.getUri());
    UIAdminContent adminContent = (UIAdminContent) getAncestorOfType(UIAdminContent.class);
    return (Node) adminContent.getSession().getItem(uri);
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentEditor uiEditor = (UIContentEditor) event.getSource();
      uiEditor.setRenderedSibling(UIContentConfig.class);
      uiEditor.clean();
    }
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentEditor uiEditor = (UIContentEditor) event.getSource();      

      String pageOwner = ContentUtil.getPageOwner();
      String portalOwner = ContentUtil.getPortalOwner();

      Session session = ((UIAdminContent)uiEditor.getAncestorOfType(
          UIAdminContent.class)).getSession();  
      String path = uiEditor.config_.getUri();
      String documentType = uiEditor.config_.getDocumentType();

      if (!portalOwner.equals(pageOwner)) {
        session = ((UIAdminContent)uiEditor.getAncestorOfType(
            UIAdminContent.class)).getDraftSession();  

        Information iprovider = findInformationProvider(uiEditor);
        iprovider.addMessage(new ExoFacesMessage("#{UIContentEditor.msg.send-to-validation}"));
      }
      

      String sitePath = ContentUtil.getLocalisedAbsoluteSitePath();
      Node homeNode = null;
      try {
        homeNode = (Node) session.getItem(sitePath);
      } catch (PathNotFoundException e) {
        return;
      }          
            
      String parentPath = path.substring(0, path.lastIndexOf("/"));
      String fileName = path.substring(path.lastIndexOf("/") + 1);
      
      Node node = ContentUtil.createNodeFromNode(homeNode, parentPath);
      
      Map structuredInputs = CMSUtils.prepareMap(uiEditor.getUIInputs());
      
      String rootPath = "/node";
      JcrInputProperty property = new JcrInputProperty(); 
      property.setJcrPath(rootPath);
      if("nt:file".equals(documentType)){        
        property.setValue(fileName);
      } else {
        node = node.addNode(fileName, "nt:file");        
        property.setValue("jcr:content");
      }            
      structuredInputs.put(rootPath, property);
      
      PortalContainer manager = PortalContainer.getInstance();
      CmsService cmsService = (CmsService) manager
          .getComponentInstanceOfType(CmsService.class);
      cmsService.storeNode(documentType, node, structuredInputs);
      homeNode.save();
      
      if(uiEditor.isNew_) {
        int pos = ContentConfigManipulationsUtils.getNewPosition();
        uiEditor.config_.setPosition("" + pos);
      }
      ContentConfigManipulationsUtils.saveContentConfig(uiEditor.config_);

      UIContentConfig uiConfig = (UIContentConfig) uiEditor
          .getSibling(UIContentConfig.class);
      uiConfig.updateConfigs(session);
      uiEditor.setRenderedSibling(UIContentConfig.class);
      uiEditor.clean();
    }
  }

}