/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.context.PortletExternalContext;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Feb 23, 2005
 * @version $Id$
 */
public class JCRUtil {
  static private Map smallNodeTypeIconMapping_;

  static private Map largeNodeTypeIconMapping_;

  static private Map propertyTypeIconMapping_;

  static private String defaultSmallNodeIcon_;

  static private String defaultLargeNodeIcon_;

  static private String defaultPropertyIcon_;

  static private String modifiedFlag_;

  static public String getNodeFlag(Node node) {
    if (modifiedFlag_ == null) {
      modifiedFlag_ = getBundle().getString(
          "UIViewController.icon.modified-flag");
    }
    if (node.isModified()) {
      return modifiedFlag_;
    }
    return "";
  }

  static public String getSmallNodeTypeIcon(Node node) {
    String nodeType = getNodeType(node);
    if (smallNodeTypeIconMapping_ == null) {
      smallNodeTypeIconMapping_ = getSubProperties("NodeType.icon.small.");
      defaultSmallNodeIcon_ = (String) smallNodeTypeIconMapping_.get("default");
    }
    String icon = getFileIcon(node, nodeType, true);
    if (icon != null)
      return icon;
    icon = (String) smallNodeTypeIconMapping_.get(nodeType);
    if (icon == null)
      icon = defaultSmallNodeIcon_;
    return icon;
  }

  static public String getLargeNodeTypeIcon(Node node) {
    String nodeType = getNodeType(node);
    if (largeNodeTypeIconMapping_ == null) {
      largeNodeTypeIconMapping_ = getSubProperties("NodeType.icon.large.");
      defaultLargeNodeIcon_ = (String) largeNodeTypeIconMapping_.get("default");
    }
    String icon = getFileIcon(node, nodeType, false);
    if (icon != null)
      return icon;
    icon = (String) largeNodeTypeIconMapping_.get(nodeType);
    if (icon == null)
      icon = defaultLargeNodeIcon_;
    return icon;
  }

  static private String getFileIcon(Node node, String nodeType, boolean isSmall) {
    Node jcrContentNode = null;
    try {
      if ("nt:file".equals(nodeType))
        jcrContentNode = node.getNode("jcr:content");
      else if ("nt:resource".equals(nodeType))
        jcrContentNode = node;
      if (jcrContentNode != null) {        
        String mimeType = null;
        if (jcrContentNode.hasProperty("jcr:mimeType"))
          mimeType = jcrContentNode.getProperty("jcr:mimeType").getString();        
        if (isSmall)
          return (String) smallNodeTypeIconMapping_.get(nodeType + "."
              + mimeType);
        return (String) largeNodeTypeIconMapping_
            .get(nodeType + "." + mimeType);
      }
    } catch (RepositoryException e) {
    }
    return null;
  }

  private static String getNodeType(Node node) {
    String nodeType = "";
    try {      
      nodeType = node.getPrimaryNodeType().getName();     
    } catch (Exception ex) {
      nodeType = "default";
    }
    return nodeType;
  }

  static public String getPropertyTypeIcon(Property property) {
    String name = "";
    try {
      name = property.getName();
    } catch (Exception ex) {
    }
    if (propertyTypeIconMapping_ == null) {
      propertyTypeIconMapping_ = getSubProperties("PropertyType.icon.");
      defaultPropertyIcon_ = (String) propertyTypeIconMapping_.get("default");
    }
    String icon = (String) propertyTypeIconMapping_.get(name);
    if (icon == null)
      icon = defaultPropertyIcon_;
    return icon;
  }

  private static Map getSubProperties(String prefix) {
    Map map = new HashMap();
    ResourceBundle res = getBundle();
    Enumeration e = res.getKeys();
    while (e.hasMoreElements()) {
      String key = (String) e.nextElement();
      if (key.startsWith(prefix)) {
        String newkey = key.substring(prefix.length(), key.length());
        newkey = newkey.replace('_', ':');
        map.put(newkey, res.getString(key));
      }
    }
    return map;
  }

  public static ResourceBundle getBundle() {
    ExternalContext eContext = FacesContext.getCurrentInstance()
        .getExternalContext();
    PortletExternalContext econtext = (PortletExternalContext) eContext;
    return econtext.getApplicationResourceBundle();
  }
  
}
