/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UINode;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.PermissionValidator;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedNode;


/**
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIContentDisplayer.java,v 1.2 2004/08/07 18:11:26 tuan08 Exp $
 */
public class UIPermissionForm extends UISimpleMulltipartForm implements
    SelectorHandler, ChangeNodeListener {

  public static final String SEARCH_GROUP = "searchGroup";

  protected UIExoPermissionSelector userOrGroup_;

  private Node node_;

  public UIPermissionForm() throws Exception {
    super("permissionForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    userOrGroup_ = new UIExoPermissionSelector("userOrGroup", null);
    userOrGroup_.addValidator(PermissionValidator.class) ;
    add(userOrGroup_);

    String[] perms = PermissionType.ALL;
    for (int i = 0; i < perms.length; i++) {
      String perm = perms[i];
      add(new UICheckBox(perm, ""));
    }
    addActionListener(AddPermissionActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP);
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP);
  }

  public String getSearchGroupAction() {
    return SEARCH_GROUP;
  }

  public String[] getPermissions() {
    return PermissionType.ALL;
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
    setRendered(false);
  }

  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIPermissionForm.class);
  }

  public void select(Selector selector) throws Exception {
    UIExoPermissionSelector editPrefPerm = (UIExoPermissionSelector) getUIInput("userOrGroup");
    editPrefPerm.setPermission(selector.getInputValue());
    selector.setRenderedSibling(UIPermissionForm.class);
  }

  static public class AddPermissionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPermissionForm uiForm = (UIPermissionForm) event.getSource();
      String userOrGroup = uiForm.userOrGroup_.getValue();
      Information iprovider = findInformationProvider(uiForm);
      if (userOrGroup == null || userOrGroup.length() == 0) {
        iprovider.addMessage(new ExoFacesMessage(
            "#{UIPermissionForm.msg.userOrGroup-is-null}"));
        return;
      }
      List<String> permsList = new ArrayList<String>();
      String[] perms = PermissionType.ALL;
      for (int i = 0; i < perms.length; i++) {
        String perm = perms[i];
        UICheckBox checkBox = (UICheckBox) uiForm.getUIInput(perm);
        if (checkBox.getSelect())
          permsList.add(perm);
      }
      String[] permsArray = permsList.toArray(new String[permsList.size()]);
      try {
        ((ExtendedNode) uiForm.node_).setChildrenPermission(userOrGroup,
            permsArray);
        uiForm.setRenderedSibling(UIPermissionsInfo.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPermissionForm uiForm = (UIPermissionForm) event.getSource();
      UIExoPermissionBrowser uiGroupBrowser = (UIExoPermissionBrowser) uiForm
          .getSibling(UIExoPermissionBrowser.class);
      uiGroupBrowser.setHandler(uiForm);
      uiForm.setRenderedSibling(UIExoPermissionBrowser.class);
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIPermissionForm uiForm = (UIPermissionForm) event.getComponent();
      uiForm.setRenderedSibling(UIPermissionsInfo.class) ;
    }
  }
}