package org.exoplatform.portlets.content.jcr.component;

import java.io.ByteArrayInputStream;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;

public class UIResourceForm extends UISimpleMulltipartForm implements
    ChangeNodeListener {

  protected UITextArea textAreaInput_;
  protected Node contentNode_;
  protected UIUploadInput uploadInput_;
  private Property mime_;

  public UIResourceForm() {
    super("textForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    textAreaInput_ = new UITextArea("text", "");

    uploadInput_ = new UIUploadInput("upload");

    addActionListener(SaveActionListener.class, SAVE_ACTION);
  }
 
  public void setContentNode(Node node) throws RepositoryException {
    contentNode_ = node;
    clearInputs();
    mime_ = node.getProperty("jcr:mimeType");
    if (mime_.getString().startsWith("text")) {
      String contentText = node.getProperty("jcr:data").getString();
      textAreaInput_.setText(contentText);
      add(textAreaInput_);
    } else {
      add(uploadInput_);
    }
  }

  public boolean isText() throws RepositoryException {
    if (mime_.getString().startsWith("text")) 
      return true;
    return false;
  }
  
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    setRendered(false);
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIResourceForm uiForm = (UIResourceForm) event.getSource();
      Information iprovider = findInformationProvider(uiForm);
      try {
        Property prop = uiForm.contentNode_.getProperty("jcr:mimeType");
        if (prop.getString().startsWith("text")) {
          String text = uiForm.textAreaInput_.getText();
          uiForm.contentNode_.setProperty("jcr:data", text);
        } else {
          byte[] content = uiForm.uploadInput_.getContent();
          uiForm.contentNode_.setProperty("jcr:data", new ByteArrayInputStream(
              content));
        }
        uiForm.setRenderedSibling(UIDocumentInfo.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

}
