package org.exoplatform.portlets.content.rules.component;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.rules.RuleService;

public class UIRuleForm extends UISimpleMulltipartForm {

  static String RULE_NAME = "rule".intern(), CONTENT = "content".intern();

  protected RuleService ruleService_;
  protected String ruleName_;
  protected UIStringInput ruleNameInput_;
  protected UITextArea ruleContentInput_;

  public UIRuleForm(RuleService ruleService) {
    super("RuleForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    ruleService_ = ruleService;

    ruleNameInput_ = new UIStringInput(RULE_NAME, "");
    ruleContentInput_ = new UITextArea(CONTENT, "");
    ruleContentInput_.setClazz("large");

    add(ruleNameInput_);
    add(ruleContentInput_);

    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
  }

  public void setRuleName(String ruleName) {
    ruleName_ = ruleName;
    try {
      String rule = ruleService_.getRuleAsText(ruleName_);
      ruleNameInput_.setText(ruleName_).setEditable(false);
      ruleContentInput_.setText(rule);
    } catch (Exception e) {
      ruleNameInput_.setText("").setEditable(true);
      ruleContentInput_.setText("");
    }
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleForm uiForm = (UIRuleForm) event.getSource();
      String name = uiForm.ruleNameInput_.getText();
      String text = uiForm.ruleContentInput_.getText();
      Information iprovider = findInformationProvider(uiForm);
      try {
        uiForm.ruleService_.addRule(name, text);
        uiForm.setRenderedSibling(UIRuleList.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleForm uiForm = (UIRuleForm) event.getSource();
      uiForm.setRenderedSibling(UIRuleList.class);
    }
  }

}
