/*
 * Created on Apr 8, 2005
 */
package org.exoplatform.portlets.content.templates.component;

import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.NodeTypeManager;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.templates.TemplateService;

/**
 * @author benjaminmestrallet
 */
public class UINodeTypeForm extends UISimpleForm {

  static String NODE_TYPE_NAME = "nodetype".intern(), DIALOG = "dialog"
      .intern();
  static String VIEW = "view".intern();

  protected UITextArea view_;
  protected UITextArea dialog_;
  protected TemplateService templateService_;

  public UINodeTypeForm(TemplateService templateService) throws Exception {
    super("nodetypeForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    templateService_ = templateService;

    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }

  public void buildTree(Session session) throws Exception {
    NodeTypeManager nodetypeManager = session.getWorkspace()
        .getNodeTypeManager();
    NodeTypeIterator iterator = nodetypeManager.getAllNodeTypes();
    List options = new ArrayList();
    while (iterator.hasNext()) {
      NodeType nodeType = iterator.nextNodeType();
      String name = nodeType.getName();
      options.add(new SelectItem(name, name));
    }
    add(new UISelectBox(NODE_TYPE_NAME, "", options));
    dialog_ = new UITextArea(DIALOG, "");
    dialog_.setClazz("large");
    add(dialog_);
    view_ = new UITextArea(VIEW, "");
    view_.setClazz("large");
    add(view_);
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getComponent();
      String name = uiForm.getUISelectBox(NODE_TYPE_NAME).getValue();
      String dialog = uiForm.dialog_.getValue();
      String view = uiForm.view_.getValue();

      Information iprovider = findInformationProvider(uiForm);
      try {
        String[] roles = {"*"};
        uiForm.templateService_.addTemplate(true, name, TemplateService.DEFAULT_DIALOG, roles, dialog);
        uiForm.templateService_.addTemplate(false, name, TemplateService.DEFAULT_VIEW, roles, view);
        uiForm.setRenderedSibling(UINodeTypesList.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getComponent();
      uiForm.setRenderedSibling(UINodeTypesList.class);
    }
  }

}