package org.exoplatform.portlets.content.versions.component;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;

public class UIVersionInfo extends UIExoCommand implements ChangeNodeListener {

  public static final String ENABLE_VERSION_ACTION = "enableVersion".intern();
  public static final String SElECT_NODE_ACTION = "selectNode".intern();
  public static final String RESTORE_VERSION_ACTION = "restoreVersion".intern();
  public static final String VIEW_VERSION_ACTION = "viewVersion".intern();
  public static final String ADD_LABEL_ACTION = "addLabel".intern();
  public static final String DELETE_VERSION_ACTION = "deleteVersion".intern();
  public static final String DIFF_ACTION = "diff".intern();
  public static final String REMOVE_LABEL_ACTION = "removeLabel".intern();
  
  static Parameter[] SELECT_NODE_PARAMS = { new Parameter(ACTION,
      SElECT_NODE_ACTION) };
  static Parameter[] ENABLE_VERSION_PARAMS = { new Parameter(ACTION,
      ENABLE_VERSION_ACTION) };
  static Parameter[] RESTORE_VERSION_PARAMS = { new Parameter(ACTION,
      RESTORE_VERSION_ACTION) };
  static Parameter[] VIEW_VERSION_PARAMS = { new Parameter(ACTION,
      VIEW_VERSION_ACTION) };
  static Parameter[] ADD_LABEL_PARAMS = { new Parameter(ACTION,
      ADD_LABEL_ACTION) };
  static Parameter[] DELETE_VERSION_PARAMS = { new Parameter(ACTION,
      DELETE_VERSION_ACTION) };
  static Parameter[] DIFF_PARAMS = { new Parameter(ACTION,
      DIFF_ACTION) };
  static Parameter[] REMOVE_LABEL_PARAMS = { new Parameter(ACTION,
      REMOVE_LABEL_ACTION) };
  static Parameter[] BACK_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  
  protected Node node_;
  protected VersionNode rootVersion_;

  public UIVersionInfo() {
    setId("UIVersionInfo");
    setRendererType("VelocityRenderer");

    addActionListener(EnableVersionListener.class, ENABLE_VERSION_ACTION);
    addActionListener(SelectNodeActionListener.class, SElECT_NODE_ACTION);
    addActionListener(RestoreVersionActionListener.class, RESTORE_VERSION_ACTION);
    addActionListener(ViewVersionActionListener.class, VIEW_VERSION_ACTION);
    addActionListener(AddLabelActionListener.class, ADD_LABEL_ACTION);
    addActionListener(DeleteVersionActionListener.class, DELETE_VERSION_ACTION);
    addActionListener(DiffActionListener.class, DIFF_ACTION);
    addActionListener(RemoveLabelActionListener.class, REMOVE_LABEL_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
    rootVersion_ = null;
    setRendered(false);
  }

  public Parameter[] getEnableVersionParams() {
    return ENABLE_VERSION_PARAMS;
  }

  public Parameter[] getSelectNodeParams() {
    return SELECT_NODE_PARAMS;
  }

  public Parameter[] getRestoreVersionParams() {
    return RESTORE_VERSION_PARAMS;
  }  

  public Parameter[] getViewVersionParams() {
    return VIEW_VERSION_PARAMS;
  }  
  
  public Parameter[] getAddLabelParams() {
    return ADD_LABEL_PARAMS;
  }  

  public Parameter[] getRemoveVersionParams() {
    return DELETE_VERSION_PARAMS;
  }
  
  public Parameter[] getDiffParams() {
    return DIFF_PARAMS;
  }  
  
  public Parameter[] getRemoveLabelParams() {
    return REMOVE_LABEL_PARAMS;
  }    
    
  public boolean canEnableVersionning() throws Exception {
    return node_.canAddMixin("mix:versionable");
  }

  public boolean isVersionned() throws RepositoryException {
    return node_.isNodeType("mix:versionable");
  }

  public VersionHistory getVersionHistory() throws Exception {
    return node_.getVersionHistory();
  }

  public boolean isBaseVersion(Version version) throws RepositoryException {
    Version baseVersion = node_.getBaseVersion(); 
    if(baseVersion.getName().equals(version.getName()))
        return true;
    return false;
  }
  
  public boolean canViewTextDiff(Version version) throws RepositoryException {
    if(isBaseVersion(version))
      return false;
    
    if(node_.isNodeType("nt:file")) {
      Node content = node_.getNode("jcr:content");
      if(content.isNodeType("nt:resource")) {
        return true;
      }      
    }
    return false;
  }  
  
  public String[] getVersionLabels(Version version) throws Exception {
    VersionHistory vH = node_.getVersionHistory();
    return vH.getVersionLabels(version);
  }
  
  public VersionNode getRootVersion() {
    if (rootVersion_ == null) {
      VersionHistory vH;
      try {
        vH = node_.getVersionHistory();
        rootVersion_ = new VersionNode(vH.getRootVersion());
      } catch (RepositoryException e) {    
        return null;
      }      
    }
    return rootVersion_;
  }

  public Node getNode() {
    return node_;
  }

  static public class SelectNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIVersionInfo uicomp = (UIVersionInfo) event.getSource();
      String path = event.getParameter(OBJECTID);
      VersionNode node = uicomp.rootVersion_.findVersionNode(path);
      node.setExpanded(!node.isExpanded());
    }
  }

  static public class EnableVersionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIVersionInfo uicomp = (UIVersionInfo) event.getSource();
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.node_.addMixin("mix:versionable");
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class RestoreVersionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIVersionInfo uicomp = (UIVersionInfo) event.getSource();
      String path = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        VersionNode node = uicomp.rootVersion_.findVersionNode(path);
        uicomp.node_.restore(node.getVersion(), true);
        Object[] args = {node.getVersion().getName()};
        iprovider.addMessage(new ExoFacesMessage("#{UIVersionInfo.msg.version-restored}", args));
      } catch (Exception e) {
        e.printStackTrace();
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class ViewVersionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIVersionInfo uicomp = (UIVersionInfo) event.getSource();
      String path = event.getParameter(OBJECTID);
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(
          UIJCRExplorer.class);      
      Information iprovider = findInformationProvider(uicomp);
      try {
        VersionNode node = uicomp.rootVersion_.findVersionNode(path);
        Node node2view = node.getVersion().getNode("jcr:frozenNode");
        uiExplorer.setSelectNode(node2view);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class AddLabelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIVersionInfo uicomp = (UIVersionInfo) event.getSource();
      String path = event.getParameter(OBJECTID);
      VersionNode node = uicomp.rootVersion_.findVersionNode(path);
      UILabelForm labelForm = (UILabelForm) uicomp.getSibling(UILabelForm.class);
      labelForm.setVersion(node.getVersion());
      uicomp.setRenderedSibling(UILabelForm.class);
    }
  }

  static public class DeleteVersionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIVersionInfo uicomp = (UIVersionInfo) event.getSource();
      String path = event.getParameter(OBJECTID);      
      Information iprovider = findInformationProvider(uicomp);      
      try {
        VersionNode node = uicomp.rootVersion_.findVersionNode(path);        
        VersionHistory vH = uicomp.node_.getVersionHistory();
        vH.removeVersion(node.getVersion().getName());
        Object[] args = {node.getVersion().getName()};
        iprovider.addMessage(new ExoFacesMessage("#{UIVersionInfo.msg.version-removed}", args));        
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }

  static public class DiffActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIVersionInfo uicomp = (UIVersionInfo) event.getSource();
      String path = event.getParameter(OBJECTID);
      VersionNode node = uicomp.rootVersion_.findVersionNode(path);
      UIDiff diffcomp = (UIDiff) uicomp.getSibling(UIDiff.class);
      diffcomp.setVersions(uicomp.node_.getBaseVersion(), node.getVersion());
      uicomp.setRenderedSibling(UIDiff.class);
    }
  }  

  static public class RemoveLabelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIVersionInfo uicomp = (UIVersionInfo) event.getSource();
      UIRemoveLabelForm comp = (UIRemoveLabelForm) uicomp.getSibling(UIRemoveLabelForm.class);
      comp.setVersionHistory(uicomp.getVersionHistory());
      uicomp.setRenderedSibling(UIRemoveLabelForm.class);
    }
  }  
  
  
}
