package org.exoplatform.services.cms.actions;

import java.util.HashMap;
import java.util.Map;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.observation.Event;
import javax.jcr.observation.EventIterator;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.access.SystemIdentity;
import org.exoplatform.services.security.SecurityService;

public abstract class BaseActionLauncherListener implements ECMEventListener {

  protected String actionName_;
  protected String srcWorkspace_;
  protected String srcPath_;
  protected String executable_;
  protected Map actionVariables_;

  public BaseActionLauncherListener(String actionName, String executable,
      String srcWorkspace, String srcPath, Map actionVariables)
      throws Exception {
    actionName_ = actionName;
    executable_ = executable;
    srcWorkspace_ = srcWorkspace;
    srcPath_ = srcPath;
    actionVariables_ = actionVariables;
  }

  public String getSrcWorkspace() {
    return srcWorkspace_;
  }

  public void onEvent(EventIterator events) {
    PortalContainer pContainer = PortalContainer.getInstance();
    RepositoryService repositoryService = (RepositoryService) pContainer
        .getComponentInstanceOfType(RepositoryService.class);
    TemplateService templateService = (TemplateService) pContainer
        .getComponentInstanceOfType(TemplateService.class);
    ActionServiceContainer actionServiceContainer = (ActionServiceContainer) pContainer
        .getComponentInstanceOfType(ActionServiceContainer.class);
    SecurityService securityService = (SecurityService) pContainer
        .getComponentInstanceOfType(SecurityService.class);
    Session jcrSession = null;
    Node node = null;
    while (events.hasNext()) {
      Event event = events.nextEvent();
      String userId = event.getUserID();
      try {
        jcrSession = repositoryService.getRepository().getSystemSession(
            srcWorkspace_);
        node = (Node) jcrSession.getItem(srcPath_);
        Node actionNode = actionServiceContainer.getAction(node, actionName_);
        Property rolesProp = actionNode.getProperty("exo:roles");
        boolean hasPermission = false;
        Value[] roles = rolesProp.getValues();
        for (int i = 0; i < roles.length; i++) {
          String role = roles[i].getString();
          if (securityService.hasMembershipInGroup(userId, role) || SystemIdentity.SYSTEM.equals(userId)) {
            hasPermission = true;
            break;
          }
        }
        if (!hasPermission)
          return;
      } catch (Exception e) {
        e.printStackTrace();
        return;
      }

      String path = null;
      try {
        path = event.getPath();
        node = (Node) jcrSession.getItem(path);

        String nodeType = node.getPrimaryNodeType().getName();
        if (templateService.isNodeTypeConfigured(nodeType)) {
          Map<String, String> variables = new HashMap<String, String>();
          variables.put("initiator", userId);
          variables.put("actionName", actionName_);
          variables.put("document-type", nodeType);
          variables.put("nodePath", path);
          variables.put("srcWorkspace", srcWorkspace_);
          variables.put("srcPath", srcPath_);
          variables.putAll(actionVariables_);
          triggerAction(userId, variables);
        }
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }

  public abstract void triggerAction(String userId, Map variables)
      throws Exception;
}
