package org.exoplatform.services.cms.categories;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.Session;
import javax.jcr.Value;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.impl.Utils;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.picocontainer.Startable;

public class CategoriesServiceImpl implements CategoriesService, Startable {

  private static final String CATEGORY_MIXIN = "exo:categorized";
  private static final String CATEGORY_PROP = "exo:category";

//  private List categories_ ;
  private InitParams initParams_ ;
  private RepositoryService repositoryService_;
  private CmsConfigurationService cmsConfig_;

  public CategoriesServiceImpl(RepositoryService repositoryService,
    CmsConfigurationService cmsConfig, InitParams params) {
//    categories_ = params.getValuesParam("cmsCategories").getValues();
    initParams_ = params ;
    repositoryService_ = repositoryService;
    cmsConfig_ = cmsConfig;
  }

  public boolean hasCategories(Node node) throws Exception {
    if (node.isNodeType(CATEGORY_MIXIN))
      return true;
    return false;
  }

  public List<Node> getCategories(Node node) {
    List<Node> cats = new ArrayList<Node>();    
    try {
      Session session = getSystemSession();
      Property categories = node.getProperty("exo:category");
      Value[] values = categories.getValues();
      for (int i = 0; i < values.length; i++) {
        cats.add(session.getNodeByUUID(values[i].getString()));
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return cats;
  }

  public void removeCategory(Node node, String categoryPath) throws Exception {
    List<Value> vals = new ArrayList<Value>();
    if (!"*".equals(categoryPath)) {
	    Session session = getSystemSession();
	    Property categories = node.getProperty("exo:category");
	    Value[] values = categories.getValues();
	    String uuid2Remove = null;
	    for (int i = 0; i < values.length; i++) {
	      String uuid = values[i].getString();
	      Node refNode = session.getNodeByUUID(uuid);
	      if(refNode.getPath().equals(categoryPath)) {
	        uuid2Remove = uuid;              
	      } else {
	        vals.add(values[i]);
	      }
	    }
	    if(uuid2Remove == null)
	      return;
    }
    node.setProperty(CATEGORY_PROP, vals.toArray(new Value[vals.size()]));
  }

  public void addCategory(Node node, String categoryPath) throws Exception {
    Session session = getSystemSession();
    Node catNode = (Node) session.getItem(categoryPath);
    Value value2add = node.getSession().getValueFactory().createValue(catNode);      
    if (!node.isNodeType(CATEGORY_MIXIN)) {
      node.addMixin(CATEGORY_MIXIN);    
      node.setProperty(CATEGORY_PROP, new Value[] {value2add});
    } else {
      List<Value> vals = new ArrayList<Value>();
      Value[] values = node.getProperty(CATEGORY_PROP).getValues();
      for (int i = 0; i < values.length; i++) {
        Value value = values[i];
        String uuid = value.getString();
        Node refNode = session.getNodeByUUID(uuid);
        if(refNode.getPath().equals(categoryPath))
          return;
        vals.add(value);
      }
      vals.add(value2add);
      node.setProperty(CATEGORY_PROP, vals.toArray(new Value[vals.size()]));
    }
  }

  public void addCategory(Node node, String categoryPath, boolean replaceAll) throws Exception {
  	if (replaceAll) {
  		removeCategory(node, "*") ;
  	}
  	addCategory(node, categoryPath) ;
  }
  
  public void start() {
    Session session = null;
    Node categoriesHome = null;
    
    List categories ;
    String categoriesPath ; 
    String key ;
    
    for (Iterator keysIter = initParams_.keySet().iterator(); keysIter.hasNext();) {
    	categories = null ;
    	categoriesPath = null ;
    	key = (String) keysIter.next() ;
    	if (key.equals("cmsCategories")) {
    		categories = initParams_.getValuesParam("cmsCategories").getValues() ;
      	categoriesPath = cmsConfig_.getJcrPath(BasePath.CMS_CATEGORIES_PATH) ;
      }
      else {
      	if (key.equals("calendarCategories")) {
	        categories = initParams_.getValuesParam("calendarCategories").getValues() ;
	        categoriesPath = cmsConfig_.getJcrPath(BasePath.CALENDAR_CATEGORIES_PATH) ;
      	}
      }
      
      if (categories != null) {
        try {
          session = getSystemSession();
          categoriesHome = (Node) session.getItem(categoriesPath);
          if (categoriesHome.hasNodes())
            return;
          for (Iterator iter = categories.iterator(); iter.hasNext();) {
            String category = (String) iter.next();
            Node cat = Utils.makePath(categoriesHome, category, "nt:folder");
            cat.addMixin("mix:referenceable");
          }
          categoriesHome.save();
        } catch (Exception e) {
          e.printStackTrace();
        }
      }
    }
    //*********************************************
//    String categoriesPath = cmsConfig_.getJcrPath(BasePath.CMS_CATEGORIES_PATH);
//    try {
//      session = getSystemSession();
//      categoriesHome = (Node) session.getItem(categoriesPath);
//      if (categoriesHome.hasNodes())
//        return;
//      for (Iterator iter = categories_.iterator(); iter.hasNext();) {
//        String category = (String) iter.next();
//        Node cat = Utils.makePath(categoriesHome, category, "nt:folder");
//        cat.addMixin("mix:referenceable");
//      }
//      categoriesHome.save();
//    } catch (Exception e) {
//      e.printStackTrace();
//    }
  }

  public void stop() {
  }

  protected Session getSystemSession() throws Exception {
    ManageableRepository jcrRepository = repositoryService_.getRepository();
    return jcrRepository.getSystemSession(cmsConfig_.getWorkspace());
  }

}
