package org.exoplatform.portlets.calendar.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.calendar.Utils;
import org.exoplatform.portlets.calendar.component.UICalendar.displayLeftPanelListener;
import org.exoplatform.services.calendar.CalendarService;

public class UICalendarSelection extends UIExoCommand {
	
	public final static String SELECT_CALENDAR_ACTION = "SELECT_CALENDAR_ACTION" ;
	public final static String MANAGE_CALENDAR_ACTION = "MANAGE_CALENDAR_ACTION" ;
	
	public final static String SELECTED_CALENDAR_NAME_PARAM = "SELECTED_CALENDAR_NAME" ;
	public final static String SELECTED_CALENDAR_TYPE_PARAM = "SELECTED_CALENDAR_TYPE" ;
	
	public final static Parameter SELECT_CALENDAR_PARAM = new Parameter(ACTION, SELECT_CALENDAR_ACTION) ;
	public final static Parameter MANAGE_CALENDAR_PARAM = new Parameter(ACTION, MANAGE_CALENDAR_ACTION) ;
	private Parameter selectedCalendarNameParam_ = new Parameter(SELECTED_CALENDAR_NAME_PARAM, "") ;
	private Parameter selectedCalendarTypeParam_ = new Parameter(SELECTED_CALENDAR_TYPE_PARAM, "") ;

	private Parameter[] selectCalendarParams_ = {SELECT_CALENDAR_PARAM, selectedCalendarNameParam_, selectedCalendarTypeParam_} ;
	private Parameter[] manageCalendarParams_ = {MANAGE_CALENDAR_PARAM} ;
	
	public UICalendarSelection() throws Exception {
		setId("UICalendarSelection") ;
		setRendererType("VelocityRenderer") ;
		setClazz("UICalendarSelection") ;

		addActionListener(selectCalendarListener.class, SELECT_CALENDAR_ACTION) ;
		addActionListener(manageCalendarListener.class, MANAGE_CALENDAR_ACTION) ;
}
	
	public boolean isCurrentCalendar(String type, String name) {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		return uiCalendarPortlet.isCurrentCalendar(type, name) ;
	}
	
	public String getPrivateType() {
		return CalendarService.PRIVATE_CALENDAR_TYPE ;
	}
	
	public String getPublicType() {
		return CalendarService.PUBLIC_CALENDAR_TYPE ;
	}

	public Collection getPublicCalendarsName() {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		return Utils.getPublicCalendarsName(uiCalendarPortlet.getCurrentUser()) ;
	}

	public Collection getPrivateCalendarsName() {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		return Utils.getPrivateCalendarsName(uiCalendarPortlet.getCurrentUser()) ;
	}

	public Parameter[] getSelectCalendarParams() {
		return selectCalendarParams_;
	}
	
	public Parameter[] getManageCalendarParams() {
		return manageCalendarParams_;
	}

	public void setSelectedCalendarNameParam(String selectedCalendarNameParam) {
		this.selectedCalendarNameParam_.setValue(selectedCalendarNameParam) ;
	}

	public void setSelectedCalendarTypeParam(String selectedCalendarTypeParam) {
		this.selectedCalendarTypeParam_.setValue(selectedCalendarTypeParam) ;
	}

	static public class selectCalendarListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarSelection uiCalendarSelection = (UICalendarSelection) uiComponent ;
			String calandarName = event.getParameter(SELECTED_CALENDAR_NAME_PARAM) ;
			String calandarType = event.getParameter(SELECTED_CALENDAR_TYPE_PARAM) ;
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendarSelection.getAncestorOfType(UICalendarPortlet.class) ;
			uiCalendarPortlet.setCurrentCalendar(calandarName) ;
			uiCalendarPortlet.setCurrentCalendarType(calandarType) ;
			UICalendar uiCalendar = (UICalendar) uiCalendarPortlet.getChildComponentOfType(UICalendar.class) ;
			uiCalendar.setRendered(true) ;
		}
	}

	static public class manageCalendarListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarSelection uiCalendarSelection = (UICalendarSelection) uiComponent ;
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendarSelection.getAncestorOfType(UICalendarPortlet.class) ;
			uiCalendarPortlet.setRenderedComponent(UICalendarManagement.class) ;
		}
	}
}
