/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.forum.component;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.faces.FacesConstants;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.forum.component.model.ForumCategoryData;
import org.exoplatform.services.forum.Category;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIAdminViewCategories.java,v 1.7 2004/10/17 13:46:24 tuan08 Exp $
 */
public class UIAdminViewCategories extends UIExoCommand {
  private static String SELECT_CATEGORY = "selectCategory" ;
  private static String ADD_CATEGORY = "addCategory" ;
  private static String DELETE_CATEGORY = "deleteCategory" ;
  private static String EDIT_CATEGORY = "editCategory" ;
  private static String ADD_FORUM = "addForum" ;
  private static String EDIT_FORUM = "editForum" ;
  private static String DELETE_FORUM = "deleteForum" ;
  
  private static Parameter[] SELECT_CATEGORY_PARAMS = { new Parameter(ACTION, SELECT_CATEGORY)} ;
  private static Parameter[] ADD_CATEGORY_PARAMS = { new Parameter(FacesConstants.ACTION, ADD_CATEGORY)} ;
  private static Parameter[] DELETE_CATEGORY_PARAMS = {new Parameter(FacesConstants.ACTION, DELETE_CATEGORY)} ;
  private static Parameter[] EDIT_CATEGORY_PARAMS = {new Parameter(FacesConstants.ACTION, EDIT_CATEGORY)} ;
  
  private static Parameter[] ADD_FORUM_PARAMS = {new Parameter(FacesConstants.ACTION, ADD_FORUM)} ;
  private static Parameter[] DELETE_FORUM_PARAMS = {new Parameter(FacesConstants.ACTION, DELETE_FORUM)} ;
  private static Parameter[] EDIT_FORUM_PARAMS = {new Parameter(FacesConstants.ACTION, EDIT_FORUM)} ;       
  
  private List datas_ ;
  
  public UIAdminViewCategories(ForumService service) throws Exception {
    super() ;
    setId("UIAdminViewCategories") ;
    setRendererType("VelocityRenderer") ;
    
    addActionListener(SelectCategoryActionListener.class, SELECT_CATEGORY);
    addActionListener(AddCategoryActionListener.class, ADD_CATEGORY)  ;
    addActionListener(EditCategoryActionListener.class, EDIT_CATEGORY)  ;
    addActionListener(DeleteCategoryActionListener.class, DELETE_CATEGORY)  ;
    addActionListener(AddForumActionListener.class, ADD_FORUM)  ;
    addActionListener(EditForumActionListener.class, EDIT_FORUM)  ;
    addActionListener(DeleteForumActionListener.class, DELETE_FORUM)  ;
    update() ;
  }
  
  public Parameter[] getSelectCategoryParams() { return SELECT_CATEGORY_PARAMS ; }
  public Parameter[] getAddCategoryParams() { return ADD_CATEGORY_PARAMS ; }
  public Parameter[] getEditCategoryParams() { return EDIT_CATEGORY_PARAMS ; }
  public Parameter[] getDeleteCategoryParams() { return DELETE_CATEGORY_PARAMS ; }
  public Parameter[] getAddForumParams() { return ADD_FORUM_PARAMS ; }
  public Parameter[] getEditForumParams() { return EDIT_FORUM_PARAMS ; }
  public Parameter[] getDeleteForumParams() { return DELETE_FORUM_PARAMS ; }
  
  public List getCategoryDatas() { return datas_ ; }
  
  public void update() throws Exception {
    datas_ = new ArrayList() ;
    ForumService service = UIForumPortlet.getForumService() ;
    List categories = service.getCategories();
    for (int i = 0; i < categories.size(); i++) {
      Category category = (Category) categories.get(i) ;
      List forums = service.getForums(category.getId()) ;
      ForumCategoryData data = new ForumCategoryData(category, forums) ;
      datas_.add(data) ;
    }
  }
  
  static public class SelectCategoryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdminViewCategories uicomp = (UIAdminViewCategories) event.getComponent() ;
      String categoryId = event.getParameter(OBJECTID) ;
      for(int i = 0; i < uicomp.datas_.size(); i++) {
        ForumCategoryData data  = (ForumCategoryData) uicomp.datas_.get(i);
        Category category = data.getCategory() ;
        if(categoryId.equals(category.getId())) {
          data.setSelected(!data.isSelected()) ;
        }
      }
    }
  }
  
  static public class AddCategoryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdminViewCategories uiCategories = (UIAdminViewCategories) event.getComponent() ;
      UICategoryForm uiForm = (UICategoryForm) uiCategories.getSibling(UICategoryForm.class) ;
      uiForm.setCategory(null) ; 
      uiCategories.setRenderedSibling(UICategoryForm.class) ;
    }
  }
  
  static public class EditCategoryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdminViewCategories uiCategories = (UIAdminViewCategories) event.getComponent() ;
      UICategoryForm uiForm = (UICategoryForm) uiCategories.getSibling(UICategoryForm.class) ;
      ForumService service = UIForumPortlet.getForumService() ;
      String categoryId = event.getParameter(OBJECTID) ;
      Category category = service.getCategory(categoryId) ;
      uiForm.setCategory(category) ; 
      uiCategories.setRenderedSibling(UICategoryForm.class) ;
    }
  }
  
  static public class DeleteCategoryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdminViewCategories uiCategories = (UIAdminViewCategories) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      String categoryId = event.getParameter(OBJECTID) ;
      service.removeCategory(categoryId) ;
      uiCategories.update();
    }
  }
  
  static public class AddForumActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdminViewCategories uiCategories = (UIAdminViewCategories) event.getComponent() ;
      UIForumForm uiForm = (UIForumForm) uiCategories.getSibling(UIForumForm.class) ;
      String categoryId = event.getParameter(OBJECTID) ;
      uiForm.setForum(categoryId, null) ; 
      uiCategories.setRenderedSibling(UIForumForm.class) ;
    }
  }
  
  static public class EditForumActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdminViewCategories uiCategories = (UIAdminViewCategories) event.getComponent() ;
      UIForumForm uiForm = (UIForumForm) uiCategories.getSibling(UIForumForm.class) ;
      ForumService service = UIForumPortlet.getForumService() ;
      
      String forumId = event.getParameter(OBJECTID) ;
      Forum forum = service.getForum(forumId) ;
      uiForm.setForum(null, forum) ; 
      uiCategories.setRenderedSibling(UIForumForm.class) ;
    }
  }
  
  static public class DeleteForumActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdminViewCategories uiCategories = (UIAdminViewCategories) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      String forumId = event.getParameter(OBJECTID) ;
      service.removeForum(forumId) ;
      uiCategories.update();
    }
  }
  
}