/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.forum.component;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.Util;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.forum.ForumACL;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumService;
import org.exoplatform.services.forum.Topic;
import org.exoplatform.services.forum.Watcher;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UITopics.java,v 1.10 2004/10/20 18:46:31 benjmestrallet Exp $
 */
public class UITopics extends UIExoCommand {
  public  static final String TOPIC_ID = "topicId";
  private static final String NEW_TOPIC = "newTopic";
  private static final String VIEW_FORUMS = "viewForums";
  private static final String WATCH_TOPIC = "watchTopic";
  private static final String STOP_WATCH_TOPIC  = "stopWatchTopic";
  private static final String WATCH_FORUM = "watchForum";
  private static final String STOP_WATCH_FORUM  = "stopWatchForum";
  
  public static Parameter[] newTopicParams = { new Parameter(ACTION , NEW_TOPIC)} ;
  public static Parameter[] viewForumParams = { new Parameter(ACTION , VIEW_FORUMS)} ;
  public static Parameter[] watchForumParams = { new Parameter(ACTION , WATCH_FORUM)} ;
  public static Parameter[] stopWatchForumParams = { new Parameter(ACTION , STOP_WATCH_FORUM)} ;
  
  private static Parameter[] viewTopicParams = { new Parameter( ACTION , "viewPosts")} ;
  private static Parameter[] deleteTopicParams = {  new Parameter(ACTION , "deleteTopic") } ;
  private static Parameter[] watchTopicParams = {  new Parameter(ACTION , "watchTopic" )} ;
  private static Parameter[] stopWatchTopicParams = {  new Parameter(ACTION , "stopWatchTopic")} ;
  
  private Forum forum_ ;
  private ForumACL acl_ ;
  private UIPageListIterator uiPageIterator_ ;  
  private User user_ ;
//  private long lastAccessTime_ = System.currentTimeMillis() + 24*60*60*1000;//tomorrow todo change on more logical
	private Map visitedTopics_ = new HashMap();

	public UITopics(OrganizationService orgService,
                  ForumService service,ForumACL acl) throws Exception {
    //setRendererType("TopicsRenderer") ;
    setRendererType("VelocityRenderer") ;
    setId("UITopics") ;       
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(uiPageIterator_);
    
    acl_ = acl ;
    String userName = Util.getRemoteUser();
    if( userName != null ){
    	user_ = orgService.getUserHandler().findUserByName( userName );
    }
    
    addActionListener(ViewPostsActionListener.class, "viewPosts")  ;
    addActionListener(ViewForumsActionListener.class, VIEW_FORUMS)  ;
    addActionListener(NewTopicActionListener.class, NEW_TOPIC);
    addActionListener(DeleteTopicActionListener.class, "deleteTopic")  ;
    addActionListener(WatchTopicActionListener.class, WATCH_TOPIC)  ;
    addActionListener(StopWatchTopicActionListener.class, STOP_WATCH_TOPIC)  ;
    addActionListener(WatchForumActionListener.class, WATCH_FORUM)  ;
    addActionListener(StopWatchForumActionListener.class, STOP_WATCH_FORUM)  ;
  }
  
  public Parameter[] getViewTopicParams(){ return viewTopicParams; }
  public Parameter[] getDeleteTopicParams(){ return deleteTopicParams; }
  public Parameter[] getWatchTopicParams(){ return watchTopicParams; }
  public Parameter[] getStopWatchTopicParams(){ return stopWatchTopicParams; }
  public Parameter[] getNewTopicParams(){ return newTopicParams; }
  public Parameter[] getViewForumParams(){ return viewForumParams; }
  
  public boolean isModerator() { return acl_.hasModeratorRole(forum_) ; }
  public boolean hasViewForumRole() { return acl_.hasViewForumRole(forum_) ; }
  public boolean hasCreateTopicRole() { return acl_.hasCreateTopicRole(forum_) ; }
  public boolean hasReplyTopicRole() { return acl_.hasReplyTopicRole(forum_) ; }
  public User getUser() { return user_; }
  public Forum getForum() { return forum_ ; }
  
  public boolean setForum(String forumId) throws Exception {
    ForumService service = UIForumPortlet.getForumService() ;
    Forum forum = service.getForum(forumId) ;
    return setForum(forum) ;
  }
  
  public boolean setForum(Forum forum) throws Exception {
    if(!acl_.hasViewForumRole(forum)) return false ;
    forum_ = forum ; 
    ForumService service = UIForumPortlet.getForumService() ;
    uiPageIterator_.setPageList(service.getTopics(forum.getId())) ;
    return true ;
  }
  
  public void reload() throws Exception {
    if (forum_ == null) return ;
    ForumService service = UIForumPortlet.getForumService() ;
    uiPageIterator_.setPageList(service.getTopics(forum_.getId())) ;
  }

  public UIPageListIterator getUIPageIterator() { return uiPageIterator_ ; }  
  
  public boolean canDecodeInvalidState()  { return forum_ != null  ;}
  
	public void visit( String topicId) { visitedTopics_.put( topicId, new Date() ); }
  
  public boolean hasNewPosts( Topic topic ) {
    boolean retVal = false;
	  Date lastVisitDate = (Date)visitedTopics_.get( topic.getId() );
    if( lastVisitDate != null ) {
      //somebody post new message in during user session but user didn't visit yet this topic
      retVal = lastVisitDate.getTime()  < topic.getLastPostDate().getTime();
    } else if(  user_ != null && user_.getLastLoginTime().getTime() < topic.getLastPostDate().getTime() ){
      //somebody post new message after login of previous user session.
      retVal = true;
    }
	  return retVal;
  }
  
  public List getTopicLists(){
    return getUIPageIterator().getObjectInCurrentPage();
  }
  
  public void viewTopicDetail(String topicId, UIComponent fromComponent) throws Exception  {
    ForumService service = UIForumPortlet.getForumService() ;
    Topic topic = service.getTopic(topicId) ;
    UIPosts uiPosts = (UIPosts) getSibling(UIPosts.class) ;
    uiPosts.setUIPostsData(forum_, topic) ;          
    visit( topicId );
    UIForumPortlet forumPortlet = (UIForumPortlet)getAncestorOfType(UIForumPortlet.class);
    forumPortlet.addHistoryElement(fromComponent);      
    forumPortlet.setRenderedComponent(UIPosts.class);   
  }
  
  static public class ViewPostsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITopics uiTopics = (UITopics) event.getComponent() ;
      String topicId = event.getParameter(OBJECTID) ;
      uiTopics.viewTopicDetail(topicId, uiTopics) ;
    }
  }
  
  static public class ViewForumsActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UITopics uiTopics = (UITopics) event.getComponent() ;			
      UIForumPortlet forumPortlet = (UIForumPortlet)uiTopics.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiTopics);      
      forumPortlet.setRenderedComponent(UIViewCategories.class); 
		}
  }
  
  static public class NewTopicActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UITopics uiTopics = (UITopics) event.getComponent() ;
			UIPostForm uiForm = (UIPostForm) uiTopics.getSibling(UIPostForm.class) ;
			uiForm.setNewTopicInfo(uiTopics.forum_) ;	    
      UIForumPortlet forumPortlet = (UIForumPortlet)uiTopics.getAncestorOfType(UIForumPortlet.class);
      forumPortlet.addHistoryElement(uiTopics);      
      forumPortlet.setRenderedComponent(UIPostForm.class);   
		}
  }
  
  static public class DeleteTopicActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UITopics uiTopics = (UITopics) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
			if(!uiTopics.acl_.hasModeratorRole(uiTopics.forum_)) return ;
			String topicId = event.getParameter(OBJECTID) ;
      service.removeTopic(topicId) ;
      uiTopics.reload() ;
		}
  }
  
  static public class WatchTopicActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UITopics uiTopics = (UITopics) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      User user = uiTopics.getUser() ;      
      Topic topic = service.getTopic(event.getParameter(OBJECTID)) ;
      Watcher watcher = service.getWatcher(topic, user.getUserName()) ;      
      if(watcher == null) {
        watcher = service.createWatcher(topic) ;
        watcher.setUserName(user.getUserName()) ;
        watcher.setAddress(user.getEmail()) ;        
      } 
      UIWatchForm uiForm = (UIWatchForm) uiTopics.getSibling(UIWatchForm.class) ;
      uiForm.setWatcher(watcher) ;
      uiTopics.setRenderedSibling(UIWatchForm.class) ;
    }
  }
  
  static public class StopWatchTopicActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITopics uiTopics = (UITopics) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      Topic topic = service.getTopic(event.getParameter(OBJECTID)) ;
      Watcher watcher = service.getWatcher(topic, uiTopics.user_.getUserName()) ;
      Information information = findInformationProvider(uiTopics);
      if(watcher  != null) {
        service.removeWatcher(watcher) ;
        Object[] args = {topic.getTopic()} ;
        information.addMessage(new ExoFacesMessage("#{UITopics.msg.stop-topic-watch-success}", args)) ;
      } else {
        information.addMessage(new ExoFacesMessage("#{UITopics.msg.stop-topic-watch-fail}")) ;
      }
    }
  }
  
  static public class WatchForumActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITopics uiTopics = (UITopics) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      User user = uiTopics.getUser();
      Watcher watcher = service.getWatcher(uiTopics.forum_, user.getUserName()) ;
      if(watcher == null) {
        watcher = service.createWatcher(uiTopics.forum_) ;
        watcher.setUserName(user.getUserName()) ;
        watcher.setAddress(user.getEmail()) ;
      } 
      UIWatchForm uiForm = (UIWatchForm) uiTopics.getSibling(UIWatchForm.class) ;
      uiForm.setWatcher(watcher) ;
      uiTopics.setRenderedSibling(UIWatchForm.class) ;
    }
  }
  
  static public class StopWatchForumActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITopics uiTopics = (UITopics) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      Watcher watcher = service.getWatcher(uiTopics.forum_, uiTopics.getUser().getUserName()) ;
      Information information = findInformationProvider(uiTopics);
      if(watcher  != null) {
        service.removeWatcher(watcher) ;
        Object[] args = {uiTopics.forum_.getForumName()} ;
        information.addMessage(new ExoFacesMessage("#{UITopics.msg.stop-forum-watch-success}", args)) ;
      } else {
        information.addMessage(new ExoFacesMessage("#{UITopics.msg.stop-forum-watch-fail}")) ;
      }
    }
  }
}