/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.forum.component;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.forum.component.model.ForumCategoryData;
import org.exoplatform.services.forum.Category;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIViewCategories.java,v 1.8 2004/10/20 18:46:31 benjmestrallet Exp $
 */
public class UIViewCategories extends UIExoCommand {
  private static String SELECT_CATEGORY = "selectCategory" ;
  private static String SELECT_FORUM = "selectForum" ;
  private static Parameter[] SELECT_CATEGORY_PARAMS = { new Parameter(ACTION, SELECT_CATEGORY)} ;
  private static Parameter[] SELECT_FORUM_PARAMS = { new Parameter(ACTION, SELECT_FORUM)} ;
  
  private List datas_ ;
  
  public UIViewCategories(ForumService service) throws Exception {
    setId("UIViewCategories") ;
    setRendererType("VelocityRenderer") ;
    addActionListener(SelectCategoryActionListener.class, SELECT_CATEGORY);
    addActionListener(ViewTopicsActionListener.class, SELECT_FORUM);
    buildTree() ;
  }
  
  public Parameter[] getSelectCategoryParams() { return SELECT_CATEGORY_PARAMS ; }
  public Parameter[] getSelectForumParams() { return SELECT_FORUM_PARAMS ; }
  
  private void buildTree() throws Exception {
    datas_ = new ArrayList() ;
    ForumService service = UIForumPortlet.getForumService() ;
    List categories = service.getCategories();
    for (int i = 0; i < categories.size(); i++) {
      Category category = (Category) categories.get(i) ;
      List forums = service.getForums(category.getId()) ;
      ForumCategoryData data = new ForumCategoryData(category, forums) ;
      datas_.add(data) ;
    }
  }
  
  public List getCategoryDatas() { return datas_ ; }
  
  public boolean hasCategory() { return datas_.size() > 0 ; }
  
  static public class SelectCategoryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewCategories uicomp = (UIViewCategories) event.getComponent() ;
      String categoryId = event.getParameter(OBJECTID) ;
      for(int i = 0; i < uicomp.datas_.size(); i++) {
        ForumCategoryData data  = (ForumCategoryData) uicomp.datas_.get(i);
        Category category = data.getCategory() ;
        if(categoryId.equals(category.getId())) {
          data.setSelected(!data.isSelected()) ;
        }
      }
    }
  }
  
  static public class ViewTopicsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewCategories uicomp = (UIViewCategories) event.getComponent() ;
      String forumId = event.getParameter(OBJECTID) ;
      for(int i = 0; i < uicomp.datas_.size(); i++) {
        ForumCategoryData data  = (ForumCategoryData) uicomp.datas_.get(i);
        Forum forum = data.getForum(forumId) ;
        if(forum != null) {
          UIForumPortlet uiController = 
            (UIForumPortlet)uicomp.getAncestorOfType(UIForumPortlet.class) ;
          UITopics uiTopics = (UITopics) uiController.getChildComponentOfType(UITopics.class) ;
          if(uiTopics.setForum(forum)) {
            uiController.addHistoryElement(uicomp.getParent());
            uiController.setRenderedComponent(UITopics.class) ;   
          } else {
            Information information = findInformationProvider(uicomp) ;            
            information.addMessage(new ExoFacesMessage("#{UIViewCategories.msg.no-view-forum-role}"));
          }
        }
      }                          
    }
  }
}