/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.forum.component;

import org.apache.lucene.document.Document;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.search.component.UISearchSummary;
import org.exoplatform.portal.Information;
import org.exoplatform.services.forum.ForumIndexerPlugin;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.indexing.IndexerPlugin;
import org.exoplatform.services.indexing.IndexingService;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id: UIViewDocument.java,v 1.2 2004/11/03 01:20:37 tuan08 Exp $
 */
public class UIViewSearchResult extends UIExoCommand {
  public final static String VIEW_TOPIC = "viewTopic" ;
  public static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION, CANCEL_ACTION)};
  public static Parameter[] VIEW_TOPIC_PARAMS = { new Parameter(ACTION, VIEW_TOPIC)};

  private String text_ ;
  private Document doc_ ;
  private IndexingService iservice_ ;
  private WikiEngineService weService_ ;
  
  public UIViewSearchResult(IndexingService iservice) {
  	setId("UIViewSearchResult") ;
    setRendererType("VelocityRenderer") ;
    iservice_ = iservice ;
    weService_ = 
      (WikiEngineService) PortalContainer.getComponent(WikiEngineService.class) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(ViewTopicActionListener.class, VIEW_TOPIC) ;
  }
  
  public Parameter[] getCancelParams() { return CANCEL_PARAMS ; }
  public Parameter[] getViewTopicParams() { return VIEW_TOPIC_PARAMS ; }
  
  public String getDocument() { return text_ ; }
  
  public void setDocument(Document doc) throws Exception {
    doc_ = doc ;
    String module = doc.getField(IndexingService.MODULE_FIELD).stringValue() ;
    IndexerPlugin plugin = iservice_.getIndexerPlugin(module) ;
    text_  =  plugin.getObjectAsText(null, doc.getField(IndexingService.IDENTIFIER_FIELD).stringValue());
    text_ = weService_.toXHTML(text_) ;
  }

  static public class ViewTopicActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewSearchResult uiView =  (UIViewSearchResult) event.getSource(); 
      String topicId = 
        uiView.doc_.getField(ForumIndexerPlugin.TOPIC_ID_FIELD).stringValue() ;
      String forumId = 
        uiView.doc_.getField(ForumIndexerPlugin.FORUM_ID_FIELD).stringValue() ;
      UIForumSearcher uiSearcher = (UIForumSearcher) uiView.getParent() ;
      UIForumPortlet uiPortlet = 
        (UIForumPortlet) uiView.getAncestorOfType(UIForumPortlet.class) ;
      UITopics uiTopics = (UITopics) uiPortlet.getChildComponentOfType(UITopics.class);
      if(uiTopics.setForum(forumId)) {
        uiTopics.viewTopicDetail(topicId, uiSearcher) ;
      } else {
        Information information = findInformationProvider(uiView) ;
        information.addMessage(new ExoFacesMessage("#{UIViewCategories.msg.no-view-forum-role}"));
      }
    }
  } 
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewSearchResult uiView =  (UIViewSearchResult) event.getSource();
      uiView.setRenderedSibling(UISearchSummary.class) ;      
    }
  } 
}