/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.pmanager.component;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.pmanager.Constant;
import org.exoplatform.services.pmanager.Project;
import org.exoplatform.services.pmanager.ProjectACL;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id$
 */
public class UIDocuments extends UIExoCommand {
  static public String CREATE_DOCUMENT = "createDocument".intern() ;  
  static public String VIEW_DOCUMENT = "viewDocument".intern() ;  
  static public String REMOVE_DOCUMENT = "removeDocument".intern() ;  
  static public String UPLOAD_DOCUMENT = "uploadDocument".intern() ;  
  
  static Parameter[] CREATE_DOCUMENT_PARAMS = { new Parameter(ACTION , CREATE_DOCUMENT)} ;
  static Parameter[] VIEW_DOCUMENT_PARAMS = { new Parameter(ACTION , VIEW_DOCUMENT)} ;
  static Parameter[] REMOVE_DOCUMENT_PARAMS = { new Parameter(ACTION , REMOVE_DOCUMENT)} ;
  static Parameter[] UPLOAD_DOCUMENT_PARAMS = { new Parameter(ACTION , UPLOAD_DOCUMENT)} ;
  static Parameter[] BACK_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  
  Project project_ ;
  Class backComponent_ ;
  
  public UIDocuments() throws Exception {
    setId("UIDocuments") ;
    setRendererType("VelocityRenderer") ;
    addActionListener(BackActionListener.class, CANCEL_ACTION)  ;
    addActionListener(CreateDocumentActionListener.class, CREATE_DOCUMENT)  ;
    addActionListener(UploadDocumentActionListener.class, UPLOAD_DOCUMENT)  ;
    addActionListener(ViewDocumentActionListener.class, VIEW_DOCUMENT)  ;
    addActionListener(RemoveDocumentActionListener.class, REMOVE_DOCUMENT)  ;
  }
 
  public Parameter[] getViewDocumentParams() { return VIEW_DOCUMENT_PARAMS ; }
  public Parameter[] getCreateDocumentParams() { return CREATE_DOCUMENT_PARAMS ; }
  public Parameter[] getRemoveDocumentParams() { return REMOVE_DOCUMENT_PARAMS ; }
  public Parameter[] getUploadDocumentParams() { return UPLOAD_DOCUMENT_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  
  public ProjectACL getProjectACL() { return project_.getProjectACL() ; }
  
  public void setData(Project project, Class backComponent) throws Exception { 
    project_ = project ;
    backComponent_ = backComponent ;
  } 
 
  public NodeIterator getDocumentIterator() throws Exception {
    return project_.getDocumentIterator() ;
  }
  
  public String getOwner(Node node) throws Exception {
    return node.getProperty(Constant.DOCUMENT_OWNER).getString() ;
  }
  
  public String getMimeType(Node node) throws Exception {
    return node.getProperty(Constant.DOCUMENT_MIMETYPE).getString() ;
  }
  
  public String getDescription(Node node) throws Exception {
    if(!node.hasProperty("exo:description")) return "No Description" ;
    return node.getProperty("exo:description").getString() ;
  }
  
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocuments uicomp = (UIDocuments) event.getComponent() ;
      UIDocumentSearcher uiSearcher =
        (UIDocumentSearcher) uicomp.getAncestorOfType(UIDocumentSearcher.class) ;
      uiSearcher.setRenderedSibling(uicomp.backComponent_) ;
    }
  }  
  
  static public class CreateDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocuments uicomp = (UIDocuments) event.getComponent() ;
      UIDocumentSearcher uiSearcher =
        (UIDocumentSearcher) uicomp.getAncestorOfType(UIDocumentSearcher.class) ;
      UIDocumentForm uiForm = 
        (UIDocumentForm) uiSearcher.getSibling(UIDocumentForm.class);
      uiForm.setData(uicomp.project_.getDocuments(), UIDocumentSearcher.class) ;
      uiSearcher.setRenderedSibling(UIDocumentForm.class) ;
    }
  }  
  
  static public class UploadDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocuments uicomp = (UIDocuments) event.getComponent() ;
      UIDocumentSearcher uiSearcher =
        (UIDocumentSearcher) uicomp.getAncestorOfType(UIDocumentSearcher.class) ;
      UIUploadForm uiForm = 
        (UIUploadForm) uiSearcher.getSibling(UIUploadForm.class);
      Node node = uicomp.project_.getDocuments() ;
      uiForm.setData(UIUploadForm.UPLOAD_DOCUMENT, node, UIDocumentSearcher.class) ;
      uiSearcher.setRenderedSibling(UIUploadForm.class) ;
    }
  }  
  
  static public class ViewDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocuments uicomp = (UIDocuments) event.getComponent() ;
      UIProjectManagerPortlet uiPortlet =
        (UIProjectManagerPortlet) uicomp.getAncestorOfType(UIProjectManagerPortlet.class) ;
      UIDocument uiDocument = (UIDocument) uiPortlet.getChildComponentOfType(UIDocument.class);
      String name = event.getParameter(OBJECTID) ;
      Node document = uicomp.project_.getDocument(name) ;
      uiDocument.setData(document, UIDocumentSearcher.class) ;
      uiPortlet.setRenderedComponent(UIDocument.class) ;
    }
  }  
  
  
  static public class RemoveDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocuments uicomp = (UIDocuments) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      uicomp.project_.removeDocument(name) ;
    }
  }
}