/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.pmanager.component;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.pmanager.Constant;
import org.exoplatform.services.pmanager.Project;
import org.exoplatform.services.pmanager.ProjectACL;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id$
 */
public class UIIssues extends UIExoCommand {
  static public String CREATE_ISSUE = "createIssue".intern() ;  
  static public String VIEW_ISSUE = "viewDocument".intern() ;  
  static public String REMOVE_ISSUE = "removeDocument".intern() ;    
  
  static Parameter[] BACK_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  static Parameter[] CREATE_ISSUE_PARAMS = { new Parameter(ACTION , CREATE_ISSUE)} ;
  static Parameter[] VIEW_ISSUE_PARAMS = { new Parameter(ACTION , VIEW_ISSUE)} ;
  static Parameter[] REMOVE_ISSUE_PARAMS = { new Parameter(ACTION , REMOVE_ISSUE)} ;
  
  Project project_ ;
  Class backComponent_ ;
  
  public UIIssues() throws Exception {
    setId("UIIssues") ;
    setRendererType("VelocityRenderer") ;
    addActionListener(BackActionListener.class, CANCEL_ACTION)  ;
    addActionListener(CreateIssueActionListener.class, CREATE_ISSUE)  ;
    addActionListener(ViewIssueActionListener.class, VIEW_ISSUE)  ;
    addActionListener(RemoveIssueActionListener.class, REMOVE_ISSUE)  ;
  }
 
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  public Parameter[] getViewIssueParams() { return VIEW_ISSUE_PARAMS ; }
  public Parameter[] getCreateIssueParams() { return CREATE_ISSUE_PARAMS ; }
  public Parameter[] getRemoveIssueParams() { return REMOVE_ISSUE_PARAMS ; }
  
  public ProjectACL getProjectACL() { return project_.getProjectACL() ; }
  
  public void setData(Project project, Class backComponent) throws Exception { 
    project_ = project ;
    backComponent_ = backComponent ;
  } 
 
  public NodeIterator getIssueIterator() throws Exception {
    return project_.getIssueIterator() ;
  }
  
  public String getCreator(Node node) throws Exception {
    return node.getProperty(Constant.ISSUE_CREATOR).getString() ;
  }
  
  public String getType(Node node) throws Exception {
    return node.getProperty("exo:type").getString() ;
  }
  
  public String getSummary(Node node) throws Exception {
    return node.getProperty("exo:summary").getString() ;
  }
  
  public String getStatus(Node node) throws Exception {
    return node.getProperty("exo:status").getString() ;
  }
  
  public String getAssignee(Node node) throws Exception {
    return node.getProperty("exo:assignee").getString() ;
  }
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIIssues uicomp = (UIIssues) event.getComponent() ;
      UIIssueSearcher uiSearcher =
        (UIIssueSearcher) uicomp.getAncestorOfType(UIIssueSearcher.class) ;
      uiSearcher.setRenderedSibling(uicomp.backComponent_) ;
    }
  }  
  
  static public class CreateIssueActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIIssues uicomp = (UIIssues) event.getComponent() ;
      UIIssueSearcher uiSearcher =
        (UIIssueSearcher) uicomp.getAncestorOfType(UIIssueSearcher.class) ;
      UIIssueForm uiForm = (UIIssueForm) uiSearcher.getSibling(UIIssueForm.class);
      uiForm.addNewIssue(uicomp.project_, UIIssueSearcher.class) ;
      uiSearcher.setRenderedSibling(UIIssueForm.class) ;
    }
  }  
  
  static public class ViewIssueActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIIssues uicomp = (UIIssues) event.getComponent() ;
      UIProjectManagerPortlet uiPortlet =
        (UIProjectManagerPortlet) uicomp.getAncestorOfType(UIProjectManagerPortlet.class) ;
      UIIssue uiIssue = (UIIssue) uiPortlet.getChildComponentOfType(UIIssue.class);
      String name = event.getParameter(OBJECTID) ;
      Node issue = uicomp.project_.getIssue(name) ;
      uiIssue.setData(uicomp.project_, issue, UIIssueSearcher.class) ;
      uiPortlet.setRenderedComponent(UIIssue.class) ;
    }
  }  
  
  static public class RemoveIssueActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIIssues uicomp = (UIIssues) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      uicomp.project_.removeIssue(name) ;
      uicomp.project_.save() ;
    }
  }  
}