/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.forum.hibernate;

import java.io.PrintWriter;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.hibernate.Query;
import org.hibernate.Session;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Nov 21, 2004
 * @version $Id$
 */
public class ForumDataTransformer {
  private Session session_ ;
  private ZipOutputStream out_ ;
  private DataTransformerImpl transformer_ ;
  private Template template_ ;
  private PrintWriter writer_ ;
  private VelocityContext categoryContext_ ;
  private VelocityContext forumContext_ ;
  private VelocityContext topicContext_ ;
  
  public ForumDataTransformer(DataTransformerImpl transformer ,
                              Session session, ZipOutputStream out) throws Exception {
    session_ = session ;
    out_ = out ;
    transformer_ = transformer ;
    template_ = transformer.getServiceTemplate() ;
    writer_ = new PrintWriter(out);
    categoryContext_ = new VelocityContext() ;
    forumContext_ = new VelocityContext() ;
    topicContext_ = new VelocityContext() ;
  }
  
  public void tranform() throws Exception {
    Query q = session_.createQuery(ForumServiceContainerImpl.queryForumOwners);
    List owners = q.list() ;
    for(int i = 0; i < owners.size(); i++ ) {
      String owner = (String) owners.get(i) ;
      transform(owner) ;
    }
  }
  
  public void transform(String owner) throws Exception {
    List  categories = session_.createQuery(ForumServiceImpl.queryCategoriesByOwner)
	.setString(0, owner).list();
	//find(ForumServiceImpl.queryCategoriesByOwner, 
    //                                 owner, Hibernate.STRING );
    for(int i = 0; i < categories.size(); i++) {
      CategoryImpl cat = (CategoryImpl) categories.get(i) ;
      transformCategory(owner, cat) ; 
    }
  }
  
  public void transformCategory(String path ,CategoryImpl category) throws Exception {
    String categoryPath = path + "/"  + category.getId() ;
    ZipEntry entry = new ZipEntry(categoryPath + "/" + category.getId() +  
                                  "-category" + transformer_.getServiceEntryExtension()) ;
    out_.putNextEntry(entry) ;
    categoryContext_.put("category", category) ;
    template_.merge(categoryContext_, writer_) ;
    writer_.flush() ;
    out_.closeEntry() ;
    List forums = session_.createQuery(ForumServiceImpl.queryForumsByCategory)
	.setString(0, category.getId()).list();
	//.find(ForumServiceImpl.queryForumsByCategory, category.getId(), Hibernate.STRING );
    for(int i = 0; i < forums.size(); i++) {
      ForumImpl forum = (ForumImpl)  forums.get(i) ;
      transformForum(categoryPath ,forum) ;
    }
  }
  
  public void transformForum(String path ,ForumImpl forum) throws Exception {
    String forumPath  =  path + "/"  +forum.getId(); 
    ZipEntry entry = new ZipEntry(forumPath+ "/" + forum.getId() +  
                                  "-forum" + transformer_.getServiceEntryExtension()) ;
    out_.putNextEntry(entry) ;
    forumContext_.put("forum", forum) ;
    template_.merge(forumContext_, writer_) ;
    writer_.flush() ;
    out_.closeEntry() ;
    String queryTopicsByForum =
      "from topic in class org.exoplatform.services.forum.hibernate.TopicImpl " +
      "where topic.forumId = ?";
    List topics = session_.createQuery(queryTopicsByForum).setString(0, forum.getId()).list();
	//.find(queryTopicsByForum, forum.getId(), Hibernate.STRING );
    for(int i = 0; i < topics.size();  i++) {
      TopicImpl topic = (TopicImpl) topics.get(i) ;
      transformTopic(forumPath, topic) ;
    }
  }
  
  public void transformTopic(String path ,TopicImpl topic) throws Exception {
    ZipEntry entry = new ZipEntry(path + "/" + topic.getId() +  
                                  "-topic" + transformer_.getServiceEntryExtension()) ;
    out_.putNextEntry(entry) ;
    List  posts = session_.createQuery(ForumServiceImpl.queryPostsByTopic).
	setString(0, topic.getId()).list();
	//.find(ForumServiceImpl.queryPostsByTopic, topic.getId(), 
    //                            Hibernate.STRING) ;
    TopicBackup backup = new TopicBackup(topic, posts) ;
    topicContext_.put("topic", backup) ;
    template_.merge(topicContext_, writer_) ;
    writer_.flush() ;
    out_.closeEntry() ;
  }
}
