/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.forum.test;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.apache.lucene.document.Document;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.forum.Category;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumIndexerPlugin;
import org.exoplatform.services.forum.ForumServiceContainer;
import org.exoplatform.services.forum.Post;
import org.exoplatform.services.forum.Topic;
import org.exoplatform.services.forum.hibernate.ForumIndexerPluginImpl;
import org.exoplatform.services.indexing.IndexingService;
import org.exoplatform.services.indexing.MultipleFieldSearchInput;
import org.exoplatform.services.indexing.RangeFieldSearchInput;
import org.exoplatform.services.indexing.Searcher;
import org.exoplatform.services.indexing.SingleFieldSearchInput;

/**
 * Created by The eXo Platform SARL Author : Tuan Nguyen
 * tuan08@users.sourceforge.net Dec 15, 2005
 */
public class TestForumIndexing extends ForumServiceTestCase {
  private IndexingService iservice_;  
  
  public TestForumIndexing(String name) {
    super(name);
  }
  
  public void setUp() throws Exception {
    if (service_ == null) {
      PortalContainer manager = PortalContainer.getInstance();
      container_ = (ForumServiceContainer)  manager.getComponentInstanceOfType(ForumServiceContainer.class);
      iservice_ = (IndexingService) manager.getComponentInstanceOfType(IndexingService.class);      
      service_ = container_.findForumService("exo");
    }
  }
  
  public void tearDown() throws Exception {
  }
  
  public void testForumIndexing() throws Exception {
    System.out.println("############ WARNING ########## \n you MUST REMOVE lucene database " +
        "dictory(/tmp/lucene/*) before run this test to make sure it runs right") ;    
    Category supportCat = 
      createCategory("Support Category","this is the customer support category");
    Forum exoConfigForum = 
      createForum(supportCat, "eXo Config Forum", "discussion about eXo Configuaration");
    Topic configIntro = 
      createTopic(exoConfigForum, "configIntro","this is topic about introduction exo Configuaration");
    Topic configBug = createTopic(exoConfigForum, "configBug", "bug configuaration topic");
    Date d = createDate(2005, GregorianCalendar.SEPTEMBER, 2, 10, 30, 30);
    Post post = 
      createPost(configBug, "some bug when config exo with tomcat", "this is the test", "hoapx",d);   
    d = createDate(2005, GregorianCalendar.SEPTEMBER, 2, 11, 0, 0);
    Post testpost = 
      createPost(configIntro, " some bugs when configuration with exo", "test","exo portal",d);
    d = createDate(2005, GregorianCalendar.OCTOBER, 1, 10, 0, 0);
    post = createPost(configBug, " help me debug..", "Help me debug", "community",d);
    
    assertTrue("expect in configIntro topic has 1 post", 
        service_.getPosts(configIntro.getId()).getAll().size() == 1);
    assertTrue("expect in configBug topic have 2 posts", 
        service_.getPosts(configBug.getId()).getAll().size() == 2);    
    
    ForumIndexerPluginImpl plugin = (ForumIndexerPluginImpl) iservice_
    .getIndexerPlugin(ForumIndexerPlugin.IDENTIFIER);    
    Thread.sleep(3000);        
    assertTrue("expect found forum indexer plugin", plugin != null);
    //---Search by single search input
    Searcher searcher = plugin.getSearcher();
    SingleFieldSearchInput singleSearchInput = 
      new SingleFieldSearchInput(ForumIndexerPlugin.SUBJECT_FIELD);
    singleSearchInput.setTerm("+bug*");
    List hits = searcher.search(singleSearchInput).getAll();
    assertEquals("expect found 2 post", 2, hits.size());
    assertTrue("expect found post", hits.size() > 0);
    Document foundDoc = null;
    for (int i = 0; i < hits.size(); i++) {
      foundDoc = (Document) hits.get(i);
      assertTrue("expect found term in SUBJECT_FIELD of document", 
          foundDoc.getField(ForumIndexerPlugin.SUBJECT_FIELD).stringValue().contains("bug"));
    }
    //-----search by range search input---
    Date lowerTerm = createDate(2005, GregorianCalendar.SEPTEMBER, 1, 10, 0, 0);    
    Date upperTerm = createDate(2005, GregorianCalendar.SEPTEMBER, 3, 11, 0, 0);    
    RangeFieldSearchInput rangeSearchInput = 
      new RangeFieldSearchInput(ForumIndexerPlugin.DATE_FIELD, lowerTerm, upperTerm);                
    searcher = plugin.getSearcher();    
    hits = searcher.search(rangeSearchInput).getAll();
    //assertEquals("expect found 2 document",2,hits.size()) ;
    System.out.println("========Size===" + hits.size());
    for (int i = 0; i < hits.size(); i++) {
      foundDoc = (Document) hits.get(i);      
      System.out.println("==========SUBJECT_FIELD=="+ 
          foundDoc.getField(ForumIndexerPlugin.SUBJECT_FIELD).stringValue());
      System.out.println("==========AUTHOR_FIELD==="  + 
          foundDoc.getField(ForumIndexerPlugin.AUTHOR_FIELD).stringValue());
    }        
    // ----search by multi search inputs-----
    List fields = new ArrayList();
    fields.add(ForumIndexerPlugin.AUTHOR_FIELD);
    fields.add(ForumIndexerPlugin.SUBJECT_FIELD);
    MultipleFieldSearchInput multiSearchInput = new MultipleFieldSearchInput(fields);
    multiSearchInput.setTerm("+exo*");
    searcher = plugin.getSearcher();
    hits = searcher.search(multiSearchInput).getAll();
    assertEquals("expec found 2 post have 'exo' in AUTHOR_FIELD or SUBJECT_FIELD ", 2, hits.size());
    // ---------search by multi input with match all field
    multiSearchInput.setMatchAllFields(true);
    searcher = plugin.getSearcher();
    hits = searcher.search(multiSearchInput).getAll();
    assertTrue("expect found 1 document", hits.size() == 1);
    foundDoc = (Document) hits.get(0);
    assertTrue("expect to found 'exo' in AUTHOR_FIELD ", 
        foundDoc.getField(ForumIndexerPlugin.AUTHOR_FIELD).stringValue().contains("exo"));
    assertTrue("expect to found 'exo' in AUTHOR_FIELD ", 
        foundDoc.getField(ForumIndexerPlugin.SUBJECT_FIELD).stringValue().contains("exo"));           
    // --- search with combination single input and multi input
    multiSearchInput.setMatchAllFields(false);
    singleSearchInput.setRequired(true);
    List inputs = new ArrayList();
    inputs.add(singleSearchInput);
    inputs.add(multiSearchInput);
    searcher = plugin.getSearcher();
    hits = searcher.search(inputs).getAll();
    assertTrue("expect to found 2 post", hits.size() == 2);
    for (int i = 0; i < hits.size(); i++) {
      foundDoc = (Document) hits.get(i);
      assertTrue("expect found 'exo' in SUBJECT_FIELD", 
          foundDoc.getField(ForumIndexerPlugin.SUBJECT_FIELD).stringValue().contains("exo"));
    }
    // ---update forum and reindex----
    d = new Date() ;
    post = createPost(configBug, " help me config with jboss", "test message", "community",d);
    Thread.sleep(1000);
    singleSearchInput.setTerm("+jboss*");
    searcher = plugin.getSearcher();
    hits = searcher.search(singleSearchInput).getAll();
    assertTrue("expect found document has SUBJECT_FIELD contain jboss before reindex",hits.size() == 1);
    // -----------reindex-------
    plugin.reindex();
    Thread.sleep(2500);
    searcher = plugin.getSearcher();
    hits = searcher.search(singleSearchInput).getAll();
    assertEquals("expect found document has SUBJECT_FIELD contain jboss after reindex", 1, hits.size());
    // --remove post in forum service, this post is removed in indexer databse,too
    service_.removePost(testpost.getId());
    singleSearchInput = new SingleFieldSearchInput(plugin.AUTHOR_FIELD);
    singleSearchInput.setTerm("+exo portal*");
    searcher = plugin.getSearcher();
    hits = searcher.search(singleSearchInput).getAll();
    assertTrue("expect no document has AUTHOR_FIELD that contains 'exo portal'", hits.size() == 0);
    
    plugin.removeIndex();
    Thread.sleep(3500);
    iservice_.optimizeDatabase();
    /*-----remove data---*/
    service_.removeCategory(supportCat.getId()) ;  
    /*------------remove lucendata------------*/    
  }
  
  private Date createDate(int year, int month, int day, int hour, int minute, int second)
  throws Exception {
    Calendar calendar = new GregorianCalendar();    
    calendar.set(GregorianCalendar.YEAR, year);
    calendar.set(GregorianCalendar.MONTH, month);
    calendar.set(GregorianCalendar.DATE, day);
    calendar.set(GregorianCalendar.HOUR, hour);
    calendar.set(GregorianCalendar.MINUTE, minute);
    calendar.set(GregorianCalendar.SECOND, second);    
    return calendar.getTime();
  }
  
  protected String getDescription() {
    return "Test Forum Indexing";
  }   
}
