/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.pmanager;

import java.util.Collection;
import javax.jcr.Node;
import org.exoplatform.services.organization.OrganizationService;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Apr 7, 2005
 * @version $Id$
 */
public class ProjectACL {
  boolean viewIssuePermission_ ;
  boolean createIssuePermission_ ;
  boolean adminIssuePermission_ ;
  boolean viewDocumentPermission_ ;
  boolean createDocumentPermission_ ;
  boolean adminDocumentPermission_ ;
  boolean adminProjectPermission_ ;
  
  public ProjectACL(String user, Project project, 
                    OrganizationService service) throws Exception {
    Node permissions = project.getPermissions() ;
    String permission = 
      permissions.getProperty(Constant.PROJECT_PERMISSION_VIEW_ISSUE).getString() ;
    viewIssuePermission_ = hasPermission(service, user, permission) ;
    permission = 
      permissions.getProperty(Constant.PROJECT_PERMISSION_CREATE_ISSUE).getString() ;
    createIssuePermission_ = hasPermission(service, user, permission) ;
    permission = 
      permissions.getProperty(Constant.PROJECT_PERMISSION_ADMIN_ISSUE).getString() ;
    adminIssuePermission_ = hasPermission(service, user, permission) ;
    permission = 
      permissions.getProperty(Constant.PROJECT_PERMISSION_VIEW_DOCUMENT).getString() ;
    viewDocumentPermission_ = hasPermission(service, user, permission) ;
    permission = 
      permissions.getProperty(Constant.PROJECT_PERMISSION_CREATE_DOCUMENT).getString() ;
    createDocumentPermission_ = hasPermission(service, user, permission) ;
    permission = 
      permissions.getProperty(Constant.PROJECT_PERMISSION_ADMIN_DOCUMENT).getString() ;
    adminDocumentPermission_ = hasPermission(service, user, permission) ;
    permission = 
      permissions.getProperty(Constant.PROJECT_PERMISSION_ADMIN_PROJECT).getString() ;
    adminProjectPermission_ = hasPermission(service, user, permission) ;
  }
  
  public boolean hasViewIssuePermission()  { return viewIssuePermission_ ;}
  
  public boolean hasCreateIssuePermission()  { return createIssuePermission_ ;}
  
  public boolean hasAdminIssuePermission() { return adminIssuePermission_  ; }
  
  public boolean hasViewDocumentPermission() { return viewDocumentPermission_ ;}
  
  public boolean hasCreateDocumentPermission() { return createDocumentPermission_ ;}
  
  public boolean hasAdminDocumentPermission(){ return adminDocumentPermission_ ; }
  
  public boolean hasAdminProjectPermission() { return adminProjectPermission_ ; }
  
  private boolean hasPermission(OrganizationService service, String remoteUser, 
                                String expression) throws Exception {
    if(expression == null || expression.length() == 0) return true ;
    String[] temp = expression.split(":") ;
    String groupId = null,  membership = null ;
    if(temp.length == 1) {
      groupId = temp[0] ;
      membership = null ;
    } else {
      membership = temp[0] ;
      groupId = temp[1] ;
    }
    if("/guest".equals(groupId)) return true ;
    if(membership == null || "*".equals(membership)) {
      Collection c = 
        service.getMembershipHandler().findMembershipsByUserAndGroup(remoteUser, groupId) ;
      if(c == null) return false ;
      return c.size() > 0 ;
    } 
    return service.getMembershipHandler().findMembershipByUserGroupAndType(remoteUser, groupId, membership)  != null;
  }
}
