/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.pmanager.impl.exont;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.OnParentVersionAction;

import org.exoplatform.services.jcr.impl.core.nodetype.NodeDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.value.DoubleValue;

/**
 * @author Tuan Nguyen
 */
public class IssueType extends NodeTypeImpl {
  static Value[] DEFAULT_DOUBLE = { new DoubleValue(0) }; 

  public IssueType(NodeTypeManager manager) throws NoSuchNodeTypeException, RepositoryException {
    super(manager);
    this.name = "exo:issue";
    this.mixin = false;
    this.orderableChild = false;
    this.primaryItemName = null;
    this.declaredSupertypes = new NodeType[1];
    this.declaredSupertypes[0] = manager.getNodeType("nt:base");

    this.declaredPropertyDefinitions = new PropertyDefinition[20];
    this.declaredPropertyDefinitions[0] = 
      new PropertyDefinitionImpl("exo:type",                  //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          null,                        //defaultValues
                          false,                       //autoCreate       
                          true,                        //mandatory        
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );

    this.declaredPropertyDefinitions[1] = 
      new PropertyDefinitionImpl("exo:creator",               //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          null,                        //defaultValues
                          false,                       //autoCreate       
                          true,                        //mandatory        
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[2] = 
      new PropertyDefinitionImpl("exo:assignee",              //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          null,                        //defaultValues
                          false,                       //autoCreate       
                          true,                        //mandatory        
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[3] = 
      new PropertyDefinitionImpl("exo:reporter",              //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          null,                        //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory        
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[4] = 
      new PropertyDefinitionImpl("exo:createdDate",          //name
                          this,                       //declaringNodeType
                          PropertyType.DATE,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          true,                       //autoCreate       
                          true,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );

    this.declaredPropertyDefinitions[5] = 
      new PropertyDefinitionImpl("exo:modifiedDate",          //name
                          this,                       //declaringNodeType
                          PropertyType.DATE,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          true,                       //autoCreate       
                          true,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );

    this.declaredPropertyDefinitions[6] = 
      new PropertyDefinitionImpl("exo:dueDate",          //name
                          this,                       //declaringNodeType
                          PropertyType.DATE,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );

    this.declaredPropertyDefinitions[7] = 
      new PropertyDefinitionImpl("exo:timeEstimate",          //name
                          this,                        //declaringNodeType
                          PropertyType.DOUBLE,         //requiredType
                          null,                        //valueConstraints 
                          DEFAULT_DOUBLE,              //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );


    this.declaredPropertyDefinitions[8] = 
      new PropertyDefinitionImpl("exo:timeSpent",          //name
                          this,                        //declaringNodeType
                          PropertyType.DOUBLE,         //requiredType
                          null,                        //valueConstraints 
                          DEFAULT_DOUBLE,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );

    this.declaredPropertyDefinitions[9] = 
      new PropertyDefinitionImpl("exo:timeRemain",          //name
                          this,                        //declaringNodeType
                          PropertyType.DOUBLE,         //requiredType
                          null,                        //valueConstraints 
                          DEFAULT_DOUBLE,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );

    this.declaredPropertyDefinitions[10] = 
      new PropertyDefinitionImpl("exo:affectedVersion",          //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[11] = 
      new PropertyDefinitionImpl("exo:targetedVersion",          //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[12] = 
      new PropertyDefinitionImpl("exo:affectedComponent",          //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[13] = 
      new PropertyDefinitionImpl("exo:summary",                //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          true,                       //autoCreate       
                          true,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[14] = 
      new PropertyDefinitionImpl("exo:description",          //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[15] = 
      new PropertyDefinitionImpl("exo:status",          //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          true,                       //autoCreate       
                          true,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[16] = 
      new PropertyDefinitionImpl("exo:resolution",          //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[17] = 
      new PropertyDefinitionImpl("exo:priority",             //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );
    this.declaredPropertyDefinitions[18] = 
      new PropertyDefinitionImpl("exo:viewPermission",             //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                  //defaultValues
                          true,                       //autoCreate       
                          true,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );

    this.declaredPropertyDefinitions[19] =
      new PropertyDefinitionImpl("exo:editPermission",             //name
                          this,                        //declaringNodeType
                          PropertyType.STRING,         //requiredType
                          null,                        //valueConstraints 
                          NULL_VALUES,                     //defaultValues
                          false,                       //autoCreate       
                          false,                        //mandatory
                          OnParentVersionAction.COPY,  //onVersion        
                          false,                       //readOnly         
                          false
      );

    this.declaredChildNodeDefinitions = new NodeDefinition[3];

    NodeType[] types = {manager.getNodeType("exo:container")};
    this.declaredChildNodeDefinitions[0] = 
      new NodeDefinitionImpl("comments",                        // name
                      this,                              // declaringNodeType
                      types,                             // requiredNodeTypes
                      manager.getNodeType("exo:container"),   // defaultNodeType
                      false,                             // autoCreate       
                      true,                              // mandatory        
                      OnParentVersionAction.VERSION,     // onVersion        
                      false,                             // protected        
                      true                               // same-name sibling
      );

    NodeType[] types1 = {manager.getNodeType("exo:container")};
    this.declaredChildNodeDefinitions[1] = 
      new NodeDefinitionImpl("tasks",                        // name
                      this,                              // declaringNodeType
                      types1,                             // requiredNodeTypes
                      manager.getNodeType("exo:container"),   // defaultNodeType
                      false,                             // autoCreate       
                      true,                              // mandatory        
                      OnParentVersionAction.VERSION,     // onVersion        
                      false,                             // protected        
                      true                               // same-name sibling
      );
    NodeType[] types2 = {manager.getNodeType("exo:container")};
    this.declaredChildNodeDefinitions[2] = 
      new NodeDefinitionImpl("resources",                     // name
                      this,                              // declaringNodeType
                      types2,                            // requiredNodeTypes
                      manager.getNodeType("exo:container"),  // defaultNodeType
                      false,                             // autoCreate       
                      false,                              // mandatory        
                      OnParentVersionAction.VERSION,     // onVersion        
                      false,                             // protected        
                      true                               // same-name sibling
      );

  }
}