/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.spider.htmlparser.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.exoplatform.services.spider.core.htmlparser.Attribute;
import org.exoplatform.services.spider.core.htmlparser.Document;
import org.exoplatform.services.spider.core.htmlparser.HTMLFactory;
import org.exoplatform.services.spider.core.htmlparser.Node;
import org.exoplatform.test.BasicTestCase;

/**
 * Created by The eXo Platform SARL
 * Author : Thuannd
 *         nhudinhthuan@yahoo.com
 * Dec 29, 2005
 */
public class TestHTMLParser extends BasicTestCase{    
  private File file_;  
  private HTMLFactory factory_;
  
  public TestHTMLParser(String name) {
    super(name) ;
  }
  
  public void setUp() throws Exception {    
    String path_ = System.getProperty("basedir")+"/src/test/conf/content.html" ;    
    file_ = new File( path_);
    
    factory_ = (HTMLFactory)Class.forName(
        "org.exoplatform.services.spider.htmlparser.impl.HTMLFactoryImpl").newInstance();
  }
  
  public void testHTMLParser() throws Exception {    
    String html = "<html>" +                                  //StructureId=0 <html>
                    "<head>" +                                //StructureId=0.0 <head>
                      "<title>TestParserHTMLString</title>"+  //StructureId=0.0.0 <title>
                      "This is text node" +                   //StructureId=0.0.1 <text>                      
                      "<!-- This is comment -->"   +          // StructureId=0.0.2 <comment>
                      "&nbsp;&nbsp;&nbsp;&nbsp;" +            // not Node(No StructureId) (space)
                      "&" + "&#36;" + "%" + "#" +             // not Node (No StructureId) (all special character)
                    "</head>" +                    
                    "<body>" +                                //StructureId=0.1 <body>
                      "<H1>HTML String</H1>" +                //StructureId=0.1.0 <H1>
                      "<table border=2 ></table>"+            //StructureId=0.1.1 <table>
                      "<p>End Html Test"+                     //StructureId=0.1.2 <p>
                    "</body>"+
                   "</html>" ;    
    
    Document document = factory_.createDocument(html) ;
    assertTrue("expect docment is created",document != null) ;
    //-----html node-----
    Node htmNode = document.findNodeByStructureId("0") ;        
    assertTrue("expect node0 is found",htmNode != null && htmNode.getName().equals("html")) ;       
    assertTrue("expect node0 has 2 child node 'head' && 'body'",
                htmNode.getChildrenSize()==2 && htmNode.getChild(0).getName().equals("head")&&
                htmNode.getChild(1).getName().equals("body")) ;
    //-------head node---
    Node headNode = document.findNodeByStructureId("0.0") ;
    assertTrue("expect found node 0.0 is 'head'",headNode !=null && headNode.getName().equals("head")) ;    
    assertEquals("expect headNode has 3 child's node",3,headNode.getChildrenSize()) ;
    
    Node titleNode = document.findNodeByStructureId("0.0.0") ;        
    assertTrue("expect node '0.0.0' is 'title'",
              titleNode.getName().equals("title") && titleNode.getTextContent().contains("TestParserHTMLString")) ;
    
    Node textNode = document.findNodeByStructureId("0.0.1") ;        
    assertTrue("expect node '0.0.1' is textNode",
                textNode != null && textNode.getTextContent().contains("This is text node")) ;
    
    Node commentNode = document.findNodeByStructureId("0.0.2") ;
    assertTrue("expect node '0.0.2' is commentNode",
                commentNode !=null && commentNode.getTextContent().contains("This is comment")) ;
    //-------body node---
    Node bodyNode = document.findNodeByStructureId("0.1") ;
    assertTrue("expect node 0.1 is 'body'",bodyNode!=null && bodyNode.getName().equals("body")) ;
    assertEquals("expect body node has 3 childs node", 3 , bodyNode.getChildrenSize());         
    Node h1Node = bodyNode.getChild(0) ;
    assertTrue("first child's node of 'body' is 'H1'",h1Node != null && h1Node.getName().equals("h1")) ;
    Node tableNode = document.findNodeByStructureId("0.1.1") ;
    assertTrue("Expect node 0.1.1 is tableNode",tableNode!= null && tableNode.getName().equals("table")) ;    
    //----set attribute for h1Node------    
    assertTrue("expect H1node hasn't got attribute",h1Node.getAttributes().size()==0) ;
    h1Node.setAttribute("align","center") ;
    h1Node.setAttribute("font-color","black") ;
    assertTrue("now, expect H1 node has got 2 attribute",h1Node.getAttributes().size() == 2) ;
    assertEquals("expect 'align' attribute's value is:","center",h1Node.getAttribute("align").getValue()) ;
    assertEquals("expect 'font-color' attribute's value is:",
                  "black",h1Node.getAttribute("font-color").getValue()) ;
    //---update attribute of H1Node---
    h1Node.setAttribute("align","left") ;
    assertTrue("now, expect H1 node stil has got 2 attribute",h1Node.getAttributes().size() == 2) ;
    assertEquals("expect 'align' attribute's value is:","left",h1Node.getAttribute("align").getValue()) ;
    //---remove attribute of H1Node----
    h1Node.removeAttribute("font-color") ;
    assertTrue("now, expect H1node has got 1 attribute",h1Node.getAttributes().size() == 1) ;
    assertTrue("expect 'font-color' attribute is removed",h1Node.getAttribute("font-color") == null) ;
    //-----attribute of tableNode----
    assertTrue("expect table node has got 1 attribute",tableNode.getAttributes().size() == 1) ;
    assertTrue("expect this attribute is 'border' && it's value is '2'",
                tableNode.getAttribute("border").getValue().equals("2")) ;
    //----remove node---
    Node pNode = bodyNode.getChild(2) ;
    assertTrue("expect node 0.1.2 is pNode",
                document.findNodeByStructureId("0.1.2").getName().equals(pNode.getName())) ;
    assertTrue("expect third node of 'bodyNode' is 'p' ",pNode != null && pNode.getName().equals("p")) ;            
    assertTrue("expect this node is node that has Id 0.1.2",pNode.getTextContent().contains("End Html Test")) ;
    bodyNode.removeChild(pNode) ;
    assertTrue("now, expect 'body' has got 2 child node",bodyNode.getChildrenSize() == 2) ;
    
    try{
      document.findNodeByStructureId("0.1.2"); //<p>
    } catch (IndexOutOfBoundsException e) {
      assertTrue("expect child node is removed",e.getMessage().equals("Structure ID have incorrect index")) ;
    }
    //---remove body node form htmNode    
    htmNode.removeChild(1) ;
    try {
      document.findNodeByStructureId("0.1") ;//<body>
      htmNode.getChild(1) ;
    }catch (IndexOutOfBoundsException e) {
      assertTrue("expect child node is removed",e.getMessage().equals("Structure ID have incorrect index")) ;
    }
    //---all node in body node is removed--
    try {      
      document.findNodeByStructureId("0.1.0") ; //<h1></h1>
    } catch (IndexOutOfBoundsException e) {      
      assertTrue("expect child node is removed",e.getMessage().equals("Structure ID have incorrect index")) ;
    }    
    try {      
      document.findNodeByStructureId("0.1.1") ; //<tabel></table>
    } catch (IndexOutOfBoundsException e) {      
      assertTrue("expect child node is removed",e.getMessage().equals("Structure ID have incorrect index")) ;
    }    
  }
    
  public void testNode() throws Exception{        
    Document document = factory_.createDocument( file_);
    assertTrue("expect document is created",document != null) ;
    assertEquals( document.getEncoding(), "utf-8");        
    Node node = document.findNodeByStructureId("0.1.0.0.1.0.0");    
    assertEquals("Contents expert is", "this is td  2", node.getTextContent().trim());    
    
    node = document.findNodeByStructureId("0.1.0.1.0.1.0.0.0");    
    Node child = document.findNodeByStructureId("0.1.0.1.0.1.0.0.0.0");    
    assertSame( child, node.getChild(0));
    assertEquals( "Copy 2006 Google", child.getTextContent().trim());            
  }
    
  public void testNodeAttributes() throws Exception {           
    Document document = factory_.createDocument( file_);    
    Node node = document.findNodeByStructureId("0.1.0");
    
    Attribute attr = node.getAttribute("border");
    assertEquals( attr.getValue(), "1");    
    
    List<Attribute> list = node.getAttributes();
//    for( Attribute ele : list)
//      System.out.println( ele.getName() +" : "+ele.getValue());
    assertEquals( list.size(), 6);
    assertEquals("Attribute expert is", "align", list.get(2).getName()); 
    node.setAttribute("align", "top");
    
    attr = node.getAttribute("align");    
    assertEquals( attr.getValue(), "top");
   
    for( Attribute ele : list)
      if(ele.getName().equals("align")) ele.setValue("right");
    node.setAttributes(list);
    
    attr = node.getAttribute("align");    
    assertEquals( attr.getValue(), "right");

    node = document.findNodeByStructureId("0.1.0.1.0");    
    list = new ArrayList<Attribute>();
    attr = factory_.createAttribute("ALIGN","center");
    list.add( attr);
    attr = factory_.createAttribute("VALIGN","top");
    list.add( attr);    
    node.setAttributes( list);
    attr = node.getAttribute("valign");
    assertEquals( "Attribute value expert is", "top", attr.getValue());    
    node.removeAttribute("valign");
  }    
  
  public void testInputStream() throws Exception {    
    /*-------------------FILE Input Stream--------*/
    InputStream fileIs = new FileInputStream(file_) ;
    assertTrue("expect file inputstream is not null",fileIs!=null) ; 
    Document fileDoc = factory_.createDocument(fileIs);  
    assertTrue("expect document is created from FileInputStream ",fileDoc != null) ;               
    /*-------------------URL Input Stream--------*/
    URL url = new URL("http://www.google.com.vn");                             
    InputStream is = url.openConnection().getInputStream() ;
    assertTrue("expect connection is opened to host",is!=null) ;
    Document googleDoc = factory_.createDocument(is);
    assertTrue("expect document is created from inputstream",googleDoc!=null) ;
    assertEquals("expect Encoding is:","ISO-8859-1",googleDoc.getEncoding()) ;
    Node htmlNode = googleDoc.findNodeByStructureId("0");
    assertTrue("expect root node is 'html',",htmlNode!=null && htmlNode.getName().equals("html")) ;
    assertTrue("expect node '0.0' is 'head'",googleDoc.findNodeByStructureId("0.0").getName().equals("head"));
    Node googleTitle = googleDoc.findNodeByStructureId("0.0.1") ;
    assertTrue("expect node '0.0.1' is 'title'",
        googleTitle.getName().equals("title") && googleTitle.getTextContent().contains("Google")) ;
    assertTrue("expect node '0.1' is 'body'",googleDoc.findNodeByStructureId("0.1").getName().equals("body"));
  }
  
  protected String getDescription() {
    return "Test HTMLParser" ;
  }
}
