package org.exoplatform.services.captchas;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Locale;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.exoplatform.container.RootContainer;

import com.octo.captcha.service.image.ImageCaptchaService;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

public class CaptchaServlet extends HttpServlet {

  public static final String LOCALE = "locale";
  public static final String CAPTCHA_ID = "captchaId";
  
  public void init(ServletConfig config) throws ServletException {
  }

  public void service(HttpServletRequest request, HttpServletResponse response)
      throws ServletException, IOException {
    ServletOutputStream os = response.getOutputStream();
    String locale = request.getParameter(LOCALE);
    String captchaID = request.getParameter(CAPTCHA_ID);
    if(captchaID == null) {
      response.sendError(HttpServletResponse.SC_NOT_FOUND);
      return; 
    }
    if(locale == null) {
      locale = request.getLocale().getLanguage();
    }
    
    ImageCaptchaService captchaService = (ImageCaptchaService) RootContainer
        .getInstance().getComponentInstanceOfType(ImageCaptchaService.class);
    BufferedImage challenge = captchaService.getImageChallengeForID(captchaID, 
        new Locale(locale));
    
    ByteArrayOutputStream jpegOutputStream = new ByteArrayOutputStream();
    JPEGImageEncoder jpegEncoder = JPEGCodec.createJPEGEncoder(jpegOutputStream);
    jpegEncoder.encode(challenge);    
    byte[] captchaChallengeAsJpeg = jpegOutputStream.toByteArray();
    
    // render the captcha challenge as a JPEG image in the response
    response.setHeader("Cache-Control", "no-store");
    response.setHeader("Pragma", "no-cache");
    response.setDateHeader("Expires", 0);
    response.setContentType("image/jpeg");
    os.write(captchaChallengeAsJpeg);    
  }

}
