/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;

/**
 * A portlet project represents a view of a project resource in terms of portlet related
 * information such as the web folder, the portlet xml file, etc. This portlet project is
 * a read-only view of the project resource, it also caches some of project-scoped
 * preferences content (such as the web folder, etc.), and registers itself to monitor any
 * changes made to the provided information. If the underlaying project is deleted or
 * closed, then this portlet project instance should unregister itself from listening to
 * the workspace changes. This does not apply to the case of renaming the project. If you
 * need to change the web folder, context root, etc., then use the corresponding exo
 * operation. To get an instance of a portlet project use
 * ExoCorePlugin.createPortletProject().
 * 
 * @author Hatim Khan
 */
public interface IPortletProject
{
  /**
   * Returns a path (relative to the project) that points to the web folder that contains
   * the web content for the portlet application. This method does not check if the file
   * actually exists.
   * 
   * @return the web content folder for the portlet application
   */
  public IPath getWebFolder();

  /**
   * Returns a path that represents the context root to be used during deployment.
   * 
   * @return the a context root path
   */
  public IPath getContextRoot();

  /**
   * Returns a path (relative to the project of course) that points to the portlet.xml
   * deployment descriptor. Please notice that the file itself might not exist.
   * 
   * @return the path to the portlet deployment descriptor file (portlet.xml)
   */
  public IPath getPortletXML();

  /**
   * Returns the value of the version attribute in the portlet-app element in the
   * portlet.xml file. Null is returned if unable to determine the value of the version or
   * if the file does not exist.
   * 
   * @return the value of the version attribute of the portlet-app tag
   */
  public String getPortletXMLVersion();

  /**
   * Returns the associated resource project.
   * 
   * @return returns the associated resource project
   */
  public IProject getProject();
}