/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.common;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

/**
 * A collection of static utility methods that are used by everybody.
 * 
 * @author Hatim Khan
 */
public class Utils
{
  public static void close(InputStream pStream)
  {
    try
    {
      pStream.close();
    }
    catch (Exception e)
    {

    }
  }

  /**
   * @param pStream the stream to read
   * @return return the input stream content as a string, it will close the stream after
   *         that
   * @throws IOException when there is a problem
   */
  public static String readStream(InputStream pStream) throws IOException
  {
    StringBuffer sb = new StringBuffer();
    BufferedReader reader = new BufferedReader(new InputStreamReader(pStream));

    try
    {
      String line = reader.readLine();
      while (line != null)
      {
        sb.append(line);
        sb.append("\n");
        line = reader.readLine();
      }
    }
    finally
    {
      close(pStream);
    }

    return sb.toString();
  }

  /**
   * Tests if the URL can be reached, the test tries to open a stream on the given URL.
   * 
   * @param pURL the URL in question
   * @return true if the URL can be reached otherwise false
   */
  public static boolean exists(URL pURL)
  {
    boolean result = false;
    InputStream stream = null;
    try
    {
      stream = pURL.openStream();
      result = true;
    }
    catch (IOException e)
    {
      // simply ignore
    }
    finally
    {
      close(stream);
    }

    return result;
  }

  /**
   * Save the given string as the content of the given file. If the file does not exist,
   * one will be created
   * 
   * @param pFile the file in question
   * @param pContent the content as a string
   * @param pMonitor progress monitor
   * @throws CoreException when things go wrong
   */
  public static void setFileContent(IFile pFile, String pContent, IProgressMonitor pMonitor)
      throws CoreException
  {
    if (pMonitor == null) pMonitor = new NullProgressMonitor();

    InputStream inputStream = new BufferedInputStream(new ByteArrayInputStream(pContent.getBytes()));
    if (!pFile.exists())
    {
      pFile.create(inputStream, true, pMonitor);
    }
    else
    {
      pFile.setContents(inputStream, true, true, pMonitor);
    }
  }

  public static String getFileContent(IFile pFile) throws CoreException, IOException
  {
    if (pFile == null) return null;
    InputStream stream = pFile.getContents();

    return readStream(stream);
  }

  /**
   * Given a file name as a string, this method tries to shorten the file name so that
   * only the last part of the file name will appear.
   * 
   * @param pFileName
   * @return a shortened file name
   */
  public static String shortenString(String pFileName)
  {
    if ((pFileName == null) || (pFileName.length() < 30)) return pFileName;

    int index0 = pFileName.indexOf("/");
    int counter = 0;
    while ((index0 != -1) && (counter < 2) && (index0 < pFileName.length() - 1))
    {
      index0 = pFileName.indexOf("/", index0 + 1);
      counter++;
    }

    String firstPart = "";
    if (index0 != -1)
    {
      firstPart = pFileName.substring(0, index0);
    }
    else
    {
      firstPart = pFileName;
    }

    int index1 = pFileName.lastIndexOf("/");
    String finalPart = firstPart;
    counter = 0;
    while ((index1 != -1) && (counter < 3) && (index1 > 0))
    {
      index1 = pFileName.lastIndexOf("/", index1 - 1);
      counter++;
    }

    if ((index1 != -1) && (index1 != index0))
    {
      finalPart = finalPart + "/..." + pFileName.substring(index1);
    }

    return finalPart;
  }
  
  public static int getIndexOf(Object pObject, Object[] pObjects)
  {
    int result = -1;
    if ((pObject == null) || (pObjects == null) || (pObjects.length < 1)) return result;
    for(int i=0,sizei = pObjects.length; i < sizei; ++i)
    {
      if (pObject == pObjects[i])
      {
        result = i;
        break;
      }
    }
    
    return result;
  }
  
  public static String emptyIfNull(String pValue)
  {
    return (pValue == null) ? "" : pValue;
  }
  
  public static boolean isEquivalentString(String p1, String p2)
  {
    if ((p1 == null) && (p2 == null)) return true;
    if (p1 == null) return false;
    if (p2 == null) return false;
    
    return p1.equals(p2);
  }
  
}