/*****************************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL All rights reserved. * Please visit
 * http://www.exoplatform.org for more license detail. *
 ****************************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.eclipse.core.portal.model.INode;

/**
 * Represents a service configuration. Although this class is not aware of XStream, please
 * do not change the name of the private fields since XStream is dependent on them to
 * serialize from/to xml.
 * 
 * @author Hatim Khan (from Tuan Nguyen work)
 */
public class ServiceConfiguration implements INode
{
  private String type;
  private String key;
  private String appearIn;
  private List valueParams;
  private List valuesParams;
  private List propertiesParams;
  private List objectParams;
  private transient Object mParent = null;

  /**
   * @return Returns the key.
   */
  public String getKey()
  {
    return key;
  }

  /**
   * @param key The key to set.
   */
  public void setKey(String key)
  {
    this.key = key;
  }

  /**
   * @return Returns a list of ObjectParam instances
   */
  public List getObjectParams()
  {
    return objectParams;
  }

  /**
   * @param objectParams The objectParams to set.
   */
  public void setObjectParams(List objectParams)
  {
    this.objectParams = objectParams;
  }

  /**
   * @return Returns a list PropertiesParam instances
   */
  public List getPropertiesParams()
  {
    return propertiesParams;
  }

  /**
   * @param propertiesParams The propertiesParams to set.
   */
  public void setPropertiesParams(List propertiesParams)
  {
    this.propertiesParams = propertiesParams;
  }

  /**
   * @return Returns the type.
   */
  public String getType()
  {
    return type;
  }

  /**
   * @param type The type to set.
   */
  public void setType(String type)
  {
    this.type = type;
  }

  /**
   * @return Returns a list of ValueParam instances
   */
  public List getValueParams()
  {
    return valueParams;
  }

  /**
   * @param valueParams The valueParams to set.
   */
  public void setValueParams(List valueParams)
  {
    this.valueParams = valueParams;
  }

  /**
   * @return Returns a list of ValuesParam instances
   */
  public List getValuesParams()
  {
    return valuesParams;
  }

  /**
   * @param valuesParams The valuesParams to set.
   */
  public void setValuesParams(List valuesParams)
  {
    this.valuesParams = valuesParams;
  }

  /**
   * @return Returns the appearIn.
   */
  public String getAppearIn()
  {
    return appearIn;
  }

  /**
   * @param appearIn The appearIn to set.
   */
  public void setAppearIn(String appearIn)
  {
    this.appearIn = appearIn;
  }

  /**
   * @return Returns the parent.
   */
  public Object getParent()
  {
    return mParent;
  }

  /**
   * @param pParent The parent to set.
   */
  public void setParent(Object pParent)
  {
    mParent = pParent;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#getChildren()
   */
  public Object[] getChildren()
  {
    List result = new ArrayList();
    addAfterAttachingToParent(result, objectParams);
    addAfterAttachingToParent(result, propertiesParams);
    addAfterAttachingToParent(result, valuesParams);
    addAfterAttachingToParent(result, valueParams);

    return result.toArray();
  }

  private void addAfterAttachingToParent(List pContainer, List pChildren)
  {
    if (pChildren == null) return;
    Iterator iterator = pChildren.iterator();
    while (iterator.hasNext())
    {
      Parameter p = (Parameter) iterator.next();
      p.setParent(this);
      pContainer.add(p);
    }
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#removeChild(java.lang.Object)
   */
  public boolean removeChild(Object pChild)
  {
    if (pChild == null) return false;
    boolean removed = false;

    if ((objectParams != null) && (objectParams.remove(pChild)))
    {
      removed = true;
    }
    else if ((propertiesParams != null) && (propertiesParams.remove(pChild)))
    {
      removed = true;
    }
    else if ((valuesParams != null) && (valuesParams.remove(pChild)))
    {
      removed = true;
    }
    else if ((valueParams != null) && (valueParams.remove(pChild)))
    {
      removed = true;
    }

    // we disconnect the child from the parent if we are able to remove it
    if (removed)
    {
      if (pChild instanceof INode) ((INode) pChild).setParent(null);
    }

    return removed;
  }

  public List getParametersKeys()
  {
    List result = new ArrayList();
    addParameterKeys(result, valueParams);
    addParameterKeys(result, valuesParams);
    addParameterKeys(result, propertiesParams);
    addParameterKeys(result, objectParams);

    return result;
  }

  private void addParameterKeys(List pTarget, List pSource)
  {
    if (pTarget == null) return;
    if (pSource == null) return;
    Iterator iterator = pSource.iterator();
    while (iterator.hasNext())
    {
      Parameter parameter = (Parameter) iterator.next();
      String name = parameter.getName();
      if ((name != null) && (name.length() > 0))
      {
        pTarget.add(name.trim());
      }
    }
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#addChild(java.lang.Object)
   */
  public void addChild(Object pChild)
  {
    if (pChild == null) return;
    if (pChild instanceof INode)
    {
      INode node = (INode) pChild;
      node.setParent(this);
    }

    if (pChild instanceof ObjectParam)
    {
      if (objectParams == null) objectParams = new ArrayList();
      objectParams.add(pChild);
    }
    else if (pChild instanceof PropertiesParam)
    {
      if (propertiesParams == null) propertiesParams = new ArrayList();
      propertiesParams.add(pChild);
    }
    else if (pChild instanceof ValuesParam)
    {
      if (valuesParams == null) valuesParams = new ArrayList();
      valuesParams.add(pChild);
    }
    else if (pChild instanceof ValueParam)
    {
      if (valueParams == null) valueParams = new ArrayList();
      valueParams.add(pChild);
    }
  }
  
  public Object getCopy()
  {
    ServiceConfiguration copy = new ServiceConfiguration();
    copy.key = this.key;
    copy.type = this.type;
    copy.appearIn = this.appearIn;
    copy.valueParams = Parameter.getCopyParameterList(this.valueParams);
    copy.valuesParams = Parameter.getCopyParameterList(this.valuesParams);  
    copy.propertiesParams = Parameter.getCopyParameterList(this.propertiesParams);
    copy.objectParams = Parameter.getCopyParameterList(this.objectParams);    
    
    return copy;
  }
}