/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.eclipse.core.common.Utils;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;

/**
 * Represent a parameter that can hold one value, a name and a desscription. Although this
 * class is not aware of XStream, please do not change the name of the private fields
 * since XStream is dependent on them to serialize from/to xml.
 * 
 * @author Hatim Khan (from Tuan Nguyen work)
 */
public class ValueParam extends Parameter
{
  private String value;
  private transient ValueHolder mValueHolder = null;

  /**
   * @return Returns the value.
   */
  public String getValue()
  {
    return value;
  }

  /**
   * @param value The value to set.
   */
  public void setValue(String value)
  {
    this.value = value;
    createValueHolder();
  }

  private void createValueHolder()
  {
    if (value == null) return;
    if (mValueHolder == null) mValueHolder = new ValueHolder();
    mValueHolder.setParent(this);
    mValueHolder.setValue(value);
  }

  public Object[] getChildren()
  {
    createValueHolder();
    List result = new ArrayList();
    if (mValueHolder != null)
    {
      result.add(mValueHolder);
    }
    return result.toArray();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#removeChild(java.lang.Object)
   */
  public boolean removeChild(Object pChild)
  {
    if ((pChild == null) || !(pChild instanceof ValueHolder) || (mValueHolder == null)) return false;
    boolean removed = false;

    if (mValueHolder == pChild)
    {
      mValueHolder = null;
      value = null;
      removed = true;
    }

    // we disconnect the child from the parent if we are able to remove it
    if (removed)
    {
      if (pChild instanceof INode) ((INode) pChild).setParent(null);
    }

    return removed;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#addChild(java.lang.Object)
   */
  public void addChild(Object pChild)
  {
    // we are not actually adding a child, we will instead replace a child
    if (pChild == null) return;

    if (pChild instanceof ValueHolder)
    {
      ValueHolder newValueHolder = (ValueHolder) pChild;
      String newValue = newValueHolder.getValue();
      if (newValue == null)
      {
        value = null;
        if (mValueHolder != null)
        {
          mValueHolder.setParent(null);
          mValueHolder = null;
        }
      }
      else
      {
        value = newValue;
        createValueHolder();
      }
    }
  }

  public Object getCopy()
  {
    ValueParam copy = new ValueParam();
    populateCopy(copy);
    copy.value = this.value;
    return copy;
  }

  public boolean isEquivalent(Object pObject)
  {
    boolean paramSame = super.isEquivalent(pObject);
    if ((!paramSame) || !(pObject instanceof ValueParam)) return false;

    return Utils.isEquivalentString(this.value, ((ValueParam) pObject).value);
  }
}