/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.extension;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

/**
 * This class provides a generic way of preparing extensions for further specialized
 * processing. The idea came from the article titled 'Notes on the Eclipse Plug-in
 * Architecture' found at the eclipse.org website.
 * 
 * @author Hatim Khan
 */
public class ProcessExtension
{
  /**
   * Given an extension point, this method collects all the extensions attached to it.
   * Then, it delegates the processing of top configuration elements for each extension to
   * the given processor.
   * 
   * @param pExtensionPointId the extension point id to inspect
   * @param pProcessor the processor to delegate to it the processing of the top
   *          configuration elements for each extension
   * @param pResultCollector an object that can be used to hold the result
   */
  public static void process(String pExtensionPointId, IExtensionProcessor pProcessor, Object pResultCollector)
  {
    IExtensionRegistry registry = Platform.getExtensionRegistry();
    IExtensionPoint extensionPoint = registry.getExtensionPoint(pExtensionPointId);
    IExtension[] extensions = extensionPoint.getExtensions();

    // For each extension ...
    for (int i = 0; i < extensions.length; i++)
    {
      IExtension extension = extensions[i];
      IConfigurationElement[] elements = extension.getConfigurationElements();

      // For each member of the extension ...
      for (int j = 0; j < elements.length; j++)
      {
        IConfigurationElement element = elements[j];
        pProcessor.process(extension, element, pResultCollector);
      }
    }
  }
}