/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.filesprovider;

import org.eclipse.core.runtime.IPath;

/**
 * This interface extends the IImportStructureProvider to allow for the ability
 * to rename certain folders/files before walking through the structure.
 * Please notice that although this interface belongs to the exo core plugin,
 * it depends on the IImportStructureProvider which is part of the eclipse ui
 * plugin. This is unfortunate, but I decided to keep this interface (along
 * with all the other file providers classes) in the exo core plugin, since it
 * suits  it better. The import structure should thought of as core services
 * rather than UI services.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public interface IImportWithRenameStructureProvider
  extends IImportStructureProvider
{
  public static final String CLASS_VERSION =
    "$Id: IImportWithRenameStructureProvider.java,v 1.1 2004/04/19 03:45:47 hatimk Exp $"; //$NON-NLS-1$

  /**   
   * Add a renaming entry that indicates which folder/file to rename while
   * provding the structure. For example, if the original structure contains a
   * directory 'test', and you add a renaming entry to change it to 'src' then
   * when walking through the structure, the directory 'src' appears  instead
   * of 'test'. This method should be called before walking through the
   * structure.
   *
   * @param pRenameFrom the path to be renamed
   * @param pRenameTo the new path
   */
  public void addRenameEntry(IPath pRenameFrom, IPath pRenameTo);
}
