/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.filesprovider;

import org.eclipse.core.runtime.IPath;

/**
 * This class holdes a structure provider along with the name of the source,
 * binary, and library folders.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class JavaProjectFilesProvider extends ProjectFilesProvider
{
  public static final String CLASS_VERSION = "$Id: JavaProjectFilesProvider.java,v 1.1 2004/04/19 03:45:47 hatimk Exp $"; //$NON-NLS-1$
  private IPath mSourceFolder;
  private IPath mBinaryFolder;
  private IPath mLibraryFolder;

  /**
   * The constructor method.
   *
   * @param pStructureProvider an import structure provider that can walk you
   *        through a directory structure. Depending on the implementation of
   *        the interface, we could be talking about a directory structure
   *        coming from a file system or from a zip file.
   * @param pSource an object that represents the root from which the structure
   *        provider starts its walk. The type of this object depends on the
   *        implementation of the structure provider. For example, for the
   *        ZipFileStructureProvider the type is ZipEntry, for the
   *        FileSystemStructureProvider the type is File.
   * @param pSourceFolder the path for the source folder relative to the root
   *        of the structure
   * @param pBinaryFolder the path for the binary folder relative to the root
   *        of the structure
   * @param pLibraryFolder the path that represents a library folder relative
   *        to the root of the structure
   */
  public JavaProjectFilesProvider(
    IImportWithRenameStructureProvider pStructureProvider, Object pSource,
    IPath pSourceFolder, IPath pBinaryFolder, IPath pLibraryFolder)
  {
    super(pStructureProvider, pSource);
    mSourceFolder = pSourceFolder;
    mBinaryFolder = pBinaryFolder;
    mLibraryFolder = pLibraryFolder;
  }

  /**   
   * Returns the path for the source folder relative to the root of the
   * structure.
   *
   * @return the path that represents a source folder
   */
  public IPath getSourceFolder()
  {
    return mSourceFolder;
  }

  /**   
   * Returns the path for the binary folder relative to the root of the
   * structure.
   *
   * @return the path that represents a binary folder
   */
  public IPath getBinaryFolder()
  {
    return mBinaryFolder;
  }

  /**   
   * Returns the path that represents a library folder relative to the root of
   * the structure.
   *
   * @return the path that represents a library folder
   */
  public IPath getLibraryFolder()
  {
    return mLibraryFolder;
  }
}
