/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.filesprovider;

/**
 * This class holdes information about an import structure provider that can be
 * used later to supply content to interested operations (such as project
 * create operation).
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ProjectFilesProvider
{
  public static final String CLASS_VERSION = "$Id: ProjectFilesProvider.java,v 1.1 2004/04/19 03:45:47 hatimk Exp $"; //$NON-NLS-1$
  private IImportWithRenameStructureProvider mStructureProvider;
  private Object mSource;
  private String mDescription;

  /**
   * The constructor method.
   *
   * @param pStructureProvider an import structure provider that can walk you
   *        through a directory structure. Depending on the implementation of
   *        the interface, we could be talking about a directory structure
   *        coming from a file system or from a zip file.
   * @param pSource an object that represents the root from which the structure
   *        provider starts its walk. The type of this object depends on the
   *        implementation of the structure provider. For example, for the
   *        ZipFileStructureProvider the type is ZipEntry, for the
   *        FileSystemStructureProvider the type is File.
   */
  public ProjectFilesProvider(
    IImportWithRenameStructureProvider pStructureProvider, Object pSource)
  {
    mStructureProvider = pStructureProvider;
    mSource = pSource;
  }

  /**   
   * Returns an import structure provider that can be used later to supply
   * content to interested operations (such as project create operation).
   *
   * @return an import structure provider that can walk you through a directory
   *         structure. Depending on the implementation of the interface, we
   *         could be talking about a directory structure coming from a file
   *         system or from a zip file.
   */
  public IImportWithRenameStructureProvider getStructureProvider()
  {
    return mStructureProvider;
  }

  /**   
   * Returns an object that represents the root from which the structure
   * provider starts its walk. The type of this object depends on the
   * implementation of the structure provider. For example, for the
   * ZipFileStructureProvider the type is ZipEntry, for the
   * FileSystemStructureProvider the type is File.
   *
   * @return an object that represents the root from which the structure
   *         provider starts its walk.
   */
  public Object getSource()
  {
    return mSource;
  }

  /**   
   * Returns a friendly description of what the content provided by this files
   * provider are all about.
   *
   * @return the description of the content provided by this files provider
   */
  public String getDescription()
  {
    return mDescription;
  }

  /**   
   * Allows you to set the description of what the content of this files
   * provider is all about.
   *
   * @param pDescription the description of the content provided by this files
   *        provider
   */
  public void setDescription(String pDescription)
  {
    mDescription = pDescription;
  }
}
