/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.filesprovider;

import org.eclipse.core.runtime.IPath;

/**
 * This class holdes a structure provider along with the name of the source
 * folder, and web folder.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class WebappProjectFilesProvider extends JavaProjectFilesProvider
{
  public static final String CLASS_VERSION = "$Id: WebappProjectFilesProvider.java,v 1.1 2004/04/19 03:45:47 hatimk Exp $"; //$NON-NLS-1$
  private IPath mWebFolder;

  /**
   * The constructor method.
   *
   * @param pStructureProvider an import structure provider that can walk you
   *        through a directory structure. Depending on the implementation of
   *        the interface, we could be talking about a directory structure
   *        coming from a file system or from a zip file.
   * @param pSource an object that represents the root from which the structure
   *        provider starts its walk. The type of this object depends on the
   *        implementation of the structure provider. For example, for the
   *        ZipFileStructureProvider the type is ZipEntry, for the
   *        FileSystemStructureProvider the type is File.
   * @param pSourceFolder the path for the source folder relative to the root
   *        of the structure
   * @param pWebFolder the path for the web folder relative to the root of the
   *        structure
   */
  public WebappProjectFilesProvider(
    IImportWithRenameStructureProvider pStructureProvider, Object pSource,
    IPath pSourceFolder, IPath pWebFolder)
  {
    super(pStructureProvider, pSource, pSourceFolder, null, null);
    mWebFolder = pWebFolder;
  }

  /**   
   * Returns the path that represents the web folder relative to the root of
   * the structure.
   *
   * @return the path that represents the web folder
   */
  public IPath getWebFolder()
  {
    return mWebFolder;
  }
}
