/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching.delegate;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.sourcelookup.ISourceContainer;
import org.eclipse.debug.core.sourcelookup.ISourcePathComputerDelegate;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;
import org.exoplatform.eclipse.core.launching.ExoLaunching;
import org.exoplatform.eclipse.core.launching.ExoSourceLookupUtil;
import org.exoplatform.eclipse.core.launching.IExoTomcatLocation;
import org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfiguration;
import org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfigurationProvider;

import java.util.HashSet;
import java.util.Set;

/**
 * @author Hatim Khan
 */
public class ExoTomcatSourcePathComputer implements ISourcePathComputerDelegate
{

  /**
   * @see org.eclipse.debug.core.sourcelookup.ISourcePathComputerDelegate#computeSourceContainers(org.eclipse.debug.core.ILaunchConfiguration,
   *      org.eclipse.core.runtime.IProgressMonitor)
   */
  public ISourceContainer[] computeSourceContainers(ILaunchConfiguration pConfig, IProgressMonitor pMonitor)
      throws CoreException
  {
    boolean shutdown = ExoTomcatLaunchConfigurationDelegate.isShutdownLaunch(pConfig);
    if (shutdown) return new ISourceContainer[0];

    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("calculating eXo Platform with Tomcat source lookup paths  ", 2);

    IExoTomcatLocation exoTomcatLocation = ExoTomcatLaunchConfigurationDelegate.getLocation(pConfig);
    IVMInstall vminstall = JavaRuntime.computeVMInstall(pConfig);
    IExoTomcatRuntimeConfigurationProvider provider = ExoTomcatLaunchConfigurationDelegate.getConfigurationProvider(pConfig);

    pMonitor.subTask("preparing Tomcat configuration source lookup paths, this might take a few seconds .. ");

    IExoTomcatRuntimeConfiguration config = provider.getConfiguration(exoTomcatLocation, vminstall);

    // check for cancellation
    if (pMonitor.isCanceled())
    {
      return new ISourceContainer[0];
    }

    pMonitor.subTask("preparing the source lookup paths ");
    pMonitor.worked(1);

    ISourceContainer[] result = prepareSourceContainers(vminstall, config, pConfig, new SubProgressMonitor(
        pMonitor, 1));
    pMonitor.done();

    return result;
  }

  private static ISourceContainer[] prepareSourceContainers(IVMInstall pVMInstall,
      IExoTomcatRuntimeConfiguration pConfig, ILaunchConfiguration pLaunchConfig, IProgressMonitor pMonitor)
      throws CoreException
  {
    Set vmLibrariesSourceLocations = ExoSourceLookupUtil.getVMInstallSourceContainers(pVMInstall);
    IJavaProject[] portletProjects = ExoLaunching.getPortletProjects(pLaunchConfig, pMonitor);
    Set portletProjectsSourceLocations = ExoSourceLookupUtil.getPortletProjectsSourceContainers(portletProjects);
    Set endorsedLibrariesSourceLocations = ExoSourceLookupUtil.translate(pConfig.getEndorsedLibraries());
    Set binLibrariesSourceLocations = ExoSourceLookupUtil.translate(pConfig.getBinLibraries());
    Set serverLibrariesSourceLocations = ExoSourceLookupUtil.translate(pConfig.getServerLibraries());
    Set commonLibrariesSourceLocations = ExoSourceLookupUtil.translate(pConfig.getCommonLibraries());
    Set sharedLibrariesSourceLocations = ExoSourceLookupUtil.translate(pConfig.getSharedLibraries());
    Set serverWebappsLibrariesSourceLocations = ExoSourceLookupUtil.translate(pConfig.getServerWebappsLibraries());
    Set defaultWebappsLibrariesSourceLocations = ExoSourceLookupUtil.translate(pConfig.getDefaultWebappsLibraries());

    Set sourceLocations = new HashSet();
    sourceLocations.addAll(vmLibrariesSourceLocations);
    sourceLocations.addAll(portletProjectsSourceLocations);
    sourceLocations.addAll(endorsedLibrariesSourceLocations);
    sourceLocations.addAll(binLibrariesSourceLocations);
    sourceLocations.addAll(serverLibrariesSourceLocations);
    sourceLocations.addAll(commonLibrariesSourceLocations);
    sourceLocations.addAll(sharedLibrariesSourceLocations);
    sourceLocations.addAll(serverWebappsLibrariesSourceLocations);
    sourceLocations.addAll(defaultWebappsLibrariesSourceLocations);

    return (ISourceContainer[]) sourceLocations.toArray(new ISourceContainer[sourceLocations.size()]);
  }

}