package org.exoplatform.eclipse.core.launching.provider;


import groovy.lang.Binding;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.IVMInstall;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.launching.IExoTomcatLocation;
import org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfiguration;
import org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfigurationProvider;
import org.exoplatform.eclipse.core.launching.ITomcatLocation;
import org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration;
import org.exoplatform.eclipse.core.launching.VMArgument;
import org.osgi.framework.Bundle;

import java.net.URL;
import java.util.List;

/**
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoTomcatRuntimeConfigurationProvider extends GenericTomcatRuntimeConfigurationProvider
    implements IExoTomcatRuntimeConfigurationProvider
{

  public static final String CLASS_VERSION = "$Id: ExoTomcatRuntimeConfigurationProvider.java,v 1.1 2004/09/20 01:39:14 hatimk Exp $"; //$NON-NLS-1$

  private URL mExoGroovyScriptLocation = null;

  //DO NOT FORGET everytime you add a new instance variable make sure you list it in the
  // reset method.
  private IExoTomcatLocation mExoTomcatLocation = null;
  private IVMInstall mVMInstall = null;
  private List mCommonLibraries = null;
  private List mDefaultWebappsLibraries = null;
  private List mVMArguments = null;

  /**
   * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(org.eclipse.core.runtime.IConfigurationElement,
   *      java.lang.String, java.lang.Object)
   */
  public void setInitializationData(IConfigurationElement pConfig, String pPropertyName, Object pData)
      throws CoreException
  {
    super.setInitializationData(pConfig, pPropertyName, pData);

    String groovylocationStr = pConfig.getAttribute("exo-groovy-script");
    String bundleName = pConfig.getDeclaringExtension().getNamespace();
    Bundle bundle = Platform.getBundle(bundleName);
    URL groovyLocationURL = bundle.getEntry(groovylocationStr);
    URL groovyLocation = null;
    try
    {
      groovyLocation = Platform.asLocalURL(groovyLocationURL);
    }
    catch (Exception e)
    {
      ExoCorePlugin.logError("Encountered an error while trying to determine the location of the groovy script file to figure out eXo Tomcat configuration.", e);
    }

    if (groovyLocation != null)
    {
      mExoGroovyScriptLocation = groovyLocation;
    }
  }

  private void prepareLibraries(IExoTomcatLocation pExoTomcatLocation, IVMInstall pVMInstall)
      throws CoreException
  {
    // lets determine if we need to recompute everything.
    if ((mExoTomcatLocation == null) || (mVMInstall == null)
        || (!mExoTomcatLocation.equals(pExoTomcatLocation)) || (mVMInstall != pVMInstall))
    {
      // yes we need to recompute
      reset();
      initializeConfiguration(pExoTomcatLocation, pVMInstall);
      mExoTomcatLocation = pExoTomcatLocation;
      mVMInstall = pVMInstall;
    }
  }

  private void initializeConfiguration(IExoTomcatLocation pExoTomcatLocation, IVMInstall pVMInstall)
      throws CoreException
  {
    Binding binding = new Binding();
    binding.setVariable("TOMCATHOME", pExoTomcatLocation.getTomcatHome());
    binding.setVariable("EXOSRC", pExoTomcatLocation.getExoSrc());
    binding.setVariable("VMInstall", pVMInstall);

    Object scriptvalue = runScript(mExoGroovyScriptLocation, binding);

    mCommonLibraries = (List) binding.getVariable("commonLibraries");
    mDefaultWebappsLibraries = (List) binding.getVariable("defaultWebappsLibraries");
    mVMArguments = (List) binding.getVariable("VMArguments");
  }

  private void reset()
  {
    mExoTomcatLocation = null;
    mVMInstall = null;
    mCommonLibraries = null;
    mDefaultWebappsLibraries = null;
    mVMArguments = null;
  }

  /**
   * @see org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfigurationProvider#getConfiguration(org.exoplatform.eclipse.core.launching.provider.ExoTomcatLocation,
   *      org.eclipse.jdt.launching.IVMInstall)
   */
  public IExoTomcatRuntimeConfiguration getConfiguration(IExoTomcatLocation pExoTomcatLocation,
      IVMInstall pVMInstall) throws CoreException
  {
    ITomcatRuntimeConfiguration tomcatConfig = getConfiguration((ITomcatLocation) pExoTomcatLocation, pVMInstall);
    prepareLibraries(pExoTomcatLocation, pVMInstall);

    IRuntimeClasspathEntry[] commonLibraries = convertToRuntimeClasspathEntries(mCommonLibraries);
    IRuntimeClasspathEntry[] defaultWebappsLibraries = convertToRuntimeClasspathEntries(mDefaultWebappsLibraries);
    VMArgument[] VMArguments = convertToVMArguments(mVMArguments);

    IExoTomcatRuntimeConfiguration config = new ExoTomcatRuntimeConfiguration(tomcatConfig, commonLibraries,
        defaultWebappsLibraries, VMArguments);
    return config;
  }

}