/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.library;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IClasspathContainer;
import org.eclipse.jdt.core.IClasspathEntry;

/**
 * This class represents an eXo library container. It is immutable. It is the
 * base class for all the different kind of eXo library containers, such as
 * portlet libraries, portal libraries, etc.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoLibraryContainer implements IClasspathContainer
{
  public static final String CLASS_VERSION =
    "$Id: ExoLibraryContainer.java,v 1.1 2004/04/19 03:45:47 hatimk Exp $"; //$NON-NLS-1$
  private IPath mPath;
  private String mDescription;
  private IClasspathEntry[] mClasspathEntries;
  private String mVersion;

  /**
   * The constructor method.
   *
   * @param pPath
   * @param pDescription
   * @param pClasspathEntries
   * @param pVersion
   */
  public ExoLibraryContainer(IPath pPath, String pDescription,
    IClasspathEntry[] pClasspathEntries, String pVersion)
  {
    mPath = pPath;
    mDescription = pDescription;
    mClasspathEntries = pClasspathEntries;
    mVersion = pVersion;
  }

  /**   
   * @see org.eclipse.jdt.core.IClasspathContainer#getClasspathEntries()
   */
  public IClasspathEntry[] getClasspathEntries()
  {
    return mClasspathEntries;
  }

  /**   
   * @see org.eclipse.jdt.core.IClasspathContainer#getDescription()
   */
  public String getDescription()
  {
    return mDescription;
  }

  /**   
   * @see org.eclipse.jdt.core.IClasspathContainer#getKind()
   */
  public int getKind()
  {
    return IClasspathContainer.K_APPLICATION;
  }

  /**   
   * @see org.eclipse.jdt.core.IClasspathContainer#getPath()
   */
  public IPath getPath()
  {
    return mPath;
  }

  /**   
   * Returns the version number that this library container represent. This
   * should we a one to one mapping with the eXo binary distribution version.
   *
   * @return version number (example: 1.0.0.beta4)
   */
  public String getVersion()
  {
    return mVersion;
  }
}
