/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import java.lang.reflect.InvocationTargetException;

/**
 * Indicates that the implementor of this interface should be treated as an
 * operation.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public interface IExoOperation
{
  public static final String CLASS_VERSION = "$Id: IExoOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$

  /**   
   * Runs this operation.  Progress should be reported to the given progress
   * monitor. A request to cancel the operation should be honored and
   * acknowledged  by throwing <code>InterruptedException</code>.
   *
   * @param pMonitor the progress monitor to use to display progress and
   *        receive requests for cancelation
   *
   * @throws CoreException
   * @throws InvocationTargetException if the run method must propagate a
   *         checked exception, it should wrap it inside an
   *         <code>InvocationTargetException</code>; runtime exceptions are
   *         automatically wrapped in an
   *         <code>InvocationTargetException</code> by the calling context
   * @throws InterruptedException if the operation detects a request to cancel,
   *         using <code>IProgressMonitor.isCanceled()</code>, it should exit
   *         by throwing  <code>InterruptedException</code>
   */
  public void run(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException;
}
