/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation.helper;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a project configuration. Usually, a project
 * configuration is required by a project create operation. A project
 * configuration consists of one or more project settings.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ProjectConfiguration
{
  public static final String CLASS_VERSION =
    "$Id: ProjectConfiguration.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private List mProjectSettings;

  /**   
   * Adds the given project settings to the collection of the settings that
   * this class represents.
   *
   * @param pSettings provides access to multiple project files providers and
   *        to the mounting point (path relative to the project) that they
   *        should exist under.
   */
  public void addProjectSettings(ProjectSettings pSettings)
  {
    if (mProjectSettings == null)
    {
      mProjectSettings = new ArrayList();
    }

    mProjectSettings.add(pSettings);
  }

  /**   
   * Returns the project settings that this class represents.
   *
   * @return the project settings that this class represents. Each settings
   *         provides access to multiple project files providers and to the
   *         mounting point (path relative to the project) that they should
   *         exist under.
   */
  public ProjectSettings[] getProjectSettings()
  {
    if (mProjectSettings == null)
    {
      return new ProjectSettings[0];
    }

    ProjectSettings[] result = new ProjectSettings[mProjectSettings.size()];
    result = (ProjectSettings[]) mProjectSettings.toArray(result);

    return result;
  }

  /**   
   * Allows you to set all project settings that this class represents. This
   * method removes any existing settings that this class represents and
   * replaces them with the ones given.
   *
   * @param pSetting An array of project settings
   */
  public void setProjectSettings(ProjectSettings[] pSetting)
  {
    if (mProjectSettings == null)
    {
      mProjectSettings = new ArrayList();
    }
    else
    {
      mProjectSettings.clear();
    }

    if ((pSetting == null) || (pSetting.length < 1))
    {
      return;
    }

    for (int i = 0, sizei = pSetting.length; i < sizei; ++i)
    {
      mProjectSettings.add(pSetting[i]);
    }
  }
}
