/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation.helper;


import org.eclipse.core.runtime.IPath;
import org.exoplatform.eclipse.core.filesprovider.JavaProjectFilesProvider;
import org.exoplatform.eclipse.core.filesprovider.WebappProjectFilesProvider;

/**
 * This class provides access to multiple webapp project files providers and to
 * the mounting point (path relative to the project) that they should exist
 * under.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class WebappProjectSettings extends JavaProjectSettings
{
  public static final String CLASS_VERSION =
    "$Id: WebappProjectSettings.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private IPath mWebFolder;

  /**
   * The constructor method.
   *
   * @param pParentFolder a relative path from the project that represents the
   *        mounting point. Since the path is relative to the project, then,
   *        for example, both the path "/test" and the path "test" are the
   *        same. A null value means the parent folder is the project folder
   *        itself.
   * @param pSourceFolder a relative path from the parent folder that
   *        represents the source folder
   * @param pWebFolder a relative path from the parent folder that represents
   *        the web folder
   */
  public WebappProjectSettings(IPath pParentFolder, IPath pSourceFolder,
    IPath pWebFolder)
  {
    super(pParentFolder, pSourceFolder,
      pWebFolder.append("WEB-INF").append("classes"),
      pWebFolder.append("WEB-INF").append("lib"));
    mWebFolder = pWebFolder;
  }

  /**   
   * Returns the web folder path relative to the parent folder.
   *
   * @return the web folder path relative to the parent folder.
   */
  public IPath getWebFolder()
  {
    return mWebFolder;
  }

  /**   
   * @see org.exoplatform.eclipse.core.operation.JavaProjectSettings#addRenaming(org.exoplatform.eclipse.core.filesprovider.JavaProjectFilesProvider)
   */
  protected void addRenaming(JavaProjectFilesProvider pProvider)
  {
    super.addRenaming(pProvider);
    if (pProvider instanceof WebappProjectFilesProvider)
    {
      WebappProjectFilesProvider webappProvider =
        (WebappProjectFilesProvider) pProvider;
      IPath providerWeb = webappProvider.getWebFolder();
      if ((providerWeb != null) && (mWebFolder != null)
          && (!mWebFolder.equals(providerWeb)))
      {
        pProvider.getStructureProvider().addRenameEntry(providerWeb, mWebFolder);
      }
    }
  }
}
