/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.xmlvalidator;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;

/**
 * This class helps in not calling the actual validator until it is really
 * needed.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PortletDescriptorDelegator implements IPortletDescriptorValidator
{
  public static final String CLASS_VERSION = "$Id: PortletDescriptorDelegator.java,v 1.1 2004/04/19 03:45:50 hatimk Exp $"; //$NON-NLS-1$
  private String mSupportedSpecVersion = null;
  private String mSupportedExoVersion = null;
  private boolean mDefault = false;
  private int mOrder;
  private IConfigurationElement mElement = null;
  private IPortletDescriptorValidator mDelegate = null;

  /**
   * The constructor method.
   *
   * @param pElement the configuration element that contains the class
   *        attribute that this class will eventually delegate the validation
   *        responsiblity to.
   */
  public PortletDescriptorDelegator(IConfigurationElement pElement)
  {
    mElement = pElement;
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#validate()
   */
  public void validate(PortletValidationProblemReporter pReporter,
    IProgressMonitor pMonitor)
  {
    // check if we already create the delegate class 
    if (mDelegate == null)
    {
      // this means we need to create the class (this will activate the plugin)
      if (mElement == null)
      {
        // something wrong.. the mElment must not be null otherwise we can not
        // continue
        ExoCorePlugin.logError("Can not delegate the validation because the configuration element is missing!!",
          new Exception());

        return;
      }
      try
      {
        mDelegate =
          (IPortletDescriptorValidator) mElement.createExecutableExtension(
            "class");
      }
      catch (CoreException e)
      {
        ExoCorePlugin.logError("Could not create a portlet validator", e);

        return;
      }
      mDelegate.setDefault(mDefault);
      mDelegate.setSupportedExoVersion(mSupportedExoVersion);
      mDelegate.setSupportedSpecVersion(mSupportedSpecVersion);
      mDelegate.setOrder(mOrder);
    }

    mDelegate.validate(pReporter, pMonitor);
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#getSupportedSpecVersion()
   */
  public String getSupportedSpecVersion()
  {
    return mSupportedSpecVersion;
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#setSupportedSpecVersion(java.lang.String)
   */
  public void setSupportedSpecVersion(String pVersion)
  {
    mSupportedSpecVersion = pVersion;
    if (mDelegate != null)
    {
      mDelegate.setSupportedSpecVersion(pVersion);
    }
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#getSupportedExoVersion()
   */
  public String getSupportedExoVersion()
  {
    return mSupportedExoVersion;
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#setSupportedExoVersion(java.lang.String)
   */
  public void setSupportedExoVersion(String pVersion)
  {
    mSupportedExoVersion = pVersion;
    if (mDelegate != null)
    {
      mDelegate.setSupportedExoVersion(pVersion);
    }
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#isDefault()
   */
  public boolean isDefault()
  {
    return mDefault;
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#setDefault(boolean)
   */
  public void setDefault(boolean pDefault)
  {
    mDefault = pDefault;
    if (mDelegate != null)
    {
      mDelegate.setDefault(pDefault);
    }
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#getOrder()
   */
  public int getOrder()
  {
    return mOrder;
  }

  /**   
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#setOrder(int)
   */
  public void setOrder(int pOrder)
  {
    mOrder = pOrder;
    if (mDelegate != null)
    {
      mDelegate.setOrder(pOrder);
    }
  }
}
