/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.editor;

import java.io.IOException;
import java.util.ArrayList;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.editor.IFormPage;
import org.eclipse.ui.part.FileEditorInput;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.core.portal.PortalSettingsBuilder;
import org.exoplatform.eclipse.ui.ExoUIPlugin;

/**
 * @author Hatim Khan
 */
public class PortalSettingsEditor extends FormEditor
{
  private PortalSettings mPortalSettings;

  /**
   * @see org.eclipse.ui.forms.editor.FormEditor#addPages()
   */
  protected void addPages()
  {
    try
    {
      // lets attempt to create the model at this step
      loadPortalSettings();
    }
    catch (Exception e1)
    {
      ExoUIPlugin.logError("could not load the portal settings", e1);
    }
    try
    {
      addPage(new PortalServicesFormPage(getPortalSettings(), this));
    }
    catch (PartInitException e)
    {
      ExoUIPlugin.logError(e);
    }

  }

  /**
   * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
   */
  public void doSave(IProgressMonitor monitor)
  {
    commitFormPages(true);
    try
    {
      savePortalSettings();
      getPortalSettings().markClean();
    }
    catch (CoreException e)
    {
      ExoUIPlugin.logError("unable to save the portal settings", e);
    }
    editorDirtyStateChanged();
  }

  /**
   * @see org.eclipse.ui.ISaveablePart#doSaveAs()
   */
  public void doSaveAs()
  {
  }

  /**
   * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
   */
  public boolean isSaveAsAllowed()
  {
    return false;
  }

  /* package */IFormPage[] getPages()
  {
    ArrayList formPages = new ArrayList();
    for (int i = 0; i < pages.size(); i++)
    {
      Object page = pages.get(i);
      if (page instanceof IFormPage) formPages.add(page);
    }
    return (IFormPage[]) formPages.toArray(new IFormPage[formPages.size()]);
  }

  private void commitFormPages(boolean onSave)
  {
    IFormPage[] pages = getPages();
    for (int i = 0; i < pages.length; i++)
    {
      IFormPage page = pages[i];
      IManagedForm mform = page.getManagedForm();
      if (mform != null && mform.isDirty()) mform.commit(true);
    }
  }

  public boolean isDirty()
  {
    return computeDirtyState();
  }

  private boolean computeDirtyState()
  {
    IFormPage page = getActivePageInstance();
    if ((page != null && page.isDirty())) return true;
    if ((getPortalSettings() != null) && (getPortalSettings().isDirty())) return true;
    return super.isDirty();
  }

  private void loadPortalSettings() throws IOException, CoreException
  {
    FileEditorInput input = (FileEditorInput) getEditorInput();
    mPortalSettings = PortalSettingsBuilder.readFrom(input.getFile());
  }

  private void savePortalSettings() throws CoreException
  {
    FileEditorInput input = (FileEditorInput) getEditorInput();
    PortalSettingsBuilder.writeTo(mPortalSettings, input.getFile());
  }

  /**
   * @return Returns the portal settings.
   */
  public PortalSettings getPortalSettings()
  {
    return mPortalSettings;
  }

}