/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.editor;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.AbstractFormPart;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.exoplatform.eclipse.core.exoservices.Configuration;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * This details page displays the services configuration entry information. This page does
 * not change the model or have any field to allow the user to enter data, however, this
 * page should receive model change events and marks itself as stale
 * 
 * @author Hatim Khan
 */
public class ServiceInfoDetailPart extends AbstractFormPart implements IDetailsPage,
    IPortalSettingsConfigurationChangedListener
{
  private ServiceConfiguration mServiceConfig = null;
  private PortalSettings mPortalSettings = null;
  private Text mKey = null;
  private Text mType = null;
  private Text mAppearIn = null;
  private Label mAppearInLabel = null;
  private Map mImages = new HashMap();

  public ServiceInfoDetailPart(PortalSettings pPortalSettings)
  {
    mPortalSettings = pPortalSettings;
  }

  protected void populateSectionTitle(Section pSection)
  {
    pSection.setText("Service Configuration Details");
    pSection.setDescription("Detailed information about the selected service configuration entry");
  }

  /**
   * @see org.eclipse.ui.forms.IDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
   */
  public void createContents(Composite pParent)
  {
    GridLayout layout = new GridLayout();
    FormToolkit toolkit = getManagedForm().getToolkit();
    layout.marginHeight = 0;
    layout.marginWidth = 0;
    pParent.setLayout(layout);

    Section section = toolkit.createSection(pParent, Section.TITLE_BAR | Section.DESCRIPTION
        | Section.TWISTIE | Section.EXPANDED);
    section.clientVerticalSpacing = 4;
    section.marginHeight = 5;
    section.marginWidth = 5;
    populateSectionTitle(section);
    GridData td = new GridData(GridData.FILL_BOTH);
    section.setLayoutData(td);

    Composite client = toolkit.createComposite(section);
    layout = new GridLayout();
    layout.marginHeight = 2;
    layout.marginWidth = 2;
    layout.numColumns = 2;

    client.setLayout(layout);
    td = new GridData(GridData.FILL_BOTH);
    client.setLayoutData(td);

    populateClientContent(client, toolkit);

    toolkit.paintBordersFor(client);
    section.setClient(client);

    // you have to call markDetailsPart() if your class implements IDetailsPage, otherwise
    // the DetailsPart will not pick up your class as a page to test for the dirty flag.
    //markDetailsPart(section);

  }

  protected void populateClientContent(Composite pClient, FormToolkit pToolkit)
  {
    pClient.addDisposeListener(new DisposeListener()
    {
      public void widgetDisposed(DisposeEvent pEvent)
      {
        disposeImages();
      }
    });

    //---------------------- key ----------------------------
    Label label = pToolkit.createLabel(pClient, "Key: ");
    label.setForeground(pToolkit.getColors().getColor(FormColors.TITLE));

    mKey = pToolkit.createText(pClient, "", SWT.LEFT | SWT.READ_ONLY);
    GridData td = new GridData(GridData.FILL_HORIZONTAL);
    mKey.setLayoutData(td);

    //---------------------- type ----------------------------
    label = pToolkit.createLabel(pClient, "Type: ");
    label.setForeground(pToolkit.getColors().getColor(FormColors.TITLE));

    mType = pToolkit.createText(pClient, "", SWT.LEFT | SWT.READ_ONLY);
    td = new GridData(GridData.FILL_HORIZONTAL);
    mType.setLayoutData(td);

    UICompositeUtil.createFormGridSpacer(pToolkit, pClient, 2);
    //---------------------- Appear In ----------------------------
    mAppearInLabel = pToolkit.createLabel(pClient, "This service configuration originally appeared in: ");
    mAppearInLabel.setForeground(pToolkit.getColors().getColor(FormColors.TITLE));
    td = new GridData(GridData.FILL_HORIZONTAL);
    td.horizontalSpan = 2;
    mAppearInLabel.setLayoutData(td);

    mAppearIn = pToolkit.createText(pClient, "", SWT.MULTI | SWT.WRAP | SWT.LEFT | SWT.READ_ONLY);
    td = new GridData(GridData.FILL_HORIZONTAL);
    td.horizontalSpan = 2;
    mAppearIn.setLayoutData(td);
    UICompositeUtil.setHeightHint(mAppearIn, 50);
  }

  /**
   * @see org.eclipse.ui.forms.IPartSelectionListener#selectionChanged(org.eclipse.ui.forms.IFormPart,
   *      org.eclipse.jface.viewers.ISelection)
   */
  public void selectionChanged(IFormPart part, ISelection pSelection)
  {
    if (pSelection instanceof IStructuredSelection)
    {
      IStructuredSelection ssel = (IStructuredSelection) pSelection;
      if (ssel.size() == 1)
      {
        ServiceConfiguration newConfig = (ServiceConfiguration) ssel.getFirstElement();
        if (newConfig != mServiceConfig)
        {
          // we need to detache from the current configuration to the new one
          mPortalSettings.removeConfigurationChangedListener(this);
          Configuration config = PortalSettings.findConfiguration(newConfig);
          mPortalSettings.addConfigurationChangedListener(config, this);
        }
        mServiceConfig = newConfig;
      }
      else
      {
        mServiceConfig = null;
        mPortalSettings.removeConfigurationChangedListener(this);
      }
    }
    else
    {
      mServiceConfig = null;
      mPortalSettings.removeConfigurationChangedListener(this);
    }
    update();
  }

  protected void update()
  {
    if ((mServiceConfig == null))
    {
      mKey.setText("");
      mType.setText("");
      mAppearIn.setText("");
      return;
    }
    else
    {
      mKey.setText(UICompositeUtil.emptyIfNull(mServiceConfig.getKey()));
      mType.setText(UICompositeUtil.emptyIfNull(mServiceConfig.getType()));
      mAppearIn.setText(UICompositeUtil.emptyIfNull(mServiceConfig.getAppearIn()));
    }
  }

  private Image getImage(ImageDescriptor pDesc)
  {
    Image image = (Image) mImages.get(pDesc);
    if (image == null)
    {
      image = pDesc.createImage();
      mImages.put(pDesc, image);
    }
    return image;
  }

  /**
   * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
   */
  private void disposeImages()
  {
    Collection collection = mImages.values();
    Iterator iterator = collection.iterator();
    while (iterator.hasNext())
    {
      Image image = (Image) iterator.next();
      if (!image.isDisposed()) image.dispose();
    }
    mImages.clear();
  }

  public void dispose()
  {
    super.dispose();
    if (mPortalSettings != null)
    {
      mPortalSettings.removeConfigurationChangedListener(this);
    }
  }

  public void refresh()
  {
    super.refresh();
    update();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementRemoved(java.lang.Object,
   *      java.lang.Object)
   */
  public void ConfigurationElementRemoved(Object pParent, Object pChild)
  {
    markStale();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementAdded(java.lang.Object)
   */
  public void ConfigurationElementAdded(Object pChild)
  {
    markStale();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementUpdated(java.lang.Object)
   */
  public void ConfigurationElementUpdated(Object pElement)
  {
    markStale();
  }

}