/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.launching;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.debug.ui.ILaunchConfigurationDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.exoplatform.eclipse.core.launching.ExoLaunching;
import org.exoplatform.eclipse.core.launching.IExoLaunchingConfigurationConstants;
import org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfigurationProvider;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

import java.io.File;

/**
 * A Tab that allows the user to select the home directory of Tomcat and its source code directory,
 * along with eXo platform source code directory.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoTomcatHomeTab extends AbstractLaunchConfigurationTab
{
  public static final String CLASS_VERSION = "$Id: ExoTomcatHomeTab.java,v 1.1 2004/09/20 01:42:56 hatimk Exp $"; //$NON-NLS-1$
  public static final String TAB_NAME = "&Home";
  private static final int TEXTUNIT = 1;
  private Image mTabImage;
  private Combo mExoVersion;
  private Text mTomcatInstallationDirText;
  private Button mTomcatInstallationDirBrowseButton;
  private Text mTomcatSrcDirText;
  private Button mTomcatSrcDirBrowseButton;
  private Text mExoSrcDirText;
  private Button mExoSrcDirBrowseButton;
  private Listener mTextModifyListener =
    new Listener()
    {
      public void handleEvent(Event pEvent)
      {
        isValid();

        // because isValid() might have updated the error messages, we will ask the 
        // parent dialog to referesh itself, so the error messages get updated
        updateDialog();
      }
    };

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite = UICompositeUtil.createFillBothComposite(pParent, 1, 1);
    UICompositeUtil.addLabel(mainComposite, "", 1);
    createExoVersionGroup(mainComposite);
    createTomcatGroup(mainComposite);
    createExoGroup(mainComposite);

    setControl(mainComposite);
  }

  /**   
   * @see ILaunchConfigurationTab#isValid(ILaunchConfiguration)
   */
  public boolean isValid(ILaunchConfiguration pConfig)
  {
    return isValid();
  }

  /**   
   * DOCUMENT ME!
   *
   * @return
   */
  public boolean isValid()
  {
    setMessage(null);
    setErrorMessage(null);

    return validateTomcatInstallationDir();
  }

  private boolean validateTomcatInstallationDir()
  {
    boolean result = true;
    if ((mTomcatInstallationDirText != null) && (!mTomcatInstallationDirText.isDisposed()))
    {
      String text = mTomcatInstallationDirText.getText();
      if ((text == null) || (text.length() < 1))
      {
        setErrorMessage("You must specify a Tomcat installation directory");
        result = false;
      }
    }

    return result;
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
   */
  public void setDefaults(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
   */
  public void initializeFrom(ILaunchConfiguration pConfiguration)
  {
    try
    {
      initializeExoVersionsFrom(pConfiguration);
      initializeTomcatInstallationDirFrom(pConfiguration);
      initializeTomcatSrcDirFrom(pConfiguration);
      initializeExoSrcDirFrom(pConfiguration);
    }
    catch (CoreException e)
    {
      ExoUIPlugin.logError(e);
    }
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
   */
  public void performApply(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    saveExoVersionsToConfig(pWorkingCopy);
    saveTomcatInstallationDirToConfig(pWorkingCopy);
    saveTomcatSrcDirToConfig(pWorkingCopy);
    saveExoSrcDirToConfig(pWorkingCopy);
  }

  private void saveExoVersionsToConfig(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    String selectedVersion = mExoVersion.getText();
    if ((selectedVersion == null) || (selectedVersion.length() < 1))
    {
      return;
    }

    IExoTomcatRuntimeConfigurationProvider provider =
      ExoLaunching.getExoTomcatRuntimeConfigurationProviderWithName(selectedVersion);
    if (provider == null)
    {
      return;
    }

    pWorkingCopy.setAttribute(IExoLaunchingConfigurationConstants.ATTR_EXO_TOMCAT_RUNTIME_CONFIGURATION_PROVIDER_ID,
      provider.getConfigurationId());
  }

  private void saveTomcatInstallationDirToConfig(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    String selectedDir = mTomcatInstallationDirText.getText();

    if ((selectedDir != null) && (selectedDir.length() < 1))
    {
      selectedDir = null;
    }
    pWorkingCopy.setAttribute(IExoLaunchingConfigurationConstants.ATTR_TOMCAT_INSTALLATION_DIR,
      selectedDir);
  }

  private void saveTomcatSrcDirToConfig(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    String selectedDir = mTomcatSrcDirText.getText();

    if ((selectedDir != null) && (selectedDir.length() < 1))
    {
      selectedDir = null;
    }
    pWorkingCopy.setAttribute(IExoLaunchingConfigurationConstants.ATTR_TOMCAT_SOURCE_DIR,
      selectedDir);
  }

  private void saveExoSrcDirToConfig(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    String selectedDir = mExoSrcDirText.getText();

    if ((selectedDir != null) && (selectedDir.length() < 1))
    {
      selectedDir = null;
    }
    pWorkingCopy.setAttribute(IExoLaunchingConfigurationConstants.ATTR_EXO_SOURCE_DIR, selectedDir);
  }

  private void initializeExoVersionsFrom(ILaunchConfiguration pConfig)
    throws CoreException
  {
    mExoVersion.removeAll();

    // first lets get all available exo tomcat runtime configuration provider names, the names
    // can be savely displayed to the user as the versions of eXo to select from
    IExoTomcatRuntimeConfigurationProvider[] providers =
      ExoLaunching.getExoTomcatRuntimeConfigurationProviders();
    if (providers == null)
    {
      // we have no providers so, display an empty list
      return;
    }

    for (int i = 0, sizei = providers.length; i < sizei; ++i)
    {
      mExoVersion.add(providers[i].getConfigurationName());
    }

    // now lets see if the configuration has a provider that was perviously specified, if so, then 
    // make it the combo current selection, otherwise make the default provider the current combo selection
    String providerId =
      pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_EXO_TOMCAT_RUNTIME_CONFIGURATION_PROVIDER_ID,
        (String) null);

    if (providerId != null)
    {
      IExoTomcatRuntimeConfigurationProvider provider =
        ExoLaunching.getExoTomcatRuntimeConfigurationProvider(providerId);
      if (provider != null)
      {
        mExoVersion.setText(provider.getConfigurationName());
      }
    }
    else
    {
      IExoTomcatRuntimeConfigurationProvider provider =
        ExoLaunching.getDefaultExoTomcatRuntimeConfigurationProvider();
      if (provider != null)
      {
        mExoVersion.setText(provider.getConfigurationName());
      }
    }
  }

  private void initializeTomcatInstallationDirFrom(ILaunchConfiguration pConfig)
    throws CoreException
  {
    mTomcatInstallationDirText.setText("");

    String location =
      pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_TOMCAT_INSTALLATION_DIR,
        (String) null);
    if ((location == null) || (location.length() < 1))
    {
      return;
    }
    mTomcatInstallationDirText.setText(location);
  }

  private void initializeTomcatSrcDirFrom(ILaunchConfiguration pConfig)
    throws CoreException
  {
    mTomcatSrcDirText.setText("");

    String location =
      pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_TOMCAT_SOURCE_DIR, (String) null);
    if ((location == null) || (location.length() < 1))
    {
      return;
    }
    mTomcatSrcDirText.setText(location);
  }

  private void initializeExoSrcDirFrom(ILaunchConfiguration pConfig)
    throws CoreException
  {
    mExoSrcDirText.setText("");

    String location =
      pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_EXO_SOURCE_DIR, (String) null);
    if ((location == null) || (location.length() < 1))
    {
      return;
    }
    mExoSrcDirText.setText(location);
  }

  private void createExoVersionGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroupWithNoGrapping(pParent, "eXo Platform Version", 2, 1);
    UICompositeUtil.addLabelWithWrapping(group,
      "Select the appropriate eXo Platform version from the supported list : ", 2);
    mExoVersion = new Combo(group, SWT.DROP_DOWN | SWT.READ_ONLY);

    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    mExoVersion.setLayoutData(gridData);
    mExoVersion.setFont(font);
    mExoVersion.addModifyListener(new ModifyListener()
      {
        public void modifyText(ModifyEvent pEvent)
        {
          updateDialog();
        }
      });
  }

  private void createTomcatGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group = UICompositeUtil.createGroupWithNoGrapping(pParent, "Tomcat", 2, 1);
    UICompositeUtil.addLabelWithWrapping(group, "Installation directory (eXo Tomcat) :", 2);
    mTomcatInstallationDirText = UICompositeUtil.addText(group, "", 1);
    UICompositeUtil.setWidthHint(mTomcatInstallationDirText, TEXTUNIT);

    // tomcat installation directory browse button
    mTomcatInstallationDirBrowseButton = new Button(group, SWT.PUSH);
    mTomcatInstallationDirBrowseButton.setText("Br&owse..");
    mTomcatInstallationDirBrowseButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent pEvent)
        {
          String loc = ""; //$NON-NLS-1$
          if (mTomcatInstallationDirText != null)
          {
            loc = mTomcatInstallationDirText.getText();
          }

          loc =
            handleLocationBrowseButtonPressed(loc, "Please choose Tomcat installation directory");
          if (loc != null)
          {
            mTomcatInstallationDirText.setText(loc);
          }
        }
      });

    mTomcatInstallationDirBrowseButton.setEnabled(true);
    mTomcatInstallationDirBrowseButton.setFont(font);

    UICompositeUtil.setButtonLayoutData(mTomcatInstallationDirBrowseButton);

    mTomcatInstallationDirText.addListener(SWT.Modify, mTextModifyListener);

    // now lets deal with Tomcat source directory widgets
    UICompositeUtil.addLabelWithWrapping(group, "Source code directory :", 2);
    mTomcatSrcDirText = UICompositeUtil.addText(group, "", 1);
    UICompositeUtil.setWidthHint(mTomcatSrcDirText, TEXTUNIT);

    // tomcat source directory browse button
    mTomcatSrcDirBrowseButton = new Button(group, SWT.PUSH);
    mTomcatSrcDirBrowseButton.setText("Bro&wse..");
    mTomcatSrcDirBrowseButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent pEvent)
        {
          String loc = ""; //$NON-NLS-1$
          if (mTomcatSrcDirText != null)
          {
            loc = mTomcatSrcDirText.getText();
          }

          loc = handleLocationBrowseButtonPressed(loc, "Please choose Tomcat source directory");
          if (loc != null)
          {
            mTomcatSrcDirText.setText(loc);
          }
        }
      });

    mTomcatSrcDirBrowseButton.setEnabled(true);
    mTomcatSrcDirBrowseButton.setFont(font);

    UICompositeUtil.setButtonLayoutData(mTomcatSrcDirBrowseButton);

    mTomcatSrcDirText.addListener(SWT.Modify, mTextModifyListener);
  }

  private void createExoGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group = UICompositeUtil.createGroupWithNoGrapping(pParent, "eXo Platform", 2, 1);
    UICompositeUtil.addLabelWithWrapping(group, "Source code directory  :", 2);
    mExoSrcDirText = UICompositeUtil.addText(group, "", 1);
    UICompositeUtil.setWidthHint(mExoSrcDirText, TEXTUNIT);

    // exo source directory browse button
    mExoSrcDirBrowseButton = new Button(group, SWT.PUSH);
    mExoSrcDirBrowseButton.setText("Brow&se..");
    mExoSrcDirBrowseButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent pEvent)
        {
          String loc = ""; //$NON-NLS-1$
          if (mExoSrcDirText != null)
          {
            loc = mExoSrcDirText.getText();
          }

          loc =
            handleLocationBrowseButtonPressed(loc, "Please choose eXo Platform source directory");
          if (loc != null)
          {
            mExoSrcDirText.setText(loc);
          }
        }
      });

    mExoSrcDirBrowseButton.setEnabled(true);
    mExoSrcDirBrowseButton.setFont(font);

    UICompositeUtil.setButtonLayoutData(mExoSrcDirBrowseButton);

    mExoSrcDirText.addListener(SWT.Modify, mTextModifyListener);
  }

  /**   
   * Open an appropriate directory browser. The browser will show the given initial location (if it
   * exists) and the given dialog message.
   *
   * @param pInitialLocation the location to start with
   * @param pDialogMessage the message to display for this dialog
   *
   * @return the location selected by the user
   */
  private String handleLocationBrowseButtonPressed(String pInitialLocation, String pDialogMessage)
  {
    DirectoryDialog dialog = new DirectoryDialog(getShell());
    dialog.setMessage(pDialogMessage);

    String dirName = pInitialLocation;

    if ((dirName != null) && (!dirName.equals(""))) //$NON-NLS-1$
    {
      File path = new File(dirName);

      if (path.exists())
      {
        dialog.setFilterPath(new Path(dirName).toOSString());
      }
    }

    return dialog.open();
  }

  private void updateDialog()
  {
    ILaunchConfigurationDialog dialog = getLaunchConfigurationDialog();
    if (dialog != null)
    {
      dialog.updateMessage();
      dialog.updateButtons();
    }
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
   */
  public String getName()
  {
    return TAB_NAME;
  }

  /**   
   * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#getImage()
   */
  public Image getImage()
  {
    if (mTabImage == null)
    {
      mTabImage = ExoUIPluginImages.DESC_OBJ_TOMCAT_HOME.createImage();
    }

    return mTabImage;
  }

  /**   
   * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#dispose()
   */
  public void dispose()
  {
    super.dispose();

    if ((mTabImage != null) && (!mTabImage.isDisposed()))
    {
      mTabImage.dispose();
      mTabImage = null;
    }
  }
  
	public void activated(ILaunchConfigurationWorkingCopy pWorkingCopy)
	{
		// do nothing
	}

	public void deactivated(ILaunchConfigurationWorkingCopy pWorkingCopy)
	{
		// do nothing
	}
  
}
