/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.library;


import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.ui.wizards.IClasspathContainerPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.ui.help.WorkbenchHelp;
import org.exoplatform.eclipse.core.library.ExoLibraryCenter;
import org.exoplatform.eclipse.core.library.ExoPortletLibraryContainer;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;

/**
 * This page provides the user with information about the selected (or a new)
 * exo Portlet Library. The information is a list of all the jar  files that
 * come with the library along with the version number of the  library. The
 * version number matches the eXo binary distribution version.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoPortletLibraryPage extends WizardPage
  implements IClasspathContainerPage
{
  public static final String CLASS_VERSION = "$Id: ExoPortletLibraryPage.java,v 1.1 2004/04/19 03:37:23 hatimk Exp $"; //$NON-NLS-1$
  private IClasspathEntry mCurrentSelection = null;

  /**
   * The constructor method.
   */
  public ExoPortletLibraryPage()
  {
    super("ExoPortletLibraryPage"); //$NON-NLS-1$
    setTitle("eXo Portlet Library");
    setDescription("Select Finish to accept this library.");
    setImageDescriptor(ExoUIPluginImages.DESC_WIZBAN_ADD_PORTLET_LIBRARY);
  }

  /**   
   * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite = createMainComposite(pParent);
    setControl(mainComposite);
    WorkbenchHelp.setHelp(pParent, IExoHelpContextIds.ADD_PORTLET_LIBRARY);

    // setting focus means that user does not need to click on a widget before F1 help can work
    pParent.setFocus();
    addLabel(mainComposite, "eXo Portlet Library");
    createVerticalSpacer(mainComposite, 1);
    addLabel(mainComposite, "The library includes the following entries :");

    // the setSelection() method will be invoked before the createControl() 
    // method. The selection will contain the IClasspathEntry if the user
    // selected the library and then pressed 'Edit' button. However, if the
    // user selected 'Add' button instead, the selection will be null. In
    // this case, will default the library to display to be the latest library
    // version available 
    IPath currentPath = new Path(ExoLibraryCenter.EXO_PORTLET_LIBRARY_PATH);

    if (mCurrentSelection != null)
    {
      currentPath = mCurrentSelection.getPath();
    }

    ExoPortletLibraryContainer container =
      ExoLibraryCenter.getPortletLibrary(currentPath);

    if (container == null)
    {
      createVerticalSpacer(mainComposite, 1);
      addLabel(mainComposite,
        "Sorry, unable to recognize any eXo portlet libraries");

      return;
    }

    mCurrentSelection = JavaCore.newContainerEntry(currentPath);

    List entryList =
      new List(mainComposite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    entryList.setLayoutData(new GridData(GridData.FILL));

    IClasspathEntry[] classpathEntries = container.getClasspathEntries();

    for (int i = classpathEntries.length - 1; i >= 0; --i)
    {
      entryList.add(classpathEntries[i].getPath().toString());
    }

    createVerticalSpacer(mainComposite, 1);
    addLabel(mainComposite,
      "The library matches the eXo platform binary distribution version:");

    List versionList =
      new List(mainComposite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    versionList.setLayoutData(new GridData(GridData.FILL));
    versionList.add(container.getVersion());
  }

  private Composite createMainComposite(Composite pParent)
  {
    Composite mainComposite = new Composite(pParent, SWT.NONE);
    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;
    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    mainComposite.setLayout(layout);
    mainComposite.setLayoutData(data);
    mainComposite.setFont(pParent.getFont());

    return mainComposite;
  }

  private void addLabel(Composite pComp, String pText)
  {
    Label label = new Label(pComp, SWT.NONE);

    label.setText(pText);

    GridData gd = new GridData(GridData.FILL);
    label.setLayoutData(gd);
  }

  /**   
   * Create some empty space.
   *
   * @param pComp the composite to add the space to
   * @param pColSpan the number of column to span
   */
  private void createVerticalSpacer(Composite pComp, int pColSpan)
  {
    Label label = new Label(pComp, SWT.NONE);
    GridData gd = new GridData();
    gd.horizontalSpan = pColSpan;
    label.setLayoutData(gd);
  }

  /**   
   * @see org.eclipse.jdt.ui.wizards.IClasspathContainerPage#finish()
   */
  public boolean finish()
  {
    return true;
  }

  /**   
   * @see org.eclipse.jdt.ui.wizards.IClasspathContainerPage#getSelection()
   */
  public IClasspathEntry getSelection()
  {
    return mCurrentSelection;
  }

  /**   
   * @see org.eclipse.jdt.ui.wizards.IClasspathContainerPage#setSelection(org.eclipse.jdt.core.IClasspathEntry)
   */
  public void setSelection(IClasspathEntry pContainerEntry)
  {
    mCurrentSelection = pContainerEntry;
  }
}
