/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.preference;


import java.io.File;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipse.ui.help.WorkbenchHelp;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.IPreferencesConstant;
import org.exoplatform.eclipse.core.operation.SimplePortletProjectStoreOperation;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.eclipse.internal.ui.wizard.ChangeWebFolderWizard;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;
import org.exoplatform.eclipse.ui.dialog.ProblemDialog;
import org.exoplatform.eclipse.ui.validator.InputTextValidator;

/**
 * This class represents a property page that is attached to resources of type
 * IJavaProject.  The property page allows the user to view/update the
 * following settings:  1- Web content folder  2- Context root name  3-
 * Deployment directory  4- Deployment overwrite options 5- eXo web.xml
 * modification options
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public final class WebappPropertyPage extends PropertyPage
{
  public static final String CLASS_VERSION =
    "$Id: WebappPropertyPage.java,v 1.1 2004/04/19 03:37:23 hatimk Exp $"; //$NON-NLS-1$
  private static final String PROBLEM_TITLE =
    "Web Application Property Problem";
  public static final String DEFAULT_WEBFOLDER_TEXT = ""; //$NON-NLS-1$
  public static final String DEFAULT_CONTEXTROOT_TEXT = ""; //$NON-NLS-1$  
  public static final String DEFAULT_DEPLOYMENTDIR_TEXT = ""; //$NON-NLS-1$    
  private Text mWebFolderText;
  private Text mContextRootText;
  private Text mDeploymentDirText;
  private Button mDeploymentOption1;
  private Button mDeploymentOption2;
  private Button mWebFolderBrowseButton;
  private Button mDeploymentDirBrowseButton;
  private Button mDeploymentOverwriteButton;
  private Button mDetectWebXMLButton;
  private Button mWebXMLOverwriteButton;
  private Label mWebXMLOverwriteDesc;
  private Label mWebXMLOverwriteDesc2;
  private Listener mTextModifyListener =
    new Listener()
    {
      public void handleEvent(Event e)
      {
        validatePageInput();
      }
    };

  private SelectionListener mWebXMLOptionsListener =
    new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        updateWebXMLOptions();
      }
    };

  /**   
   * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
   */
  protected Control createContents(Composite pParent)
  {
    WorkbenchHelp.setHelp(pParent, IExoHelpContextIds.WEBAPP_PROPERTY_PAGE);

    TabFolder folder = new TabFolder(pParent, SWT.NONE);

    Composite deploymentTab = createDeploymentSettingsTab(folder);
    Composite dialogSettingsTab = createDialogSettingsTab(folder);

    updateControlsStateFromPersistedData();
    updateWebXMLOptions();
    validatePageInput();

    TabItem tab1 = new TabItem(folder, SWT.NONE);
    tab1.setText("Deployment Settings"); //$NON-NLS-1$
    tab1.setControl(deploymentTab);

    TabItem tab2 = new TabItem(folder, SWT.NONE);
    tab2.setText("Dialog Settings"); //$NON-NLS-1$
    tab2.setControl(dialogSettingsTab);

    return folder;
  }

  protected Composite createDeploymentSettingsTab(Composite pParent)
  {
    Composite topComp = UICompositeUtil.createFillBothComposite(pParent, 1, 1);

    createWebFolderGroup(topComp);
    createContextRootGroup(topComp);
    createDeploymentDirectoryGroup(topComp);
    createDeploymentOptionGroup(topComp);

    return topComp;
  }

  protected Composite createDialogSettingsTab(Composite pParent)
  {
    Composite topComp = UICompositeUtil.createFillBothComposite(pParent, 1, 1);

    createOverwritePromptGroup(topComp);
    createOverwriteWebXMLPromptGroup(topComp);

    return topComp;
  }

  /**   
   * @see org.eclipse.jface.preference.IPreferencePage#performOk()
   */
  public boolean performOk()
  {
    updatePersistedDataFromControls();

    // I am not checking if the inputs are valid here. Because if the user
    // say remove the exo-express, then all java project property pages will
    // refuse to close until the deployment directory is correct. And that is
    // not what we want the user experience to be.
    return super.performOk();
  }

  /**   
   * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
   */
  protected void performDefaults()
  {
    updateControlsStateFromPersistedData();
    super.performDefaults();
  }

  private IProject getProjectElement()
  {
    IAdaptable adaptable = getElement();
    IProject elem = (IProject) adaptable.getAdapter(IProject.class);

    return elem;
  }

  private String getPersistedWebFolder(IProject pProject)
  {
    String path = null;
    try
    {
      IPortletProject portletProject =
        ExoCorePlugin.createPortletProject(pProject);
      IPath ipath = portletProject.getWebFolder();
      if (ipath != null)
      {
        path = ipath.toString();
      }
    }
    catch (Exception e)
    {
    }

    return path;
  }

  private String getPersistedContextRoot(IProject pProject)
  {
    String path = null;
    try
    {
      IPortletProject portletProject =
        ExoCorePlugin.createPortletProject(pProject);
      IPath ipath = portletProject.getContextRoot();
      if (ipath != null)
      {
        path = ipath.toString();
      }
    }
    catch (Exception e)
    {
    }

    return path;
  }

  private String getPersistedDeploymentDir(IProject pProject)
  {
    String path = null;
    try
    {
      path =
        pProject.getPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_DIR);
    }
    catch (Exception e)
    {
    }

    return path;
  }

  private int getPersistedDeploymentOption(IProject pProject)
  {
    int option = IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_DEFAULT;
    try
    {
      option =
        Integer.parseInt(pProject.getPersistentProperty(
            IExoProjectRuntimeProperties.DEPLOYMENT_OPTION));
    }
    catch (Exception e)
    {
    }

    return option;
  }

  private boolean getPersistedDeploymentOverwriteOption(IProject pProject)
  {
    boolean overwritePrompt =
      IExoProjectRuntimeProperties.OVERWRITE_PROMPT_DEFAULT;
    try
    {
      String overwritePromptString =
        pProject.getPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_PROMPT);
      if (overwritePromptString != null)
      {
        overwritePrompt = new Boolean(overwritePromptString).booleanValue();
      }
    }
    catch (Exception e)
    {
    }

    return overwritePrompt;
  }

  private boolean getPersistedOverwriteWebXMLPrompt(IProject pProject)
  {
    boolean result =
      IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT_DEFAULT;
    String resultString = null;
    try
    {
      resultString =
        pProject.getPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT);
      if ((resultString == null) || (resultString.length() < 1))
      {
        pProject.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT,
          String.valueOf(result));
      }
      else
      {
        result = Boolean.valueOf(resultString).booleanValue();
      }
    }
    catch (Exception e)
    {
    }

    return result;
  }

  private boolean getPersistedDetectWebXMLOption(IProject pProject)
  {
    boolean result =
      IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION_DEFAULT;
    String resultString = null;
    try
    {
      resultString =
        pProject.getPersistentProperty(IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION);
      if ((resultString == null) || (resultString.length() < 1))
      {
        pProject.setPersistentProperty(IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION,
          String.valueOf(result));
      }
      else
      {
        result = Boolean.valueOf(resultString).booleanValue();
      }
    }
    catch (Exception e)
    {
    }

    return result;
  }

  private void updateControlsStateFromPersistedData()
  {
    IProject project = getProjectElement();
    String webfolder = getPersistedWebFolder(project);
    if ((mWebFolderText != null) && (webfolder != null))
    {
      mWebFolderText.setText(webfolder);
    }

    String contextroot = getPersistedContextRoot(project);
    if ((mContextRootText != null) && (contextroot != null))
    {
      mContextRootText.setText(contextroot);
    }

    String deploymentdir = getPersistedDeploymentDir(project);
    if ((mDeploymentDirText != null) && (deploymentdir != null))
    {
      mDeploymentDirText.setText(deploymentdir);
    }

    int deploymentOption = getPersistedDeploymentOption(project);
    if ((mDeploymentOption1 != null) && (mDeploymentOption2 != null))
    {
      switch (deploymentOption)
      {
        case IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_PACKAGED:
          mDeploymentOption1.setSelection(false);
          mDeploymentOption2.setSelection(true);

          break;

        default:
          mDeploymentOption1.setSelection(true);
          mDeploymentOption2.setSelection(false);
      }
    }

    boolean overwritePrompt = getPersistedDeploymentOverwriteOption(project);
    if (mDeploymentOverwriteButton != null)
    {
      mDeploymentOverwriteButton.setSelection(overwritePrompt);
    }

    boolean overwriteWebXMLPrompt = getPersistedOverwriteWebXMLPrompt(project);
    if (mWebXMLOverwriteButton != null)
    {
      mWebXMLOverwriteButton.setSelection(overwriteWebXMLPrompt);
    }

    boolean detectWebXML = getPersistedDetectWebXMLOption(project);
    if (mDetectWebXMLButton != null)
    {
      mDetectWebXMLButton.setSelection(detectWebXML);
    }
  }

  private void updatePersistedDataFromControls()
  {
    IProject project = getProjectElement();
    String webfolder = null;
    String contextroot = null;
    String deploymentdir = null;
    int deploymentOption =
      IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_DEFAULT;
    boolean deploymentOverwritePrompt =
      IExoProjectRuntimeProperties.OVERWRITE_PROMPT_DEFAULT;
    boolean webxmlOverwritePrompt =
      IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT_DEFAULT;
    boolean detectWebXMLOption =
      IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION_DEFAULT;

    try
    {
      if (mWebFolderText != null)
      {
        webfolder = mWebFolderText.getText();
        if ((webfolder != null) && (webfolder.length() < 1))
        {
          webfolder = null;
        }
      }

      if (mContextRootText != null)
      {
        contextroot = mContextRootText.getText();
        if ((contextroot != null) && (contextroot.length() < 1))
        {
          contextroot = null;
        }
      }

      handlePortletProjectStoreUpdate(webfolder, contextroot);

      if (mDeploymentDirText != null)
      {
        deploymentdir = mDeploymentDirText.getText();
        if ((deploymentdir != null) && (deploymentdir.length() < 1))
        {
          deploymentdir = null;
        }
      }
      project.setPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_DIR,
        deploymentdir);

      if ((mDeploymentOption1 != null) && (mDeploymentOption2 != null))
      {
        if (mDeploymentOption2.getSelection())
        {
          deploymentOption =
            IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_PACKAGED;
        }
      }
      project.setPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_OPTION,
        String.valueOf(deploymentOption));

      if (mDeploymentOverwriteButton != null)
      {
        deploymentOverwritePrompt = mDeploymentOverwriteButton.getSelection();
      }
      project.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_PROMPT,
        String.valueOf(deploymentOverwritePrompt));

      if (mWebXMLOverwriteButton != null)
      {
        webxmlOverwritePrompt = mWebXMLOverwriteButton.getSelection();
      }
      project.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT,
        String.valueOf(webxmlOverwritePrompt));

      if (mDetectWebXMLButton != null)
      {
        detectWebXMLOption = mDetectWebXMLButton.getSelection();
      }
      project.setPersistentProperty(IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION,
        String.valueOf(detectWebXMLOption));
    }
    catch (Exception e)
    {
      ExoUIPlugin.logError(e);
    }
  }

  private void handlePortletProjectStoreUpdate(String pWebFolder,
    String pContextRoot)
  {
    Shell shell = getShell();
    try
    {
      IProject project = getProjectElement();
      SimplePortletProjectStoreOperation op =
        new SimplePortletProjectStoreOperation(project);
      op.addPropertyToModify(IPreferencesConstant.PORTLET_PROJECT_STORE_CONTEXTROOT,
        pContextRoot);
      op.addPropertyToModify(IPreferencesConstant.PORTLET_PROJECT_STORE_WEBFOLDER,
        pWebFolder);

      op.run(new NullProgressMonitor());

      IStatus status = op.getStatus();
      if (!status.isOK())
      {
        ProblemDialog.open(shell, PROBLEM_TITLE, null, status);
      }
    }
    catch (InvocationTargetException target)
    {
      Throwable cause = target.getTargetException();
      if (cause instanceof CoreException)
      {
        IStatus status = ((CoreException) cause).getStatus();
        ProblemDialog.open(shell, PROBLEM_TITLE, null, status);
        ExoUIPlugin.log(status);
      }
      else
      {
        String message =
          (cause == null) ? target.getLocalizedMessage() : cause.toString();
        MessageDialog.openError(shell, PROBLEM_TITLE,
          "Encountered a problem while trying to update some of the eXo portlet settings\n"
          + message);
        ExoUIPlugin.logError(cause);
      }
    }
    catch (CoreException e)
    {
      IStatus status = e.getStatus();
      ProblemDialog.open(shell, PROBLEM_TITLE, null, status);
      ExoUIPlugin.log(status);
    }
    catch (InterruptedException e)
    {
      // no action is required if user requests cancelation
    }
  }

  private void createWebFolderGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Group group =
      UICompositeUtil.createGroup(pParent, "Web content folder setting", 2, 1);

    UICompositeUtil.addLabel(group,
      "The name of the folder that contains the web content\n (jsp pages, WEB-INF directory, etc.) : ",
      2);

    mWebFolderText =
      UICompositeUtil.addReadOnlyText(group, DEFAULT_WEBFOLDER_TEXT, 1);

    // browse button
    mWebFolderBrowseButton = new Button(group, SWT.PUSH);
    mWebFolderBrowseButton.setText("C&hange..");
    mWebFolderBrowseButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent event)
        {
          String loc = ""; //$NON-NLS-1$
          if (mWebFolderText != null)
          {
            loc = mWebFolderText.getText();
          }

          handleWebFolderBrowseButtonPressed(loc);
        }
      });

    mWebFolderBrowseButton.setEnabled(true);
    mWebFolderBrowseButton.setFont(font);
    setButtonLayoutData(mWebFolderBrowseButton);
  }

  private void createContextRootGroup(Composite pParent)
  {
    Group group =
      UICompositeUtil.createGroup(pParent, "Context root setting", 2, 1);

    UICompositeUtil.addLabelWithWrapping(group,
      "The context root to use during deployment : ", 2);
    mContextRootText =
      UICompositeUtil.addText(group, DEFAULT_CONTEXTROOT_TEXT, 2);
    mContextRootText.addListener(SWT.Modify, mTextModifyListener);
  }

  private void createDeploymentDirectoryGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroup(pParent, "Deployment Directory", 3, 1);
    String description =
      "The directory to use to deploy this web application to : ";
    UICompositeUtil.addLabelWithWrapping(group, description, 3);
    mDeploymentDirText =
      UICompositeUtil.addText(group, DEFAULT_DEPLOYMENTDIR_TEXT, 2);

    // browse button
    mDeploymentDirBrowseButton = new Button(group, SWT.PUSH);
    mDeploymentDirBrowseButton.setText("Br&owse..");
    mDeploymentDirBrowseButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent event)
        {
          String loc = ""; //$NON-NLS-1$
          if (mDeploymentDirText != null)
          {
            loc = mDeploymentDirText.getText();
          }

          handleDeploymentDirBrowseButtonPressed(loc);
        }
      });

    mDeploymentDirBrowseButton.setEnabled(true);
    mDeploymentDirBrowseButton.setFont(font);
    setButtonLayoutData(mDeploymentDirBrowseButton);
    mDeploymentDirText.addListener(SWT.Modify, mTextModifyListener);
  }

  private void createDeploymentOptionGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroup(pParent, "Deployment Option", 1, 1);
    UICompositeUtil.addLabel(group, "The type of deployment that you prefer :",
      1);
    mDeploymentOption1 =
      UICompositeUtil.createRadioButton(group, "&Exploded Archive");
    mDeploymentOption1.setSelection(true);
    mDeploymentOption2 =
      UICompositeUtil.createRadioButton(group, "&Packaged Archive");
    mDeploymentOption2.setSelection(false);
  }

  private void createOverwritePromptGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroup(pParent,
        "Deployment Overwrite Dialog Option", 1, 1);
    mDeploymentOverwriteButton =
      UICompositeUtil.createCheckboxButton(group,
        "A&lways prompt me before overwriting any deployment file.");

    String overwriteDesc =
      "If you uncheck the box, then deployment files will always be "
      + "overwritten. The selection you make for this checkbox only "
      + "takes effect for the selected project. Each project has its "
      + "own deployment overwrite option.";
    UICompositeUtil.addLabelWithWrapping(group, overwriteDesc, 1);
  }

  private void createOverwriteWebXMLPromptGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroup(pParent,
        "eXo web.xml Modification Dialog Option", 1, 1);
    mDetectWebXMLButton =
      UICompositeUtil.createCheckboxButton(group,
        "Al&ways detect if the web.xml file needs modification.");

    mWebXMLOverwriteButton =
      UICompositeUtil.createCheckboxButton(group,
        "Alw&ays prompt me before modifying the web.xml file.");

    String overwriteDesc =
      "If you uncheck the box, then web.xml file will be modified without prompting you first. "
      + "The selection you make for this checkbox only "
      + "takes effect for the selected project. Each project has its "
      + "own eXo web.xml modification option. ";

    mWebXMLOverwriteDesc =
      UICompositeUtil.addLabelWithWrapping(group, overwriteDesc, 1);

    UICompositeUtil.addLabelWithWrapping(group, "", 1);

    String warningDesc =
      "WARNING: The existing format of web.xml will NOT be preserved. However, all the comments in the "
      + "web.xml will not be affected.";
    mWebXMLOverwriteDesc2 =
      UICompositeUtil.addLabelWithWrapping(group, warningDesc, 1);

    mDetectWebXMLButton.addSelectionListener(mWebXMLOptionsListener);
    mWebXMLOverwriteButton.addSelectionListener(mWebXMLOptionsListener);
  }

  /**   
   * Open an appropriate directory browser
   *
   * @param pLocation the location to start with
   */
  private void handleDeploymentDirBrowseButtonPressed(String pLocation)
  {
    DirectoryDialog dialog = new DirectoryDialog(mDeploymentDirText.getShell());
    dialog.setMessage("Please choose the deployment directory location");

    String dirName = pLocation;

    if (!dirName.equals("")) //$NON-NLS-1$
    {
      File path = new File(dirName);

      if (path.exists())
      {
        dialog.setFilterPath(new Path(dirName).toOSString());
      }
    }

    String selectedDirectory = dialog.open();

    if (selectedDirectory != null)
    {
      mDeploymentDirText.setText(selectedDirectory);
    }
  }

  private void handleWebFolderBrowseButtonPressed(String pInitialLocation)
  {
    ChangeWebFolderWizard wizard = new ChangeWebFolderWizard();
    wizard.init(getProjectElement());

    WizardDialog dialog = new WizardDialog(getShell(), wizard);
    dialog.open();

    IPath webfolder = wizard.getSelectedWebFolder();
    if (!webfolder.isEmpty())
    {
      if (mWebFolderText != null)
      {
        mWebFolderText.setText(webfolder.toString());
      }
    }
  }

  private void updateWebXMLOptions()
  {
    if ((mDetectWebXMLButton != null) && (mWebXMLOverwriteButton != null))
    {
      if (mDetectWebXMLButton.getSelection())
      {
        mWebXMLOverwriteButton.setEnabled(true);
        mWebXMLOverwriteDesc.setEnabled(true);
        mWebXMLOverwriteDesc2.setEnabled(true);
        
      }
      else
      {
        mWebXMLOverwriteButton.setEnabled(false);
        mWebXMLOverwriteDesc.setEnabled(false);
        mWebXMLOverwriteDesc2.setEnabled(false);        
      }
    }
  }

  protected boolean validatePageInput()
  {
    String contextRoot = DEFAULT_CONTEXTROOT_TEXT;
    String deploymentDir = DEFAULT_DEPLOYMENTDIR_TEXT;
    if (mContextRootText != null)
    {
      contextRoot = mContextRootText.getText();
    }
    if (mDeploymentDirText != null)
    {
      deploymentDir = mDeploymentDirText.getText();
    }
    
    // validate context root
    IStatus validationStatus =
      InputTextValidator.validateContextRootName(contextRoot);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    // validate deployment directory
    validationStatus =
      InputTextValidator.validateDeploymentDirectory(deploymentDir);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }
    
    setErrorMessage(null);
    setMessage(null);

    return true;
  }

  /**
   * Filter to remove any IFile resources.
   */
  private class FileFilter extends ViewerFilter
  {
    /* (non-Javadoc)
     * Method declared on ViewerFilter.
     */
    public boolean select(Viewer viewer, Object parentElement, Object element)
    {
      IResource resource = null;
      if (element instanceof IResource)
      {
        resource = (IResource) element;
      }
      else
      {
        if (element instanceof IAdaptable)
        {
          IAdaptable adaptable = (IAdaptable) element;
          resource = (IResource) adaptable.getAdapter(IResource.class);
        }
      }

      if (resource != null)
      {
        if (getProjectElement() != resource.getProject())
        {
          return false;
        }

        return resource.getType() != IResource.FILE;
      }
      else
      {
        return false;
      }
    }
  }
}
