/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.preference.helper;


import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * Represents a boolean field editor within a group border, and a description
 * under the checkbox and its label.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class GroupBooleanFieldEditor extends BooleanFieldEditor
{
  public static final String CLASS_VERSION = "$Id: GroupBooleanFieldEditor.java,v 1.1 2004/04/19 03:37:23 hatimk Exp $"; //$NON-NLS-1$
  private String mDescription = ""; //$NON-NLS-1$
  private String mGroupTitle = ""; //$NON-NLS-1$
  private Composite mMainComposite;

  /**
   * Creates a boolean field editor in the default style but within a group
   * border, and a description under the checkbox and its label.
   *
   * @param pName the name of the preference this field editor works on
   * @param pCheckBoxLabel the label text of the field editor
   * @param pGroupLabel the label to appear at the group header
   * @param pDescription the text to appear under the checkbox and its label
   * @param pParent the parent of the field editor's control
   */
  public GroupBooleanFieldEditor(String pName, String pCheckBoxLabel,
    String pGroupLabel, String pDescription, Composite pParent)
  {
    mDescription = pDescription;
    mGroupTitle = pGroupLabel;
    init(pName, pCheckBoxLabel);
    createControl(pParent);
  }

  /* (non-Javadoc)
   * Method declared on FieldEditor.
   */
  protected void doFillIntoGrid(Composite pParent, int pNumColumns)
  {
    Group group =
      UICompositeUtil.createGroupWithNoGrapping(pParent, mGroupTitle, 2,
        pNumColumns);
    super.doFillIntoGrid(group, 2);        
    UICompositeUtil.addLabelWithWrapping(group, mDescription, 2);

    mMainComposite = group;
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#adjustForNumColumns(int)
   */
  protected void adjustForNumColumns(int pNumColumns)
  {
    if (mMainComposite != null)
    {
      ((GridData) mMainComposite.getLayoutData()).horizontalSpan = pNumColumns;
    }
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#getNumberOfControls()
   */
  public int getNumberOfControls()
  {
    return 1;
  }
  
}
