/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.sample;

import org.eclipse.jface.resource.ImageDescriptor;

/**
 * An immutable class that represents a sample java web application to be
 * installed or created.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class SampleWebappProject
{
  public static final String CLASS_VERSION =
    "$Id: SampleWebappProject.java,v 1.1 2004/04/19 03:37:23 hatimk Exp $"; //$NON-NLS-1$
  private String mKind;
  private String mTitle;
  private String mDescription;
  private ImageDescriptor mIcon;
  private Object mCachedToken;

  /**
   * The constructor method. Should not be called by clients. If you need to
   * get instances of this object, then use
   * SampleWebappInitializer.getSampleWebappProjects() method.
   *
   * @param pCachedToken cached information that can only be used and
   *        interpreted  by the SampleWebappInitializer class
   * @param pKind the kind of webapp
   * @param pTitle the title of this sample
   * @param pDescription the description of this sample
   * @param pImage an icon that represents this sample
   */
  public SampleWebappProject(Object pCachedToken, String pKind, String pTitle,
    String pDescription, ImageDescriptor pImage)
  {
    mCachedToken = pCachedToken;
    mKind = pKind;
    mTitle = pTitle;
    mDescription = pDescription;
    mIcon = pImage;
  }

  /**   
   * A more elaborate description of the content of this sample. This
   * description is written with the user in mind. The description will be
   * presented by a wizard (that creates a web application such as the portlet
   * application wizard) next to the title of the sample. By looking at the
   * description, the user should be able to tell what the sample is for and
   * what it contains.
   *
   * @return the description of this sample
   */
  public String getDescription()
  {
    return mDescription;
  }

  /**   
   * An Icon descriptor that is associated with this sample.
   *
   * @return an Icon descriptor that is associated with this sample
   */
  public ImageDescriptor getIcon()
  {
    return mIcon;
  }

  /**   
   * There are two kinds: portlet-sample, and portlet-default. The value
   * 'portlet-sample' indicates that the content of this webapp is to be used
   * as a sample, while if the value 'portlet-default' indicates that the
   * content of this webapp is to be used as the default content. The portlet
   * application wizard will use a default content if the user didn't select
   * any sample webapp.
   *
   * @return the kind of this sample
   */
  public String getKind()
  {
    return mKind;
  }

  /**   
   * Title that provides a brief description of the sample content. This title
   * is presented to the user by a wizard that creates a web application such
   * as the portlet application wizard. The title should be short.
   *
   * @return
   */
  public String getTitle()
  {
    return mTitle;
  }

  /**   
   * Cached information that can only be used and interpreted  by the
   * SampleWebappInitializer class
   *
   * @return cached information that can only be used and interpreted  by the
   *         SampleWebappInitializer class
   */
  public Object getCachedToken()
  {
    return mCachedToken;
  }
}
