/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.wizard;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.internal.ui.actions.WorkbenchRunnableAdapter;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.Wizard;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.operation.ChangeWebFolderOperation;
import org.exoplatform.eclipse.core.operation.helper.WebappProjectSettings;
import org.exoplatform.eclipse.internal.ui.sample.IExoSampleConstant;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappInitializer;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappProject;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;

import java.lang.reflect.InvocationTargetException;

/**
 * DOCUMENT ME!
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ChangeWebFolderWizard extends Wizard
{
  public static final String CLASS_VERSION = "$Id: ChangeWebFolderWizard.java,v 1.1 2004/04/19 03:37:24 hatimk Exp $"; //$NON-NLS-1$
  private boolean mFatalErrorOccurred = false;
  private SelectWebFolderPage mWebFolderPage;
  private IProject mProject;
  private IPath mWebFolder;
  
  /**
   * The constructor method.
   */
  public ChangeWebFolderWizard()
  {
    super();
    setWindowTitle("Select a folder as the web content folder");
    setNeedsProgressMonitor(true);
    setDefaultPageImageDescriptor(ExoUIPluginImages.DESC_WIZBAN_CHANGE_WEB_FOLDER);
  }

  /**   
   * @see org.eclipse.jface.wizard.Wizard#addPages()
   */
  public void addPages()
  {
    super.addPages();

    mWebFolderPage = new SelectWebFolderPage("SelectWebFolderPage"); //$NON-NLS-1$
    mWebFolderPage.setTitle("Web Content Selection Page");
    mWebFolderPage.setDescription(
      "Specify the new web content folder from the given list, or create a new one.");
    mWebFolderPage.init(mProject);
    mWebFolderPage.setSelectTitle(
      "Select the new folder that contains the web content (jsp pages, WEB-INF directory, etc.):");
    mWebFolderPage.setGroupTitle("Web content folder selection");

    addPage(mWebFolderPage);
  }

  /**   
   * This method is run in the wizard container's context as a workspace
   * runnable.
   *
   * @param pMonitor to report progress
   *
   * @throws CoreException something went wrong
   * @throws InvocationTargetException
   * @throws InterruptedException we got interrupted
   */
  protected void finishPage(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    try
    {
      pMonitor.beginTask("modifying web content folder : ", 700);

      // see if we need to upgrade the web content folder
      IResource resource = mWebFolderPage.getResult();
      if ((resource != null) && (!resource.getProjectRelativePath().isEmpty()))
      {
        IPath webpath = resource.getProjectRelativePath();
        pMonitor.subTask("modifying web content folder");

        SampleWebappProject[] webapps =
          SampleWebappInitializer.getSampleWebappProjects(IExoSampleConstant.KIND_PORTLET_UPGRADE);
        WebappProjectSettings websettings =
          SampleWebappInitializer.getUpgradeWebappSettings(webapps[0], null,
            null, webpath);
        ChangeWebFolderOperation op =
          new ChangeWebFolderOperation(mProject, websettings,
            mWebFolderPage.getMakeDefaultBinaryOption(),
            mWebFolderPage.getAddJarsToClasspathOption(), new NoOverwriteQuery());
        op.run(new SubProgressMonitor(pMonitor, 700));
        mWebFolder = webpath;
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  /**   
   * @see Wizard#performFinish
   */
  public boolean performFinish()
  {
    IWorkspaceRunnable op =
      new IWorkspaceRunnable()
      {
        public void run(IProgressMonitor monitor)
          throws CoreException, OperationCanceledException
        {
          try
          {
            finishPage(monitor);
          }
          catch (InvocationTargetException eTarget)
          {
            Throwable e = eTarget.getTargetException();
            throw new CoreException(ExoUIPlugin.createErrorStatus(
                e.getMessage(), e));
          }
          catch (InterruptedException e)
          {
            throw new OperationCanceledException(e.getMessage());
          }
        }
      };

    try
    {
      getContainer().run(false, true, new WorkbenchRunnableAdapter(op));
      mFatalErrorOccurred = false;
    }
    catch (InvocationTargetException e)
    {
      handleException(e.getTargetException());
      mFatalErrorOccurred = true;

      return false;
    }
    catch (InterruptedException e)
    {
      mFatalErrorOccurred = true;

      return false;
    }

    return true;
  }

  /**   
   * The project to change its web content
   *
   * @param pSelectedProject the project to change its web content
   */
  public void init(IProject pSelectedProject)
  {
    mProject = pSelectedProject;
  }

  /**   
   * Returns the path of the selected web folder relative to the project,
   * otherwise  an empty path is returned.
   *
   * @return the path of the selected web folder relative to the project
   */
  public IPath getSelectedWebFolder()
  {
    return (mWebFolder == null)? new Path(""): mWebFolder;
  }

  /**   
   * @see org.eclipse.jface.wizard.IWizard#canFinish()
   */
  public boolean canFinish()
  {
    return super.canFinish() && !mFatalErrorOccurred;
  }

  protected void handleException(Throwable pTarget)
  {
    String title = "Error while changing web content folder";

    if (pTarget instanceof CoreException)
    {
      IStatus status = ((CoreException) pTarget).getStatus();
      ExoUIPlugin.log(status);
      ErrorDialog.openError(getShell(), title, null, status);
    }
    else
    {
      ExoUIPlugin.logError(pTarget);
      MessageDialog.openError(getShell(), title, title + " : " + pTarget);
    }
  }

  // always answer not to overwrite any files
  protected class NoOverwriteQuery implements IOverwriteQuery
  {
    public String queryOverwrite(String file)
    {
      String[] returnCodes = { YES, NO, ALL, NO_ALL, CANCEL };
      int returnVal = 3;

      return (returnVal < 0) ? CANCEL : returnCodes[returnVal];
    }
  }
}
