/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import org.eclipse.jface.wizard.Wizard;
import org.exoplatform.eclipse.core.exoservices.Property;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;

/**
 * @author Hatim Khan
 */
public class EditParameterPropertyValueWizard extends Wizard
{
  public static final int ADD_MODE = EditParameterPropertyValuePage.ADD_MODE;
  public static final int EDIT_MODE = EditParameterPropertyValuePage.EDIT_MODE;
  
  public static final String EDIT_SERVICE_PARAMETER_TITLE = "Edit an eXo Platform service configuration parameter value";
  public static final String ADD_SERVICE_PARAMETER_TITLE = "Add an eXo Platform service configuration parameter value";

  private Object mPropertyParent;
  private Property mProperty;
  private EditParameterPropertyValuePage mPropertyPage;
  private int mMode = EDIT_MODE;
  private String mDidYouKnow;

  /**
   * The constructor method.
   */
  public EditParameterPropertyValueWizard(int pMode, String pDidYouKnow, Object pPropertyParent, Property pProperty)
  {
    super();
    mPropertyParent = pPropertyParent;
    mProperty = pProperty;
    mMode = pMode;
    mDidYouKnow = pDidYouKnow;
    setWindowTitle((mMode == ADD_MODE)? ADD_SERVICE_PARAMETER_TITLE: EDIT_SERVICE_PARAMETER_TITLE);
    setNeedsProgressMonitor(false);
    setForcePreviousAndNextButtons(false);
    setDefaultPageImageDescriptor(ExoUIPluginImages.DESC_WIZBAN_EXO_SERVICES_PARAM);
  }

  /**
   * @see org.eclipse.jface.wizard.Wizard#addPages()
   */
  public void addPages()
  {
    super.addPages();

    mPropertyPage = createPage();
    if (mPropertyPage != null) addPage(mPropertyPage);
  }

  private EditParameterPropertyValuePage createPage()
  {
    return new EditParameterPropertyValuePage(mMode, mDidYouKnow, mPropertyParent, mProperty);
  }

  /**
   * @see org.eclipse.jface.wizard.IWizard#performFinish()
   */
  public boolean performFinish()
  {
    if (mPropertyPage != null) mPropertyPage.finish();
    return true;
  }

  public boolean isDirty()
  {
    return (mPropertyPage != null) ? mPropertyPage.isDirty() : false;
  }
}