/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.exoplatform.eclipse.core.exoservices.ObjectParam;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.common.DidYouKnowMessages;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * @author Hatim Khan
 */
public class EditServiceObjectParamPage extends EditServiceParameterPage
{
  public static final String TITLE_EDIT = "Edit an eXo Platform service configuration ObjectParam";
  public static final String TITLE_ADD = "Add an eXo Platform service configuration ObjectParam";
  public static final String MESSAGE_EDIT = "You have the option of editing the name, description, type and values of this ObjectParam parameter";
  public static final String MESSAGE_ADD = "Please provide the name, type, the description (optional) and the values of this new ObjectParam parameter";

  public static final String PAGE_NAME = "EditServiceObjectParamPage";
  public static final String DID_YOU_KNOW_ADD = DidYouKnowMessages.getString("EditServiceObjectParamPage.add");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_EDIT = DidYouKnowMessages.getString("EditServiceObjectParamPage.edit");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_ADD_PROPERTY = DidYouKnowMessages.getString("EditServiceObjectParamPage.property.add");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_EDIT_PROPERTY = DidYouKnowMessages.getString("EditServiceObjectParamPage.property.edit");//$NON-NLS-1$

  private List mOriginalValues;
  private ObjectParam mCopyParameter;
  private String mOriginalType;
  private ParameterPropertyValueListPageDelegate mDelegate;

  private Text mTypeText;

  public EditServiceObjectParamPage(int pMode, PortalSettings pPortalSettings, Object pParameterParent,
      Parameter pParameter)
  {
    super(PAGE_NAME, (pMode == ADD_MODE) ? DID_YOU_KNOW_ADD : DID_YOU_KNOW_EDIT, pMode, pPortalSettings,
        pParameterParent, pParameter);
    setTitle((pMode == ADD_MODE) ? TITLE_ADD : TITLE_EDIT);
    setMessage((pMode == ADD_MODE) ? MESSAGE_ADD : MESSAGE_EDIT);
    mDelegate = new ParameterPropertyValueListPageDelegate(DID_YOU_KNOW_ADD_PROPERTY,
        DID_YOU_KNOW_EDIT_PROPERTY);
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterPage#createNewParameter()
   */
  protected Parameter createNewParameter()
  {
    return new ObjectParam();
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterPage#addValuesContent(org.eclipse.swt.widgets.Composite)
   */
  protected void addValuesContent(Composite pClient)
  {
    createTypeContent(pClient);
    mDelegate.addValuesContent(pClient);
  }

  protected void createTypeContent(Composite pParent)
  {
    UICompositeUtil.addLabel(pParent, "Type : ", 1);

    mTypeText = UICompositeUtil.addText(pParent, "", 1);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    mTypeText.setLayoutData(gd);
  }

  protected void updateFromModel()
  {
    super.updateFromModel();

    String type = "";

    if ((mParameter != null) && (mParameter instanceof ObjectParam))
    {
      type = UICompositeUtil.emptyIfNull(((ObjectParam) mParameter).getType()).trim();
      mOriginalValues = ((ObjectParam) mParameter).getProperties();
      mCopyParameter = (ObjectParam) mParameter.getCopy();
      mDelegate.updateFromModel(mCopyParameter);
    }

    mOriginalType = type;
    mTypeText.setText(type);
  }

  public void finish()
  {
    // first lets detect if the user changed any of the values
    if (changeOcurred())
    {
      String type = UICompositeUtil.emptyIfNull(mTypeText.getText()).trim();
      List newValues = new ArrayList();
      if (mCopyParameter != null)
          UICompositeUtil.addAllIfNotNull(newValues, mCopyParameter.getProperitiesCopy());
      if (mParameter instanceof ObjectParam)
      {
        ObjectParam oparam = (ObjectParam) mParameter;
        oparam.setProperties(newValues);
        oparam.setType(type);
      }
      mDirty = true;
    }
    super.finish();
  }

  private boolean changeOcurred()
  {
    String type = UICompositeUtil.emptyIfNull(mTypeText.getText()).trim();
    if (!type.equals(mOriginalType)) return true;
    if ((mOriginalValues == null) && (mCopyParameter == null)) return false;
    if ((mOriginalValues == null) && ((mCopyParameter != null) && (mCopyParameter.getProperties() == null)))
        return false;
    if (mOriginalValues == null) return true;
    if (mCopyParameter == null) return true;

    return (!mCopyParameter.isEquivalentProperitiesList(mOriginalValues));
  }

  protected void attachListeners()
  {
    super.attachListeners();
    mDelegate.attachListeners();
    mTypeText.addListener(SWT.Modify, new Listener()
    {
      public void handleEvent(Event pEvent)
      {
        setPageComplete(validateInput());
      }
    });
  }

  protected boolean validateInput()
  {
    boolean isValid = super.validateInput();
    if (!isValid) return false;

    String type = UICompositeUtil.emptyIfNull(mTypeText.getText()).trim();

    // check if both are empty, if so then display an error
    if ((type.length() < 1))
    {
      setErrorMessage("The type of the parameter can NOT be empty.");
      return false;
    }

    // if we get to this point then validation is complete and we don't have any errors
    // so update the error message
    setErrorMessage(null);
    return true;
  }

}