/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.widgets.Composite;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.PropertiesParam;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.common.DidYouKnowMessages;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * @author Hatim Khan
 */
public class EditServicePropertiesParamPage extends EditServiceParameterPage
{
  public static final String TITLE_EDIT = "Edit an eXo Platform service configuration PropertiesParam";
  public static final String TITLE_ADD = "Add an eXo Platform service configuration PropertiesParam";
  public static final String MESSAGE_EDIT = "You have the option of editing the name, description and values of this PropertiesParam parameter";
  public static final String MESSAGE_ADD = "Please provide the name, the description (optional) and the values of this new PropertiesParam parameter";
  public static final String PAGE_NAME = "EditServicePropertiesParamPage";
  public static final String DID_YOU_KNOW_ADD = DidYouKnowMessages.getString("EditServicePropertiesParamPage.add");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_EDIT = DidYouKnowMessages.getString("EditServicePropertiesParamPage.edit");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_ADD_PROPERTY = DidYouKnowMessages.getString("EditServicePropertiesParamPage.property.add");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_EDIT_PROPERTY = DidYouKnowMessages.getString("EditServicePropertiesParamPage.property.edit");//$NON-NLS-1$

  private List mOriginalValues;
  private PropertiesParam mCopyParameter;
  private ParameterPropertyValueListPageDelegate mDelegate;

  public EditServicePropertiesParamPage(int pMode, PortalSettings pPortalSettings, Object pParameterParent,
      Parameter pParameter)
  {
    super(PAGE_NAME, (pMode == ADD_MODE) ? DID_YOU_KNOW_ADD : DID_YOU_KNOW_EDIT, pMode, pPortalSettings,
        pParameterParent, pParameter);
    setTitle((pMode == ADD_MODE) ? TITLE_ADD : TITLE_EDIT);
    setMessage((pMode == ADD_MODE) ? MESSAGE_ADD : MESSAGE_EDIT);
    mDelegate = new ParameterPropertyValueListPageDelegate(DID_YOU_KNOW_ADD_PROPERTY,
        DID_YOU_KNOW_EDIT_PROPERTY);
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterPage#createNewParameter()
   */
  protected Parameter createNewParameter()
  {
    return new PropertiesParam();
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterPage#addValuesContent(org.eclipse.swt.widgets.Composite)
   */
  protected void addValuesContent(Composite pClient)
  {
    mDelegate.addValuesContent(pClient);
  }

  protected void updateFromModel()
  {
    super.updateFromModel();

    if ((mParameter != null) && (mParameter instanceof PropertiesParam))
    {
      mOriginalValues = ((PropertiesParam) mParameter).getProperties();
      mCopyParameter = (PropertiesParam) mParameter.getCopy();
      mDelegate.updateFromModel(mCopyParameter);
    }
  }

  public void finish()
  {
    // first lets detect if the user changed any of the values
    if (changeOcurred())
    {
      List newValues = new ArrayList();
      if (mCopyParameter != null)
          UICompositeUtil.addAllIfNotNull(newValues, mCopyParameter.getProperitiesCopy());
      if (mParameter instanceof PropertiesParam)
      {
        ((PropertiesParam) mParameter).setProperties(newValues);
      }
      mDirty = true;
    }
    super.finish();
  }

  private boolean changeOcurred()
  {
    if ((mOriginalValues == null) && (mCopyParameter == null)) return false;
    if ((mOriginalValues == null) && ((mCopyParameter != null) && (mCopyParameter.getProperties() == null)))
        return false;
    if (mOriginalValues == null) return true;
    if (mCopyParameter == null) return true;

    return !mCopyParameter.isEquivalentProperitiesList(mOriginalValues);
  }

  protected void attachListeners()
  {
    super.attachListeners();
    mDelegate.attachListeners();
  }
}