/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import org.eclipse.jface.wizard.Wizard;
import org.exoplatform.eclipse.core.exoservices.Configuration;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;

/**
 * @author Hatim Khan
 */
public class EditServiceWizard extends Wizard
{
  public static int ADD_MODE = EditServicePage.ADD_MODE;
  public static int EDIT_MODE = EditServicePage.EDIT_MODE;
  public static final String ADD_TITLE = "Add a new eXo Platform service configuration";
  public static final String EDIT_TITLE = "Edit an eXo Platform service configuration";

  private PortalSettings mPortalSettings;
  private Configuration mConfiguration;
  private ServiceConfiguration mServiceConfiguration;
  private int mMode = ADD_MODE;

  private EditServicePage mServicePage;

  public EditServiceWizard(PortalSettings pPortalSettings, Configuration pConfiguration)
  {
    this(ADD_MODE, pPortalSettings, pConfiguration, null);
  }

  public EditServiceWizard(PortalSettings pPortalSettings, Configuration pConfiguration,
      ServiceConfiguration pServiceConfiguration)
  {
    this(EDIT_MODE, pPortalSettings, pConfiguration, pServiceConfiguration);
  }

  private EditServiceWizard(int pMode, PortalSettings pPortalSettings, Configuration pConfiguration,
      ServiceConfiguration pServiceConfiguration)
  {
    super();
    mPortalSettings = pPortalSettings;
    mConfiguration = pConfiguration;
    mServiceConfiguration = pServiceConfiguration;
    mMode = pMode;

    setWindowTitle((mMode == ADD_MODE) ? ADD_TITLE : EDIT_TITLE);
    setNeedsProgressMonitor(false);
    setForcePreviousAndNextButtons(false);
    setDefaultPageImageDescriptor(ExoUIPluginImages.DESC_WIZBAN_EXO_SERVICES);
  }

  /**
   * @see org.eclipse.jface.wizard.Wizard#addPages()
   */
  public void addPages()
  {
    super.addPages();
    if (mMode == EDIT_MODE) mServicePage = new EditServicePage(mPortalSettings, mConfiguration,
        mServiceConfiguration);
    else mServicePage = new EditServicePage(mPortalSettings, mConfiguration);
    addPage(mServicePage);
  }

  /**
   * @see org.eclipse.jface.wizard.IWizard#performFinish()
   */
  public boolean performFinish()
  {
    mServicePage.finish();
    return true;
  }

  public boolean isDirty()
  {
    return mServicePage.isDirty();
  }
}