/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.io.File;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.eclipse.ui.help.WorkbenchHelp;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;
import org.exoplatform.eclipse.ui.IExoUIPreferencesConstant;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;
import org.exoplatform.eclipse.ui.validator.InputTextValidator;

/**
 * @author Hatim
 */
public class NewMavenPortletsProjectSettingPage extends WizardNewProjectCreationPage
{

  private Text mContextRootText;
  private Text mDeploymentDirText;
  private Button mBrowseButton;

  private Listener mTextModifyListener = new Listener()
  {
    public void handleEvent(Event e)
    {
      setPageComplete(validatePage());
    }
  };

  /**
   * Creates a wizard page.
   * 
   * @param pPageName the name of this page
   */
  public NewMavenPortletsProjectSettingPage(String pPageName)
  {
    super(pPageName);
    setPageComplete(false);
  }

  /**
   * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {

    super.createControl(pParent);

    // because we want to overwrite the super help context we need to
    // reassign it here
    Control mainComposite = getControl();
    WorkbenchHelp.setHelp(mainComposite, IExoHelpContextIds.CREATE_PORTLET_PROJECT);

    createContextRootGroup((Composite) mainComposite);
    createDeploymentDirectoryGroup((Composite) mainComposite);
    setPageComplete(validatePage());

    // Show description on opening instead of error messages
    setErrorMessage(null);
    setMessage(null);
  }

  /**
   * Returns the value of the context root field with leading and trailing spaces removed.
   * 
   * @return the source folder
   */
  public String getContextRoot()
  {
    if (mContextRootText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mContextRootText.getText().trim();
    }
  }

  /**
   * Returns the value of the deployment directory location with leading and trailing
   * spaces removed.
   * 
   * @return the deployment directory location
   */
  public String getDeploymentFolder()
  {
    if (mDeploymentDirText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mDeploymentDirText.getText().trim();
    }
  }

  protected boolean validatePage()
  {
    boolean parentResult = super.validatePage();
    if (!parentResult)
    {
      return parentResult;
    }

    // validate context root
    String contextRoot = getContextRoot();
    IStatus validationStatus = InputTextValidator.validateContextRootName(contextRoot);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    // validate deployment directory
    String deploymentDir = getDeploymentFolder();
    validationStatus = InputTextValidator.validateDeploymentDirectory(deploymentDir);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    setErrorMessage(null);
    setMessage(null);

    return true;
  }

  private void createContextRootGroup(Composite pParent)
  {
    Group group = UICompositeUtil.createGroup(pParent, "Context root setting", 1, 1);

    UICompositeUtil.addLabelWithWrapping(group, "Please provide the context root to use for this web application during deployment. "
        + "Note: the context root is not used during the project creation. It will be used "
        + "later when you deploy the application. Therefore, you can leave it empty if you "
        + "want, and supply a value later in the property page of this project. You can n"
        + "access the property page by selecting the 'Project -> Properties' menu option", 1);
    mContextRootText = UICompositeUtil.addText(group, "", 1);
    mContextRootText.addListener(SWT.Modify, mTextModifyListener);
  }

  private void createDeploymentDirectoryGroup(Composite pParent)
  {
    Font font = pParent.getFont();
    Group group = UICompositeUtil.createGroup(pParent, "Deployment Directory", 2, 1);

    UICompositeUtil.addLabelWithWrapping(group, "Please provide the directory that will be used to deploy this web application. "
        + "Note: the deployment directory is not used during the project creation. It will "
        + "be used later when you deploy the application. Therefore, you can leave it empty "
        + "if you want, and supply a value later in the property page of this project. You "
        + "can access the property page by selecting the 'Project -> Properties' menu option."
        + "\n\n"
        + "If you installed eXo express on C:\\exo-tomcat, then the deployment directory "
        + "should be C:\\exo-tomcat\\webapps", 2);
    mDeploymentDirText = UICompositeUtil.addText(group, "", 1);

    // browse button
    mBrowseButton = new Button(group, SWT.PUSH);
    mBrowseButton.setText("B&rowse...");
    mBrowseButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent event)
      {
        String loc = ""; //$NON-NLS-1$
        if (mDeploymentDirText != null)
        {
          loc = mDeploymentDirText.getText();
        }

        handleLocationBrowseButtonPressed(loc);
      }
    });

    mBrowseButton.setEnabled(true);
    mBrowseButton.setFont(font);
    UICompositeUtil.setButtonLayoutData(mBrowseButton);

    // Set the initial value first before listener
    // to avoid handling an event during the creation.
    String initialLocation = ExoUIPlugin.getDefault().getPreferenceStore().getString(IExoUIPreferencesConstant.PREF_DEPLOYMENT_DIR);
    mDeploymentDirText.setText(initialLocation);
    mDeploymentDirText.addListener(SWT.Modify, mTextModifyListener);
  }

  /**
   * Open an appropriate directory browser
   * 
   * @param pLocation the location to start with
   */
  private void handleLocationBrowseButtonPressed(String pLocation)
  {
    DirectoryDialog dialog = new DirectoryDialog(mDeploymentDirText.getShell());
    dialog.setMessage("Please choose the deployment directory location");

    String dirName = pLocation;

    if (!dirName.equals("")) //$NON-NLS-1$
    {
      File path = new File(dirName);

      if (path.exists())
      {
        dialog.setFilterPath(new Path(dirName).toOSString());
      }
    }

    String selectedDirectory = dialog.open();

    if (selectedDirectory != null)
    {
      mDeploymentDirText.setText(selectedDirectory);
    }
  }

}