/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.exoplatform.eclipse.core.operation.helper.JavaProjectConfiguration;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappInitializer;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappProject;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;

/**
 * This wizard allows the user to create a portlet project that is maven friendly.
 * 
 * @author Hatim
 */
public class NewMavenPortletsProjectWizard extends NewPortletsProjectWizard
{
  private NewMavenPortletsProjectSettingPage mProjectSettingPage;
  private NewPortletsProjectSamplePage mProjectSamplePage;

  /**
   * The constructor method.
   */
  public NewMavenPortletsProjectWizard()
  {
    super();
    setWindowTitle("Create a Maven Friendly Project that contains Portlet(s)");
    setNeedsProgressMonitor(true);
    setDefaultPageImageDescriptor(ExoUIPluginImages.DESC_WIZBAN_NEW_PORTLET_PROJECT);
  }

  /**
   * @see org.eclipse.jface.wizard.Wizard#addPages()
   */
  public void addPages()
  {
    mProjectSettingPage = new NewMavenPortletsProjectSettingPage("NewMavenPortletsProjectSettingPage"); //$NON-NLS-1$
    mProjectSettingPage.setTitle("Project Settings Page");
    mProjectSettingPage.setDescription("Supply the information necessary to create the project such as name and path");

    mProjectSamplePage = new NewPortletsProjectSamplePage("NewMavenPortletsProjectSamplePage"); //$NON-NLS-1$
    mProjectSamplePage.setTitle("Sample Portlets");
    mProjectSamplePage.setDescription("Select a sample portlets project for a quicker start.");

    addPage(mProjectSettingPage);
    addPage(mProjectSamplePage);
  }

  protected String getProjectName()
  {
    return mProjectSettingPage.getProjectName();
  }

  protected IPath getProjectLocationPath()
  {
    if (mProjectSettingPage.useDefaults())
    {
      // this is to cover the case where the user choses the default
      // workspace as the location of the project
      return null;
    }
    else
    {
      return mProjectSettingPage.getLocationPath();
    }
  }

  protected String getDeploymentFolder()
  {
    return mProjectSettingPage.getDeploymentFolder();
  }

  protected String getDeploymentOption()
  {
    return String.valueOf(IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_EXPLODED);
  }

  protected IPath getSourceFolder()
  {
    return new Path("src/java");
  }

  protected IPath getWebFolder()
  {
    return new Path("src/webapp");
  }

  protected IPath getContextRootPath()
  {
    return new Path(mProjectSettingPage.getContextRoot());
  }

  protected JavaProjectConfiguration getJavaProjectConfiguration()
  {
    SampleWebappProject webappProject;
    SampleWebappProject[] sampleWebapps = mProjectSamplePage.getSelectedSampleWebappProjects();
    if (sampleWebapps.length > 0)
    {
      // we don't support the creation of multiple projects in this release
      webappProject = sampleWebapps[0];
    }
    else
    {
      webappProject = mProjectSamplePage.getDefaultProject();
    }

    JavaProjectConfiguration javaConfig = SampleWebappInitializer.getJavaProjectConfiguration(webappProject, null, getSourceFolder(), getWebFolder());
    // set up the default binary folder
    javaConfig.setDefaultBinaryFolder(new Path("target/classes"));
    return javaConfig;
  }

}