/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.wizard;

import java.io.File;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.help.WorkbenchHelp;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;
import org.exoplatform.eclipse.ui.IExoUIPreferencesConstant;
import org.exoplatform.eclipse.ui.validator.InputTextValidator;

/**
 * This page allows the user to specify the deployment directory. The page remembers the
 * last deployment directory that was specified.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class NewPortletsProjectDeploymentPage extends WizardPage
{
  public static final String CLASS_VERSION = "$Id: NewPortletsProjectDeploymentPage.java,v 1.1 2004/04/19 03:37:24 hatimk Exp $"; //$NON-NLS-1$
  private Text mDeploymentDirText;
  private Button mBrowseButton;
  private Button mDeploymentOption1;
  private Button mDeploymentOption2;
  private Listener mLocationModifyListener = new Listener()
  {
    public void handleEvent(Event e)
    {
      setPageComplete(validatePage());
    }
  };

  /**
   * Creates a wizard page.
   * 
   * @param pPageName the name of this page
   */
  public NewPortletsProjectDeploymentPage(String pPageName)
  {
    super(pPageName);
    setPageComplete(false);
  }

  /**
   * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite = createFillBothComposite(pParent, 1);
    initializeDialogUnits(pParent);
    createDeploymentDirectoryGroup(mainComposite, 1);
    createDeploymentOptionGroup(mainComposite, 1);
    setPageComplete(validatePage());

    WorkbenchHelp.setHelp(mainComposite, IExoHelpContextIds.SELECT_PORTLET_DEPLOYMENT);
    setControl(mainComposite);
  }

  /**
   * Returns the value of the deployment directory location with leading and trailing
   * spaces removed.
   * 
   * @return the deployment directory location
   */
  public String getDeploymentFolder()
  {
    if (mDeploymentDirText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mDeploymentDirText.getText().trim();
    }
  }

  /**
   * Returns the deployment option. The exploded option is returned by default if none is
   * selected by the user. To see all possible values please visit
   * IExoProjectRuntimeProperties interface.
   * 
   * @return the deployment option
   */
  public int getDeploymentOption()
  {
    if (mDeploymentOption1 == null)
    {
      return IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_EXPLODED;
    }
    if (mDeploymentOption1.getSelection())
    {
      return IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_EXPLODED;
    }
    else
    {
      return IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_PACKAGED;
    }
  }

  /*
   * see @DialogPage.setVisible(boolean)
   */
  public void setVisible(boolean visible)
  {
    super.setVisible(visible);
    if ((visible) && (mDeploymentDirText != null))
    {
      mDeploymentDirText.setFocus();
    }
  }

  /**
   * Returns whether this page's controls currently all contain valid values.
   * 
   * @return <code>true</code> if all controls are valid, and <code>false</code> if at
   *         least one is invalid
   */
  protected boolean validatePage()
  {
    // validate deployment directory
    String deploymentDir = getDeploymentFolder();
    IStatus validationStatus = InputTextValidator.validateDeploymentDirectory(deploymentDir);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    setErrorMessage(null);
    setMessage(null);

    return true;
  }

  private void createDeploymentOptionGroup(Composite pParent, int pHorizontalSpan)
  {
    Font font = pParent.getFont();

    Composite group = createGroup(pParent, "Deployment Option", 1, pHorizontalSpan);
    addLabel(group, "Please select the type of deployment that you prefer :\n\n", 1);
    mDeploymentOption1 = createRadioButton(group, "&Exploded Archive");
    mDeploymentOption1.setSelection(true);

    String option1Desc = "When you deploy your application using this option, the whole directory \n"
        + "structure including files and subdirectories in your project's web content \n"
        + "folder will be copied to the deployment folder under a directory with the \n"
        + "same name as the context root that you provided earlier.";
    addLabel(group, option1Desc, 1, 20);
    createVerticalSpacer(group, 1);
    mDeploymentOption2 = createRadioButton(group, "&Packaged Archive");
    mDeploymentOption2.setSelection(false);

    String option2Desc = "When you deploy your application using this option, a WAR file is created \n"
        + "that contains all the files and subdirectories from your project's web content \n"
        + "folder. The WAR file will have the same name as the context root that you \n"
        + "provided earlier. The WAR file is then put in the deployment directory.";
    addLabel(group, option2Desc, 1, 20);
  }

  private void createDeploymentDirectoryGroup(Composite pParent, int pHorizontalSpan)
  {
    Font font = pParent.getFont();

    Composite group = createGroup(pParent, "Deployment Directory", 3, pHorizontalSpan);
    String description = "Please provide the directory that will be used to deploy this web application. \n"
        + "Note: the deployment directory is not used during the project creation. It will \n"
        + "be used later when you deploy the application. Therefore, you can leave it empty \n"
        + "if you want, and supply a value later in the property page of this project. You \n"
        + "can access the property page by selecting the 'Project -> Properties' menu option." + "\n\n"
        + "If you installed eXo express on C:\\exo-express, then the deployment directory \n"
        + "should be C:\\exo-express\\webapps";

    addLabel(group, description, 3);
    mDeploymentDirText = addText(group, "");

    // browse button
    mBrowseButton = new Button(group, SWT.PUSH);
    mBrowseButton.setText("B&rowse..");
    mBrowseButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent event)
      {
        String loc = ""; //$NON-NLS-1$
        if (mDeploymentDirText != null)
        {
          loc = mDeploymentDirText.getText();
        }

        handleLocationBrowseButtonPressed(loc);
      }
    });

    mBrowseButton.setEnabled(true);
    mBrowseButton.setFont(font);
    setButtonLayoutData(mBrowseButton);

    // Set the initial value first before listener
    // to avoid handling an event during the creation.
    String initialLocation = ExoUIPlugin.getDefault().getPreferenceStore().getString(IExoUIPreferencesConstant.PREF_DEPLOYMENT_DIR);
    mDeploymentDirText.setText(initialLocation);
    mDeploymentDirText.addListener(SWT.Modify, mLocationModifyListener);
  }

  /**
   * Open an appropriate directory browser
   * 
   * @param pLocation the location to start with
   */
  private void handleLocationBrowseButtonPressed(String pLocation)
  {
    DirectoryDialog dialog = new DirectoryDialog(mDeploymentDirText.getShell());
    dialog.setMessage("Please choose the deployment directory location");

    String dirName = pLocation;

    if (!dirName.equals("")) //$NON-NLS-1$
    {
      File path = new File(dirName);

      if (path.exists())
      {
        dialog.setFilterPath(new Path(dirName).toOSString());
      }
    }

    String selectedDirectory = dialog.open();

    if (selectedDirectory != null)
    {
      mDeploymentDirText.setText(selectedDirectory);
    }
  }

  private Group createGroup(Composite pParent, String pTitle, int pNumColumns, int pHorizontalSpan)
  {
    Group group = new Group(pParent, SWT.NONE);
    group.setText(pTitle);

    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;

    //    data.grabExcessHorizontalSpace = true;
    //    data.grabExcessVerticalSpace = true;
    data.horizontalSpan = pHorizontalSpan;
    group.setLayout(layout);
    group.setLayoutData(data);
    group.setFont(pParent.getFont());

    return group;
  }

  private Composite createFillBothComposite(Composite pParent, int pNumColumns)
  {
    Composite composite = new Composite(pParent, SWT.NONE);
    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.makeColumnsEqualWidth = true;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;
    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    composite.setLayout(layout);
    composite.setLayoutData(data);
    composite.setFont(pParent.getFont());

    return composite;
  }

  private Label addLabel(Composite pComp, String pText, int pHorizontalSpan)
  {
    return addLabel(pComp, pText, pHorizontalSpan, 0);
  }

  private Label addLabel(Composite pComp, String pText, int pHorizontalSpan, int pHorizontalIndent)
  {
    Label label = new Label(pComp, SWT.NONE);

    label.setText(pText);

    GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
    gridData.horizontalSpan = pHorizontalSpan;
    gridData.horizontalIndent = pHorizontalIndent;
    label.setLayoutData(gridData);

    return label;
  }

  private Text addText(Composite pComp, String pTextDefaultValue)
  {
    Text result = new Text(pComp, SWT.SINGLE | SWT.BORDER);
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    result.setLayoutData(gridData);
    result.setText(pTextDefaultValue);

    return result;
  }

  /**
   * Utility method that creates a radio button instance and sets the default layout data.
   * 
   * @param pParent the parent for the new button
   * @param pLabel the label for the new button
   * 
   * @return the newly-created button
   */
  private Button createRadioButton(Composite pParent, String pLabel)
  {
    Button button = new Button(pParent, SWT.RADIO | SWT.LEFT);
    button.setText(pLabel);

    //    button.addSelectionListener(this);
    GridData data = new GridData();
    button.setLayoutData(data);

    return button;
  }

  /**
   * Create some empty space.
   * 
   * @param pComp the composite to add the space to
   * @param pColSpan the number of column to span
   */
  private void createVerticalSpacer(Composite pComp, int pColSpan)
  {
    Label label = new Label(pComp, SWT.NONE);
    GridData gd = new GridData();
    gd.horizontalSpan = pColSpan;
    label.setLayoutData(gd);
  }
}