/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.io.File;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.exoplatform.eclipse.core.exoservices.IExoPlatformInstallationInfo;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.IExoUIPreferencesConstant;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;
import org.exoplatform.eclipse.ui.validator.InputTextValidator;

/**
 * @author Hatim Khan
 */
public class NewTomcatPortalProjectSettingPage extends WizardNewProjectCreationPage
{

  private Text mContextRootText;
  private Text mInstallationDirText;
  private Button mBrowseButton;
  private Combo mJDKVersion;
  

  private Listener mTextModifyListener = new Listener()
  {
    public void handleEvent(Event e)
    {
      setPageComplete(validatePage());
    }
  };

  /**
   * Creates a wizard page.
   * 
   * @param pPageName the name of this page
   */
  public NewTomcatPortalProjectSettingPage(String pPageName)
  {
    super(pPageName);
    setPageComplete(false);
  }

  /**
   * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {

    super.createControl(pParent);

    // because we want to overwrite the super help context we need to
    // reassign it here
    Control mainComposite = getControl();
    //WorkbenchHelp.setHelp(mainComposite, IExoHelpContextIds.CREATE_PORTLET_PROJECT);
    
    createInstallationDirectoryGroup((Composite) mainComposite);
    createJDKVersionGroup((Composite) mainComposite);
    createContextRootGroup((Composite) mainComposite);
    setPageComplete(validatePage());

    // Show description on opening instead of error messages
    setErrorMessage(null);
    setMessage(null);
  }

  /**
   * Returns the value of the context root field with leading and trailing spaces removed.
   * 
   * @return the source folder
   */
  public String getContextRoot()
  {
    if (mContextRootText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mContextRootText.getText().trim();
    }
  }

  /**
   * @return the selected jdk version
   */
  public String getJDKVersion()
  {
    if (mJDKVersion == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mJDKVersion.getText().trim();
    }
  }
  
  /**
   * Returns the value of the installation directory location with leading and trailing
   * spaces removed.
   * 
   * @return the installation directory location
   */
  public String getInstallationFolder()
  {
    if (mInstallationDirText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mInstallationDirText.getText().trim();
    }
  }

  protected boolean validatePage()
  {
    boolean parentResult = super.validatePage();
    if (!parentResult)
    {
      return parentResult;
    }

    // validate context root
    String contextRoot = getContextRoot();
    IStatus validationStatus = InputTextValidator.validateContextRootName(contextRoot);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    // validate installation directory
    String installationDir = getInstallationFolder();
    validationStatus = InputTextValidator.validateInstallationDirectory(installationDir);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    setErrorMessage(null);
    setMessage(null);

    return true;
  }

  private void createContextRootGroup(Composite pParent)
  {
    Group group = UICompositeUtil.createGroup(pParent, "Context root setting", 1, 1);
    // don't grap vertical space
    GridData gd = (GridData) group.getLayoutData();
    gd.grabExcessVerticalSpace= false;

    UICompositeUtil.addLabelWithWrapping(group, "Please provide the context root to use for this portal. (example: /myportal)", 1);
    mContextRootText = UICompositeUtil.addText(group, "", 1);
    mContextRootText.addListener(SWT.Modify, mTextModifyListener);
  }

  private void createJDKVersionGroup(Composite pParent)
  {
    Font font = pParent.getFont();
    Group group = UICompositeUtil.createGroup(pParent, "eXo Tomcat JDK Version", 1, 1);
    // don't grap vertical space
    GridData gd = (GridData) group.getLayoutData();
    gd.grabExcessVerticalSpace= false;

    UICompositeUtil.addLabelWithWrapping(group, "Please select the JDK version that eXo Tomcat is using.", 1);

    mJDKVersion = new Combo(group, SWT.DROP_DOWN | SWT.READ_ONLY);

    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    mJDKVersion.setLayoutData(gridData);
    mJDKVersion.setFont(font);
    mJDKVersion.removeAll();
    mJDKVersion.add(IExoPlatformInstallationInfo.JDK_1_4);
    mJDKVersion.add(IExoPlatformInstallationInfo.JDK_1_5);
    mJDKVersion.select(0);
  }
  
  private void createInstallationDirectoryGroup(Composite pParent)
  {
    Font font = pParent.getFont();
    Group group = UICompositeUtil.createGroup(pParent, "eXo Tomcat Installation Directory", 2, 1);
    // don't grap vertical space
    GridData gd = (GridData) group.getLayoutData();
    gd.grabExcessVerticalSpace= false;

    UICompositeUtil.addLabelWithWrapping(group, "Please provide the installation directory of eXo Tomcat. The wizard will then "
        + "scan the installation directory to obtain the portal settings to use for the portal project that you are about to create.", 2);
    mInstallationDirText = UICompositeUtil.addText(group, "", 1);

    // browse button
    mBrowseButton = new Button(group, SWT.PUSH);
    mBrowseButton.setText("B&rowse...");
    mBrowseButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent event)
      {
        String loc = ""; //$NON-NLS-1$
        if (mInstallationDirText != null)
        {
          loc = mInstallationDirText.getText();
        }

        handleLocationBrowseButtonPressed(loc);
      }
    });

    mBrowseButton.setEnabled(true);
    mBrowseButton.setFont(font);
    UICompositeUtil.setButtonLayoutData(mBrowseButton);

    // Set the initial value first before listener
    // to avoid handling an event during the creation.
    String initialLocation = ExoUIPlugin.getDefault().getPreferenceStore().getString(IExoUIPreferencesConstant.PREF_INSTALLATION_DIR);
    mInstallationDirText.setText(initialLocation);
    mInstallationDirText.addListener(SWT.Modify, mTextModifyListener);
  }

  /**
   * Open an appropriate directory browser
   * 
   * @param pLocation the location to start with
   */
  private void handleLocationBrowseButtonPressed(String pLocation)
  {
    DirectoryDialog dialog = new DirectoryDialog(mInstallationDirText.getShell());
    dialog.setMessage("Please choose the installation directory location");

    String dirName = pLocation;

    if (!dirName.equals("")) //$NON-NLS-1$
    {
      File path = new File(dirName);

      if (path.exists())
      {
        dialog.setFilterPath(new Path(dirName).toOSString());
      }
    }

    String selectedDirectory = dialog.open();

    if (selectedDirectory != null)
    {
      mInstallationDirText.setText(selectedDirectory);
    }
  }

}