/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;

import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.internal.ui.launchConfigurations.OrganizeFavoritesAction;
import org.eclipse.debug.ui.DebugUITools;
import org.eclipse.debug.ui.actions.AbstractLaunchHistoryAction;
import org.eclipse.debug.ui.actions.OpenLaunchDialogAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchWindow;
import org.exoplatform.eclipse.ui.launching.ExoUILaunching;


/**
 * An implementation of an action that displays a drop-down launch history for the debug eXo
 * platform launch group.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class DebugExoServerMenuActionDelegate extends AbstractLaunchHistoryAction
{
  public static final String CLASS_VERSION = "$Id: DebugExoServerMenuActionDelegate.java,v 1.1 2004/09/20 01:42:55 hatimk Exp $"; //$NON-NLS-1$
  private IWorkbenchWindow mWindow;

	/**
	 * The constructor method.
	 */
	public DebugExoServerMenuActionDelegate()
	{
		super(ExoUILaunching.DEBUG_EXO_LAUNCH_GROUP_ID);
	}

  /**
   * A launch history action that also includes an action to open the launch configuration dialog.
   *
   * @param pCategoryId the category id for this launch group
   */
  public DebugExoServerMenuActionDelegate(String pCategoryId)
  {
    super(pCategoryId);
  }

  /**   
   * Launch the last launch, or open the launch config dialog if none.
   *
   * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
   */
  public void run(IAction action)
  {
    ILaunchConfiguration configuration = getLastLaunch();
    if (configuration == null)
    {
      DebugUITools.openLaunchConfigurationDialogOnGroup(mWindow.getShell(),
        new StructuredSelection(), getLaunchGroupIdentifier());
    }
    else
    {
      DebugUITools.launch(configuration, getMode());
    }
  }

  /**   
   * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
   */
  public void init(IWorkbenchWindow pWindow)
  {
    mWindow = pWindow;
  }

  /**   
   * Fills the drop-down menu with an action to open the launch configuration dialog.
   *
   * @param menu the menu to fill
   */
  protected void fillMenu(Menu menu)
  {
    super.fillMenu(menu);

    // Separator between history and common actions
    if (menu.getItemCount() > 0)
    {
      addSeparator(menu);
    }

    addToMenu(menu, new OpenLaunchDialogAction(getLaunchGroupIdentifier()), -1);
	addToMenu(menu, new OrganizeFavoritesAction(getLaunchGroupIdentifier()), -1);    
  }
}
