/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.ui.DebugUITools;
import org.eclipse.debug.ui.IDebugUIConstants;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.exoplatform.eclipse.core.launching.IExoLaunchingConfigurationConstants;
import org.exoplatform.eclipse.ui.ExoUIPlugin;

/**
 * Use this action to stop a running eXo platform.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class StopExoServerAction extends Action
{
  public static final String CLASS_VERSION = "$Id: StopExoServerAction.java,v 1.1 2004/09/20 01:42:55 hatimk Exp $"; //$NON-NLS-1$
  private ILaunch mLaunch;
  private IWorkbenchWindow mWindow;

  /**
   * The constructor method.
   *
   * @param pLaunch the launch that represents the running eXo Platform to stop
   * @param pWindow
   */
  public StopExoServerAction(ILaunch pLaunch, IWorkbenchWindow pWindow)
  {
    mLaunch = pLaunch;
    mWindow = pWindow;

    ILaunchConfiguration config = mLaunch.getLaunchConfiguration();
    if (config != null)
    {
      setText(config.getName());
      setImageDescriptor(DebugUITools.getDefaultImageDescriptor(config));
    }
  }

  /**   
   * @see org.eclipse.jface.action.Action#run()
   */
  public void run()
  {
    if (mLaunch.isTerminated())
    {
      MessageDialog.openError(mWindow.getShell(), "Stopping eXo Platform Error",
        "eXo Platform seems to be already stopped.");
    }
    else
    {
      ILaunchConfiguration config = mLaunch.getLaunchConfiguration();
      if (config != null)
      {
        try
        {
          ILaunchConfigurationWorkingCopy workingCopy = config.getWorkingCopy();
          workingCopy.setAttribute(IExoLaunchingConfigurationConstants.ATTR_EXO_SHUTDOWN, true);
          workingCopy.setAttribute(IDebugUIConstants.ATTR_PRIVATE, true);
          workingCopy.launch(ILaunchManager.DEBUG_MODE, null);

          String name = workingCopy.getName();
          workingCopy.rename("Stop " + name);
        }
        catch (CoreException e)
        {
          MessageDialog.openError(mWindow.getShell(), "Stopping eXo Platform Error",
            "Encountered a problem while trying to stop eXo Platform. " + e);
          ExoUIPlugin.logError(e);
        }
      }
    }
  }
}
