/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.exoplatform.eclipse.core.operation.ValidatePortletDeploymentDescriptorOperation;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.dialog.ProblemDialog;

import java.lang.reflect.InvocationTargetException;

/**
 * Simply validate the selected portlet deployment descriptor. The actual
 * validation is done by a exo core operation.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ValidatePortletActionDelegate implements IObjectActionDelegate
{
  public static final String CLASS_VERSION = "$Id: ValidatePortletActionDelegate.java,v 1.1 2004/04/19 03:37:26 hatimk Exp $"; //$NON-NLS-1$
  private static final String VALIDATION_PROBLEM_TITLE =
    "Portlet Validation Problem";
  IWorkbenchPart mActivePart;
  ISelection mCurrentSelection;

  /**   
   * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
   *      org.eclipse.ui.IWorkbenchPart)
   */
  public void setActivePart(IAction pAction, IWorkbenchPart pTargetPart)
  {
    mActivePart = pTargetPart;
  }

  /**   
   * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
   */
  public void run(IAction pAction)
  {
    if (mCurrentSelection instanceof IStructuredSelection)
    {
      Object firstElement =
        ((IStructuredSelection) mCurrentSelection).getFirstElement();

      if (!(firstElement instanceof IFile))
      {
        return;
      }

      IFile portletFile = (IFile) firstElement;
      handleValidation(portletFile);
    }
  }

  private void handleValidation(IFile pPortletFile)
  {
    Shell shell = mActivePart.getSite().getShell();
    ValidatePortletDeploymentDescriptorOperation op = null;
    try
    {
      op = new ValidatePortletDeploymentDescriptorOperation(pPortletFile);
      new ProgressMonitorDialog(shell).run(true, true,
        new ExoRunnableAdapter(op));

      IStatus status = op.getStatus();
      if (!status.isOK())
      {
        ProblemDialog.open(shell, VALIDATION_PROBLEM_TITLE, null, status);
      }
    }
    catch (InvocationTargetException target)
    {
      Throwable cause = target.getTargetException();
      if (cause instanceof CoreException)
      {
        IStatus status = ((CoreException) cause).getStatus();
        ProblemDialog.open(shell, VALIDATION_PROBLEM_TITLE, null, status);
        ExoUIPlugin.log(status);
      }
      else
      {
        String message =
          (cause == null) ? target.getLocalizedMessage() : cause.toString();
        MessageDialog.openError(shell, VALIDATION_PROBLEM_TITLE,
          "Encountered a problem while trying to validate all portlets\n"
          + message);
        ExoUIPlugin.logError(cause);
      }
    }
    catch (InterruptedException e)
    {
      // no action is required if user requests cancelation
    }
  }

  /**   
   * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
   *      org.eclipse.jface.viewers.ISelection)
   */
  public void selectionChanged(IAction pAction, ISelection pSelection)
  {
    mCurrentSelection = pSelection;
  }
}
