/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.operation.WebXMLModifyOperation;
import org.exoplatform.eclipse.core.operation.helper.IWebXMLModifyStrategy;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.common.WebXMLModifyDialog;
import org.exoplatform.eclipse.ui.dialog.ProblemDialog;

import java.lang.reflect.InvocationTargetException;

/**
 * This action takes care of modifying web.xml so that it follows the eXo
 * Platform portlet container requirement.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class WebXMLModifyAction extends Action implements IWebXMLModifyStrategy
{
  public static final String CLASS_VERSION =
    "$Id: WebXMLModifyAction.java,v 1.1 2004/04/19 03:37:26 hatimk Exp $"; //$NON-NLS-1$
  public static final String MESSAGE_TITLE = "eXo web.xml Modification";
  private IProject mProject;
  private IFile mWebXML;
  private Shell mShell;
  private int mOverwrite = WebXMLModifyDialog.NO_BUTTON_INDEX;

  /**
   * The constructor method.
   *
   * @param pShell
   * @param pProject
   */
  public WebXMLModifyAction(Shell pShell, IProject pProject)
  {
    mShell = pShell;
    mProject = pProject;
  }

  /**
   */
  public void run()
  {
    super.run();

    WebXMLModifyOperation op = null;
    try
    {
      IPortletProject portletProject =
        ExoCorePlugin.createPortletProject(mProject);

      op = new WebXMLModifyOperation(portletProject, this);
      new ProgressMonitorDialog(mShell).run(true, true,
        new ExoRunnableAdapter(op));

      IStatus status = op.getStatus();
      if (!status.isOK())
      {
        ProblemDialog.open(mShell, MESSAGE_TITLE, null, status); //$NON-NLS-1$
      }
    }
    catch (InvocationTargetException target)
    {
      Throwable cause = target.getTargetException();
      if (cause instanceof CoreException)
      {
        IStatus status = ((CoreException) cause).getStatus();
        ProblemDialog.open(mShell, MESSAGE_TITLE, null, status); //$NON-NLS-1$        
        ExoUIPlugin.log(status);
      }
      else
      {
        String message =
          (cause == null) ? target.getLocalizedMessage() : cause.toString();
        MessageDialog.openError(mShell, MESSAGE_TITLE,
          "Encountered a problem while trying to detect/modify web.xml "
          + "as part of the eXo portlet deployment requirement: \n" + message);
        ExoUIPlugin.logError(cause);
      }
    }
    catch (CoreException e)
    {
      IStatus status = e.getStatus();
      ProblemDialog.open(mShell, MESSAGE_TITLE, null, status); //$NON-NLS-1$        
      ExoUIPlugin.log(status);
    }
    catch (InterruptedException e)
    {
      // no action is required if user requests cancelation
    }
  }

  private void showWarning(final String pMessage)
  {
    Display display = mShell.getDisplay();
    display.syncExec(new Runnable()
      {
        public void run()
        {
          MessageDialog.openWarning(mShell, MESSAGE_TITLE, pMessage);
        }
      });
  }

  private void showError(final String pMessage)
  {
    Display display = mShell.getDisplay();
    display.syncExec(new Runnable()
      {
        public void run()
        {
          MessageDialog.openError(mShell, MESSAGE_TITLE, pMessage);
        }
      });
  }

  private int openDialog(IFile pWebXML)
  {
    mWebXML = pWebXML;
    mShell.getDisplay().syncExec(new Runnable()
      {
        public void run()
        {
          String title = "eXo web.xml Modification";
          String msg =
            "web.xml must be modified to follow eXo Platform "
            + "portlet container requirement. Do you want to modify it?";
          WebXMLModifyDialog dialog =
            new WebXMLModifyDialog(mShell, title, null, msg,
              WebXMLModifyDialog.QUESTION, 0, mWebXML);
          mOverwrite = dialog.open();
        }
      });

    return mOverwrite;
  }

  /**   
   * @see org.exoplatform.eclipse.core.operation.helper.IWebXMLModifyStrategy#readyToModify()
   */
  public boolean isOkToModify(IFile pWebXML)
  {
    // you need to check if the user requested to be prompted before
    // a change is made, if so, display a dialog to confirm the user wants
    // to processed, if not then don't continue
    if (openDialog(pWebXML) == WebXMLModifyDialog.NO_BUTTON_INDEX)
    {
      return false;
    }
    else
    {
      return true;
    }
  }
}
