/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.ui.common;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.portal.model.INode;

/**
 * @author Hatim Khan
 */
public class PortalConfigurationContentProvider implements ITreeContentProvider
{

  private boolean mTrimParameterValues = false;

  public PortalConfigurationContentProvider()
  {
    this(false);
  }

  public PortalConfigurationContentProvider(boolean pTrimParameterValues)
  {
    mTrimParameterValues = pTrimParameterValues;
  }

  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
   */
  public Object[] getChildren(Object pParentElement)
  {
    if ((mTrimParameterValues) && (pParentElement instanceof Parameter)) return null;
    if (pParentElement instanceof INode) return ((INode) pParentElement).getChildren();
    else return null;
  }

  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
   */
  public Object getParent(Object pElement)
  {
    if (pElement instanceof INode) return ((INode) pElement).getParent();
    else return null;
  }

  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
   */
  public boolean hasChildren(Object pElement)
  {
    Object[] children = null;
    if ((mTrimParameterValues) && (pElement instanceof Parameter))
    {
      // don't return any child
    }
    else if (pElement instanceof INode)
    {
      children = ((INode) pElement).getChildren();
    }
    return (children != null) ? (children.length > 0) : false;
  }

  /**
   * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
   */
  public Object[] getElements(Object pInputElement)
  {
    return getChildren(pInputElement);
  }

  /**
   * @see org.eclipse.jface.viewers.IContentProvider#dispose()
   */
  public void dispose()
  {
  }

  /**
   * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
   *      java.lang.Object, java.lang.Object)
   */
  public void inputChanged(Viewer pViewer, Object pOldInput, Object pNewInput)
  {
  }

}