/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.common;

import java.util.Collection;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormText;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

/**
 * A utility class to help compose UI elements
 * 
 * @author Hatim Khan
 * @version $Revision: 1.2 $
 */
public final class UICompositeUtil
{
  public static final String CLASS_VERSION = "$Id: UICompositeUtil.java,v 1.2 2004/09/20 01:42:57 hatimk Exp $"; //$NON-NLS-1$

  /**
   * DOCUMENT ME!
   * 
   * @param pParent
   * @param pNumColumns
   * @param pHorizontalSpan
   * 
   * @return
   */
  public static Composite createFillBothComposite(Composite pParent, int pNumColumns, int pHorizontalSpan)
  {
    Composite composite = new Composite(pParent, SWT.NONE);
    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.makeColumnsEqualWidth = true;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;
    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    data.horizontalSpan = pHorizontalSpan;
    composite.setLayout(layout);
    composite.setLayoutData(data);
    composite.setFont(pParent.getFont());

    return composite;
  }

  /**
   * DOCUMENT ME!
   * 
   * @param pParent
   * @param pTitle
   * @param pNumColumns
   * @param pHorizontalSpan
   * 
   * @return
   */
  public static Group createGroup(Composite pParent, String pTitle, int pNumColumns, int pHorizontalSpan)
  {
    Group group = new Group(pParent, SWT.NONE);
    group.setText(pTitle);

    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.numColumns = pNumColumns;
    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;

    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    data.horizontalSpan = pHorizontalSpan;
    group.setLayout(layout);
    group.setLayoutData(data);
    group.setFont(pParent.getFont());

    return group;
  }

  /**
   * Utility method that creates a group composite that fills both vertically and
   * horizontally but does not grap excess spaces.
   * 
   * @param pParent the parent composite
   * @param pTitle the title of the group
   * @param pNumColumns number of columns that this group will contain
   * @param pHorizontalSpan the number of columns that this group should span horizontally
   * 
   * @return the group composite that fills both vertically and horizontally but does not
   *         grap excess spaces.
   */
  public static Group createGroupWithNoGrapping(Composite pParent, String pTitle, int pNumColumns,
      int pHorizontalSpan)
  {
    Group group = new Group(pParent, SWT.NONE);
    group.setText(pTitle);

    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;

    data.grabExcessHorizontalSpace = false;
    data.grabExcessVerticalSpace = false;
    data.horizontalSpan = pHorizontalSpan;
    group.setLayout(layout);
    group.setLayoutData(data);
    group.setFont(pParent.getFont());

    return group;
  }

  /**
   * DOCUMENT ME!
   * 
   * @param pComp
   * @param pText
   * @param pHorizontalSpan
   * 
   * @return
   */
  public static Label addLabel(Composite pComp, String pText, int pHorizontalSpan)
  {
    return addLabel(pComp, pText, pHorizontalSpan, 0);
  }

  /**
   * DOCUMENT ME!
   * 
   * @param pComp
   * @param pText
   * @param pHorizontalSpan
   * @param pHorizontalIndent
   * 
   * @return
   */
  public static Label addLabel(Composite pComp, String pText, int pHorizontalSpan, int pHorizontalIndent)
  {
    Label label = new Label(pComp, SWT.NONE);

    label.setText(pText);

    GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
    gridData.horizontalSpan = pHorizontalSpan;
    gridData.horizontalIndent = pHorizontalIndent;
    label.setLayoutData(gridData);

    return label;
  }

  /**
   * Returns a label that wrapps. The original width of the label is determined by the
   * IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH. Use this method to create labels that
   * wrapp, so you don't have to insert '\n' in the text yourself.
   * 
   * @param pComp the parent control
   * @param pText the text of the label
   * @param pHorizontalSpan how many columns to expand this label into
   * 
   * @return a label that wrapps
   */
  public static Label addLabelWithWrapping(Composite pComp, String pText, int pHorizontalSpan)
  {
    return addLabelWithWrapping(pComp, pText, pHorizontalSpan, 0);
  }

  /**
   * Returns a label that wrapps. The original width of the label is determined by the
   * IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH. Use this method to create labels that
   * wrapp, so you don't have to insert '\n' in the text yourself.
   * 
   * @param pComp the parent control
   * @param pText the text of the label
   * @param pHorizontalSpan how many columns to expand this label into
   * @param pHorizontalIndent how many pixel to use to indent this label
   * 
   * @return a label that wrapps
   */
  public static Label addLabelWithWrapping(Composite pComp, String pText, int pHorizontalSpan,
      int pHorizontalIndent)
  {
    Label label = null;

    if (pText != null)
    {
      label = new Label(pComp, SWT.WRAP);

      // Compute a font metric to be used to figure out the width hint for the label
      GC gc = new GC(label);
      gc.setFont(pComp.getFont());

      FontMetrics fontMetrics = gc.getFontMetrics();
      gc.dispose();

      // knowing the font metrics start building the label
      label.setText(pText);

      //      GridData data =
      //        new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_BEGINNING);
      GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
      data.widthHint = org.eclipse.jface.dialogs.Dialog.convertHorizontalDLUsToPixels(fontMetrics, IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
      label.setFont(pComp.getFont());
      data.horizontalSpan = pHorizontalSpan;
      data.horizontalIndent = pHorizontalIndent;
      label.setLayoutData(data);
    }

    return label;
  }

  /**
   * DOCUMENT ME!
   * 
   * @param pComp
   * @param pTextDefaultValue
   * @param pHorizontalSpan
   * 
   * @return
   */
  public static Text addReadOnlyText(Composite pComp, String pTextDefaultValue, int pHorizontalSpan)
  {
    Text result = new Text(pComp, SWT.SINGLE | SWT.BORDER | SWT.READ_ONLY);
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = pHorizontalSpan;
    result.setLayoutData(gridData);
    result.setText(pTextDefaultValue);

    return result;
  }

  /**
   * DOCUMENT ME!
   * 
   * @param pComp
   * @param pTextDefaultValue
   * @param pHorizontalSpan
   * 
   * @return
   */
  public static Text addText(Composite pComp, String pTextDefaultValue, int pHorizontalSpan)
  {
    Text result = new Text(pComp, SWT.SINGLE | SWT.BORDER);
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = pHorizontalSpan;
    result.setLayoutData(gridData);
    result.setText(pTextDefaultValue);

    return result;
  }

  /**
   * Sets the <code>GridData</code> on the specified button to be one that is spaced for
   * the current dialog page units.
   * 
   * @param pButton the button to set the <code>GridData</code>
   * 
   * @return the <code>GridData</code> set on the specified button
   */
  public static GridData setButtonLayoutData(Button pButton)
  {
    GridData data = new GridData(GridData.END);
    // Compute a font metric to be used to figure out the width hint for the label
    GC gc = new GC(pButton);
    gc.setFont(pButton.getFont());

    FontMetrics fontMetrics = gc.getFontMetrics();
    gc.dispose();

    int widthHint = org.eclipse.jface.dialogs.Dialog.convertHorizontalDLUsToPixels(fontMetrics, IDialogConstants.BUTTON_WIDTH);
    data.widthHint = Math.max(widthHint, pButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).x);
    data.heightHint = org.eclipse.jface.dialogs.Dialog.convertVerticalDLUsToPixels(fontMetrics, IDialogConstants.BUTTON_HEIGHT);
    pButton.setLayoutData(data);

    return data;
  }

  /**
   * @param pControl
   * @param pDefaultUnits
   */
  public static void setWidthHint(Control pControl, int pDefaultUnits)
  {
    Object dataObject = pControl.getLayoutData();
    if ((dataObject == null) || (!(dataObject instanceof GridData)))
    {
      // the grid data must be set first
      return;
    }

    GridData data = (GridData) dataObject;

    // Compute a font metric to be used to figure out the width hint
    GC gc = new GC(pControl);
    gc.setFont(pControl.getFont());

    FontMetrics fontMetrics = gc.getFontMetrics();
    gc.dispose();

    data.widthHint = org.eclipse.jface.dialogs.Dialog.convertHorizontalDLUsToPixels(fontMetrics, pDefaultUnits);
    pControl.setLayoutData(data);
  }

  /**
   * Sets the height hint of the given control to the given units instead of pixles. So
   * this method takes care of the conversion from dialog units to pixels.
   * 
   * @param pControl the control
   * @param pUnits the units in dialog units instead of pixels
   */
  public static void setHeightHint(Control pControl, int pUnits)
  {
    Object dataObject = pControl.getLayoutData();
    if ((dataObject == null) || (!(dataObject instanceof GridData)))
    {
      // the grid data must be set first
      return;
    }

    GridData data = (GridData) dataObject;

    // Compute a font metric to be used to figure out the width hint
    GC gc = new GC(pControl);
    gc.setFont(pControl.getFont());

    FontMetrics fontMetrics = gc.getFontMetrics();
    gc.dispose();

    data.heightHint = org.eclipse.jface.dialogs.Dialog.convertHorizontalDLUsToPixels(fontMetrics, pUnits);
    pControl.setLayoutData(data);
  }

  /**
   * Utility method that creates a radio button instance and sets the default layout data.
   * 
   * @param pParent the parent for the new button
   * @param pLabel the label for the new button
   * 
   * @return the newly-created button
   */
  public static Button createRadioButton(Composite pParent, String pLabel)
  {
    Button button = new Button(pParent, SWT.RADIO | SWT.LEFT);
    button.setText(pLabel);

    //    button.addSelectionListener(this);
    GridData data = new GridData();
    button.setLayoutData(data);

    return button;
  }

  /**
   * Utility method that creates a checkbox button instance and sets the default layout
   * data.
   * 
   * @param pParent the parent for the new button
   * @param pLabel the label for the new button
   * 
   * @return the newly-created button
   */
  public static Button createCheckboxButton(Composite pParent, String pLabel)
  {
    Button button = new Button(pParent, SWT.CHECK | SWT.LEFT);
    button.setText(pLabel);

    GridData data = new GridData(GridData.FILL_HORIZONTAL);
    button.setLayoutData(data);

    return button;
  }

  /**
   * Return the number of rows available in the current display using the current font.
   * 
   * @param pParent The Composite whose Font will be queried.
   * 
   * @return int The result of the display size divided by the font size.
   */
  public static int availableRows(Composite pParent)
  {
    int fontHeight = (pParent.getFont().getFontData())[0].getHeight();
    int displayHeight = pParent.getDisplay().getClientArea().height;

    return displayHeight / fontHeight;
  }

  /**
   * Return whether or not the font in the parent is the size of a result font (i.e.
   * smaller than the High Contrast Font). This method is used to make layout decisions
   * based on screen space.
   * 
   * @param pParent The Composite whose Font will be queried.
   * 
   * @return boolean. True if there are more than 50 lines of possible text in the
   *         display.
   */
  public static boolean inRegularFontMode(Composite pParent)
  {
    return availableRows(pParent) > 50;
  }

  /**
   * A convenient method to recursively visit all childern and enable them. Notice that
   * the given parent will be affected too.
   * 
   * @param pParent the composite to be affected
   */
  public static void enableAllWidgets(Composite pParent)
  {
    setEnableForAllWidgets(pParent, true);
  }

  /**
   * A convenient method to recursively visit all childern and disable them. Notice that
   * the given parent will be affected too.
   * 
   * @param pParent the composite to be affected
   */
  public static void disableAllWidgets(Composite pParent)
  {
    setEnableForAllWidgets(pParent, false);
  }

  /**
   * A convenient method to recursively visit all childern and set the enable flag to the
   * given value. Notice that the given parent will be affected too.
   * 
   * @param pParent the composite to be affected
   * @param pState the state to set the enable flag to
   */
  private static void setEnableForAllWidgets(Composite pParent, boolean pState)
  {
    if (pParent == null)
    {
      return;
    }

    pParent.setEnabled(pState);

    Control[] widgets = pParent.getChildren();

    if (widgets != null)
    {
      for (int i = 0, sizei = widgets.length; i < sizei; ++i)
      {
        if (widgets[i] instanceof Composite)
        {
          setEnableForAllWidgets((Composite) widgets[i], pState);
        }
        else
        {
          widgets[i].setEnabled(pState);
        }
      }
    }
  }

  public static Composite createContainer(Composite pParent, int pSpan, int pMarginWidth, int pMarginHeight,
      FormToolkit pToolkit)
  {
    Composite container = pToolkit.createComposite(pParent);
    GridLayout layout = new GridLayout();
    layout.marginWidth = pMarginWidth;
    layout.marginHeight = pMarginHeight;
    layout.numColumns = pSpan;
    container.setLayout(layout);
    return container;
  }

  public static void setWidthHeight(Control pControl, int pWidth, int pHeight)
  {
    Object layoutData = pControl.getLayoutData();
    if (layoutData instanceof GridData)
    {
      GridData layoutd = (GridData) layoutData;
      layoutd.heightHint = pHeight;
      layoutd.widthHint = pWidth;
    }
  }

  public static String emptyIfNull(String pValue)
  {
    return (pValue == null) ? "" : pValue;
  }

  public static void addAllIfNotNull(Collection p1, Collection p2)
  {
    if ((p1 == null) || (p2 == null)) return;
    p1.addAll(p2);
  }

  public static void createFormGridSpacer(FormToolkit pToolkit, Composite pParent, int pSpan)
  {
    Label spacer = pToolkit.createLabel(pParent, ""); //$NON-NLS-1$
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = pSpan;
    spacer.setLayoutData(gd);
  }

  public static String escapeForXML(String pString)
  {
    if (pString == null) return "";
    String string = pString.replaceAll("&[a-zA-Z]*;", " ");
    string = string.replaceAll("<", "&lt;");
    return string.replaceAll(">", "&gt;");
  }

  public static String escapeBackslash(String pString)
  {
    if (pString == null) return "";
    String string = pString;
    string = string.replaceAll("\t", "\\\\t");
    string = string.replaceAll("\r", "\\\\r");
    string = string.replaceAll("\f", "\\\\f");
    string = string.replaceAll("\b", "\\\\b");
    string = string.replaceAll("\n", "\\\\n");
    return string;
  }

  /**
   * @param pSelection the selection to inspect
   * @return returns the selected object if any is found, otherwise null is returned
   */
  public static Object getSelection(ISelection pSelection)
  {
    return UICompositeUtil.getSelection(pSelection, (Class[]) null);
  }

  /**
   * @param pSelection the ISelection that must be IStructureSelection otherwise null is
   *        returned
   * @param pClass the selection is expected to have the same class name as the given
   *        pClass otherwise null is returned
   * @return the selection (if any) but only if it matches the type of the class given,
   *         null otherwise
   */
  public static Object getSelection(ISelection pSelection, Class pClass)
  {
    return UICompositeUtil.getSelection(pSelection, new Class[] { pClass });
  }

  /**
   * @param pSelection the ISelection that must be IStructureSelection otherwise null is
   *        returned
   * @param pClass[] the selection is expected to have the same class name as the any of
   *        the given classes otherwise null is returned
   * @return the selection (if any) but only if it matches the type of any of the classes
   *         given, null otherwise
   */
  public static Object getSelection(ISelection pSelection, Class[] pClasses)
  {
    Object result = null;
    if ((pSelection == null) || (pSelection.isEmpty())) return result;

    if (pSelection instanceof IStructuredSelection)
    {
      IStructuredSelection ss = (IStructuredSelection) pSelection;
      if (ss.size() == 1)
      {
        Object selection = ss.getFirstElement();
        if ((pClasses != null) && (pClasses.length > 0))
        {
          String foundType = selection.getClass().getName();
          for (int i = 0, sizei = pClasses.length; i < sizei; ++i)
          {
            String desiredType = pClasses[i].getName();
            if (desiredType.equalsIgnoreCase(foundType))
            {
              result = selection;
              break;
            }
          }
        }
        else
        {
          result = selection;
        }

      }
    }

    return result;
  }

  /**
   * This is a simple method that creates an expandable widget in the given parent, and
   * then populates the client area of the widget with the given text. The text is
   * expected to contain tags that follow the form conventions for formatted texts. The
   * text is not expected to have any images.
   */
  public static ExpandableComposite createFormattedTextExpandable(Composite pParent, FormToolkit pToolkit,
      String pTextWithTags)
  {
    ExpandableComposite expandable = pToolkit.createExpandableComposite(pParent, ExpandableComposite.TWISTIE
        | ExpandableComposite.EXPANDED);
    expandable.clientVerticalSpacing = 10;
    expandable.marginHeight = 0;
    expandable.marginWidth = 5;

    TableWrapLayout layout = new TableWrapLayout();
    expandable.setLayout(layout);
    TableWrapData td = new TableWrapData(TableWrapData.FILL_GRAB);
    expandable.setLayoutData(td);

    // create the form text and populate it with the given text
    FormText rtext = pToolkit.createFormText(expandable, false);
    rtext.setText(pTextWithTags, true, false);

    pToolkit.paintBordersFor(rtext);
    expandable.setClient(rtext);
    return expandable;
  }

}