/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.common;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;

/**
 * A dialog to prompt the user of the modification of web.xml <display-name> 
 * and other tags such as <listner> <servlet> and <servlet-mapping>
 * The dialog contains a checkbox, if the user unselect the checkbox, then
 * the dialog will never appear again, and it will always assume that the
 * answer to the overwrite is yes.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class WebXMLModifyDialog extends MessageDialog
  implements SelectionListener
{
  public static final String CLASS_VERSION = "$Id: WebXMLModifyDialog.java,v 1.1 2004/04/19 03:37:39 hatimk Exp $"; //$NON-NLS-1$
  private IResource mResource;
  private Button mOverwritePrompt;
  private boolean mCurrentSelection =
    IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT_DEFAULT;
  public static final int YES_BUTTON_INDEX = 0;
  public static final int NO_BUTTON_INDEX = 1;

  /**
   * The constructor method.
   *
   * @param pParentShell  
   * @param pDialogTitle  
   * @param pDialogTitleImage  
   * @param pDialogMessage  
   * @param pDialogImageType  
   * @param pDefaultIndex  
   * @param pResource  
   */
  public WebXMLModifyDialog(Shell pParentShell, String pDialogTitle,
    Image pDialogTitleImage, String pDialogMessage, int pDialogImageType,
    int pDefaultIndex, IResource pResource)
  {
    super(pParentShell, pDialogTitle, pDialogTitleImage, pDialogMessage,
      pDialogImageType,
      new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL, },
      pDefaultIndex);
    mResource = pResource;
  }

  /**   
   * @see org.eclipse.jface.window.Window#open()
   */
  public int open()
  {
    boolean prompt = getOverwriteWebXMLPrompt(mResource.getProject());
    if (prompt == false)
    {
      return YES_BUTTON_INDEX;
    }

    int select = super.open();
    saveOverwriteWebXMLPrompt(mResource.getProject());

    return select;
  }

  /**   
   * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
   */
  public void widgetSelected(SelectionEvent e)
  {
    Button button = (Button) e.widget;
    mCurrentSelection = button.getSelection();
    updateButtonsEnablement(mCurrentSelection);
  }

  /**   
   * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
   */
  public void widgetDefaultSelected(SelectionEvent e)
  {
  }

  protected void updateButtonsEnablement(boolean pOverwritePromptSelection)
  {
    if (!pOverwritePromptSelection)
    {
      getButton(YES_BUTTON_INDEX).setEnabled(true);
      getButton(NO_BUTTON_INDEX).setEnabled(false);
      getButton(YES_BUTTON_INDEX).setFocus();
    }
    else
    {
      getButton(YES_BUTTON_INDEX).setEnabled(true);
      getButton(NO_BUTTON_INDEX).setEnabled(true);
      getButton(YES_BUTTON_INDEX).setFocus();
    }
  }

  /**   
   * @see org.eclipse.jface.dialogs.IconAndMessageDialog#createButtonBar(org.eclipse.swt.widgets.Composite)
   */
  protected Control createButtonBar(Composite pParent)
  {
    Composite main = UICompositeUtil.createFillBothComposite(pParent, 1, 2);
    Composite parent = (Composite) super.createButtonBar(main);
    createOverwritePromptGroup(main, 1);

    return main;
  }

  private void createOverwritePromptGroup(Composite pParent, int pHorizontalSpan)
  {
    Font font = pParent.getFont();

    Composite group =
      createGroup(pParent, "Overwrite Option", 1, pHorizontalSpan);
    mOverwritePrompt =
      createCheckboxButton(group,
        "A&lways prompt me before modifying the web.xml file.");

    boolean selection = getOverwriteWebXMLPrompt(mResource.getProject());
    mOverwritePrompt.setSelection(selection);
    mCurrentSelection = selection;
    updateButtonsEnablement(mCurrentSelection);

    String overwriteDesc =
      "If you uncheck the box, then web.xml file will be modified without prompting you first. "+
      "If you wish to be prompted again, you can find an option in the property page "+
      "of this project. NOTE: The selection you make for this checkbox only takes effect for the selected project. "
      + "Each project has its own web.xml overwrite option.";
    addLabel(group, overwriteDesc, 1);

    addLabel(group, "", 1);

    String warningDesc =
      "WARNING: The existing format of web.xml will NOT be preserved. However, all the comments in the " +
      "web.xml will not be affected.";
    addLabel(group, warningDesc, 1);
  }

  /**   
   * Utility method that creates a checkbox button instance and sets the
   * default layout data.
   *
   * @param pParent the parent for the new button
   * @param pLabel the label for the new button
   *
   * @return the newly-created button
   */
  private Button createCheckboxButton(Composite pParent, String pLabel)
  {
    Button button = new Button(pParent, SWT.CHECK | SWT.LEFT);
    button.setText(pLabel);

    GridData data = new GridData();
    button.setLayoutData(data);
    button.addSelectionListener(this);

    return button;
  }

  private Group createGroup(Composite pParent, String pTitle, int pNumColumns,
    int pHorizontalSpan)
  {
    Group group = new Group(pParent, SWT.NONE);
    group.setText(pTitle);

    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;

    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    data.horizontalSpan = pHorizontalSpan;
    group.setLayout(layout);
    group.setLayoutData(data);
    group.setFont(pParent.getFont());

    return group;
  }

  private Label addLabel(Composite pComp, String pText, int pHorizontalSpan)
  {
    return addLabel(pComp, pText, pHorizontalSpan, 0);
  }

  private Label addLabel(Composite pComp, String pText, int pHorizontalSpan,
    int pHorizontalIndent)
  {
    Label label = null;

    if (pText != null)
    {
      label = new Label(pComp, SWT.WRAP);

      // Compute a font metric to be used to figure out the width hint for the label
      GC gc = new GC(label);
      gc.setFont(pComp.getFont());

      FontMetrics fontMetrics = gc.getFontMetrics();
      gc.dispose();

      // knowing the font metrics start building the label      
      label.setText(pText);

      GridData data =
        new GridData(GridData.HORIZONTAL_ALIGN_FILL
          | GridData.VERTICAL_ALIGN_BEGINNING);
      data.widthHint =
        org.eclipse.jface.dialogs.Dialog.convertHorizontalDLUsToPixels(fontMetrics,
          IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
      label.setFont(pComp.getFont());
      data.horizontalSpan = pHorizontalSpan;
      data.horizontalIndent = pHorizontalIndent;
      label.setLayoutData(data);
    }

    return label;
  }

  private void saveOverwriteWebXMLPrompt(IProject pProject)
  {
    boolean overwriteSelection = mCurrentSelection;
    try
    {
      pProject.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT,
        String.valueOf(overwriteSelection));
    }
    catch (CoreException e)
    {
    }
  }

  private boolean getOverwriteWebXMLPrompt(IProject pProject)
  {
    boolean result =
      IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT_DEFAULT;
    String resultString = null;
    try
    {
      resultString =
        pProject.getPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT);
      if ((resultString == null) || (resultString.length() < 1))
      {
        pProject.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT,
          String.valueOf(result));
      }
      else
      {
        result = Boolean.valueOf(resultString).booleanValue();
      }
    }
    catch (Exception e)
    {
    }

    return result;
  }
}
