/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.ui.dialog;

import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.exoplatform.eclipse.core.exoservices.ValuesParam;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * @author Hatim Khan
 */
public class EditValueHolderDialog extends Dialog
{
  private Object mValueHolderParent;
  private ValueHolder mValueHolder;
  private String mOriginalValue;
  private boolean mDirty;

  private Text mValueText;
  private String mWindowTitle;

  public EditValueHolderDialog(Shell pParentShell, String pWindowTitle, Object pValueHolderParent,
      ValueHolder pValueHolder)
  {
    super(pParentShell);
    setShellStyle(getShellStyle() | SWT.RESIZE);
    mWindowTitle = pWindowTitle;
    mValueHolderParent = pValueHolderParent;
    mValueHolder = pValueHolder;
  }

  /**
   * Creates and returns the contents of the upper part of this dialog (above the button
   * bar).
   * <p>
   * The <code>Dialog</code> implementation of this framework method creates and returns
   * a new <code>Composite</code> with no margins and spacing.
   * </p>
   * 
   * @param parent the parent composite to contain the dialog area
   * @return the dialog area control
   */
  protected Control createDialogArea(Composite parent)
  {
    Composite composite = (Composite) super.createDialogArea(parent);

    createValueContent(composite);
    updateFromModel();    
    return composite;
  }

  private void createValueContent(Composite pParent)
  {
    Composite container = new Composite(pParent, SWT.NONE);
    GridLayout layout = new GridLayout(2, false);
    container.setLayout(layout);
    container.setLayoutData(new GridData(GridData.FILL_BOTH));
    container.setFont(pParent.getFont());

    UICompositeUtil.addLabel(container, "Value : ", 1);

    mValueText = UICompositeUtil.addText(container, "", 1);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    mValueText.setLayoutData(gd);
    UICompositeUtil.setWidthHint(mValueText, IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);    
  }

  protected void configureShell(Shell pNewShell)
  {
    super.configureShell(pNewShell);
    pNewShell.setText(mWindowTitle);
  }

  protected void okPressed()
  {
    // first lets detect if the user changed the value
    String value = UICompositeUtil.emptyIfNull(mValueText.getText());
    if ((!value.equals(mOriginalValue)) || (mValueHolder == null))
    {
      mDirty = true;
      if (mValueHolder == null)
      {
        mValueHolder = new ValueHolder();
        mValueHolder.setValue(value);
        if ((mValueHolderParent != null) && (mValueHolderParent instanceof INode))
        {
          INode node = (INode) mValueHolderParent;
          node.addChild(mValueHolder);
        }
      }
      else
      {
        // we can only handle the ValuesParam as the parent
        if ((mValueHolderParent != null) && (mValueHolderParent instanceof ValuesParam))
        {
          ValuesParam parent = (ValuesParam) mValueHolderParent;
          Object[] children = parent.getChildren();
          if (children != null)
          {
            int index = -1;
            for (int i = 0, sizei = children.length; i < sizei; ++i)
            {
              ValueHolder holder = (ValueHolder) children[i];
              if (holder.equals(mValueHolder))
              {
                index = i;
                break;
              }
            }

            // now do the acutal update if the index is found
            if (index > -1)
            {
              List values = parent.getValues();
              values.set(index, value);
            }
          }
        }
        mValueHolder.setValue(value);
      }
    }
    super.okPressed();
  }

  private void updateFromModel()
  {
    String value = "";
    if (mValueHolder != null)
    {
      value = UICompositeUtil.emptyIfNull(mValueHolder.getValue());
    }
    mOriginalValue = value;
    mValueText.setText(value);
    mDirty = false;
  }

  /**
   * @return Returns the dirty.
   */
  public boolean isDirty()
  {
    return mDirty;
  }
}