/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.launching;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.ui.IDebugUIConstants;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.launching.ExoLaunching;


/**
 * A place to gather common utilties methods for usage with launching eXo platform.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoUILaunching
{
  public static final String CLASS_VERSION = "$Id: ExoUILaunching.java,v 1.1 2004/09/20 01:42:59 hatimk Exp $"; //$NON-NLS-1$
  public static final String DEBUG_EXO_LAUNCH_GROUP_ID = "org.exoplatform.eclipse.ui.launching.DebugExoServerLaunchGroup"; //$NON-NLS-1$
  
	/**
	 * Returns all eXo Platform launches, this method returns launches even if they are terminated, but
	 * it will not return those that do not have a launch configuration associated with them or those that
	 * are marked as private.
	 * 
	 * @return the launch instances of eXo Platform   
	 */
	public static ILaunch[] getExoLaunches()
	{
		ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();

		ILaunch[] launches = manager.getLaunches();
		if (launches == null)
		{
			return new ILaunch[0];
		}

		List result = new ArrayList();

		for (int i = 0, sizei = launches.length; i < sizei; ++i)
		{
			ILaunchConfiguration config = launches[i].getLaunchConfiguration();
			if (config == null)
			{
				continue;
			}
			
			try
			{
				boolean isPrivate = config.getAttribute(IDebugUIConstants.ATTR_PRIVATE, false);
				if (isPrivate) 
				{
					continue;
				} 
			}
			catch (CoreException e)
			{
				ExoCorePlugin.logError("for some reason, we can not determine if this launch is a private launch",
					e);
			}
			
			ILaunchConfigurationType type;
			try
			{
				type = config.getType();
				if (ExoLaunching.isExoConfigurationType(type.getIdentifier()))
				{
					result.add(launches[i]);
				}
			}
			catch (CoreException e)
			{
				ExoCorePlugin.logError("for some reason, we have a launch configuration, but we can't find its type",
					e);
			}
		}

		return (ILaunch[]) result.toArray(new ILaunch[result.size()]);
	}
  
}
