/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.addrbook.component;

import java.util.ArrayList;
import java.util.Collection;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.addrbook.AddrBookService;
import org.exoplatform.services.addrbook.model.AddrGroup;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id: UIAccount.java,v 1.9 2004/10/23 01:58:56 tuan08 Exp $
 */
public class UIAddrBookExplorer extends UIExoCommand {

	private AddrBookService service_;
	private int configState_ = 0;
	private AddrGroup currentAddrGroup_;
	private AddrGroup parentAddrGroup_;
	private Collection childrenAddrGroup_;
	private UIAddrBooksList uiViewAddrBookInAddrGroup_;
	public final static String CONFIG_ADDRGROUP_ACTION = "configAddrGroup";
	public final static String EXIT_CONFIG_ADDRGROUP_ACTION = "exitConfigAddrGroup";
	public final static String CHANGE_ADDRGROUP_ACTION = "changeAddrGroup";
	public final static String ADD_ADDRGROUP_ACTION = "addAddrGroup";
	public final static String ADDRGROUP_ID = "addrGroupId";
	public final static String ROOT = "/";

	public String getFamily() {
		return "org.exoplatform.portlets.addrbook.component.UIAddrBookExplorer";
	}

	public UIAddrBookExplorer(
		AddrBookService service,
		UIAddrBooksList uiViewAddrBookInAddrGroup)
		throws Exception {
		setId("UIAddrBookExplorer");
		//setClazz("UIAddrBookExplorer");
		setRendererType("AddrBookExplorerRenderer");
	
		service_ = service;
		uiViewAddrBookInAddrGroup_ = uiViewAddrBookInAddrGroup;

		getChildren().add(uiViewAddrBookInAddrGroup_);

		childrenAddrGroup_ = service_.findAddrGroupsByParent(null);
		addActionListener(
			ConfigAddrGroupActionListener.class,
			CONFIG_ADDRGROUP_ACTION);
		addActionListener(
			ExitConfigAddrGroupActionListener.class,
			EXIT_CONFIG_ADDRGROUP_ACTION);
		addActionListener(
			ChangeAddrGroupActionListener.class,
			CHANGE_ADDRGROUP_ACTION);
		addActionListener(AddAddrGroupActionListener.class, ADD_ADDRGROUP_ACTION);
		
	}
	public int getConfigState() {
		return configState_;
	}
	public AddrGroup getCurrentAddrGroup() {
		return currentAddrGroup_;
	}
	public void setCurrentAddrGroup(AddrGroup addrGroup) {
	    currentAddrGroup_=addrGroup;
	}
	
	public AddrGroup getParentAddrGroup() {
		return parentAddrGroup_;
	}
	public Collection getChildrenAddrGroup() {
		return childrenAddrGroup_;
	}

	public void update() throws Exception {
		if (currentAddrGroup_ != null) {
			currentAddrGroup_ = service_.findAddrGroupById(currentAddrGroup_.getId());
			parentAddrGroup_ =
				service_.findAddrGroupById(currentAddrGroup_.getParentId());
			childrenAddrGroup_ = service_.findAddrGroupsByParent(currentAddrGroup_);
		} else {
			currentAddrGroup_ = null;
			childrenAddrGroup_ = service_.findAddrGroupsByParent(null);
			parentAddrGroup_ = null;
		}
	}

	private void changeAddrGroup(String addrGroupId) throws Exception {
		if (addrGroupId != null && addrGroupId.length() > 0 && !addrGroupId.equals("/")) {
			currentAddrGroup_ = service_.findAddrGroupById(addrGroupId);
			if (currentAddrGroup_.getParentId() != null) {
				parentAddrGroup_ =
					service_.findAddrGroupById(currentAddrGroup_.getParentId());
			} else {
				parentAddrGroup_ = null;
			}
			childrenAddrGroup_ = service_.findAddrGroupsByParent(currentAddrGroup_);
			uiViewAddrBookInAddrGroup_.setAddrGroup(currentAddrGroup_);
			uiViewAddrBookInAddrGroup_.refresh();
		} else {
			currentAddrGroup_ = null;
			childrenAddrGroup_ = service_.findAddrGroupsByParent(null);
			parentAddrGroup_ = null;
			uiViewAddrBookInAddrGroup_.setAddrGroup(currentAddrGroup_);
			uiViewAddrBookInAddrGroup_.refresh();
		}
	}

	static public class ChangeAddrGroupActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBookExplorer uiExplorer =
				(UIAddrBookExplorer) event.getComponent();
			String addrGroupId = event.getParameter(ADDRGROUP_ID);
			uiExplorer.changeAddrGroup(addrGroupId);
		}
	}

	static public class AddAddrGroupActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBookExplorer uiExplorer =
				(UIAddrBookExplorer) event.getComponent();
			UIAddrGroupForm uiAddrGroupForm =
				(UIAddrGroupForm) uiExplorer.getSibling(UIAddrGroupForm.class);
			uiAddrGroupForm.setNewFormData(uiExplorer.currentAddrGroup_);
			uiExplorer.setRenderedSibling(UIAddrGroupForm.class);
		}
	}

	static public class ConfigAddrGroupActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBookExplorer uiExplorer =
				(UIAddrBookExplorer) event.getComponent();
			uiExplorer.configState_ = 1;
		}
	}

	static public class ExitConfigAddrGroupActionListener
		extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBookExplorer uiExplorer =
				(UIAddrBookExplorer) event.getComponent();
			uiExplorer.configState_ = 0;
		}
	}
}