/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.addrbook.component;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIHtmlTextArea;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Cell;
import org.exoplatform.faces.core.component.model.ComponentCell;
import org.exoplatform.faces.core.component.model.FormButton;
import org.exoplatform.faces.core.component.model.HeaderRow;
import org.exoplatform.faces.core.component.model.LabelCell;
import org.exoplatform.faces.core.component.model.ListComponentCell;
import org.exoplatform.faces.core.component.model.Row;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.addrbook.AddrBookService;
import org.exoplatform.services.addrbook.model.AddrBook;
import org.exoplatform.services.addrbook.model.AddrGroup;

/**
 * Jun 22, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIFileAddrBookEditor.java,v 1.3 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIAddrBookForm extends UISimpleForm {
	private AddrBookService service_;
	private UIStringInput name_;
	private UIStringInput address_;
	private UIStringInput telBusiness_;
	private UIStringInput telHome_;
	private UIStringInput zipCode_;
	private UIStringInput email_;
	private AddrBook addrBook_;
	private AddrGroup addrGroup_;

	public UIAddrBookForm(AddrBookService service) throws Exception {
		super("addrBookForm", "post", null);
		setId("UIAddrBookForm");
		setClazz("UIAddrBookForm");
		service_ = service;
		name_ = new UIStringInput("name", "");
		address_ = new UIStringInput("address", "");
		telBusiness_ = new UIStringInput("telBusiness", "");
		telHome_ = new UIStringInput("telHome", "");
		zipCode_ = new UIStringInput("zipCode", "");
		email_ = new UIStringInput("email", "");

		add(
			new HeaderRow().add(
				new Cell("#{UIAddrBookForm.header.title}").addColspan("2")));
		add(
			new Row().add(new LabelCell("#{UIAddrBookForm.label.name}")).add(
				new ComponentCell(this, name_)));
		add(
			new Row().add(new LabelCell("#{UIAddrBookForm.label.address}")).add(
				new ComponentCell(this, address_)));
		add(
			new Row().add(new LabelCell("#{UIAddrBookForm.label.telBusiness}")).add(
				new ComponentCell(this, telBusiness_)));
		add(
			new Row().add(new LabelCell("#{UIAddrBookForm.label.telHome}")).add(
				new ComponentCell(this, telHome_)));
		add(
			new Row().add(new LabelCell("#{UIAddrBookForm.label.zipCode}")).add(
				new ComponentCell(this, zipCode_)));
		add(
			new Row().add(new LabelCell("#{UIAddrBookForm.label.email}")).add(
				new ComponentCell(this, email_)));
		add(
			new Row().add(
				new ListComponentCell()
					.add(
						new FormButton(
							"#{UIAddrBookForm.button.save}",
							SAVE_ACTION))
					.add(
						new FormButton(
							"#{UIAddrBookForm.button.cancel}",
							CANCEL_ACTION))
					.addColspan("2")
					.addAlign("center")));
		addActionListener(CancelActionListener.class, CANCEL_ACTION);
		addActionListener(SaveActionListener.class, SAVE_ACTION);

	}

	public void setNewAddrBookInfo(String addrGroupId) throws Exception {
		if (!addrGroupId.equals("/")) {
			addrGroup_ = service_.findAddrGroupById(addrGroupId);
		}else {
			addrGroup_ = null ;
		}

		addrBook_ = null;
		name_.setText("");
		address_.setText("");
		telBusiness_.setText("");
		telHome_.setText("");
		zipCode_.setText("");
		email_.setText("");
	}

	public void setEditAddrBookInfo(String addrBookId) throws Exception {
		//addrGroup_ = addrGroup ;
		addrBook_ = service_.findAddrBookById(addrBookId);
		name_.setText(addrBook_.getName());
		address_.setText(addrBook_.getAddress());
		telBusiness_.setText(addrBook_.getTelBusiness());
		telHome_.setText(addrBook_.getTelHome());
		zipCode_.setText(addrBook_.getZipCode());
		email_.setText(addrBook_.getEmail());
	}

	//public boolean canDecodeInvalidState()  { return addrGroup_ != null  || content_ != null; }

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBookForm uiForm = (UIAddrBookForm) event.getComponent();
			UIAddrBookPortlet addrbookPortlet =
				(UIAddrBookPortlet) uiForm.getParent();

			addrbookPortlet.setRenderedComponent(UIAddrBookExplorer.class);
		}
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBookForm uiAddrBookForm = (UIAddrBookForm) event.getComponent();
			String user =
				FacesContext
					.getCurrentInstance()
					.getExternalContext()
					.getRemoteUser();
			if (user == null)
				user = "guest";
			if (uiAddrBookForm.addrBook_ == null) {
				uiAddrBookForm.addrBook_ =
					uiAddrBookForm.service_.createAddrBookInstance();
				uiAddrBookForm.addrBook_.setName(
					uiAddrBookForm.name_.getValue());
				uiAddrBookForm.addrBook_.setAddress(
					uiAddrBookForm.address_.getValue());
				uiAddrBookForm.addrBook_.setTelBusiness(
					uiAddrBookForm.telBusiness_.getValue());
				uiAddrBookForm.addrBook_.setTelHome(
					uiAddrBookForm.telHome_.getValue());
				uiAddrBookForm.addrBook_.setZipCode(
					uiAddrBookForm.zipCode_.getValue());
				uiAddrBookForm.addrBook_.setEmail(
					uiAddrBookForm.email_.getValue());
				uiAddrBookForm.addrBook_.setCreatedBy(user);
				uiAddrBookForm.service_.createAddrBook(
					uiAddrBookForm.addrGroup_,
					uiAddrBookForm.addrBook_);
			} else {
				uiAddrBookForm.addrBook_.setName(
					uiAddrBookForm.name_.getValue());
				uiAddrBookForm.addrBook_.setAddress(
					uiAddrBookForm.address_.getValue());
				uiAddrBookForm.addrBook_.setTelBusiness(
					uiAddrBookForm.telBusiness_.getValue());
				uiAddrBookForm.addrBook_.setTelHome(
					uiAddrBookForm.telHome_.getValue());
				uiAddrBookForm.addrBook_.setZipCode(
					uiAddrBookForm.zipCode_.getValue());
				uiAddrBookForm.addrBook_.setEmail(
					uiAddrBookForm.email_.getValue());
				uiAddrBookForm.service_.updateAddrBook(uiAddrBookForm.addrBook_);
			}
			UIAddrBookExplorer uiAddrBookExplorer =
				(UIAddrBookExplorer) uiAddrBookForm.getSibling(
					UIAddrBookExplorer.class);
			//uiAddrBookExplorer.update();
			UIAddrBooksList uiList =
				(UIAddrBooksList) uiAddrBookExplorer.getChildComponentOfType(
					UIAddrBooksList.class);
			uiList.refresh();
			uiAddrBookForm.setRenderedSibling(UIAddrBookExplorer.class);
			//uiAddrBookExplorer.setRendered(true);
			//uiAddrBookForm.setRendered(false);

		}
	}

}