/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.addrbook.component;

import java.util.ArrayList;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIGrid;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.addrbook.AddrBookService;
import org.exoplatform.services.addrbook.AddrBookService;
import org.exoplatform.services.addrbook.model.AddrBook;
import org.exoplatform.services.addrbook.model.AddrGroup;
/**
 * @author: Daniel Summer
 */
public class UIAddrBooksList extends UIGrid {
	public static final PageList EMPTY_LIST =
		new ObjectPageList(new ArrayList(), 20);
	private static Parameter[] ADD = { new Parameter(ACTION, "add")};
	private static Parameter[] VIEW = { new Parameter(ACTION, "view")};
	private static Parameter[] EDIT = { new Parameter(ACTION, "edit")};
	private static Parameter[] DELETE = { new Parameter(ACTION, "delete")};

	public final static String ADDRBOOK_ID = "addrBookId";
	public final static String ADDRGROUP_ID = "addrGroupId";
	private UIPageListIterator uiPageIterator_;

	private AddrBookService service_;
	private AddrGroup currentAddrGroup_;

	public UIAddrBooksList(AddrBookService service) throws Exception {
		setId("UIAddrBooksList");
		setClazz("UIAddrBooksList");
		service_ = service;

		uiPageIterator_ = new UIPageListIterator(new AddrBookDataHandler());

		add(
			new Rows(uiPageIterator_.getPageListDataHandler(), "even", "odd")
				.add(new Column("#{UIAddrBooksList.header.name}", "name"))
				.add(
					new Column(
						"#{UIAddrBooksList.header.createddate}",
						"createdDate"))
				.add(
					new Column(
						"#{UIAddrBooksList.header.createdby}",
						"createdBy"))
				.add(
					new ActionColumn(
						"#{UIAddrBooksList.header.action}",
						"addrBookId")
						.add(new Button("#{UIAddrBooksList.button.view}",VIEW))
						.add(new Button("#{UIAddrBooksList.button.edit}", EDIT))
						.add(
							new Button(
								"#{UIAddrBooksList.button.delete}",
								DELETE))));
		add(
			new Row().add(
				new ComponentCell(this, uiPageIterator_).addColspan(
					"4").addStyle(
					"text-align: right")));
		add(
			new Row().add(
				new ListComponentCell()
					.add(new Button("#{UIAddrBooksList.button.add}", ADD))
					.addColspan("4")
					.addAlign("center")));
                addActionListener(ViewActionListener.class, "view");
		addActionListener(AddActionListener.class, "add");
		addActionListener(EditActionListener.class, "edit");
		addActionListener(DeleteActionListener.class, "delete");
		refresh();
	}
	public UIPageListIterator getUIPageIterator() {
		return uiPageIterator_;
	}

	public void setAddrBookList(PageList list) throws Exception {
		uiPageIterator_.setPageList(list);
	}
	public void setAddrGroup(AddrGroup currentAddrGroup) {
		currentAddrGroup_ = currentAddrGroup;
	}

	public void refresh() throws Exception {
		if (currentAddrGroup_ == null) {
			uiPageIterator_.setPageList(service_.findAddrBooksByAddrGroup("/"));
		} else {
			uiPageIterator_.setPageList(
				service_.findAddrBooksByAddrGroup(currentAddrGroup_.getId()));
		}
	}

	static public class AddrBookDataHandler extends PageListDataHandler {
		private AddrBook addrBook_;

		public String getData(String fieldName) {
			if ("addrBookId".equals(fieldName))
				return addrBook_.getAddrBookId();
			if ("name".equals(fieldName))
				return addrBook_.getName();
			if ("createdDate".equals(fieldName))
				return addrBook_.getCreatedDate().toGMTString();
			if ("createdBy".equals(fieldName))
				return addrBook_.getCreatedBy();
			return null;
		}

		public void setCurrentObject(Object o) {
			addrBook_ = (AddrBook) o;
		}
	}
	
	static public class ViewActionListener extends ExoActionListener{
	       public void execute(ExoActionEvent event) throws Exception {
	       	        UIAddrBooksList uiAddrBooksList =
				(UIAddrBooksList) event.getComponent();
			String addrBookId = event.getParameter(OBJECTID) ;
			AddrBook addr = uiAddrBooksList.service_.findAddrBookById(addrBookId);
			UIAddrBookExplorer uiExplorer =
				(UIAddrBookExplorer) uiAddrBooksList.getParent();
			UIAddrUserInfo uiAddrUserInfo =
				(UIAddrUserInfo) uiExplorer.getSibling(UIAddrUserInfo.class);
			uiAddrUserInfo.setAddrBookInfo(addr);
			uiExplorer.setRenderedSibling(UIAddrUserInfo.class);
		}
	}

	static public class AddActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBooksList uiAddrBooksList =
				(UIAddrBooksList) event.getComponent();
			//String addrGroupId = event.getParameter(OBJECTID);
			UIAddrBookExplorer uiExplorer =
				(UIAddrBookExplorer) uiAddrBooksList.getParent();
			UIAddrBookForm uiAddrBookForm =
				(UIAddrBookForm) uiExplorer.getSibling(UIAddrBookForm.class);
			if(uiAddrBooksList.currentAddrGroup_!=null){	
			
			uiAddrBookForm.setNewAddrBookInfo(
				uiAddrBooksList.currentAddrGroup_.getId());
			}else {
				uiAddrBookForm.setNewAddrBookInfo("/");
			}
			uiExplorer.setRenderedSibling(UIAddrBookForm.class);
		}
	}

	static public class EditActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBooksList uiAddrBooksList =
				(UIAddrBooksList) event.getComponent();
			String addrBookId = event.getParameter(OBJECTID);
			UIAddrBookExplorer uiExplorer =
				(UIAddrBookExplorer) uiAddrBooksList.getParent();
			UIAddrBookForm uiAddrBookForm =
				(UIAddrBookForm) uiExplorer.getSibling(UIAddrBookForm.class);
			uiAddrBookForm.setEditAddrBookInfo(addrBookId);
			uiExplorer.setRenderedSibling(UIAddrBookForm.class);
		}
	}

	static public class DeleteActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrBooksList uiAddrBooksList =
				(UIAddrBooksList) event.getComponent();
			String addrBookId = event.getParameter(OBJECTID);
			uiAddrBooksList.service_.removeAddrBookById(addrBookId);
			uiAddrBooksList.refresh();
		}
	}
}