/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.addrbook.renderer.html;

import java.io.IOException;
import java.util.*;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.renderer.html.HtmlBasicRenderer;
import org.exoplatform.portlets.addrbook.component.UIAddrBookExplorer;
import org.exoplatform.services.addrbook.model.AddrGroup;
import org.exoplatform.services.addrbook.model.AddrBook;
/**
 * @email:   danny_xcz@163.com
 * @version: $Id: AddrBookExplorerRenderer.java,v 1.12 2004/10/22 14:23:31 tuan08 Exp $
 */
public class AddrBookExplorerRenderer extends HtmlBasicRenderer {

	private static Parameter[] CONFIG_ADDRGROUP =
		{ new Parameter(ACTION, "configAddrGroup")};
	private static Parameter[] EXIT_CONFIG_ADDRGROUP =
		{ new Parameter(ACTION, "exitConfigAddrGroup")};

	private static Parameter CHANGE_ADDRGROUP =
		new Parameter(ACTION, "changeAddrGroup");
	private static Parameter[] ADD_ADDRGROUP =
		{ new Parameter(ACTION, "addAddrGroup")};

	private static Parameter SEARCH = new Parameter(ACTION, "search");

	final public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {
		ResourceBundle res =
			getApplicationResourceBundle(context.getExternalContext());
		ResponseWriter w = context.getResponseWriter();
		UIAddrBookExplorer uiExplorer = (UIAddrBookExplorer) component;
		Collection childrenAddrGroup = uiExplorer.getChildrenAddrGroup();
		AddrGroup currentAddrGroup = uiExplorer.getCurrentAddrGroup();
		AddrGroup parentAddrGroup = uiExplorer.getParentAddrGroup();
		String groupName = "ROOT";
		if (currentAddrGroup != null)
			groupName = currentAddrGroup.getName();
		Object[] args = { groupName };
		//The head line
		w.write("<table class='UIAddrBookExplorer'>");
		w.write("<tr>");
		w.write("<th class='header' colspan='2'>");
		w.write(
			ft_.format(res.getString("UIAddrBookExplorer.label.addrGroup"), args));
			

		w.write("</th>");
		w.write("</tr>");
		//The middle line
		w.write("<tr>");
		w.write("<td class='tree'>");
		
		if (uiExplorer.getConfigState() == 0) {
			linkRenderer_.render(
				w,
				uiExplorer,
				res.getString("UIAddrBookExplorer.button.config-addrGroup"),
				CONFIG_ADDRGROUP);
		} else if (uiExplorer.getConfigState() == 1) {
			linkRenderer_.render(
				w,
				uiExplorer,
				res.getString("UIAddrBookExplorer.button.exit-config-addrGroup"),
				EXIT_CONFIG_ADDRGROUP);
		}
		
		
		if (uiExplorer.getConfigState() == 0) {
			addAddrGroupTreeTable(
				parentAddrGroup,
				childrenAddrGroup,
				w,
				res,
				uiExplorer);
		} else if (uiExplorer.getConfigState() == 1) {
			addConfigAddrGroupTreeTable(
				parentAddrGroup,
				childrenAddrGroup,
				w,
				res,
				uiExplorer);
		}
		w.write("</td>");
		w.write("<td>");
	   // if(currentAddrGroup!=null){
	    
		renderChildren(context, uiExplorer);
		//}
		//else {
		//	w.write("no addrGroup selected!");
		//}
		w.write("</td>");
		w.write("</tr>");
		//The bottom line
		w.write("<tr>");
		w.write("<td colspan='2' class='footer'>");
		//String parentAddrGroupId = "" ;
		//if (currentAddrGroup != null) parentAddrGroupId = currentAddrGroup.getId() ;
		// Parameter groupParam = new Parameter(UIAddrBookExplorer.ADDRGROUP_ID, parentAddrGroupId) ;
		//w.write("search");

		w.write("</td>");
		w.write("</tr>");

		w.write("</table>");
	}

	private void addAddrGroupTreeTable(
		AddrGroup parentAddrGroup,
		Collection childrenAddrGroup,
		ResponseWriter w,
		ResourceBundle res,
		UIAddrBookExplorer uiAddrBookExplorer)
		throws IOException {
		Parameter addrGroupIdParam =
			new Parameter(UIAddrBookExplorer.ADDRGROUP_ID, "");
		Parameter[] changeAddrGroupParams = { CHANGE_ADDRGROUP, addrGroupIdParam };

		Iterator i = childrenAddrGroup.iterator();
		String parentAddrGroupId = "";
		if (parentAddrGroup != null){
			parentAddrGroupId = parentAddrGroup.getId();
		}else {
			parentAddrGroupId = "/";
		}
		String groupIcon = res.getString("UIAddrBookExplorer.img.addrGroup-icon");
		w.write("<table>");
		w.write("<tr>");
		w.write("<td>");
		addrGroupIdParam.setValue(parentAddrGroupId);
		//uiAddrBookExplorer.appendLink(w, "[..]", changeAddrGroupParams, "");
		w.write("</td>");
		w.write("</tr>");
		while (i.hasNext()) {
			AddrGroup addrGroup = (AddrGroup) i.next();
			w.write("<tr>");
			w.write("<td style='padding-left: 10px'>");
			w.write(groupIcon);
			addrGroupIdParam.setValue(addrGroup.getId());
//			uiAddrBookExplorer.appendLink( w, addrGroup.getName(), changeAddrGroupParams, "");
			w.write("</td>");
			w.write("</tr>");
		}
		w.write("</table>");

	}

	private void addConfigAddrGroupTreeTable(
		AddrGroup parentAddrGroup,
		Collection childrenAddrGroup,
		ResponseWriter w,
		ResourceBundle res,
		UIAddrBookExplorer uiAddrBookExplorer)
		throws IOException {
		Parameter addrGroupIdParam =
			new Parameter(UIAddrBookExplorer.ADDRGROUP_ID, "");
		Parameter[] changeAddrGroupParams = { CHANGE_ADDRGROUP, addrGroupIdParam };

		Iterator i = childrenAddrGroup.iterator();
		String parentAddrGroupId = "";
		if (parentAddrGroup != null)
			parentAddrGroupId = parentAddrGroup.getId();
		String groupIcon = res.getString("UIAddrBookExplorer.img.addrGroup-icon");
		w.write("<table>");
		w.write("<tr>");
		w.write("<td>");
		addrGroupIdParam.setValue(parentAddrGroupId);
//		uiAddrBookExplorer.appendLink(w, "[..]", changeAddrGroupParams, "");
		w.write("</td>");
		w.write("</tr>");
		while (i.hasNext()) {
			AddrGroup addrGroup = (AddrGroup) i.next();
			w.write("<tr>");
			w.write("<td style='padding-left: 10px'>");
			w.write(groupIcon);
			addrGroupIdParam.setValue(addrGroup.getId());
//			uiAddrBookExplorer.appendLink( w, addrGroup.getName(), changeAddrGroupParams,"");
			w.write("</td>");
			w.write("</tr>");
		}
		w.write("<tr>");
		w.write("<td>");
		linkRenderer_.render(
			w,
			uiAddrBookExplorer,
			res.getString("UIAddrBookExplorer.button.add-addrGroup"),
			ADD_ADDRGROUP);
		w.write("</td>");
		w.write("</tr>");
		w.write("</table>");

	}

}