/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.bookmark.component;

import java.util.ArrayList;
import java.util.Collection;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.bookmark.BookmarkService;
import org.exoplatform.services.bookmark.model.BookmarkCategory;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id: UIAccount.java,v 1.9 2004/10/23 01:58:56 tuan08 Exp $
 */
public class UIBookmarkExplorer extends UIExoCommand {

	private BookmarkService service_;
	private int configState_ = 0;
	private BookmarkCategory currentBookmarkCategory_;
	private BookmarkCategory parentBookmarkCategory_;
	private Collection childrenBookmarkCategory_;
	private UIBookmarksList uiViewBookmarkInBookmarkCategory_;
	public final static String CONFIG_CATEGORY_ACTION = "configBookmarkCategory";
	public final static String EXIT_CONFIG_CATEGORY_ACTION = "exitConfigBookmarkCategory";
	public final static String CHANGE_CATEGORY_ACTION = "changeBookmarkCategory";
	public final static String ADD_CATEGORY_ACTION = "addBookmarkCategory";
	public final static String CATEGORY_ID = "sectionId";
	public final static String ROOT = "/";

	public String getFamily() {
		return "org.exoplatform.portlets.bookmark.component.UIBookmarkExplorer";
	}

	public UIBookmarkExplorer(
		BookmarkService service,
		UIBookmarksList uiViewBookmarkInBookmarkCategory)
		throws Exception {
		setId("UIBookmarkExplorer");
		//setClazz("UIBookmarkExplorer");
		setRendererType("BookmarkExplorerRenderer");
	
		service_ = service;
		uiViewBookmarkInBookmarkCategory_ = uiViewBookmarkInBookmarkCategory;

		getChildren().add(uiViewBookmarkInBookmarkCategory_);

		childrenBookmarkCategory_ = service_.findBookmarkCategorysByParent(null);
		addActionListener(
			ConfigBookmarkCategoryActionListener.class,
			CONFIG_CATEGORY_ACTION);
		addActionListener(
			ExitConfigBookmarkCategoryActionListener.class,
			EXIT_CONFIG_CATEGORY_ACTION);
		addActionListener(
			ChangeBookmarkCategoryActionListener.class,
			CHANGE_CATEGORY_ACTION);
		addActionListener(AddBookmarkCategoryActionListener.class, ADD_CATEGORY_ACTION);
		
	}
	
	public int getConfigState() {
		return configState_;
	}

	public BookmarkCategory getCurrentBookmarkCategory() {
		return currentBookmarkCategory_;
	}
	public void setCurrentBookmarkCategory(BookmarkCategory category) {
	    currentBookmarkCategory_=category;
	}
	
	public BookmarkCategory getParentBookmarkCategory() {
		return parentBookmarkCategory_;
	}
	public Collection getChildrenBookmarkCategory() {
		return childrenBookmarkCategory_;
	}

	public void update() throws Exception {
		if (currentBookmarkCategory_ != null) {
			currentBookmarkCategory_ = service_.findBookmarkCategoryById(currentBookmarkCategory_.getId());
			parentBookmarkCategory_ =
				service_.findBookmarkCategoryById(currentBookmarkCategory_.getParentId());
			childrenBookmarkCategory_ = service_.findBookmarkCategorysByParent(currentBookmarkCategory_);
		} else {
			currentBookmarkCategory_ = null;
			childrenBookmarkCategory_ = service_.findBookmarkCategorysByParent(null);
			parentBookmarkCategory_ = null;
		}
	}

	private void changeBookmarkCategory(String sectionId) throws Exception {
		if (sectionId != null && sectionId.length() > 0 && ! sectionId.equals("/")) {
			currentBookmarkCategory_ = service_.findBookmarkCategoryById(sectionId);
			if (currentBookmarkCategory_.getParentId() != null) {
				parentBookmarkCategory_ =
					service_.findBookmarkCategoryById(currentBookmarkCategory_.getParentId());
			} else {
				parentBookmarkCategory_ = null;
			}
			childrenBookmarkCategory_ = service_.findBookmarkCategorysByParent(currentBookmarkCategory_);
			uiViewBookmarkInBookmarkCategory_.setBookmarkCategory(currentBookmarkCategory_);
			uiViewBookmarkInBookmarkCategory_.refresh();
		} else {
			currentBookmarkCategory_ = null;
			childrenBookmarkCategory_ = service_.findBookmarkCategorysByParent(null);
			parentBookmarkCategory_ = null;
			//uiViewBookmarkInBookmarkCategory_.setBookmarkCategory(currentBookmarkCategory_);
			uiViewBookmarkInBookmarkCategory_.setBookmarkCategory(currentBookmarkCategory_);
			uiViewBookmarkInBookmarkCategory_.refresh();
		}
	}

	static public class ChangeBookmarkCategoryActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarkExplorer uiExplorer =
				(UIBookmarkExplorer) event.getComponent();
			String sectionId = event.getParameter(CATEGORY_ID);
			uiExplorer.changeBookmarkCategory(sectionId);
		}
	}

	static public class AddBookmarkCategoryActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarkExplorer uiExplorer =
				(UIBookmarkExplorer) event.getComponent();
			UIBookmarkCategoryForm uiBookmarkCategoryForm =
				(UIBookmarkCategoryForm) uiExplorer.getSibling(UIBookmarkCategoryForm.class);
			uiBookmarkCategoryForm.setNewFormData(uiExplorer.currentBookmarkCategory_);
			uiExplorer.setRenderedSibling(UIBookmarkCategoryForm.class);
		}
	}
	static public class ConfigBookmarkCategoryActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarkExplorer uiExplorer =
				(UIBookmarkExplorer) event.getComponent();
			uiExplorer.configState_ = 1;
		}
	}

	static public class ExitConfigBookmarkCategoryActionListener
		extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarkExplorer uiExplorer =
				(UIBookmarkExplorer) event.getComponent();
			uiExplorer.configState_ = 0;
		}
	}


}