/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.bookmark.component;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIHtmlTextArea;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Cell;
import org.exoplatform.faces.core.component.model.ComponentCell;
import org.exoplatform.faces.core.component.model.FormButton;
import org.exoplatform.faces.core.component.model.HeaderRow;
import org.exoplatform.faces.core.component.model.LabelCell;
import org.exoplatform.faces.core.component.model.ListComponentCell;
import org.exoplatform.faces.core.component.model.Row;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.bookmark.BookmarkService;
import org.exoplatform.services.bookmark.model.Bookmark;
import org.exoplatform.services.bookmark.model.BookmarkCategory;

/**
 * Jun 22, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIFileBookmarkEditor.java,v 1.3 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIBookmarkForm extends UISimpleForm {
	private BookmarkService service_;
	private UIStringInput name_;
	private UIStringInput url_;
	private Bookmark bookmark_;
	private BookmarkCategory category_;

	public UIBookmarkForm(BookmarkService service) throws Exception {
		super("bookmarkForm", "post", null);
		setId("UIBookmarkForm");
		setClazz("UIBookmarkForm");
		service_ = service;
		name_ = new UIStringInput("name", "");
                url_  = new UIStringInput("url","");

		add(
			new HeaderRow().add(
				new Cell("#{UIBookmarkForm.header.title}").addColspan("2")));
		add(
			new Row().add(new LabelCell("#{UIBookmarkForm.label.name}")).add(
				new ComponentCell(this, name_)));
		add(
			new Row()
				.add(
					new LabelCell("#{UIBookmarkForm.label.url}"))
				.add(new ComponentCell(this, url_)));
		add(
			new Row().add(
				new ListComponentCell()
					.add(
						new FormButton(
							"#{UIBookmarkForm.button.save}",
							SAVE_ACTION))
					.add(
						new FormButton(
							"#{UIBookmarkForm.button.cancel}",
							CANCEL_ACTION))
					.addColspan("2")
					.addAlign("center")));
		addActionListener(CancelActionListener.class, CANCEL_ACTION);
		addActionListener(SaveActionListener.class, SAVE_ACTION);

	}

	public void setNewBookmarkInfo(String categoryId) throws Exception {
		if (!categoryId.equals("/")) {
			category_ = service_.findBookmarkCategoryById(categoryId);
		}else {
			category_ = null ;
		}

		bookmark_ = null;
		name_.setText("");
		url_.setText("");
	}

	public void setEditBookmarkInfo(String bookmarkId) throws Exception {
		//category_ = category ;
		bookmark_ = service_.findBookmarkById(bookmarkId);
		name_.setText(bookmark_.getName());
		url_.setText(bookmark_.getURL());
	}

	//public boolean canDecodeInvalidState()  { return category_ != null  || bookmark_ != null; }

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarkForm uiForm = (UIBookmarkForm) event.getComponent();
			UIBookmarkPortlet bookmarkPortlet =
				(UIBookmarkPortlet) uiForm.getParent();

			bookmarkPortlet.setRenderedComponent(UIBookmarkExplorer.class);
		}
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarkForm uiBookmarkForm = (UIBookmarkForm) event.getComponent();
			String user =
				FacesContext
					.getCurrentInstance()
					.getExternalContext()
					.getRemoteUser();
			if (user == null)
				user = "guest";
			if (uiBookmarkForm.bookmark_ == null) {
				uiBookmarkForm.bookmark_ =
					uiBookmarkForm.service_.createBookmarkInstance();
				uiBookmarkForm.bookmark_.setName(
					uiBookmarkForm.name_.getValue());
				uiBookmarkForm.bookmark_.setURL(
					uiBookmarkForm.url_.getValue());
				uiBookmarkForm.bookmark_.setCreatedBy(user);
				uiBookmarkForm.service_.createBookmark(
					uiBookmarkForm.category_,
					uiBookmarkForm.bookmark_);
			} else {
				uiBookmarkForm.bookmark_.setName(
					uiBookmarkForm.name_.getValue());
				uiBookmarkForm.bookmark_.setURL(
					uiBookmarkForm.url_.getValue());
				uiBookmarkForm.service_.updateBookmark(uiBookmarkForm.bookmark_);
			}
			UIBookmarkExplorer uiBookmarkExplorer =
				(UIBookmarkExplorer) uiBookmarkForm.getSibling(
					UIBookmarkExplorer.class);
			//uiBookmarkExplorer.update();
			UIBookmarksList uiList =
				(UIBookmarksList) uiBookmarkExplorer.getChildComponentOfType(
					UIBookmarksList.class);
			uiList.refresh();
			uiBookmarkForm.setRenderedSibling(UIBookmarkExplorer.class);
			//uiBookmarkExplorer.setRendered(true);
			//uiBookmarkForm.setRendered(false);

		}
	}

}