/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.bookmark.component;

import java.util.ArrayList;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.ActionResponse;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIGrid;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.bookmark.BookmarkService;
import org.exoplatform.services.bookmark.model.Bookmark;
import org.exoplatform.services.bookmark.model.BookmarkCategory;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIListUser.java,v 1.18 2004/10/22 14:23:30 tuan08 Exp $
 */
public class UIBookmarksList extends UIGrid {
	public static final PageList EMPTY_LIST =
		new ObjectPageList(new ArrayList(), 20);
	private static Parameter[] ADD = { new Parameter(ACTION, "add")};
	//private static Parameter[] LINKTO = { new Parameter(ACTION, "linkto")};
	private static Parameter[] EDIT = { new Parameter(ACTION, "edit")};
	private static Parameter[] DELETE = { new Parameter(ACTION, "delete")};

	public final static String BOOKMARK_ID = "bookmarkId";
	public final static String CATEGORY_ID = "categoryId";
	private UIPageListIterator uiPageIterator_;

	private BookmarkService service_;
	private BookmarkCategory currentBookmarkCategory_;

	public UIBookmarksList(BookmarkService service) throws Exception {
		setId("UIBookmarksList");
		setClazz("UIBookmarksList");
		service_ = service;

		uiPageIterator_ = new UIPageListIterator(new BookmarkDataHandler());

		add(
			new Rows(uiPageIterator_.getPageListDataHandler(), "even", "odd")
				.add(new Column("#{UIBookmarksList.header.name}", "name"))
				.add(new Column("#{UIBookmarksList.header.url}", "url"))
				.add(
					new Column(
						"#{UIBookmarksList.header.createddate}",
						"createdDate"))
				.add(
					new Column(
						"#{UIBookmarksList.header.createdby}",
						"createdBy"))
				.add(
					new ActionColumn(
						"#{UIBookmarksList.header.action}",
						"bookmarkId")
						//.add(new Button("#{UIBookmarksList.button.linkto}", LINKTO))
						.add(new Button("#{UIBookmarksList.button.edit}", EDIT))
						.add(
							new Button(
								"#{UIBookmarksList.button.delete}",
								DELETE))));
		add(
			new Row().add(
				new ComponentCell(this, uiPageIterator_).addColspan(
					"5").addStyle(
					"text-align: right")));
		add(
			new Row().add(
				new ListComponentCell()
					.add(new Button("#{UIBookmarksList.button.add}", ADD))
					.addColspan("5")
					.addAlign("center")));
                //addActionListener(LinkToActionListener.class, "linkto");
		addActionListener(AddActionListener.class, "add");
		addActionListener(EditActionListener.class, "edit");
		addActionListener(DeleteActionListener.class, "delete");
		refresh();
	}
	public UIPageListIterator getUIPageIterator() {
		return uiPageIterator_;
	}

	public void setBookmarkList(PageList list) throws Exception {
		uiPageIterator_.setPageList(list);
	}
	public void setBookmarkCategory(BookmarkCategory currentBookmarkCategory) {
		currentBookmarkCategory_ = currentBookmarkCategory;
	}

	public void refresh() throws Exception {
		if (currentBookmarkCategory_ == null) {
			uiPageIterator_.setPageList(service_.findBookmarksByBookmarkCategory("/"));
		} else {
			uiPageIterator_.setPageList(
				service_.findBookmarksByBookmarkCategory(currentBookmarkCategory_.getId()));
		}
	}

	static public class BookmarkDataHandler extends PageListDataHandler {
		private Bookmark bookmark_;

		public String getData(String fieldName) {
			if ("bookmarkId".equals(fieldName))
				return bookmark_.getBookmarkId();
			if ("name".equals(fieldName))
				return bookmark_.getName();
			if ("url".equals(fieldName))
			   if(bookmark_.getURL().startsWith("http"))
			         return "<a href=\""+bookmark_.getURL()+"\" target=\"blank\">" + bookmark_.getURL() + "</a>";
			       else
				return "<a href=\"http://"+bookmark_.getURL()+"\" target=\"blank\">" + bookmark_.getURL() + "</a>";
			if ("createdDate".equals(fieldName))
				return bookmark_.getCreatedDate().toGMTString();
			if ("createdBy".equals(fieldName))
				return bookmark_.getCreatedBy();
			return null;
		}

		public void setCurrentObject(Object o) {
			bookmark_ = (Bookmark) o;
		}
	}

	static public class AddActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarksList uiBookmarksList =
				(UIBookmarksList) event.getComponent();
			//String categoryId = event.getParameter(OBJECTID);
			UIBookmarkExplorer uiExplorer =
				(UIBookmarkExplorer) uiBookmarksList.getParent();
			UIBookmarkForm uiBookmarkForm =
				(UIBookmarkForm) uiExplorer.getSibling(UIBookmarkForm.class);
			if(uiBookmarksList.currentBookmarkCategory_!=null){	
			
			uiBookmarkForm.setNewBookmarkInfo(
				uiBookmarksList.currentBookmarkCategory_.getId());
			}else {
				uiBookmarkForm.setNewBookmarkInfo("/");
			}
			uiExplorer.setRenderedSibling(UIBookmarkForm.class);
		}
	}
	
	static public class LinkToActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarksList uiBookmarksList =
				(UIBookmarksList) event.getComponent();
			String bookmarkId = event.getParameter(OBJECTID);
			Bookmark b = uiBookmarksList.service_.findBookmarkById(bookmarkId);
			String u = "http://" + b.getURL();
                        ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext();
                        ActionResponse response = (ActionResponse) eContext.getResponse();
                        response.sendRedirect(u);

		}
	}
	
	static public class EditActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarksList uiBookmarksList =
				(UIBookmarksList) event.getComponent();
			String bookmarkId = event.getParameter(OBJECTID);
			UIBookmarkExplorer uiExplorer =
				(UIBookmarkExplorer) uiBookmarksList.getParent();
			UIBookmarkForm uiBookmarkForm =
				(UIBookmarkForm) uiExplorer.getSibling(UIBookmarkForm.class);
			uiBookmarkForm.setEditBookmarkInfo(bookmarkId);
			uiExplorer.setRenderedSibling(UIBookmarkForm.class);
		}
	}

	static public class DeleteActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarksList uiBookmarksList =
				(UIBookmarksList) event.getComponent();
			String bookmarkId = event.getParameter(OBJECTID);
			uiBookmarksList.service_.removeBookmark(bookmarkId);
			uiBookmarksList.refresh();
		}
	}
}