/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.ftp.component;

import java.util.*;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;

import org.exoplatform.faces.core.component.*;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.ftp.component.util.FtpConnection;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id: UIQuickSearch.java,v 1.7 2004/09/19 19:08:06 tuan08 Exp $
 */
public class UIFileForm extends UISimpleForm {
	static final public String SAVE_ACTION = "save";
	static final public String CANCEL_ACTION = "cancel";

	private FtpConnection ftpConn_;
	private UIStringInput nameInput_;

	private String oldFileName_;
	private String actionType_;

	public UIFileForm(FtpConnection ftpConn, ResourceBundle res)
		throws Exception {
		super("fileForm", "post", null);
		setId("UIFileForm");
		//setClazz("UIFileForm");
		ftpConn_ = ftpConn;
		nameInput_ = new UIStringInput("name", "");

		add(
			new HeaderRow().add(
				new Cell(res.getString("UIFileForm.header.edit")).addColspan(
					"2")));
		add(
			new Row().add(
				new LabelCell(res.getString("UIFileForm.label.name"))).add(
				new ComponentCell(this, nameInput_)));
		add(
			new Row().add(
				new ListComponentCell()
					.add(
						new FormButton(
							res.getString("UIFileForm.button.save"),
							SAVE_ACTION))
					.add(
						new FormButton(
							res.getString("UIFileForm.button.cancel"),
							CANCEL_ACTION))
					.addColspan("2")
					.addAlign("center")));

		addActionListener(SaveActionListener.class, SAVE_ACTION);
		addActionListener(CancelActionListener.class, CANCEL_ACTION);
	}

	public void setNewFileData() {
		oldFileName_ = "";
		nameInput_.setText("");
		actionType_ = "newfile";
	}

	public void setEditFileData(String s) {
		oldFileName_ = s;
		nameInput_.setText(s);
		actionType_ = "editfile";
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIFileForm uiFileForm = (UIFileForm) event.getSource();
			String fileName = uiFileForm.nameInput_.getText();

			if (fileName != null && !fileName.equals("")) {

				if (uiFileForm.actionType_.equals("editfile")) {
					uiFileForm.ftpConn_.rename(
						uiFileForm.oldFileName_,
						fileName);
				}else if(uiFileForm.actionType_.equals("newfile")){
					//uiFileForm.ftpConn_.rename(
					//	uiFileForm.oldFileName_,
					//	fileName);
				}

			}

			UIFtpPortlet ftpPortlet =
				(UIFtpPortlet) uiFileForm.getAncestorOfType(UIFtpPortlet.class);
			UIFtp uiFtp =
				(UIFtp) ftpPortlet.getChildComponentOfType(UIFtp.class);
			uiFtp.update();
			ftpPortlet.setRenderedComponent(UIFtp.class);

		}
	}

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIFileForm uiFileForm = (UIFileForm) event.getSource();
			UIFtpPortlet ftpPortlet =
				(UIFtpPortlet) uiFileForm.getAncestorOfType(UIFtpPortlet.class);
			ftpPortlet.setRenderedComponent(UIFtp.class);

		}
	}
}