/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.publish.component;

import java.util.ArrayList;
import java.util.Collection;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.publish.PublishService;
import org.exoplatform.services.publish.model.Section;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id: UIAccount.java,v 1.9 2004/10/23 01:58:56 tuan08 Exp $
 */
public class UIContentExplorer extends UIExoCommand {

	private PublishService service_;
	private int configState_ = 0;
	private Section currentSection_;
	private Section parentSection_;
	private Collection childrenSection_;
	private UIContentsList uiViewContentInSection_;
	public final static String CONFIG_SECTION_ACTION = "configSection";
	public final static String EXIT_CONFIG_SECTION_ACTION = "exitConfigSection";
	public final static String CHANGE_SECTION_ACTION = "changeSection";
	public final static String ADD_SECTION_ACTION = "addSection";
	public final static String SECTION_ID = "sectionId";
	public final static String ROOT = "/";

	public String getFamily() {
		return "org.exoplatform.portlets.publish.component.UIContentExplorer";
	}

	public UIContentExplorer(
		PublishService service,
		UIContentsList uiViewContentInSection)
		throws Exception {
		setId("UIContentExplorer");
		//setClazz("UIContentExplorer");
		setRendererType("ContentExplorerRenderer");
	
		service_ = service;
		uiViewContentInSection_ = uiViewContentInSection;

		getChildren().add(uiViewContentInSection_);

		childrenSection_ = service_.findSectionsByParent(null);
		addActionListener(
			ConfigSectionActionListener.class,
			CONFIG_SECTION_ACTION);
		addActionListener(
			ExitConfigSectionActionListener.class,
			EXIT_CONFIG_SECTION_ACTION);
		addActionListener(
			ChangeSectionActionListener.class,
			CHANGE_SECTION_ACTION);
		addActionListener(AddSectionActionListener.class, ADD_SECTION_ACTION);
		
	}
	public int getConfigState() {
		return configState_;
	}
	public Section getCurrentSection() {
		return currentSection_;
	}
	public void setCurrentSection(Section section) {
	    currentSection_=section;
	}
	
	public Section getParentSection() {
		return parentSection_;
	}
	public Collection getChildrenSection() {
		return childrenSection_;
	}

	public void update() throws Exception {
		if (currentSection_ != null) {
			currentSection_ = service_.findSectionById(currentSection_.getId());
			parentSection_ =
				service_.findSectionById(currentSection_.getParentId());
			childrenSection_ = service_.findSectionsByParent(currentSection_);
		} else {
			currentSection_ = null;
			childrenSection_ = service_.findSectionsByParent(null);
			parentSection_ = null;
		}
	}

	private void changeSection(String sectionId) throws Exception {
		if (sectionId != null && sectionId.length() > 0 && !sectionId.equals("/") ) {
			currentSection_ = service_.findSectionById(sectionId);
			if (currentSection_.getParentId() != null) {
				parentSection_ =
					service_.findSectionById(currentSection_.getParentId());
			} else {
				parentSection_ = null;
			}
			childrenSection_ = service_.findSectionsByParent(currentSection_);
			uiViewContentInSection_.setSection(currentSection_);
			uiViewContentInSection_.refresh();
		} else {
			currentSection_ = null;
			childrenSection_ = service_.findSectionsByParent(null);
			uiViewContentInSection_.setSection(null);
			uiViewContentInSection_.refresh();
			parentSection_ = null;
			//uiViewContentInSection_.setSection(currentSection_);
		}
	}

	static public class ChangeSectionActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentExplorer uiExplorer =
				(UIContentExplorer) event.getComponent();
			String sectionId = event.getParameter(SECTION_ID);
			uiExplorer.changeSection(sectionId);
		}
	}

	static public class AddSectionActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentExplorer uiExplorer =
				(UIContentExplorer) event.getComponent();
			UISectionForm uiSectionForm =
				(UISectionForm) uiExplorer.getSibling(UISectionForm.class);
			uiSectionForm.setNewFormData(uiExplorer.currentSection_);
			uiExplorer.setRenderedSibling(UISectionForm.class);
		}
	}

	static public class ConfigSectionActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentExplorer uiExplorer =
				(UIContentExplorer) event.getComponent();
			uiExplorer.configState_ = 1;
		}
	}

	static public class ExitConfigSectionActionListener
		extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentExplorer uiExplorer =
				(UIContentExplorer) event.getComponent();
			uiExplorer.configState_ = 0;
		}
	}
}