/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.publish.component;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIHtmlTextArea;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Cell;
import org.exoplatform.faces.core.component.model.ComponentCell;
import org.exoplatform.faces.core.component.model.FormButton;
import org.exoplatform.faces.core.component.model.HeaderRow;
import org.exoplatform.faces.core.component.model.LabelCell;
import org.exoplatform.faces.core.component.model.ListComponentCell;
import org.exoplatform.faces.core.component.model.Row;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.publish.PublishService;
import org.exoplatform.services.publish.model.Content;
import org.exoplatform.services.publish.model.Section;

/**
 * Jun 22, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIFileContentEditor.java,v 1.3 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIContentForm extends UISimpleForm {
	private PublishService service_;
	private UIStringInput title_;
	private UIHtmlTextArea introText_;
	private UIHtmlTextArea fullText_;
	private Content content_;
	private Section section_;

	public UIContentForm(PublishService service) throws Exception {
		super("contentForm", "post", null);
		setId("UIContentForm");
		//setClazz("UIContentForm");
		service_ = service;
		title_ = new UIStringInput("title", "");
		introText_ = new UIHtmlTextArea("introText", "", "600", "100");
		fullText_ = new UIHtmlTextArea("fullText", "", "600", "400");

		add(
			new HeaderRow().add(
				new Cell("#{UIContentForm.header.title}").addColspan("2")));
		add(
			new Row().add(new LabelCell("#{UIContentForm.label.title}")).add(
				new ComponentCell(this, title_)));
		add(
			new Row()
				.add(
					new LabelCell("#{UIContentForm.label.fulltext}").addValign(
						"top"))
				.add(new ComponentCell(this, fullText_)));
		add(
			new Row().add(
				new ListComponentCell()
					.add(
						new FormButton(
							"#{UIContentForm.button.save}",
							SAVE_ACTION))
					.add(
						new FormButton(
							"#{UIContentForm.button.cancel}",
							CANCEL_ACTION))
					.addColspan("2")
					.addAlign("center")));
		addActionListener(CancelActionListener.class, CANCEL_ACTION);
		addActionListener(SaveActionListener.class, SAVE_ACTION);

	}

	public void setNewContentInfo(String sectionId) throws Exception {
		if (!sectionId.equals("/")) {
			section_ = service_.findSectionById(sectionId);
		}else {
			section_ = null ;
		}

		content_ = null;
		title_.setText("");
		introText_.setText("");
		fullText_.setText("");
	}

	public void setEditContentInfo(String contentId) throws Exception {
		//section_ = section ;
		content_ = service_.findContentById(contentId);
		title_.setText(content_.getTitle());
		introText_.setText(content_.getIntroText());
		fullText_.setText(content_.getFullText());
	}

	//public boolean canDecodeInvalidState()  { return section_ != null  || content_ != null; }

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentForm uiForm = (UIContentForm) event.getComponent();
			UIPublishPortlet publishPortlet =
				(UIPublishPortlet) uiForm.getParent();

			publishPortlet.setRenderedComponent(UIContentExplorer.class);
		}
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentForm uiContentForm = (UIContentForm) event.getComponent();
			String user =
				FacesContext
					.getCurrentInstance()
					.getExternalContext()
					.getRemoteUser();
			if (user == null)
				user = "guest";
			if (uiContentForm.content_ == null) {
				uiContentForm.content_ =
					uiContentForm.service_.createContentInstance();
				uiContentForm.content_.setTitle(
					uiContentForm.title_.getValue());
				uiContentForm.content_.setIntroText(
					uiContentForm.introText_.getValue());
				uiContentForm.content_.setFullText(
					uiContentForm.fullText_.getValue());
				uiContentForm.content_.setCreatedBy(user);
				uiContentForm.service_.createContent(
					uiContentForm.section_,
					uiContentForm.content_);
			} else {
				uiContentForm.content_.setTitle(
					uiContentForm.title_.getValue());
				uiContentForm.content_.setIntroText(
					uiContentForm.introText_.getValue());
				uiContentForm.content_.setFullText(
					uiContentForm.fullText_.getValue());
				uiContentForm.service_.updateContent(uiContentForm.content_);
			}
			UIContentExplorer uiContentExplorer =
				(UIContentExplorer) uiContentForm.getSibling(
					UIContentExplorer.class);
			//uiContentExplorer.update();
			UIContentsList uiList =
				(UIContentsList) uiContentExplorer.getChildComponentOfType(
					UIContentsList.class);
			uiList.refresh();
			uiContentForm.setRenderedSibling(UIContentExplorer.class);
			//uiContentExplorer.setRendered(true);
			//uiContentForm.setRendered(false);

		}
	}

}