/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.addrbook.hibernate;

import java.util.Date;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.commons.exception.UniqueObjectException;

import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.idgenerator.IDGeneratorService;
import org.exoplatform.commons.utils.*;
import org.exoplatform.services.addrbook.AddrBookService;
import org.exoplatform.services.addrbook.model.*;
import org.exoplatform.services.addrbook.impl.*;
import org.hibernate.Hibernate;
import org.hibernate.Session;

/**
 * @author Bob
 * Created on 2005-1-4
 *
 */
public class AddrBookServiceImpl implements AddrBookService {

	private HibernateService hService_;
	
	private IDGeneratorService idService_;
	
	private static final String queryFindAddrBookById =
		"from b in class org.exoplatform.services.addrbook.impl.AddrBookImpl "
			+ "where b.addrBookId = ? ";
	
	private static final String queryFindAddrBooksByGroup =
		"from b in class org.exoplatform.services.addrbook.impl.AddrBookImpl "
			+ "where b.addrGroupId = ? ";

	private static final String queryFindAddrGroupById =
		"from g in class org.exoplatform.services.addrbook.impl.AddrGroupImpl "
			+ "where g.id = ? ";
	
	private static final String queryFindRootAddrGroups =
		"from g in class org.exoplatform.services.addrbook.impl.AddrGroupImpl "
			+ "where g.parentId is null";

	private static final String queryFindAddrGroupsByParent =
		"from g in class org.exoplatform.services.addrbook.impl.AddrGroupImpl "
			+ "where g.parentId = ? ";

	static private String[] MAPPING =
		{
			"org/exoplatform/services/addrbook/impl/AddrBookImpl.hbm.xml",
			"org/exoplatform/services/addrbook/impl/AddrGroupImpl.hbm.xml" };

	public AddrBookServiceImpl(	HibernateService hibernateService, IDGeneratorService idService) {     
//			hibernateService.addMappingFiles(MAPPING);
//			hService_ = hibernateService;
//			idService_ = idService;
		}

	public AddrBook createAddrBookInstance() {
		return new AddrBookImpl();
	}

	public AddrBook createAddrBook(AddrGroup group, AddrBook book)
			throws Exception {
		Date now = new Date();
		AddrBookImpl impl = (AddrBookImpl) book;
		impl.setAddrBookId(idService_.generateStringID(impl));
		if(group != null) {
			impl.setAddrGroupId(group.getId());
		} else {
			impl.setAddrGroupId("/");
		}
		impl.setCreatedDate(now);
    hService_.save( impl);  
    org.hibernate.Session session = hService_.openSession();
		session.save(impl);
		session.flush();
		
		return impl;
	}

	public void updateAddrBook(AddrBook book) throws Exception {
    hService_.update( book);
    org.hibernate.Session session = hService_.openSession();
		session.update(book);
		session.flush();

	}

	public AddrBook removeAddrBook(AddrBook book) throws Exception {   
    org.hibernate.Session session = hService_.openSession();
		session.delete(book);
		session.flush();

		return book;
	}

	public AddrBook removeAddrBookById(String id) throws Exception {
		org.hibernate.Session session = hService_.openSession();
		AddrBook book = (AddrBook) session.get(AddrBookImpl.class, id);
		session.delete(book);
		session.flush();

		return book;
	}
	
	public void removeAddrBookByGroup(AddrGroup group)
	throws Exception {
		org.hibernate.Session session = hService_.openSession();
		
		removeAddrBookByGroup(group,session);
		
		session.flush();
	}


	public AddrBook findAddrBookById(String id) throws Exception {
    org.hibernate.Session session = hService_.openSession();
		AddrBook book =
			(AddrBook) hService_.findOne(
				session,
				queryFindAddrBookById,
				id);
		return book;
	}

	public PageList findAddrBooksByAddrGroup(String groupId) throws Exception {
		String queryFindAddrBooksInAddrGroup =
			"from b in class org.exoplatform.services.addrbook.impl.AddrBookImpl, "
				+ "where b.addrGroupId =  '"
				+ groupId
				+ "'";

		String countAddrBooksInAddrGroup =
			"select count(b) "
				+ "from b in class org.exoplatform.services.addrbook.impl.AddrBookImpl, "
				+ "where b.addrGroupId =  '"
				+ groupId
				+ "'";

		return new DBObjectPageList(
			hService_,
			20,
			queryFindAddrBooksInAddrGroup,
			countAddrBooksInAddrGroup);
	}

	/*public PageList findAddrBooks(AddrbookQuery q) throws Exception {
		Session session = hService_.openSession();
		
		String name = refactorString(q.getName());
		String englishName = refactorString(q.getEnglishName());
		String handset = refactorString(q.getHandset());
		String province = refactorString(q.getProvince());
		String city = refactorString(q.getCity());
		String mail = refactorString(q.getMail());
		String nickName = refactorString(q.getNickName());
		
		String groupId = q.getGroupId();
		String userId = q.getUserId();
		
		if(groupId == null || groupId.trim().equals(""))
			groupId = null;
		if(name == null)
			name = "%";
		
		StringBuffer b = new StringBuffer();
		b
			.append("from c in class org.exoplatform.services.addrbook.impl.AddrBookImpl ")
			.append(" where c.name like :name")
			.append(englishName == null ? "" : " and c.englishName like :englishName")
			.append(handset == null ? "" : " and c.handset like :handset")
			.append(province == null ? "" : " and c.province like :province")
			.append(city == null ? "" : " and c.city like :city")
			.append(mail == null ? "" : " and c.mail like :mail")
			.append(nickName == null ? "" : " and c.nickName like :nickName")
			.append(userId == null ? "" : " and c.userId = :userId")
			.append(groupId == null ? "" : " and c.groupId = :groupId");

		Query query = session.createQuery(b.toString());
		query.setString("name", name);
		if(englishName != null)
			query.setString("englishName", englishName);
		if(handset != null)
			query.setString("handset", handset);
		if(province != null)
			query.setString("province", province);
		if(city != null)
			query.setString("city", city);
		if(mail != null)
			query.setString("mail", mail);
		if(nickName != null)
			query.setString("nickName", nickName);
		if(groupId != null)
			query.setString("groupId", groupId);

		//System.out.println("sql:" + b.toString());
		
		query.setFirstResult(0);
		query.setMaxResults(q.getMaxSize());

		return new ObjectPageList(query.list(), 20);
	}*/

	public AddrGroup createAddrGroupInstance() {
		return new AddrGroupImpl();
	}

	public AddrGroup createAddrGroup(AddrGroup group) throws Exception {
    org.hibernate.Session session = hService_.openSession();
		String groupId = "/" + group.getName();
		Object o = session.get(AddrGroupImpl.class, groupId);
		if (o != null) {
			Object[] args = { group.getName()};
			throw new UniqueObjectException(
				"AddrbookService.unique-addrgroup-exception",
				args);
		}

		session.save(group, groupId);
		session.flush();

		return group;
	}

	public AddrGroup addChild(AddrGroup parent, AddrGroup child)
			throws Exception {
    org.hibernate.Session session = hService_.openSession();
		AddrGroup parentGroup =
			(AddrGroup) session.get(AddrGroupImpl.class, parent.getId());
		String groupId = parentGroup.getId() + "/" + child.getName();
		Object o = session.get(AddrGroupImpl.class, groupId);
		if (o != null) {
			Object[] args = { child.getName()};
			throw new UniqueObjectException(
				"AddrbookService.unique-addrgroup-exception",
				args);
		}
		AddrGroupImpl childImpl = (AddrGroupImpl) child;

		childImpl.setParentId(parentGroup.getId());
		session.save(childImpl, groupId);
		session.flush();
		return parentGroup;

	}

	public void updateAddrGroup(AddrGroup group) throws Exception {
    org.hibernate.Session session = hService_.openSession();
		session.update(group);
		session.flush();

	}

	public AddrGroup removeAddrGroup(AddrGroup group) throws Exception {
//    Session session = hService_.openSession();
//
//		/*======= delete section from parent ========*/
//		List sections =
//			session.find(
//				queryFindAddrGroupsByParent,
//				group.getId(),
//				Hibernate.STRING);
//		Iterator i = sections.iterator();
//		while (i.hasNext()) {
//			AddrGroup g = (AddrGroup) i.next();
//			removeAddrGroup(g);
//		}
//		/*==========================================*/
//
//		/*======= delete self and self's books =======*/
//		session.delete(group);
//		removeAddrBookByGroup(group, session);
//		/*============================================*/
//
//		session.flush();
		
		return group;
	}

	public AddrGroup findAddrGroupById(String id) throws Exception {
    org.hibernate.Session session = hService_.openSession();
		AddrGroup group =
			(AddrGroup) hService_.findOne(
				session,
				queryFindAddrGroupById,
				id);
		return group;
	}

	public Collection findAddrGroupsByParent(AddrGroup parent) throws Exception {
    return null;
//    Session session = hService_.openSession();
//		if (parent == null) {
//			return session.find(queryFindRootAddrGroups);
//		}
//		return session.load(
//			queryFindAddrGroupsByParent,
//			parent.getId(),
//			Hibernate.STRING);
	}
	
	/*private String refactorString(String str){
		if (str == null || str.trim().equals("") || str.length() == 0) {
			return null;
		}
		else{
			return "%" + str.replace('*','%') + "%";
		}
		
	}*/
	
	private void removeAddrBookByGroup(AddrGroup group, org.hibernate.Session session)
	throws Exception {
//		session.delete(
//		queryFindAddrBooksByGroup,
//		group.getId(),
//		org.hibernate.Hibernate.STRING);
	}

}
