/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.addrbook.hibernate;

import java.util.Collection;

import junit.framework.TestCase;
import org.exoplatform.services.addrbook.*;
import org.exoplatform.services.addrbook.model.*;
import org.exoplatform.services.addrbook.impl.*;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.commons.utils.PageList;
import org.hibernate.Session;

/**
 * @author Bob
 * Created on 2005-1-6
 *
 */
public class TestAddrBookServiceImpl extends TestCase {

	private static AddrBookService service;
	private static HibernateService hservice_;

	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		if (service == null) {
			PortalContainer manager = PortalContainer.getInstance();
			hservice_ =
				(HibernateService) manager.getComponentInstanceOfType(
					HibernateService.class);
			service =
				(AddrBookService) manager.getComponentInstanceOfType(
						AddrBookService.class);
		}
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
	    hservice_.closeSession();
	}
	
	public void testAddrGroup() throws Exception  {
		
		AddrGroup addrGroupParent = service.createAddrGroupInstance();
		addrGroupParent.setName("AddrGroupParent");
		service.createAddrGroup(addrGroupParent);
		assertTrue(((AddrGroupImpl) addrGroupParent).getId() != null);

		AddrGroup addrGroupChild = service.createAddrGroupInstance();
		addrGroupChild.setName("AddrGroupChild");

		service.addChild(addrGroupParent, addrGroupChild);

		addrGroupParent = service.findAddrGroupById(addrGroupParent.getId());
		assertEquals(addrGroupParent.getName(), "AddrGroupParent");

		addrGroupChild = service.findAddrGroupById(addrGroupChild.getId());
		assertEquals(addrGroupChild.getParentId(), addrGroupParent.getId());

		service.updateAddrGroup(addrGroupChild);

		service.removeAddrGroup(addrGroupChild);
		assertNotNull(service.findAddrGroupById(addrGroupParent.getId()));

		service.addChild(addrGroupParent, addrGroupChild);
		Collection addrGroups = service.findAddrGroupsByParent(addrGroupParent);
		assertEquals(1, addrGroups.size());

		service.removeAddrGroup(addrGroupParent);
		assertNull(service.findAddrGroupById(addrGroupParent.getId()));
		assertNull(service.findAddrGroupById(addrGroupChild.getId()));
	}
	
	public void testAddrBook() throws Exception{
		Session session = hservice_.openSession();
		
		String BOOK = "test";
		org.hibernate.Transaction tx = session.beginTransaction();
		AddrBook addrBook = service.createAddrBookInstance();
		addrBook.setName(BOOK);
		addrBook.setAddress("zhen");
		addrBook.setTelBusiness("123456");
		addrBook.setTelHome("123456");
		addrBook.setZipCode("13013056789");
		addrBook.setEmail("zzb205@163.com");
		
		AddrGroup addrGroup = service.createAddrGroupInstance();
		addrGroup.setName("testAddrGroup");
		
		service.createAddrGroup(addrGroup);
		service.createAddrBook(addrGroup,addrBook);
		tx.commit();
		
		AddrBook u = service.findAddrBookById(addrBook.getAddrBookId());
		assertEquals(u.getName(), BOOK);

		
		/*AddrBookQuery query = new AddrBookQuery();
		query.setName(BOOK);
		query.setEnglishName(BOOK);
		query.setHandset("13056789");
		query.setNickName("Spin*h");
		query.setGroupId("/testAddrGroup");
		query.setMail("zzb205");
		query.setProvince("jiang");
		PageList addrBooks = service.findAddrBooks(query);
		assertTrue("findAddrBooks:Expect 1 addrBook found ", addrBooks.getAvailable() >= 1);*/
		PageList addrBooks = null;
		addrBooks = service.findAddrBooksByAddrGroup(addrGroup.getId());
		assertTrue("findAddrBooksByAddrGroup:Expect 1 addrBook found ", addrBooks.getAvailable() >= 1);

		service.removeAddrBook(addrBook);
		service.removeAddrGroup(addrGroup);
	}

}
