/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.css.sac;

import java.io.IOException;
import java.util.Locale;

import org.exoplatform.services.css.sac.OperationNotYetSupported;
import org.w3c.css.sac.CSSException;
import org.w3c.css.sac.ConditionFactory;
import org.w3c.css.sac.DocumentHandler;
import org.w3c.css.sac.ErrorHandler;
import org.w3c.css.sac.InputSource;
import org.w3c.css.sac.LexicalUnit;
import org.w3c.css.sac.Parser;
import org.w3c.css.sac.SelectorFactory;
import org.w3c.css.sac.SelectorList;


/**
 * @author Ove Ranheim (oranheim@yahoo.no)
 * @since Jul 2, 2004 9:01:04 AM
 */
public class ParserImpl implements Parser {
    
    static final public String PARSER_VERSION = "http://www.w3.org/TR/REC-CSS2";
    
    private Locale _locale;
    private DocumentHandler _documentHandler;
    private SelectorFactory _selectorFactory;
    private ConditionFactory _conditionFactory;
    private ErrorHandler _errorHandler;
    private InputSource _inputSource;

    /* 
     * @see org.w3c.css.sac.Parser#setLocale(java.util.Locale)
     */
    public void setLocale(Locale locale) throws CSSException {
        _locale = locale;
    }

    /* 
     * @see org.w3c.css.sac.Parser#setDocumentHandler(org.w3c.css.sac.DocumentHandler)
     */
    public void setDocumentHandler(DocumentHandler handler) {
        _documentHandler = handler;
    }

    /* 
     * @see org.w3c.css.sac.Parser#setSelectorFactory(org.w3c.css.sac.SelectorFactory)
     */
    public void setSelectorFactory(SelectorFactory selectorFactory) {
        _selectorFactory = selectorFactory;
    }

    /* 
     * @see org.w3c.css.sac.Parser#setConditionFactory(org.w3c.css.sac.ConditionFactory)
     */
    public void setConditionFactory(ConditionFactory conditionFactory) {
        _conditionFactory = conditionFactory;
    }

    /* 
     * @see org.w3c.css.sac.Parser#setErrorHandler(org.w3c.css.sac.ErrorHandler)
     */
    public void setErrorHandler(ErrorHandler handler) {
        _errorHandler = handler;
    }

    /* 
     * @see org.w3c.css.sac.Parser#parseStyleSheet(org.w3c.css.sac.InputSource)
     */
    public void parseStyleSheet(InputSource source) throws CSSException, IOException {
        _inputSource = source;
    }

    /* 
     * @see org.w3c.css.sac.Parser#parseStyleSheet(java.lang.String)
     */
    public void parseStyleSheet(String uri) throws CSSException, IOException {
        _inputSource = new InputSource(uri);
    }

    /* 
     * @see org.w3c.css.sac.Parser#parseStyleDeclaration(org.w3c.css.sac.InputSource)
     */
    public void parseStyleDeclaration(InputSource source) throws CSSException, IOException {
        _inputSource = source;
    }

    /* 
     * @see org.w3c.css.sac.Parser#parseRule(org.w3c.css.sac.InputSource)
     */
    public void parseRule(InputSource source) throws CSSException, IOException {
        _inputSource = source;
    }

    /* 
     * @see org.w3c.css.sac.Parser#getParserVersion()
     */
    public String getParserVersion() {
        return PARSER_VERSION;
    }

    /* 
     * @see org.w3c.css.sac.Parser#parseSelectors(org.w3c.css.sac.InputSource)
     */
    public SelectorList parseSelectors(InputSource source) throws CSSException, IOException {
        throw new OperationNotYetSupported();
    }

    /* 
     * @see org.w3c.css.sac.Parser#parsePropertyValue(org.w3c.css.sac.InputSource)
     */
    public LexicalUnit parsePropertyValue(InputSource source) throws CSSException, IOException {
        throw new OperationNotYetSupported();
    }

    /* 
     * @see org.w3c.css.sac.Parser#parsePriority(org.w3c.css.sac.InputSource)
     */
    public boolean parsePriority(InputSource source) throws CSSException, IOException {
        throw new OperationNotYetSupported();
    }

}
