/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.       *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.publish.hibernate;

import java.util.Collection;

import junit.framework.TestCase;
import org.exoplatform.services.publish.*;
import org.exoplatform.services.publish.model.Content;
import org.exoplatform.services.publish.model.Section;
import org.exoplatform.services.publish.impl.*;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.database.HibernateService;
//import org.exoplatform.services.idgenerator.IDGeneratorService;
//import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.hibernate.Session;
import org.hibernate.Transaction;

//import net.sf.hibernate.Session;

/**
 * @author Bob
 *
 */
 
public class TestPublishServiceImpl extends TestCase {

	//protected Section section;
	//protected Content content;
	private static PublishService service;
	private static HibernateService hservice_;
	
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		if (service == null) {
			PortalContainer manager = PortalContainer.getInstance();
			hservice_ =
				(HibernateService) manager.getComponentInstanceOfType(
					HibernateService.class);
			service =
				(PublishService) manager.getComponentInstanceOfType(
						PublishService.class);
		}
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
	    hservice_.closeSession();
	}

	public void testSection() throws Exception  {
		Section sectionParent = service.createSectionInstance();
		sectionParent.setName("SectionParent");
		sectionParent.setDescription("Section Parent");
		service.createSection(sectionParent);
		assertTrue(((SectionImpl) sectionParent).getId() != null);

		Section sectionChild = service.createSectionInstance();
		sectionChild.setName("SectionChild");
		sectionChild.setDescription("Child Parent");

		service.addChild(sectionParent, sectionChild);

		sectionParent = service.findSectionById(sectionParent.getId());
		assertEquals(sectionParent.getName(), "SectionParent");

		sectionChild = service.findSectionById(sectionChild.getId());
		assertEquals(sectionChild.getParentId(), sectionParent.getId());

		sectionChild.setDescription("Changed Description");
		service.updateSection(sectionChild);
		/*
		assertEquals(
			service.findSectionById(sectionChild.getId()).getLabel(),
			"SectionRenamed");
		*/

		service.removeSection(sectionChild);
		assertNotNull(service.findSectionById(sectionParent.getId()));

		service.addChild(sectionParent, sectionChild);
		Collection sections = service.findSectionsByParent(sectionParent);
		assertEquals(1, sections.size());

		service.removeSection(sectionParent);
		assertNull(service.findSectionById(sectionParent.getId()));
		assertNull(service.findSectionById(sectionChild.getId()));
	}
	
	public void testContent() throws Exception{
		Session session = hservice_.openSession();
		
		String CONTENT = "test";
		Transaction tx = session.beginTransaction();
		Content content = service.createContentInstance();
		content.setTitle(CONTENT);
		content.setIntroText("intro text");
		content.setFullText("full text");
		content.setCreatedBy("bob");
		
		Section section = service.createSectionInstance();
		section.setName("test section");
		section.setDescription("this is test");
		
		service.createSection(section);
		service.createContent(section,content);
		tx.commit();
		
		Content u = service.findContentById(content.getContentId());
		assertEquals(u.getTitle(), "test");

		ContentQuery query = new ContentQuery();
		PageList contents = service.findContents(query);
		assertTrue("findContents:Expect 1 content found ", contents.getAvailable() >= 1);
		
		//contents = null;
		//contents = service.findContentsByAuthor("bob");
		//assertTrue("findContentsByAuthor:Expect 1 content found ", contents.getAvailable() >= 1);
		
		contents = null;
		contents = service.findContentsBySection(section.getId());
		assertTrue("findContentsBySection:Expect 1 content found ", contents.getAvailable() >= 1);

		service.removeContent(content.getContentId());
		service.removeSection(section);
	}

}
