/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * (c) Copyright 2002, Hewlett-Packard Development Company, LP
 * Please look at license.txt and license_hp.txt in info directory for more license detail.
 **/
package org.exoplatform.services.rdf.model;

import org.exoplatform.services.rdf.model.RDFModelService;
import com.hp.hpl.jena.db.IDBConnection;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.ontology.daml.DAMLModel;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ModelMaker;
import com.hp.hpl.jena.rdf.model.ModelSpec;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.shared.PrefixMapping;
import com.hp.hpl.jena.shared.ReificationStyle;

/**
 * Created by the Exo Development team.
 * 
 * @author Ove Ranheim (oranheim@users.sourceforge.net)
 * @since 07.apr.2004 18:38:35
 */
public class RDFModelServiceImpl implements RDFModelService {

    public RDFModelServiceImpl() {
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#finalStandard()
     */
    public ReificationStyle finalStandard() {
        return ModelFactory.Standard;
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#finalConvenient()
     */
    public ReificationStyle finalConvenient() {
        return ModelFactory.Convenient;
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#finalMinimal()
     */
    public ReificationStyle finalMinimal() {
        return ModelFactory.Minimal;
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#setDefaultModelPrefixes(com.hp.hpl.jena.shared.PrefixMapping)
     */
    public PrefixMapping setDefaultModelPrefixes(PrefixMapping pm) {
        return ModelFactory.setDefaultModelPrefixes(pm);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#getDefaultModelPrefixes()
     */
    public PrefixMapping getDefaultModelPrefixes() {
        return ModelFactory.getDefaultModelPrefixes();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createSpec(com.hp.hpl.jena.rdf.model.Model)
     */
    public ModelSpec createSpec(Model desc) {
        return ModelFactory.createSpec(desc);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createSpec(com.hp.hpl.jena.rdf.model.Resource,
     *      com.hp.hpl.jena.rdf.model.Model)
     */
    public ModelSpec createSpec(Resource root, Model desc) {
        return ModelFactory.createSpec(root, desc);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createModel(com.hp.hpl.jena.rdf.model.ModelSpec)
     */
    public Model createModel(ModelSpec desc) {
        return ModelFactory.createModel(desc);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createModelOver(com.hp.hpl.jena.rdf.model.ModelSpec,
     *      java.lang.String)
     */
    public Model createModelOver(ModelSpec desc, String name) {
        return ModelFactory.createModelOver(desc, name);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createDefaultModel()
     */
    public Model createDefaultModel() {
        return ModelFactory.createDefaultModel();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createDefaultModel(com.hp.hpl.jena.shared.ReificationStyle)
     */
    public Model createDefaultModel(ReificationStyle style) {
        return ModelFactory.createDefaultModel(style);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#withHiddenStatements(com.hp.hpl.jena.rdf.model.Model)
     */
    public Model withHiddenStatements(Model m) {
        return ModelFactory.withHiddenStatements(m);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createNonreifyingModel()
     */
    public Model createNonreifyingModel() {
        return ModelFactory.createNonreifyingModel();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createModelForGraph(com.hp.hpl.jena.graph.Graph)
     */
    public Model createModelForGraph(Graph g) {
        return ModelFactory.createModelForGraph(g);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createFileModelMaker(java.lang.String)
     */
    public ModelMaker createFileModelMaker(String root) {
        return ModelFactory.createFileModelMaker(root);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createFileModelMaker(java.lang.String,
     *      com.hp.hpl.jena.shared.ReificationStyle)
     */
    public ModelMaker createFileModelMaker(String root, ReificationStyle style) {
        return ModelFactory.createFileModelMaker(root, style);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createMemModelMaker()
     */
    public ModelMaker createMemModelMaker() {
        return ModelFactory.createMemModelMaker();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createMemModelMaker(com.hp.hpl.jena.shared.ReificationStyle)
     */
    public ModelMaker createMemModelMaker(ReificationStyle style) {
        return ModelFactory.createMemModelMaker(style);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createModelRDBMaker(com.hp.hpl.jena.db.IDBConnection)
     */
    public ModelMaker createModelRDBMaker(IDBConnection c) {
        return ModelFactory.createModelRDBMaker(c);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createModelRDBMaker(com.hp.hpl.jena.db.IDBConnection,
     *      com.hp.hpl.jena.shared.ReificationStyle)
     */
    public ModelMaker createModelRDBMaker(IDBConnection c, ReificationStyle style) {
        return ModelFactory.createModelRDBMaker(c, style);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createSimpleRDBConnection(java.lang.String,
     *      java.lang.String, java.lang.String, java.lang.String)
     */
    public IDBConnection createSimpleRDBConnection(String url, String user, String password, String dbType) {
        return ModelFactory.createSimpleRDBConnection(url, user, password, dbType);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createSimpleRDBConnection()
     */
    public IDBConnection createSimpleRDBConnection() {
        return ModelFactory.createSimpleRDBConnection();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createRDFSModel(com.hp.hpl.jena.rdf.model.Model)
     */
    public InfModel createRDFSModel(Model model) {
        return ModelFactory.createRDFSModel(model);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createRDFSModel(com.hp.hpl.jena.rdf.model.Model,
     *      com.hp.hpl.jena.rdf.model.Model)
     */
    public InfModel createRDFSModel(Model schema, Model model) {
        return ModelFactory.createRDFSModel(schema, model);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createInfModel(com.hp.hpl.jena.reasoner.Reasoner,
     *      com.hp.hpl.jena.rdf.model.Model)
     */
    public InfModel createInfModel(Reasoner reasoner, Model model) {
        return ModelFactory.createInfModel(reasoner, model);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createInfModel(com.hp.hpl.jena.reasoner.Reasoner,
     *      com.hp.hpl.jena.rdf.model.Model, com.hp.hpl.jena.rdf.model.Model)
     */
    public InfModel createInfModel(Reasoner reasoner, Model schema, Model model) {
        return ModelFactory.createInfModel(reasoner, model);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createOntologyModel()
     */
    public OntModel createOntologyModel() {
        return ModelFactory.createOntologyModel();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createOntologyModel(java.lang.String)
     */
    public OntModel createOntologyModel(String languageURI) {
        return ModelFactory.createOntologyModel(languageURI);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createOntologyModel(com.hp.hpl.jena.ontology.OntModelSpec,
     *      com.hp.hpl.jena.rdf.model.ModelMaker,
     *      com.hp.hpl.jena.rdf.model.Model)
     */
    public OntModel createOntologyModel(OntModelSpec spec, ModelMaker maker, Model base) {
        return ModelFactory.createOntologyModel();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createOntologyModel(com.hp.hpl.jena.ontology.OntModelSpec,
     *      com.hp.hpl.jena.rdf.model.Model)
     */
    public OntModel createOntologyModel(OntModelSpec spec, Model base) {
        return ModelFactory.createOntologyModel(spec, base);
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactory V2.1
     * 
     * @see exo.services.rdf.RDFService#createDAMLModel()
     */
    public DAMLModel createDAMLModel() {
        return ModelFactory.createDAMLModel();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactoryBase V2.1
     * 
     * @see exo.services.rdf.RDFService#guessDBURL()
     */
    public String guessDBURL() {
        return ModelFactory.guessDBURL();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactoryBase V2.1
     * 
     * @see exo.services.rdf.RDFService#guessDBUser()
     */
    public String guessDBUser() {
        return ModelFactory.guessDBUser();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactoryBase V2.1
     * 
     * @see exo.services.rdf.RDFService#guessDBPassword()
     */
    public String guessDBPassword() {
        return ModelFactory.guessDBPassword();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactoryBase V2.1
     * 
     * @see exo.services.rdf.RDFService#guessDBType()
     */
    public String guessDBType() {
        return ModelFactory.guessDBType();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactoryBase V2.1
     * 
     * @see exo.services.rdf.RDFService#guessDBDriver()
     */
    public String guessDBDriver() {
        return ModelFactory.guessDBDriver();
    }

    /*
     * Refactored from com.hp.hpl.jena.rdf.model.ModelFactoryBase V2.1
     * 
     * @see exo.services.rdf.RDFService#extractType(java.lang.String)
     */
    public String extractType(String dbURL) {
        return ModelFactory.extractType(dbURL);
    }

}