/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.vietspider.veis.impl.core.text.codec;

/**
 * @by thuannd (nhudinhthuan@yahoo.com)
 * @since Jul 4, 2005
 */
public class VietnameseConverter {
  
  private String[] precomposed  = {
      "à","á","ả","ã","ạ",
      "â","ầ","ấ","ẩ","ẫ","ậ",
      "ă","ằ","ắ","ẳ","ẵ","ặ" ,
      "è","é","ẻ","ẽ","ẹ",
      "ê","ề","ế","ể","ễ","ệ",
      "ò","ó","ỏ","õ","ọ",
      "ô","ồ","ố","ổ","ỗ","ộ",
      "ơ","ờ","ớ","ở","ỡ","ợ",
      "ù","ú","ủ","ũ","ụ",
      "ư","ừ","ứ","ử","ữ","ự",
      "ì","í","ỉ","ĩ","ị",
      "ỳ","ý","ỷ","ỹ","ỵ", "đ",
      
      "À","Á","Ả","Ã","Ạ",
      "Â","Ầ","Ấ","Ẩ","Ẫ","Ậ",
      "Ă","Ằ","Ắ","Ẳ","Ẵ","Ặ" ,
      "È","É","Ẻ","Ẽ","Ẹ",
      "Ê","Ề","Ế","Ể","Ễ","Ệ",
      "Ò","Ó","Ỏ","Õ","Ọ",
      "Ô","Ồ","Ố","Ổ","Ỗ","Ộ",
      "Ơ","Ờ","Ớ","Ở","Ỡ","Ợ",
      "Ù","Ú","Ủ","Ũ","Ụ",
      "Ư","Ừ","Ứ","Ử","Ữ","Ự",
      "Ì","Í","Ỉ","Ĩ","Ị ",
      "Ỳ","Ý","Ỷ","Ỹ","Ỵ", "Đ"
  };
  private String[] precomposedSign  = {
      "&#224;̀","&#224;́","&#224;̉","&#224;̃","&#224;̣",
      "&#226;","&#226;̀","&#226;́","&#226;̉","&#226;̃","&#226;̣",
      "&#259;","&#259;̀","&#259;́","&#259;̉","&#259;̃","&#259;̣" ,
      "&#232;̀","&#232;́","&#232;̉","&#232;̃","&#232;̣",
      "&#234;","&#234;̀","&#234;́","&#234;̉","&#234;̃","&#234;̣",
      "&#242;̀","&#242;́","&#242;̉","&#242;̃","&#242;̣",
      "&#244;","&#244;̀","&#244;́","&#244;̉","&#244;̃","&#244;̣",
      "&#417;","&#417;̀","&#417;́","&#417;̉","&#417;̃","&#417;̣",
      "&#249;̀","&#249;́","&#249;̉","&#249;̃","&#249;̣",
      "&#432;","&#432;̀","&#432;́","&#432;̉","&#432;̃","&#432;̣",
      "&#236;̀","&#236;́","&#236;̉","&#236;̃","&#236;̣",
      "&#7923;̀","&#7923;́","&#7923;̉","&#7923;̃","&#7923;̣", "&#273;",
      
      "&#192;̀","&#192;́","&#192;̉","&#192;̃","&#192;̣",
      "&#7846;","&#7846;̀","&#7846;́","&#7846;̉","&#7846;̃","&#7846;̣",
      "&#258;","&#258;̀","&#258;́","&#258;̉","&#258;̃","&#258;̣" ,
      "&#200;̀","&#200;́","&#200;̉","&#200;̃","&#200;̣",
      "&#202;","&#202;̀","&#202;́","&#202;̉","&#202;̃","&#202;̣",
      "&#210;̀","&#210;́","&#210;̉","&#210;̃","&#210;̣",
      "&#212;","&#212;̀","&#212;́","&#212;̉","&#212;̃","&#212;̣",
      "&#416;","&#416;̀","&#416;́","&#416;̉","&#416;̃","&#416;̣",
      "&#217;̀","&#217;́","&#217;̉","&#217;̃","&#217;̣",
      "&#431;","&#431;̀","&#431;́","&#431;̉","&#431;̃","&#431;̣",
      "&#204;̀","&#204;́","&#204;̉","&#204;̃","&#204;̣ ",
      "&#7922;̀","&#7922;́","&#7922;̉","&#7922;̃","&#7922;̣", "&#272;"
  };
  
  private String[] precomposedAnsi={
      "a&#768;","a&#769;","a&#777;","a&#771;","a&#803;",
      "â","â&#768;","â&#769;","â&#777;","â&#771;","â&#803;",
      "ă","ă&#768;","ă&#769;","ă&#777;","ă&#771;","ă&#803;" ,
      "e&#768;","e&#769;","e&#777;","e&#771;","e&#803;",
      "ê","ê&#768;","ê&#769;","ê&#777;","ê&#771;","ê&#803;",
      "o&#768;","o&#769;","o&#777;","o&#771;","o&#803;",
      "ô","ô&#768;","ô&#769;","ô&#777;","ô&#771;","ô&#803;",
      "ơ","ơ&#768;","ơ&#769;","ơ&#777;","ơ&#771;","ơ&#803;",
      "u&#768;","u&#769;","u&#777;","u&#771;","u&#803;",
      "ư","ư&#768;","ư&#769;","ư&#777;","ư&#771;","ư&#803;",
      "i&#768;","i&#769;","i&#777;","i&#771;","i&#803;",
      "y&#768;","y&#769;","y&#777;","y&#771;","y&#803;", "&#273;",
      
      "A&#768;","A&#769;","A&#777;","A&#771;","A&#803;",
      "Â","Â&#768;","Â&#769;","Â&#777;","Â&#771;","Â&#803;",
      "Ă","Ă&#768;","Ă&#769;","Ă&#777;","Ă&#771;","Ă&#803;" ,
      "E&#768;","E&#769;","E&#777;","E&#771;","E&#803;",
      "Ê","Ê&#768;","Ê&#769;","Ê&#777;","Ê&#771;","Ê&#803;",
      "O&#768;","O&#769;","O&#777;","O&#771;","O&#803;",
      "Ô","Ô&#768;","Ô&#769;","Ô&#777;","Ô&#771;","Ô&#803;",
      "Ơ","Ơ&#768;","Ơ&#769;","Ơ&#777;","Ơ&#771;","Ơ&#803;",
      "U&#768;","U&#769;","U&#777;","U&#771;","U&#803;",
      "Ư","Ư&#768;","Ư&#769;","Ư&#777;","Ư&#771;","Ư&#803;",
      "I&#768;","I&#769;","I&#777;","I&#771;","I&#803;",
      "Y&#768;","Y&#769;","Y&#777;","Y&#771;","Y&#803;", "&#272;"
  };
  private String[] composed = {
      "à","á","ả","ã","ạ",
      "â","ầ","ấ","ẩ","ẫ","ậ",
      "ă","ằ","ắ","ẳ","ẵ","ặ" ,
      "è","é","ẻ","ẽ","ẹ",
      "ê","ề","ế","ể","ễ","ệ",
      "ò","ó","ỏ","õ","ọ",
      "ô","ồ","ố","ổ","ỗ","ộ",
      "ơ","ờ","ớ","ở","ỡ","ợ",
      "ù","ú","ủ","ũ","ụ",
      "ư","ừ","ứ","ử","ữ","ự",
      "ì","í","ỉ","ĩ","ị",
      "ỳ","ý","ỷ","ỹ","ỵ",      "đ",
      
      "À","Á","Ả","Ã","Ạ",
      "Â","Ầ","Ấ","Ẩ","Ẫ","Ậ",
      "Ă","Ằ","Ắ","Ẳ","Ẵ","Ặ" ,
      "È","É","Ẻ","Ẽ","Ẹ",
      "Ê","Ề","Ế","Ể","Ễ","Ệ",
      "Ò","Ó","Ỏ","Õ","Ọ",
      "Ô","Ồ","Ố","Ổ","Ỗ","Ộ",
      "Ơ","Ờ","Ớ","Ở","Ỡ","Ợ",
      "Ù","Ú","Ủ","Ũ","Ụ",
      "Ư","Ừ","Ứ","Ử","Ữ","Ự",
      "Ì","Í","Ỉ","Ĩ","Ị",
      "Ỳ","Ý","Ỷ","Ỹ","Ỵ","Đ"
  };
  private String[] composedAnsi= {
      "&#224;", "&#225;", "&#7843;", "&#227;", "&#7841;",
      "&#226;", "&#7847;", "&#7845;", "&#7849;", "&#7851;", "&#7853;",
      "&#259;", "&#7857;", "&#7855;", "&#7859;", "&#7861;", "&#7863;",
      "&#232;", "&#233;", "&#7867;", "&#7869;", "&#7865;",
      "&#234;", "&#7873;", "&#7871;", "&#7875;", "&#7877;", "&#7879;",
      "&#242;", "&#243;", "&#7887;", "&#245;", "&#7885;",
      "&#244;", "&#7891;", "&#7889;", "&#7893;", "&#7895;", "&#7897;",
      "&#417;", "&#7901;", "&#7899;", "&#7903;", "&#7905;", "&#7907;",
      "&#249;", "&#250;", "&#7911;", "&#361;", "&#7909;",
      "&#432;", "&#7915;", "&#7913;", "&#7917;", "&#7919;", "&#7921;",
      "&#236;", "&#237;", "&#7881;", "&#297;", "&#7883;",
      "&#7923;", "&#253;", "&#7927;", "&#7929;", "&#7925;", "&#273;",
      
      "&#192;", "&#193;", "&#7842;", "&#195;", "&#7840;", "&#194;",
      "&#7846;", "&#7844;", "&#7848;", "&#7850;", "&#7852;",
      "&#258;", "&#7856;", "&#7854;", "&#7858;", "&#7860;", "&#7862;",
      "&#200;", "&#201;", "&#7866;", "&#7868;", "&#7864;",
      "&#202;", "&#7872;", "&#7870;", "&#7874;", "&#7876;", "&#7878;",
      "&#210;", "&#211;", "&#7886;", "&#213;", "&#7884;",
      "&#212;", "&#7890;", "&#7888;", "&#7892;", "&#7894;", "&#7896;",
      "&#416;", "&#7900;", "&#7898;", "&#7902;", "&#7904;", "&#7906;",
      "&#217;", "&#218;", "&#7910;", "&#360;", "&#7908;",
      "&#431;", "&#7914;", "&#7912;", "&#7916;", "&#7918;", "&#7920;",
      "&#204;", "&#205;", "&#7880;", "&#296;", "&#7882;",
      "&#7922;", "&#221;", "&#7926;", "&#7928;", "&#7924;", "&#272;"
  };
  
//  private String ansiSign [] = {
//      "&nbsp;","&quot;","&#8216;","&#8217;","&amp;"
//  };
//  
//  private  String sign[] = {
//      " ","\"","�","�","&"
//  };
  
  protected String replace(String string, String[] pattern, String[] match){    
    for( int i = 0; i< pattern.length; i++)      
      string = string.replaceAll( pattern[i], match[i]);
    return string;
  }
  
  public String[] convert( String string){
    String [] data = new String[5];
    data[0] = string;
    data[1] = this.replace( string, this.composed, this.precomposed);
    data[2] = this.replace(string, this.composed, this.precomposedAnsi);
    data[3] = this.replace(string, this.composed, this.precomposedSign);
    data[4] = this.replace(string, this.composed, this.composedAnsi);
    return data;
  }
}
