/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.vietspider.veis.impl.ui.publisher;

import java.io.File;
import java.net.URL;
import java.util.List;
import java.util.Properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TreeItem;
import org.vietspider.veis.impl.core.io.base.DataBuffer;
import org.vietspider.veis.impl.core.io.base.UtilFile;
import org.vietspider.veis.impl.core.io.local.RSSProperties;
import org.vietspider.veis.impl.core.rss.RssClient;
import org.vietspider.veis.impl.core.rss.dom.RssChannel;
import org.vietspider.veis.impl.core.rss.dom.RssItem;
import org.vietspider.veis.impl.core.text.codec.UtilConvert;
import org.vietspider.veis.impl.ui.resources.ResourceManager;
import org.vietspider.veis.impl.ui.widget.ApplicationFactory;
import org.vietspider.veis.impl.ui.widget.ImageLoader;

/**
 * @by thuannd (nhudinhthuan@yahoo.com)
 * @since Jul 26, 2005
 */
public class RSS extends RSSAction implements Runnable{  
  
  private int state = -1;  
  private Thread t;

  public RSS(ApplicationFactory factory, Composite parent, int style) {
    super(factory, parent, style);  
    try{
      load();
    }catch( Exception exp){
      exp.printStackTrace();
    }
    t = new Thread( this);
    t.start();
    
  }
  
  protected void load() throws Exception {
    treeRSS.removeAll();
    UtilFile util = new UtilFile();    
    String path = util.getFolder("data.rss") ; 
    File file = new File( path);
    File [] lstFile = file.listFiles();
    UtilConvert convert = new UtilConvert();    
    Image image = (new ImageLoader()).load( this.getDisplay(), "domain.gif");  
    if( lstFile == null) return;
    for( int i=0; i< lstFile.length; i++){ 
      TreeItem item  = new TreeItem( treeRSS, SWT.NONE);
      if( image != null) item.setImage( image);
      item.setFont(new Font( this.getDisplay(), "Times New Roman", 12, SWT.NORMAL));
      item.setText( convert.deUnicode( lstFile[i].getName()));                
      createSource( lstFile[i], item);     
    }    
  }
  
  public void createSource(  File file, TreeItem parent){   
    String[] lstName = file.list();
    UtilConvert convert = new UtilConvert();
    if( lstName == null || lstName.length < 1)  return ;       
    Image image = (new ImageLoader()).load( this.getDisplay(), "source.gif");    
    for( int i=0; i< lstName.length; i++){
      TreeItem item  = new TreeItem( parent, SWT.NONE);      
      if( image != null) item.setImage( image);
      item.setText( convert.deUnicode( lstName[i]).trim());
      item.setFont(new Font( this.getDisplay(), "Times New Roman", 11, SWT.NORMAL));
    }       
  }
  
  protected void deleteRSS(){ 
    MessageBox msg = new MessageBox ( this.getShell(), SWT.APPLICATION_MODAL | SWT.YES | SWT.NO);
    ResourceManager resource = new ResourceManager( "veis");
    msg.setMessage ( resource.getLabel( this.getClass().getName()+".msgRSSDelete"));    
    if(msg.open() == SWT.NO) return;
    
    TreeItem [] items = this.treeRSS.getSelection();    
    if( items == null  || items.length < 1) return;
    TreeItem item = items[0];       
    try{
      UtilFile util = new UtilFile();    
      String folder = util.getFolder("data.rss") ;    
      UtilConvert convert = new UtilConvert();
      String p = "";
      if( item.getParentItem() != null) {
        p = folder + File.separator + convert.enUnicode( item.getParentItem().getText())
             + File.separator + convert.enUnicode( item.getText());
      }else{
        p = folder + File.separator + convert.enUnicode( item.getText());
      }     
      File f  = new File( p);
      if( !f.exists()) return;
      if( f.isDirectory()){
        util.deleteFolder( p);
      }else{
        f.delete();
      }
      this.load();
    }catch( Exception exp){
      exp.printStackTrace();
    }   
  }
  
  public RssChannel getChannel( String path){
    try{
      RSSProperties io = new RSSProperties();
      Properties p = io.get( path);
      String cate = p.getProperty("Domain");
      String name = p.getProperty("Name");
      RssClient client = new RssClient( new URL( p.get("URL").toString()), cate, name);
      return new RssChannel( client);
    }catch( Exception exp){
      exp.printStackTrace();
      return null;
    }
  }
  
  public String getContent( RssChannel channel){
    String html =  "<html><head>\n<style type=\"text/css\">\n"
      +"a {text-decoration:none; color: #000000}"
      +"a:hover { color:#FF6600;}a:Visited {color: 84689D;}\n"                   
      +".title { font-family: 'arial'; font-size: 12.5pt; font-weight: bold; margin-top: 0 }\n"                 
      +".des   { font-family: 'Times New Roman'; font-size: 12pt; color: #000000}\n"                  
      +".pAuthor { font-family: 'arial'; font-size: 10pt; text-align: Right; font-weight: bold }\n"
      +".name { font-family: 'arial'; font-size: 9pt; text-align: Right; font-style: italic; }\n"                    
      +"\n</style>\n"
      +"<meta http-equiv=\"content-type\" content=\"text/head; charset=utf-8\">\n"
      +"</head><div align=\"center\"><table  valign=\"center\">\n";
      try{
        channel.update();
        List<RssItem> list = channel.getItemList();        
        for( int i=0; i<list.size(); i++){
          html += this.getHtmlText( list.get(i).getTitle(), list.get(i).getLink().toString(),
                    list.get(i).getPubDate().toString(), list.get(i).getDescription() );
        }
      }catch( Exception exp){
        exp.printStackTrace();
      }
     html += "</table></div></body></html>";  
    return html;
  }
  
  private String getHtmlText( String title, String url , String time, String des){
    return "<tr>"
        +  "<td width=\"100%\" vAlign=\"top\" align=\"left\" colspan= \"2\">\n"       
        +  "<a style=\"text-decoration:none\" href=\""
        +  url+"\" class=\"title\">\n"+ title+".</a>"
        +"</td></tr><tr>\n"
        +  "<td width=\"100%\" vAlign=\"top\" align=\"right\" colspan= \"2\" class=\"name\">\n"
        +  time+ "</td></tr>\n"
        + "<tr><td vAlign=\"top\" align=\"left\" colspan= \"2\" class=\"des\">"
        +"<p align=\"justify\">"+des+".</p><br></td></tr>";    
  }
  
  public boolean save(){
    String c = this.getContent( this.getChannel( this.path));
    UtilFile util = new UtilFile();   
    DataBuffer buffer = new DataBuffer();
    try{
      String  tp = util.getFile("data.temp.rss", "view.htm");
      buffer.saveByte( tp, c.getBytes("utf-8"));      
      return true;      
    }catch( Exception exp){
      exp.printStackTrace();
      return false;
    }    
  }
  
  public String getTitle(){
    return this.title;
  }
  
  public void run(){
    while( true){
      try{        
        if( state == 0){
          save();
          state = 1;
        }
        Thread.sleep( 1000);
      }catch( Exception exp){
        exp.printStackTrace();
      }
    }
  }
  
  public int getState(){
    return this.state;
  }
  
  public void setState( int i){
    this.state = i;
  }

}
