/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.vietspider.veis.impl.ui.publisher;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.vietspider.veis.impl.core.io.base.UtilFile;
import org.vietspider.veis.impl.core.io.local.RSSProperties;
import org.vietspider.veis.impl.core.text.codec.UtilConvert;
import org.vietspider.veis.impl.ui.publisher.action.RSSSaveEvent;
import org.vietspider.veis.impl.ui.publisher.action.RSSSaveListener;
import org.vietspider.veis.impl.ui.widget.ApplicationFactory;

/**
 * @by thuannd (nhudinhthuan@yahoo.com)
 * @since Jul 26, 2005
 */
public class RSSDialogViewer {
  
  private Shell shell;   
  private Combo cboDomain;
  private Text txtName, txtURL, txtDes; 
  private List<RSSSaveListener> listeners;
  
  public RSSDialogViewer(Shell parent){
    listeners = new ArrayList<RSSSaveListener>();
    shell = new Shell( parent);  
    shell.setLocation( parent.getLocation().x+250, parent.getLocation().y+300);
    
    ApplicationFactory factory = new ApplicationFactory( shell, this.getClass().getName());    
    
    shell.setLayout( null);    
    
    factory.createLabel( "lblDomain").setBounds( 5, 10, 70, 20); 
    cboDomain = factory.createCombo( SWT.DROP_DOWN);    
    cboDomain.setBounds( 80, 10, 200, 20);   
    
    factory.createLabel( "lblName").setBounds( 5, 40, 70, 20); 
    txtName = factory.createText();  
    txtName.setBounds( 80, 40, 200, 20); 
    
    factory.createLabel( "lblURL").setBounds( 5, 70, 70, 20); 
    txtURL = factory.createText();   
    txtURL.setBounds( 80, 70, 400, 20); 
    
    factory.createLabel( "lblDes").setBounds( 5, 100, 70, 20);  
    txtDes = factory.createText( SWT.MULTI | SWT.BORDER);    
    txtDes.setBounds( 80, 100, 400, 70); 
    
    Button ok = factory.createButton("butOk", new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {  
        save();
      }
    });
    ok.setBounds( 150, 190, 100, 25); 
    
    Button cancel = factory.createButton("butCancel", new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) { 
        reset();
      }
    });        
    cancel.setBounds( 270, 190, 100, 25);
    
    load();
    
    shell.setSize( 500, 250);
    shell.open();
  }
  private void reset(){
    this.cboDomain.setText("");
    this.txtDes.setText("");
    this.txtName.setText("");
    this.txtURL.setText("");
  }
  public void load(){
    try{
      UtilFile util = new UtilFile();    
      String path = util.getFolder("data.rss") ; 
      File file = new File( path);
      File [] lstFile = file.listFiles();
      UtilConvert convert = new UtilConvert();     
      if( lstFile == null || lstFile.length < 1) return;
      String [] name = new String[ lstFile.length];
      for( int i=0; i< lstFile.length; i++)
        name[i] = convert.deUnicode( lstFile[i].getName()); 
      this.cboDomain.setItems( name);
    }catch( Exception exp){
      exp.printStackTrace();
    }
  }
  
  public void setData( String path){
    try{
      if( path == null || path.trim().equals("")) return;
      RSSProperties io = new RSSProperties();      
      Properties p = io.get( path);
      String cate = p.getProperty("Domain");
      String name = p.getProperty("Name");
      String url = p.getProperty( "URL");
      String des = p.getProperty("Description");
      UtilConvert convert = new UtilConvert();
      if( cate != null) this.cboDomain.setText( convert.deUnicode( cate));
      if( name != null) this.txtName.setText( convert.deUnicode( name));
      if( url != null) this.txtURL.setText( url);
      if( des != null) this.txtDes.setText( convert.deUnicode( des));
    }catch( Exception exp){
      exp.printStackTrace();     
    }
  }
  
  
//event
  public void fireSaveEvent( RSSSaveEvent e) {
    synchronized(listeners)  {
      for(int i = 0; i < listeners.size(); i++){
        try{
          RSSSaveListener listener = listeners.get(i);         
          listener.save(e);       
          break;
        }catch(Exception exp){                  
        }       
      }
    }
  }
  public void addSaveListener( RSSSaveListener listener) {      
    synchronized(listeners) {
      listeners.remove(listener);
      listeners.add(listener);
    }
  }
  public void removeSaveListener( RSSSaveListener listener) {
    synchronized(listeners) {
      listeners.remove(listener);
    }
  }
  public void save( RSSSaveEvent ev) {    
    fireSaveEvent(ev);    
  } 
  private void save(){    
    try{
      if( this.txtName.getText().equals("")) return;
      if( this.cboDomain.getText().equals("")) return;
      if( this.txtURL.getText().equals("")) return;
      
      RSSProperties pro = new RSSProperties();
      UtilConvert convert = new UtilConvert();
      String name = this.txtName.getText();
      String cate = this.cboDomain.getText();
      name = convert.enUnicode( name);
      cate = convert.enUnicode( cate);
      pro.set( cate, name, this.txtURL.getText(), this.txtDes.getText());
      RSSSaveEvent event = new RSSSaveEvent();    
      fireSaveEvent(event); 
    }catch( Exception exp){
      exp.printStackTrace();
    }   
    shell.dispose();
  }
}
