package org.vietspider.veis.impl.ui.spider;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.vietspider.veis.impl.core.io.local.FavoritesCollection;
import org.vietspider.veis.impl.core.io.local.FavoritesFolder;
import org.vietspider.veis.impl.core.text.codec.UtilConvert;
import org.vietspider.veis.impl.ui.spider.action.FavoritesAddEvent;
import org.vietspider.veis.impl.ui.spider.action.FavoritesAddListener;
import org.vietspider.veis.impl.ui.spider.action.FavoritesSaveEvent;
import org.vietspider.veis.impl.ui.spider.action.FavoritesSaveListener;
import org.vietspider.veis.impl.ui.widget.ApplicationFactory;

public class FavoritesViewer {
  
  private Shell shell;
  private List lstFav, lstAgent;
  private Vector<FavoritesAddListener> listeners;
  private Vector<FavoritesSaveListener> listeners2;
  
  public FavoritesViewer(Shell parent){
    shell = new org.eclipse.swt.widgets.Shell( parent);
    shell.setLocation( parent.getLocation().x+150, parent.getLocation().y+50);    
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;		
    shell.setLayout( gridLayout);
    
    listeners = new Vector<FavoritesAddListener>();
    listeners2 = new Vector<FavoritesSaveListener>();
    
    SashForm sashMain = new SashForm( shell, SWT.HORIZONTAL);
    GridData gridData = new GridData(GridData.FILL_BOTH);	   
    gridData.grabExcessHorizontalSpace = true;	
    sashMain.setLayoutData(gridData);
    
    ApplicationFactory factory = new ApplicationFactory( sashMain, this.getClass().getName());
    
    Group group = factory.createGroup("groupCategory", null,  new FillLayout());    
    factory.setComposite( group);
    this.lstFav = factory.createList( SWT.SINGLE | SWT.BORDER, new String[]{}, new SelectionAdapter(){
      public void widgetSelected(SelectionEvent evt) {
        selectFav();  
      }   
    });  
    
    factory.setComposite( sashMain);    
    group = factory.createGroup("groupAgent", null,  new FillLayout());
    factory.setComposite( group);    
    lstAgent = factory.createList( SWT.SINGLE | SWT.BORDER, new String[]{});
    
    Composite butCom = new Composite( shell, SWT.NONE);
    gridData = new GridData(GridData.FILL_HORIZONTAL);		
    butCom.setLayoutData(gridData);
    RowLayout rowLayout = new RowLayout();
    butCom.setLayout( rowLayout);
    rowLayout.justify = true;
    
    factory.setComposite( butCom); 
    factory.createButton( "butSave", new SelectionAdapter(){
      public void widgetSelected(SelectionEvent evt) {
        createdSaveAction();
      }   
    });  
    
    factory.createButton( "butAdd", new SelectionAdapter(){
      public void widgetSelected(SelectionEvent evt) {
        createdAddAction();
      }   
    }); 
    
    factory.createButton( "butRemove", new SelectionAdapter(){
      public void widgetSelected(SelectionEvent evt) {
        removeFav();
      }   
    }); 
    
    factory.createButton( "butClear", new SelectionAdapter(){
      public void widgetSelected(SelectionEvent evt) {
        removeAll();
      }   
    });  
      
    sashMain.setWeights( new int[]{ 120, 170});
    
    shell.setSize( 400, 400);		
    shell.open();
    this.loadFav();
  }
  private String getFav(){
    String [] data = lstFav.getSelection();
    if( data.length > 0)
      return data[0];
    return "";
  }
  private void removeFav(){
    try{			
      (new FavoritesFolder()).delDomain( ( new UtilConvert()).enUnicode(this.getFav()) );
      loadFav();
    }catch(Exception exp){
      exp.printStackTrace();
    }
  }	
  private void removeAll(){
    try{			 
      (new FavoritesCollection()).deleteAll();
      this.lstAgent.setItems( new String[]{});
      loadFav();
    }catch(Exception exp){
      exp.printStackTrace();
    }
  }
  private void selectFav(){
    try{		
      UtilConvert convert = new UtilConvert();
      String [] agent = (new FavoritesCollection()).getAllAgent( convert.enUnicode(this.getFav()));	     
      for( int i=0; i<agent.length; i++)
        agent[i] = convert.deUnicode( agent[i]);				
      this.lstAgent.setItems( agent);
    }catch( Exception exp){
      exp.printStackTrace();
    }
  }
  public void loadFav(){
    try{			
      String [] domain = (new FavoritesCollection()).getFav();
      UtilConvert convert = new UtilConvert();
      for( int i=0; i<domain.length; i++)
        domain[i] = convert.deUnicode( domain[i]);
      this.lstFav.setItems( domain);
    }catch( Exception exp){
      exp.printStackTrace();
    }
  }
  //add
  public void fireAddEvent( FavoritesAddEvent e) {
    synchronized(listeners)  {
      for(int i = 0; i < listeners.size(); i++){
        try{
          FavoritesAddListener listener = listeners.get(i);
          listener.add(e);
          break;
        }catch(Exception exp){               	  
        }       
      }
    }
  }
  public void addAddListener(  FavoritesAddListener listener) {    	
    synchronized(listeners) {
      listeners.remove(listener);
      listeners.add(listener);
    }
  }
  public void removeAddListener(  FavoritesAddListener listener) {
    synchronized(listeners) {
      listeners.remove(listener);
    }
  }
  public void add (FavoritesAddEvent ev) {	    		 
    fireAddEvent(ev);	   
  }	
  public void createdAddAction()  {
    FavoritesAddEvent e = new FavoritesAddEvent( this.getFav());
    fireAddEvent(e);
    this.shell.dispose();
  }	
  //save
  public void fireSaveEvent( FavoritesSaveEvent e) {
    synchronized(listeners2)  {
      for(int i = 0; i < listeners2.size(); i++){
        try{
          FavoritesSaveListener listener = listeners2.get(i);
          listener.save(e);
          break;
        }catch(Exception exp){               	  
        }       
      }
    }
  }
  public void addSaveListener(  FavoritesSaveListener listener) {    	
    synchronized(listeners2) {
      listeners2.remove(listener);
      listeners2.add(listener);
    }
  }
  public void removeSaveListener(  FavoritesSaveListener listener) {
    synchronized(listeners2) {
      listeners2.remove(listener);
    }
  }
  public void save (FavoritesSaveEvent ev) {	    		 
    fireSaveEvent(ev);	   
  }	
  public void createdSaveAction()  {
    FavoritesSaveEvent e = new FavoritesSaveEvent();
    fireSaveEvent(e);
    this.shell.dispose();
  }	
  
}
