package org.exoplatform.filters.jcr;

import java.io.IOException;
import java.util.*;

import javax.jcr.*;
import javax.naming.*;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.exoplatform.container.monitor.jvm.*;

import org.exoplatform.services.jcr.core.*;
import org.exoplatform.services.jcr.config.*;


public class JcrBrowserFilter implements Filter {

  public void init(FilterConfig filterConfig) {
  }

  public void doFilter(ServletRequest servletRequest,
      ServletResponse servletResponse, FilterChain filterChain)
      throws IOException, ServletException {

    HttpServletRequest httpRequest = (HttpServletRequest) servletRequest;
    HttpSession session =  httpRequest.getSession();
    HttpServletResponse httpResponse = (HttpServletResponse) servletResponse;
    String p = httpRequest.getServletPath();

System.out.println("JcrBrowserFilter: path: " + p);

    if (p.equals("/ws.jsp")) {
      String s = httpRequest.getParameter("rep");
      if (s != null) {
        try {
          InitialContext ctx = new InitialContext();
          J2EEServerInfo eesi = new J2EEServerInfo();
          httpRequest.getSession().setAttribute("repDisplayName", s);
          if (eesi.getServerName().equals("tomcat") && (s.indexOf("/") < 0))
            s = "java:comp/env/jcr/" + s;
          httpRequest.getSession().setAttribute("repName", s);
          ManageableRepository repository = (ManageableRepository) ctx.lookup(s);
          httpRequest.getSession().setAttribute("rep", repository);

          httpRequest.getSession().removeAttribute("wsNames");
          httpRequest.getSession().removeAttribute("wsData");

          // building workspace names list
          
          String[] wss = repository.getWorkspaceNames();
          if (wss.length > 0) {
            ArrayList wsNames = new ArrayList();
            for (int i = 0; i < wss.length; i++) {
              wsNames.add(wss[i]);
            }
            httpRequest.getSession().setAttribute("wsNames", wsNames);
          }
        
          // building configuration tree

          RepositoryEntry cfg = repository.getConfiguration();
          List wsl = cfg.getWorkspaceEntries();
          if (wsl.size() > 0) {
            ArrayList wsData = new ArrayList();

            String[] wsens = new String[3];
            wsens[0] = "container";
            wsens[1] = "query handler";
            wsens[2] = "cache";
            for (int i = 0; i < wsl.size(); i++) {
              ArrayList wsEntries = new ArrayList();

              WorkspaceEntry wse = (WorkspaceEntry) wsl.get(i);
              MappedParametrizedObjectEntry[] wses = new MappedParametrizedObjectEntry[3];
              wses[0] = wse.getContainer();
              wses[1] = wse.getQueryHandler();
              wses[2] = wse.getCache();

              for (int j = 0; j < 3; j++) {
                if (wses[j] != null) {
                  ArrayList params = new ArrayList();
                  List prms = wses[j].getParameters();

                  String wsen = wsens[j];
                  if (wses[j].getType() != null)
                    wsen = wsen + " {" + wses[j].getType() + "}";
                  params.add(wsen);

                  if (prms.size() > 0) {
                    ArrayList prmNames = new ArrayList();
                    ArrayList prmValues = new ArrayList();
                    for (int k = 0; k < prms.size(); k++) {
                      SimpleParameterEntry pe = (SimpleParameterEntry) prms.get(k);
                      prmNames.add(pe.getName());
                      prmValues.add(pe.getValue());
                    }
                    params.add(prmNames);
                    params.add(prmValues);
                  }

                  wsEntries.add(params);
                }
              }

              wsData.add(wsEntries);
            }
            httpRequest.getSession().setAttribute("wsData", wsData);
          }
              
        } catch (Exception e) {
          httpResponse.sendRedirect("notfound.jsp");
        }
      }
      else
        httpResponse.sendRedirect("notfound.jsp");
    }
    else
    if (p.equals("/view.jsp")) {
      Session ses = null;
      Node node = null;
      
      String s = httpRequest.getParameter("rep");
      if (s != null) {
        try {
          InitialContext ctx = new InitialContext();
          Repository repository = (Repository) ctx.lookup(s);
          
          Credentials credentials = new SimpleCredentials("admin", "admin".toCharArray());  
          s = httpRequest.getParameter("ws");
          if (s == null)
            s = "";
          ses = repository.login(credentials, s);
          httpRequest.getSession().setAttribute("ses", ses);
        } catch (Exception e) {
          httpRequest.getSession().setAttribute("msg", e.toString());
          httpResponse.sendRedirect("reperr.jsp");
        }
      } else {
        ses = (Session) httpRequest.getSession().getAttribute("ses");
      }
      if (ses != null) {
        s = httpRequest.getParameter("node");
        try {
          if (s == null) {
            node = ses.getRootNode();
          } else
          if (s.equals("/")) {
            node = ses.getRootNode();
          } else {
            node = ses.getRootNode().getNode(s.substring(1));
          }

          httpRequest.getSession().removeAttribute("parentPath");
          httpRequest.getSession().removeAttribute("prNames");
          httpRequest.getSession().removeAttribute("prValues");
          httpRequest.getSession().removeAttribute("chNames");
          httpRequest.getSession().removeAttribute("chPaths");

          boolean isRoot = node == ses.getRootNode();
          httpRequest.getSession().setAttribute("wsName", ses.getWorkspace().getName());
          httpRequest.getSession().setAttribute("isRoot", new Boolean(isRoot));
          httpRequest.getSession().setAttribute("nodePath", node.getPath());
          if (!isRoot)
            httpRequest.getSession().setAttribute("parentPath", node.getParent().getPath());

          // building properties list

          if (node.hasProperties()) {
            ArrayList prNames = new ArrayList();
            ArrayList prValues = new ArrayList();

            PropertyIterator pi = node.getProperties();
            while (pi.hasNext()) { 
              Property tp = pi.nextProperty();
              prNames.add(tp.getName());
              String pv = "";
              try {
                pv = tp.getString();
              } catch (Exception e) {
                try {
                  Value[] pvs = tp.getValues();
                  for(int i = 0; i < pvs.length; i++) {
                    if (i > 0)
                      pv = pv + ", ";
                    pv = pv + pvs[i].getString();
                  }
                } catch (Exception e1) { pv = "&lt; non-viewable &gt;"; }
              }
              prValues.add(pv);
            }

            httpRequest.getSession().setAttribute("prNames", prNames);
            httpRequest.getSession().setAttribute("prValues", prValues);
          }

          // building children list

          if (node.hasNodes()) { 
            ArrayList chNames = new ArrayList();
            ArrayList chPaths = new ArrayList();

            NodeIterator ni = node.getNodes();
            while (ni.hasNext()) { 
              Node tn = ni.nextNode();
              chNames.add(tn.getName());
              chPaths.add(tn.getPath());
            }

            httpRequest.getSession().setAttribute("chNames", chNames);
            httpRequest.getSession().setAttribute("chPaths", chPaths);
          }
          
        } catch (Exception e) {
          httpRequest.getSession().setAttribute("msg", e.toString());
          httpResponse.sendRedirect("reperr.jsp");
        }
      } else {
        httpRequest.getSession().setAttribute("msg", "Session expired");
        httpResponse.sendRedirect("reperr.jsp");
      }
    }

    filterChain.doFilter(servletRequest, servletResponse);
  }

  public void destroy() {
  }

}
