/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl;

import java.io.IOException;
import java.io.InputStream;

import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.exoplatform.commons.utils.ExceptionUtil;
import org.exoplatform.commons.utils.IOUtil;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;
/**
 * Created by The eXo Platform SARL        .
 * Sun, Dec 28, 2003 @ 14:57 
 * @author: Tuan Nguyen
 * @version: $Id: JCRContentServlet.java 3510 2005-12-29 17:32:57Z geaz $
 * @email: tuan08@yahoo.com
 */
public class JCRContentServlet extends HttpServlet {
  
  public void init(ServletConfig config) throws ServletException {
    
  }
  
  public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    ServletOutputStream os = response.getOutputStream() ;
    try {
      String contextPath = request.getContextPath();
      String servletPath = request.getPathInfo() ;
      String[] info = parse(servletPath) ;
      if(info.length != 2) {
        response.setContentType("text/plain");
        os.println("Cannot parse the path: " + servletPath ) ;
      } else {
        RepositoryService service = getRepositoryService(contextPath.substring(1)) ;
        ManageableRepository repository = service.getRepository();         
        Session session = repository.getSystemSession(info[0]);
        Node node = (Node)session.getItem(info[1]) ;
        NodeType type = node.getPrimaryNodeType() ;
        if("nt:resource".equals(type.getName())) {
          if(node.hasProperty("jcr:data")) {
            String mimeType = "application/octet-stream" ;
            if(node.hasProperty("jcr:mimeType")) {
              mimeType = node.getProperty("jcr:mimeType").getString() ;
            }
            response.setContentType(mimeType);
            InputStream is = node.getProperty("jcr:data").getStream() ;
            byte[] buf = IOUtil.getStreamContentAsBytes(is);
            os.write(buf) ;
          } else {
            response.setContentType("text/plain");
            os.println("There is no jcr:content property for node : " + info[1]) ;
          }
        } else {
          response.setContentType("text/plain");
          os.println("There is no content for node : " + info[1]) ;
        }
      }
    } catch (Throwable ex) {
      String s = ExceptionUtil.getStackTrace(ex, 20) ;
      os.println(s) ;
    } finally {
      PortalContainer.setInstance(null) ;
    }
  }
  
  /**
   * The method split the URL in 3 parts:
   *   - the "jcr" token for identifying the servlet
   *   - the second one being the workspace name
   *   - the third one being the item path
   */
  protected String[]  parse(String path) {
    path = path.substring(1) ;
    String[] result = path.split("/", 2) ;
    result[1] = "/" + result[1] ;
    return result ;
  }
  
  protected RepositoryService getRepositoryService(String portal) throws Exception {
    PortalContainer pcontainer = RootContainer.getInstance().getPortalContainer(portal) ;
    if(pcontainer == null){
      throw new Exception("Cannot find the portal container:  " + portal) ;
    }
    PortalContainer.setInstance(pcontainer) ;    
    RepositoryService service = 
      (RepositoryService) pcontainer.getComponentInstanceOfType(RepositoryService.class) ;    
    return service ;
  }
}