/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl;

import java.util.List;

import javax.jcr.RepositoryException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.jmx.MX4JComponentAdapterFactory;
import org.exoplatform.services.jcr.access.AccessControlPolicy;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.RepositoryEntry;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NamespaceRegistryImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.UUIDGenerator;
import org.exoplatform.services.jcr.impl.core.access.AccessManager;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: RepositoryContainer.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class RepositoryContainer extends ExoContainer {

  private String name;
  
  private MBeanServer mbeanServer;

  public RepositoryContainer(ExoContainer parent, RepositoryEntry config)
      throws RepositoryException, RepositoryConfigurationException {

    //super(parent);
    super(new MX4JComponentAdapterFactory(), parent);

    System.out.println("set>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> " +
		" "+config.getAccessControl());

    
    // Defaults:
    if(config.getAccessControl() == null)
      config.setAccessControl(AccessControlPolicy.OPTIONAL);

    registerComponentInstance(config);

    this.name = config.getName();
    this.mbeanServer = MBeanServerFactory.createMBeanServer("jcrrep" + name + "mx");

    registerComponentImplementation(UUIDGenerator.class);
    registerComponentImplementation(NamespaceRegistryImpl.class);
    registerComponentImplementation(LocationFactory.class);
    registerComponentImplementation(ValueFactoryImpl.class);
    registerComponentImplementation(NodeTypeManagerImpl.class);
    registerComponentImplementation(AccessManager.class);
    
    //System.out.println("REPCONTAINER "+this+" "+parent+" access control "+config.getAccessControl());


    // Workspaces
    List wsEntries = config.getWorkspaceEntries();
    for (int i = 0; i < wsEntries.size(); i++) {
      WorkspaceEntry wsEntry = (WorkspaceEntry) wsEntries.get(i);

      WorkspaceContainer workspaceContainer = new WorkspaceContainer(this,
          wsEntry);

      registerComponentInstance(wsEntry.getName(), workspaceContainer);

    }

    // Repository
    RepositoryImpl repository = new RepositoryImpl(this);
    registerComponentInstance(repository);

    // SystemSession
    init(config, repository);
  }

  public MBeanServer getMBeanServer() {
    return this.mbeanServer;
  }

  public void init(RepositoryEntry config, RepositoryImpl repository)
      throws RepositoryException, RepositoryConfigurationException {

    //   	 After repository instantiation
    List wsEntries = config.getWorkspaceEntries();
    for (int i = 0; i < wsEntries.size(); i++) {
      WorkspaceEntry wsEntry = (WorkspaceEntry) wsEntries.get(i);
      WorkspaceContainer workspaceContainer = (WorkspaceContainer) getComponentInstance(wsEntry
          .getName());

      workspaceContainer.init(wsEntry, repository, wsEntry.getName().equals(
          config.getSystemWorkspaceName()));

    }
    
    // Start container
//    try {
//      start();
//    } catch (IllegalStateException e) {
      //log.info("Presumably repository container "+name+" is already started");
//    }

    
  }

  /**
   * @return Returns the name.
   */
  public String getName() {
    return name;
  }
}