/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.version;

import java.util.Comparator;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.TreeSet;

import javax.jcr.ItemNotFoundException;
import javax.jcr.MergeException;
import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.util.TraversingItemVisitor;

import org.exoplatform.services.jcr.impl.core.JCRPath;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: MergeVisitor.java 3547 2006-01-03 08:42:56Z geaz $
 */

public class MergeVisitor extends TraversingItemVisitor {
  
//  protected final Log log = LogFactory.getLog(this.getClass());

  private static int NONE = -1;

  private static int LEAVE = 0;

  private static int FAIL = 1;

  private static int UPDATE = 2;

  private boolean bestEffort;

  private Map failed;

  private Set versionableStates;
  
  private SessionImpl destSession;
  
  private NodeImpl destNode;
  
  protected Stack parents;

  private NodeImpl parent;
  
  private NodeImpl mergedNode;

  public MergeVisitor(NodeImpl parent, Map failed, boolean bestEffort)
      throws RepositoryException {
    //super(context, true);
    this.destSession = (SessionImpl)parent.getSession();
    this.parent = parent;

    this.failed = failed;
    this.bestEffort = bestEffort;
    this.versionableStates = new TreeSet(new VersionableStateComparator());
    this.parents = new Stack();

  }
  
  /**
   * @return Returns the mergedNode.
   */
  public NodeImpl getMergedNode() {
    return mergedNode;
  }

  protected void entering(Node node, int level) throws RepositoryException {

    System.out.println("MergeVisitor entering node "+node.getPath()+" "+node.getSession().getWorkspace().getName());

    try {
      destNode = ((NodeImpl)node).getCorrespondingNode((SessionImpl)destSession);
    } catch (ItemNotFoundException e) {
      return;
    }
    
    // TODO (one more pass) 
    // If N (destNode) does not have a corresponding node then the merge
    // result for N is leave.

    if(destNode.isNodeType("mix:versionable")) {
      VersionImpl srcBase = (VersionImpl)node.getBaseVersion();
      VersionImpl destBase = (VersionImpl)destNode.getBaseVersion();
      //System.out.println("MERGE compare >>> "+destNode.getPath()+" "+bestEffort+
      //    " src ver:"+srcBase.getName()+" dest Ver:"+destBase.getName());

      if (destNode.isCheckedOut()) { // CHECKED-OUT
        if (destBase.getName().equals(srcBase.getName())
            || destBase.isSuccessorOrSameOf(srcBase)) {
          addNode(destNode, level);
          versionableStates.add(new VersionableState(destNode.getLocation(),
              LEAVE));
        } else {
          addNode(destNode, level);
          if (bestEffort) {
            failed.put(destNode.getUUID(), srcBase.getUUID());
          } else
            throw new MergeException("Merging of node " + node.getPath()
                + " failed");
          versionableStates.add(new VersionableState(destNode.getLocation(),
              FAIL));
        }
      } else { // CHECKED-IN
        if (srcBase.isSuccessorOrSameOf(destBase)) {
          addNode((NodeImpl) node, level);
          //super.entering(node, level);
          versionableStates.add(new VersionableState(destNode.getLocation(),
              UPDATE));
        } else if (destBase.getName().equals(srcBase.getName())
            || destBase.isSuccessorOrSameOf(srcBase)) {
          addNode(destNode, level);
          versionableStates.add(new VersionableState(destNode.getLocation(),
              LEAVE));
        } else {
          addNode(destNode, level);
          if (bestEffort) {
            failed.put(destNode.getUUID(), srcBase.getUUID());
          } else
            throw new MergeException("Merging of node " + node.getPath()
                + " failed");
          versionableStates.add(new VersionableState(destNode.getLocation(),
              FAIL));
        }
      }
    } else { // node is not versionable
      int nearestState = getNearestVersionableState(destNode.getLocation());
      if(nearestState == NONE || nearestState == UPDATE)
        addNode((NodeImpl)node, level);
        //super.entering(node, level);
      else
        addNode(destNode, level);
    }
  }

  protected void entering(Property property, int level)
      throws RepositoryException {
    
    
    PropertyImpl propImpl = (PropertyImpl) property;

    if (getNearestVersionableState(propImpl.getLocation()) == UPDATE ||
        getNearestVersionableState(propImpl.getLocation()) == NONE) {
      System.out.println("MergeVisitor update property "+propImpl.dump());
      addProperty(propImpl);
    } else {
      
      addProperty((PropertyImpl)destNode.getProperty(property.getName()));
//      System.out.println("MergeVisitor repeat property "+
//          ((PropertyImpl)destNode.getProperty(property.getName())).dump());
    }
      
  }

  protected void leaving(Property property, int level)
      throws RepositoryException {
  }

  protected void leaving(Node node, int level) throws RepositoryException {
    parents.pop();
  }
  
  private void addNode(NodeImpl node, int level) throws RepositoryException {
    NodeImpl cParent;
    if(level == 0) {
      this.mergedNode = node;
      cParent = parent;
    } else
      cParent = curParent();
      
    //NodeImpl newNode = cParent.createChildNode(node.getName(),
    //    node.getPrimaryNodeType().getName(), node.getMixinTypeNames(),
    //    node.getInternalUUID(), true, false);
    NodeImpl newNode = cParent.createChildNodeInmemory(
        node.storedNodeData().getQPath().getName(), 
        node.storedNodeData().getPrimaryTypeName(),
        node.storedNodeData().getMixinTypeNames(),
        node.storedNodeData().getUUID(),
        node.storedNodeData().getQPath().getIndex(),
        false, false, false, false);   
    System.out.println("MergeVisitor new node "+newNode.getPath()+" "+newNode.getSession().getWorkspace().getName());

    parents.push(newNode);
  }
  
  private void addProperty(PropertyImpl prop) throws RepositoryException {
    curParent().createChildProperty(prop.getName(), 
        prop.getValueArray(), prop.getType());
    System.out.println("MergeVisitor CREATE property "+prop.dump() );

  } 

  private int getNearestVersionableState(JCRPath path) {
    for(Iterator it = versionableStates.iterator(); it.hasNext();) {
      VersionableState state = (VersionableState)it.next();
      if(path.isDescendantOf(state.path, false))
        return state.result;
    }
    return NONE;
  }
  
  protected NodeImpl curParent() {
    return (NodeImpl) parents.peek();
  }

  private class VersionableState {
    private int result;

    private JCRPath path;

    private VersionableState(JCRPath path, int result) {
      this.path = path;
      this.result = result;
    }
  }
  
  private class VersionableStateComparator implements Comparator {
    public int compare(Object o1, Object o2) {
      VersionableState nc1 = (VersionableState) o1;
      VersionableState nc2 = (VersionableState) o2;
      return nc1.path.getAsString(true).compareTo(nc2.path.getAsString(true));
    }
  }

}