/*
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.inmemory;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import javax.jcr.ItemExistsException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.core.JCRPath;
import org.exoplatform.services.jcr.impl.storage.WorkspaceDataContainerBase;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.log.LogUtil;
import org.exoplatform.services.log.impl.ExoLog;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: WorkspaceContainerImpl.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class WorkspaceContainerImpl extends WorkspaceDataContainerBase {
  private ExoLog log;

  private TreeMap items;

  private TreeMap uuids;

  //    private Set references;
  private String name;

  public WorkspaceContainerImpl(WorkspaceEntry wsEntry, LogService logService)
      throws RepositoryException {

    //log = LogUtil.getLog("org.exoplatform.services.jcr");
    this.log = (ExoLog) logService.getLog(this.getClass());

    this.name = wsEntry.getUniqueName();
    log.debug("ContainerImpl() name: " + name);
    items = WorkspaceContainerRegistry.getInstance()
        .getWorkspaceContainer(name);
    //        references = new HashSet();
    uuids = new TreeMap();
  }

  public void beginTransaction() {
  }

  public void close() {
  }

  public void commit() {
  }

  public void rollback() {
  }

  public String getName() {
    return name;
  }

  public String getInfo() {
    String str = "Info: Inmemory (for testing only) based container \n";
    str += "Name: " + name + "\n";
    return str;
  }

  public ItemData getItem(InternalQPath qPath) {
    log.debug("InmemoryContainer finding " + qPath.getAsString());
    Object o = items.get(qPath.getAsString());
    log.debug("InmemoryContainer FOUND " + qPath.getAsString() + " " + o);
    return (ItemData) o;
  }

  synchronized public NodeData add(NodeData item) throws RepositoryException {

    //    JCRPath loc = ((ItemImpl)item).getLocation();
    if (items.get(item.getQPath().getAsString()) != null)
      throw new ItemExistsException("WorkspaceContainerImpl.add(Item) item '"
          + item.getQPath().getAsString() + "' already exists!");

    //    NodeImpl node = (NodeImpl) item;
    //    InternalQName primaryTypeName = InternalQName.parse(node.getPrimaryNodeType().getName()); 
    //   	NodeData res =  
    //   		new TransientNodeData(node.getLocation().getInternalPath(), 
    //   				node.getLocation().getUUID(), -1, primaryTypeName);

    items.put(item.getQPath().getAsString(), item);
    log.debug("InmemoryContainer added node " + item.getQPath().getAsString());
    Iterator props = item.getChildProperties().iterator();
    while (props.hasNext()) {
      add((PropertyData) props.next());
    }
    Iterator nodes = item.getChildNodes().iterator();
    while (nodes.hasNext()) {
      add((NodeData) nodes.next());
    }

    //log.debug("InmemoryContainer added " + item + " to workspace container: " + name);//+ " "+((ItemData)items.get(loc)).getLocation().getInternalPath());
    return item;
  }

  synchronized public void delete(ItemData data) {
    /*    	
     ItemData item = getItem(loc);
     if(item instanceof NodeData) {
     NodeData node = (NodeData) item;
     Iterator props = node.getPropertyLocations().iterator();
     while(props.hasNext()) {
     items.remove((ItemLocation)props.next());

     }
     Iterator nodes = node.getChildNodeLocations().iterator();
     while(nodes.hasNext()) {
     String childId = null;
     try {
     String childPath = ((ItemLocation)nodes.next()).getInternalPath();
     childId = ((ItemLocation)nodes.next()).getUUID();
     delete(new ItemLocation(childPath, childId));
     } catch (PathNotFoundException e) {
     log.error("InmemoryContainer remove failed "+childId);
     }
     }
     }
     */
    items.remove(data.getQPath().getAsString());
    log.debug("InmemoryContainer removed " + data.getQPath().getAsString());
  }

  synchronized public PropertyData update(PropertyData item)
      throws RepositoryException {

    //JCRPath loc = ((ItemImpl)item).getLocation();
    //PropertyImpl prop = (PropertyImpl)item;

    //PropertyData res = new TransientPropertyData(loc.getInternalPath(), 
    //		loc.getUUID(), -1, prop.getType());

    items.put(item.getQPath().getAsString(), item);
    log.debug("InmemoryContainer updated " + item);

    return item;
  }

  public NodeData addNode(InternalQPath qPath, String uuid,
      InternalQName primaryType) throws ItemExistsException,
      RepositoryException {
    throw new RepositoryException("Does not implemented");
  }

  public NodeData update(NodeData node) throws ItemExistsException,
      RepositoryException {
    throw new RepositoryException("Does not implemented");
  }

  public PropertyData add(PropertyData prop) throws RepositoryException {
    items.put(prop.getQPath().getAsString(), prop);
    log.debug("InmemoryContainer added property "
        + prop.getQPath().getAsString());
    return prop;
    //throw new RepositoryException("Does not implemented");

  }

  public PropertyData updateProperty(JCRPath loc, List valueDatas)
      throws RepositoryException {
    throw new RepositoryException("Does not implemented");

  }

  synchronized public List getReferences(String uuid)
      throws RepositoryException {
    ArrayList refs = new ArrayList();
    Iterator it = items.values().iterator();
    while (it.hasNext()) {
      ItemData itemData = (ItemData) it.next();
      ValueData uuidVal = (ValueData) ((PropertyData) itemData).getValues()
          .get(0);
      if ((itemData instanceof PropertyData)
          && ((PropertyData) itemData).getType() == PropertyType.REFERENCE
          && new String(uuidVal.getAsByteArray()).equals(uuid)) {
        //                 ((PropertyData)itemData).getPropertyValues()[0].getString().equals(uuid)) {

        //                 refs.add(new ItemLocation(itemData.getLocation().getInternalPath(),
        //                 		itemData.getLocation().getUUID()).makeParentLocation());
      }
    }
    return refs;
  }

  public NodeData getNodeByUUID(String uuid) {
    Iterator itemsIterator = items.values().iterator();
    while (itemsIterator.hasNext()) {
      ItemData data = (ItemData) itemsIterator.next();
      if (data.getUUID().equals(uuid) && (data instanceof NodeData))
        return (NodeData) data;
    }
    return null;
  }

  public ItemData getItemByUUID(String uuid) {
    Iterator itemsIterator = items.values().iterator();
    while (itemsIterator.hasNext()) {
      ItemData data = (ItemData) itemsIterator.next();
      if (data.getUUID().equals(uuid))
        return data;
    }
    return null;
  }
  /*
   public String toString() {
   String str = "Inmemory WorkspaceContainer Data: \n";
   Iterator i = items.keySet().iterator();
   while(i.hasNext()) {
   //            String s = (String)i.next();
   JCRPath d = (JCRPath)i.next();
   str+=d.getInternalPath()+"\n"; //s+":"+items.get(s)+"\n";
   }
   return str;
   }
   */
}