/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.rdb.container;

import java.util.List;

import javax.jcr.InvalidItemStateException;
import javax.jcr.ItemExistsException;
import javax.jcr.RepositoryException;

import org.apache.commons.logging.Log;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.component.ComponentLifecycle;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.datamodel.IllegalNameException;
import org.exoplatform.services.jcr.datamodel.IllegalPathException;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.storage.WorkspaceDataContainerBase;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.ItemRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.NodeRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.PropertyRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.ValueRecord;
import org.exoplatform.services.log.LogService;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: RDBWorkspaceContainerImpl.java 3547 2006-01-03 08:42:56Z geaz $
 */
public abstract class RDBWorkspaceContainerImpl extends
    WorkspaceDataContainerBase implements ComponentLifecycle {


  protected Log log;

  //protected static final Log log = LogFactory.getLog("RDBWorkspaceContainer");

  protected String name;

  protected String srcName;

  protected SessionFactory sessionFactory;

  protected Session hibernateSession;

  protected Transaction tx = null;

  public RDBWorkspaceContainerImpl(WorkspaceEntry wsConfig,
  //HibernateConfigurationsService hibernateContainer,
      LogService logService) throws RepositoryException,
      RepositoryConfigurationException {
    log = logService.getLog("org.exoplatform.services.jcr");
    //log = logService.getLog(this.getClass());

    this.name = wsConfig.getName();
    srcName = (String) wsConfig.getContainer().getParameterValue("sourceName");
    log.info("RDBWorkspacecontainerImpl (cEntry)" + name
        + " hibernate container =" + srcName);

  }

  ////// ComponentLifecycle

  public void initComponent(ExoContainer container) throws Exception {
    HibernateService hibernateService = null;
    //System.out.println("Call initComponent " + srcName);

    if (srcName == null || srcName.length() == 0 || "default".equals(srcName)) {
      hibernateService = (HibernateService) container
          .getComponentInstanceOfType(HibernateService.class);
    } else {
      hibernateService = (HibernateService) container
          .getComponentInstance(srcName);
    }

    if (hibernateService == null)
      throw new RepositoryException("Cant obtain hibernate service for "
          + srcName);

    this.sessionFactory = hibernateService.getSessionFactory();

    this.hibernateSession = sessionFactory.openSession();

  }

  public void startComponent(ExoContainer arg0) throws Exception {
  }

  public void stopComponent(ExoContainer arg0) throws Exception {
  }

  public void destroyComponent(ExoContainer arg0) throws Exception {
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#beginTransaction()
   */
  public void beginTransaction() {
    this.hibernateSession = sessionFactory.openSession();
    tx = hibernateSession.beginTransaction();
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#close()
   */
  public void close() {
    hibernateSession.close();
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#commit()
   */
  public void commit() {
    if (tx != null && tx.isActive())
      tx.commit();
    hibernateSession.close();
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#rollback()
   */
  public void rollback() {
    if (tx != null && tx.isActive())
      tx.rollback();
    hibernateSession.close();
  }

  /**
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getName()
   */
  public String getName() {
    return name;
  }

  public String getInfo() {
    String str = "Info: Relational Database (Hibernate 3) based container ver1.0 \n";
    str += "\tName: " + name + "\n";
    str += "\tHibernate configuration: \n";
    str += "\tDatasource name (hibernate.datasource.name): "
    //				+ hibernateConfiguration
        //						.getHibernateProperty("hibernate.datasource.name")
        + "\n";
    return str;
  }

  public ItemData getItem(InternalQPath qPath) {
    log.debug("RDB Container looks up item : " + qPath.getAsString());
    try {

      checkTransaction();

      ItemRecord data = findItemByPath(qPath.getAsString(), hibernateSession);

      if (data != null && (data instanceof NodeData))
        initNode((NodeRecord) data);

      return data;

    } catch (Exception e) {
      e.printStackTrace();
      throw new RuntimeException("RDB Container.getItem() failed: <"
          + qPath.getAsString() + "> " + e.getMessage());
    }
  }

  public ItemData getItemByUUID(String uuid) {

    try {
      checkTransaction();

      ItemRecord item = findItemByUUID(uuid, hibernateSession);

      if (item != null && (item instanceof NodeData))
        initNode((NodeRecord) item);
      return item;

    } catch (Exception e) {
      e.printStackTrace();
      throw new RuntimeException("RDB Container.getItem() failed: <" + uuid
          + "> " + e.getMessage());
    }

  }

  synchronized public void delete(ItemData data) throws RepositoryException {

    checkTransaction();

    ItemData storedData = findItemByUUID(data.getUUID(), hibernateSession);
    if (storedData != null)
      hibernateSession.delete(storedData);
    log.debug("RDB Container removed " + data.getQPath().getAsString());
  }

  synchronized public NodeData add(NodeData data) throws RepositoryException,
      ItemExistsException, InvalidItemStateException {

    checkTransaction();

    return traverseNode(data);
  }

  /**
   * Traverses all node's descendants and adds to the container
   * 
   * @param data
   * @return
   * @throws RepositoryException
   */
  private NodeRecord traverseNode(NodeData data) throws RepositoryException,
      InvalidItemStateException {

    NodeRecord nodeRecord = addNodeRecord(data, hibernateSession);

    for (int i = 0; i < data.getChildProperties().size(); i++) {
      PropertyData prop = (PropertyData) data.getChildProperties().get(i);
      PropertyRecord propRecord = addPropertyRecord(prop, nodeRecord,
          hibernateSession);
    }

    for (int i = 0; i < data.getChildNodes().size(); i++) {
      traverseNode((NodeData) data.getChildNodes().get(i));
    }

    return nodeRecord;

  }

  synchronized public PropertyData add(PropertyData prop)
      throws RepositoryException, ItemExistsException,
      InvalidItemStateException {

    checkTransaction();

    NodeRecord parent = findParentNodeRecord(prop);

    PropertyRecord propRecord = addPropertyRecord(prop, parent,
        hibernateSession);

    log.debug("RDB Container added property " + prop);

    return propRecord;
  }

  synchronized public PropertyData update(PropertyData prop)
      throws RepositoryException {

    checkTransaction();

    log.debug("RDB Container updating " + prop);

    PropertyRecord propertyRecord = findPropertyByUUID(prop.getUUID(),
        hibernateSession);

    List oldValues = propertyRecord.getValues();
    for (int i = 0; i < oldValues.size(); i++)
      hibernateSession.delete(oldValues.get(i));

    propertyRecord.setType(prop.getType());
    propertyRecord.setValues(prop.getValues());

    hibernateSession.persist(propertyRecord);

    return propertyRecord;
  }

  synchronized public NodeData update(NodeData node)
      throws RepositoryException, ItemExistsException {

    checkTransaction();

    NodeRecord nodeRecord = findNodeByUUID(node.getUUID(), hibernateSession);

    log.debug("RDB Container updating " + node.getQPath().getAsString() + " "
        + node.getOrderNumber() + " " + node.getUUID());

    nodeRecord.setOrderNumber(node.getOrderNumber());
    nodeRecord.setPath(node.getQPath().getAsString());

    hibernateSession.persist(nodeRecord);

    return nodeRecord;

  }

  synchronized public List getReferences(String uuid)
      throws RepositoryException {

    checkTransaction();
    return findReferences(uuid, hibernateSession);

  }

  protected NodeRecord findParentNodeRecord(ItemData data)
      throws RepositoryException, InvalidItemStateException {
    NodeRecord parentNodeRecord = null;
    if (!data.getQPath().getAsString().equals(Constants.ROOT_URI)) {
      String parentPath = data.getQPath().makeParentPath().getAsString();
      log.debug("Finding parent record " + parentPath);

      parentNodeRecord = findNodeByPath(parentPath, hibernateSession);

      if (parentNodeRecord == null)
        throw new InvalidItemStateException("Parent not found for "
            + data.getQPath().getAsString()+" container: "+name
            + " probably was removed by other client");
    }
    return parentNodeRecord;
  }

  /**
   * Inits node's types(primary & mixin)
   * 
   * @param nodeRecord
   * @param hibernateSession
   * @throws HibernateException
   * @throws RepositoryException
   */
  public void initNode(NodeRecord data) throws HibernateException,
      RepositoryException, IllegalPathException, IllegalNameException {

    // PRIMARY
    InternalQPath propPath = InternalQPath.makeChildPath(data.getQPath(),
        Constants.PRIMARY_TYPE_URI);
    PropertyRecord ptProp = findPropertyByPath(propPath.getAsString(),
        hibernateSession);

    if (ptProp == null)
      throw new RepositoryException("FATAL ERROR Not found primary type "
          + propPath.getAsString());

    ValueRecord ptValue = (ValueRecord) ptProp.getValues().get(0);
    InternalQName ptName = InternalQName.parse(new String(ptValue
        .getAsByteArray()));
    data.setPrimaryType(ptName);

    // MIXIN
    InternalQPath mtPath = InternalQPath.makeChildPath(data.getQPath(),
        Constants.MIXIN_TYPE_URI);
    PropertyRecord mtProp = findPropertyByPath(mtPath.getAsString(),
        hibernateSession);

    InternalQName[] mixinNames;
    if (mtProp == null) {
      mixinNames = new InternalQName[0];
    } else {

      List ptValues = mtProp.getValues();
      mixinNames = new InternalQName[ptValues.size()];
      for (int i = 0; i < ptValues.size(); i++) {
        ValueRecord val = (ValueRecord) ptValues.get(i);
        mixinNames[i] = InternalQName.parse(new String(val.getAsByteArray()));
      }
    }
    data.setMixinTypes(mixinNames);

    // ACL
    if (isAccessControllable(mixinNames)) {
      InternalQPath ownerPath = InternalQPath.makeChildPath(data.getQPath(),
          Constants.NODE_OWNER_URI);
      PropertyRecord ownerRecord = findPropertyByPath(ownerPath.getAsString(),
          hibernateSession);
      InternalQPath permPath = InternalQPath.makeChildPath(data.getQPath(),
          Constants.NODE_PERM_URI);
      PropertyRecord permissionsRecord = findPropertyByPath(permPath
          .getAsString(), hibernateSession);

      //System.out.println("Access Controllable: " + data.getQPath().getAsString()+ 
      //    ownerRecord + " " + permissionsRecord);

      data.setChildrenACL(new AccessControlList(ownerRecord, permissionsRecord));
    }

  }

  private boolean isAccessControllable(InternalQName[] mixinNames) {
    for (int i = 0; i < mixinNames.length; i++) {
      if (mixinNames[i].getAsString().equals(Constants.ACCESS_TYPE_URI)) {
        return true;
      }
    }
    return false;
  }

  private void checkTransaction() throws RepositoryException {
    if (!tx.isActive())
      throw new RepositoryException(
          "Transaction is not opened. Call beginTransaction");

  }

  public abstract boolean isMultiplicityAllowed();

  protected abstract NodeRecord addNodeRecord(NodeData data,
      Session hibernateSession) throws RepositoryException,
      ItemExistsException, InvalidItemStateException;

  protected abstract PropertyRecord addPropertyRecord(PropertyData prop,
      NodeRecord parent, Session hibernateSession) throws RepositoryException,
      ItemExistsException;

  protected abstract ItemRecord findItemByPath(String path,
      Session hibernateSession);

  protected abstract ItemRecord findItemByUUID(String uuid,
      Session hibernateSession);

  protected abstract NodeRecord findNodeByPath(String path,
      Session hibernateSession);

  protected abstract NodeRecord findNodeByUUID(String uuid,
      Session hibernateSession);

  protected abstract PropertyRecord findPropertyByPath(String path,
      Session hibernateSession);

  protected abstract PropertyRecord findPropertyByUUID(String uuid,
      Session hibernateSession);

  protected abstract List findReferences(String uuid, Session hibernateSession);

}