/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.xml;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.xml.parsers.SAXParserFactory;

import org.apache.commons.codec.binary.Base64;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.core.JCRPath;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: SysNodeImporter.java 3697 2006-01-10 17:18:09Z peterit $
 */

class SysNodeImporter implements ContentHandler {

  private Stack     tree;

  private String    curPropName;

  private String    curPropType;

  private ArrayList curPropValues;

  // private JCRPath curLocation;
  private List      nodeInfos;

  private Session   session;

  private NodeImpl parent;
  private NodeImpl rootNode;
  
  private Exception causeException = null;

  SysNodeImporter(NodeImpl parent) throws RepositoryException {
    //super(parent);
    this.parent = parent;
    this.tree = new Stack();
    this.nodeInfos = new ArrayList();
    this.session = parent.getSession();
    this.curPropValues = new ArrayList();
  }

  private NodeImpl _parse(InputStream stream) throws IOException, SAXException, RepositoryException {

    // parse XML
    XMLReader reader;
    try {
      SAXParserFactory factory = SAXParserFactory.newInstance();
      factory.setNamespaceAware(true);
      reader = factory.newSAXParser().getXMLReader();
    } catch (Exception e) {
      throw new SAXException(e.getMessage());
    }
    reader.setContentHandler(this);
    reader.parse(new InputSource(stream));

    // build node
    return buildNode();
  }

  NodeImpl getNode() throws RepositoryException {
    
    return rootNode;
  }
  
  private NodeImpl buildNode() throws RepositoryException {

    // build node from parsed infos
    NodeImpl node = null;
    //Stack buildTree = new Stack();

    for (int i = 0; i < nodeInfos.size(); i++) {
      NodeInfo info = (NodeInfo) nodeInfos.get(i);
      
      // System.out.println("Set items "+nodeInfos.size()+" "+parent.getPath()+" "+info.getRelPath());

//      String mix = "";
//      String[] mixStr = parent.getMixinTypeNames();
//      for (int j=0; j<mixStr.length; j++) {
//        mix += ", " + mixStr[j];
//      }
//      System.err.println("=== parent node: " + parent.getPath() + 
//          ", PrimaryTypeName: " + parent.getPrimaryNodeType().getName() +
//          ", MixinTypeNames: " + mix + ", mix:versionable: " + parent.isNodeType("mix:versionable"));
      
//      mix = "";
//      mixStr = info.getMixinTypeNames();
//      for (int j=0; j<mixStr.length; j++) {
//        mix += ", " + mixStr[j];
//      }
//      System.err.println("=== importChildNode: " + info.getRelPath() + 
//          ", PrimaryTypeName: " + info.getPrimaryTypeName() +
//          ", MixinTypeNames: " + mix);
                
      SessionImpl sessionImpl = (SessionImpl) session;
      LocationFactory loFactory = sessionImpl.getLocationFactory();
      
      //JCRPath relPath = loFactory.parseRelPath(info.getRelPath());
      JCRPath relPath = loFactory.createJCRPath(parent.getLocation(), info.getRelPath());
      InternalQName name = relPath.getInternalPath().getName(); 
      InternalQName primaryTypeName = loFactory.parseJCRName(info.getPrimaryTypeName()).getInternalName();
      InternalQName[] mixinTypeNames = new InternalQName[info.getMixinTypeNames().length];
      for (int mi=0; mi<mixinTypeNames.length; mi++) {
        InternalQName mixinTypeName = loFactory.parseJCRName(info.getMixinTypeNames()[mi]).getInternalName();
        mixinTypeNames[mi] = mixinTypeName;
      }
           
      JCRPath parentPath = relPath.makeParentPath();
      NodeImpl parentNode = (NodeImpl) sessionImpl.getItem(parentPath.getAsString(false));    
      
//      mix = "";
//      mixStr = parentNode.getMixinTypeNames();
//      for (int j=0; j<mixStr.length; j++) {
//        mix += ", " + mixStr[j];
//      }
//      System.err.println("=== parent node: " + parent.getPath() + 
//          ", PrimaryTypeName: " + parent.getPrimaryNodeType().getName() +
//          ", MixinTypeNames: " + mix + ", mix:versionable: " + parent.isNodeType("mix:versionable"));
      
      NodeImpl n = parentNode.createChildNodeInmemory(name, primaryTypeName, mixinTypeNames,
          null,   // uuid 
          -1,      // index, relPath.getIndex()
          true,  // registerInDataManager
          true,  // doExternalValidation
          false,  // addAutoCreated 
          true);  // cacheInSession
                     
//      mix = "";
//      mixStr = n.getMixinTypeNames();
//      for (int j=0; j<mixStr.length; j++) {
//        mix += ", " + mixStr[j];
//      }
//      System.err.println("===> created node: " + n.getPath() + 
//          ", PrimaryTypeName: " + n.getPrimaryNodeType().getName() +
//          ", MixinTypeNames: " + mix);
      
      List props = info.getProperties();
      for (int j = 0; j < props.size(); j++) {
        PropertyInfo prop = (PropertyInfo) props.get(j);
        
//        if (prop.getName().equals("jcr:isCheckedOut")) {
//          System.err.println("===> jcr:isCheckedOut");
//        }
        
        List valueList = prop.getValues();
        Value[] values = new Value[valueList.size()];
        for (int k = 0; k < valueList.size(); k++) {
          String val = new String((StringBuffer) valueList.get(k));
          if (prop.getType() == PropertyType.BINARY) {
            values[k] = session.getValueFactory().createValue(
                new ByteArrayInputStream(Base64.decodeBase64(val.getBytes())));
          } else {
            values[k] = session.getValueFactory().createValue(val, prop.getType());
          }
        }
                
//        String prVals = "";
//        Value[] v = values;
//        for (int vi=0; vi<v.length; vi++) {
//          prVals += ", " + v[vi].getString();
//        }
//        System.err.println("== app prop: " + prop.getName() + 
//            ", type: " + prop.getType() + 
//            ", values: " + prVals);
        
        PropertyImpl pr = n.createChildProperty(prop.getName(), values, prop.getType());
        
//        prVals = "";
//        try {
//          v = pr.getValues();
//        } catch(ValueFormatException e) {
//          v = new Value[1];
//          v[0] = pr.getValue();
//        }        
//        for (int vi=0; vi<v.length; vi++) {
//          prVals += ", " + v[vi].getString();
//        }
//        System.err.println("==> app prop: " + pr.getPath() + 
//            ", type: " + pr.getType() + ", " + pr +
//            ", values: " + prVals);
                
      }
      if (i == 0) {
        node = n;
      }
    }

    return node;
  }

  public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
      throws SAXException {
    if (qName.equals("sv:node")) {
      try {
        String name = atts.getValue("sv:name");
        String relPath;
        if (!tree.isEmpty())
          relPath = ((NodeInfo) tree.peek()).getRelPath() + "/" + name;
        else
          relPath = name;
        
//        for (int i=0; i<atts.getLength(); i++) {
//          System.err.println("===== sv:node ATTR: " + atts.getQName(i) + 
//              "=" + atts.getValue(i));
//        }
//        System.err.println("======= sv:node; sv:name: " + name + ", relPath: " + relPath);
        NodeInfo info = new NodeInfo(relPath);
        tree.push(info);
        nodeInfos.add(info);
      } catch (Exception e) {
        e.printStackTrace();
        throw new SAXException(e.getMessage());
      }
    } else if (qName.equals("sv:property")) {
      // TODO manage UUID
      try {
        curPropType = atts.getValue("sv:type");
        curPropName = atts.getValue("sv:name");
        curPropValues = new ArrayList();
        
//        for (int i=0; i<atts.getLength(); i++) {
//          System.err.println("========== sv:property ATTR: " + atts.getQName(i) + 
//              "=" + atts.getValue(i));
//        }
//        System.err.println("============ sv:property; sv:name: " + curPropName + 
//            ", sv:type: " + curPropType);
      } catch (Exception e) {
        e.printStackTrace();
        throw new SAXException(e.getMessage(), e);
      }
    } else if (qName.equals("sv:value")) {
//      System.err.println("============ sv:value");
      // nothing to do
    } else {
//      System.err.println("============ NOT ALLOWED " + qName);
      throw new SAXException("'" + qName
          + "' is not allowed. Only sv:node, sv:property and sv:value are allowed");
    }
  }

  public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
    if (qName.equals("sv:node")) {
      // nodeInfos.add(tree.pop());
      tree.pop();
    } else if (qName.equals("sv:property")) {
      ((NodeInfo) tree.peek()).addProperty(curPropName, curPropType, curPropValues);
    }
  }

  public void characters(char[] ch, int start, int length) throws SAXException {
    // property values
    StringBuffer curPropValue = new StringBuffer();
    curPropValue.append(ch, start, length);
    curPropValues.add(curPropValue);
  }

  public void setDocumentLocator(Locator locator) {
  }

  public void startDocument() throws SAXException {
    nodeInfos.clear();
  }

  public void endDocument() throws SAXException {
    // so, build the node parsed before
    // and put it in the session
    try {
      rootNode = buildNode();
    } catch(Exception e) {
      //e.printStackTrace();
      //causeException = e;
      throw new SAXException("Error create node from System View document. Exception: " + 
          e.getMessage(), e);
    }
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException {
  }

  public void endPrefixMapping(String prefix) throws SAXException {
  }

  public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
  }

  public void processingInstruction(String target, String data) throws SAXException {
  }

  public void skippedEntity(String name) throws SAXException {
  }

  private class NodeInfo {
    private String    relPath;

    private ArrayList properties = new ArrayList();

    public NodeInfo(String relPath) {
      this.relPath = relPath;
    }

    public void addProperty(String name, String typeName, ArrayList strValues) {
      int type = ExtendedPropertyType.valueFromName(typeName);
      PropertyInfo prop = new PropertyInfo(name, type, strValues);
      properties.add(prop);
    }

    public ArrayList getProperties() {
      return properties;
    }

    public String getRelPath() {
      return relPath;
    }

    public String getPrimaryTypeName() throws RepositoryException {
      for (int i = 0; i < properties.size(); i++) {
        PropertyInfo prop = (PropertyInfo) properties.get(i);
        if (prop.getName().equals("jcr:primaryType"))
          return new String((StringBuffer) prop.getValues().get(0));
      }
      throw new RepositoryException("No primary type ");
    }

    public String[] getMixinTypeNames() {
      ArrayList mixins = new ArrayList();
      for (int i = 0; i < properties.size(); i++) {
        PropertyInfo prop = (PropertyInfo) properties.get(i);
        if (prop.getName().equals("jcr:mixinTypes")) {
          for (int j = 0; j < prop.getValues().size(); j++)
            mixins.add(new String((StringBuffer) prop.getValues().get(j)));
        }
      }

      String[] mixinProps = new String[mixins.size()];
      for (int i = 0; i < mixins.size(); i++) {
        mixinProps[i] = (String) mixins.get(i);
      }
      return mixinProps;
    }

  }

  private class PropertyInfo {

    private String name;

    private int    type;

    private List   values;

    public PropertyInfo(String name, int type, List values) {
      super();
      this.name = name;
      this.type = type;
      this.values = values;
    }

    /**
     * @return Returns the name.
     */
    public String getName() {
      return name;
    }

    /**
     * @return Returns the type.
     */
    public int getType() {
      return type;
    }

    /**
     * @return Returns the values.
     */
    public List getValues() {
      return values;
    }
  }

}
