package org.exoplatform.services.webdav.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Calendar;

import javax.jcr.Node;
import javax.jcr.Property;

import org.apache.jackrabbit.server.io.AbstractImportCommand;
import org.apache.jackrabbit.server.io.ImportContext;

public class MyCommand extends AbstractImportCommand {

  
  public boolean execute(ImportContext context) throws Exception {
    Node parentNode = context.getNode().getParent();
    InputStream in = context.getInputStream();
    if (in == null) {
        // assume already consumed
        return false;
    }
    if (!canHandle(context.getContentType())) {
        // ignore imports
        return false;
    }
    // check 'file' node
    Node fileNode = parentNode.hasNode(context.getSystemId())
        ? parentNode.getNode(context.getSystemId())
        : parentNode.addNode(context.getSystemId(), "nt:file");

    if (importResource(context, fileNode, in)) {
        context.setInputStream(null);
        // set current node
        context.setNode(fileNode);
    }
    return false;
}  
  
  /**
   * Imports a resource by creating a new nt:resource node.
   *
   * @param ctx the import context
   * @param parentNode the parent node
   * @param in the input stream
   * @return <code>true</code>
   * @throws Exception in an error occurrs
   */
  public boolean importResource(ImportContext ctx, Node parentNode,
                                InputStream in)
          throws Exception {
      Node content = parentNode.hasNode(JCR_CONTENT)
              ? parentNode.getNode(JCR_CONTENT)
              : parentNode.addNode(JCR_CONTENT, NT_RESOURCE);
      content.setProperty(JCR_MIMETYPE, ctx.getContentType());
      Property data = content.setProperty(JCR_DATA, in);
      in.read();
      Calendar lastMod = Calendar.getInstance();
      if (ctx.getModificationTime() != 0) {
          lastMod.setTimeInMillis(ctx.getModificationTime());
      }
      content.setProperty(JCR_LASTMODIFIED, lastMod);
      
      return true;
  }

  /**
   * Can handle all content type thus returning <code>true</code>.
   * 
   * @param contentType
   * @return <code>true</code>
   */
  public boolean canHandle(String contentType) {
      return true;
  }


}
