/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.file.explorer.component;

import java.io.File;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.file.explorer.component.model.NodeDescriptor;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIContentEditor.java,v 1.1 2004/07/16 09:51:34 oranheim Exp $
 */
public class UIFileNodeEditor extends UISimpleForm implements ExplorerListener {
	final static public String CREATE_DIR_ACTION = "createDir" ;
	final static public String CREATE_FILE_ACTION = "createFile" ;
  private static String DIR = "dir" , FILE = "file" ;
  protected UIStringInput dirInput ;
  protected UIStringInput fileInput ;

  public UIFileNodeEditor() {
    super("contentForm", "post", null) ;
    setId("UIContentEditor") ;    
    setRendererType("SimpleFormVelocityRenderer");
    add( new UIStringInput(DIR, "") );
    add( new UIStringInput(FILE, "") ) ;
    dirInput =  getUIStringInput(DIR);
    fileInput =  getUIStringInput(FILE);
    addActionListener(CreateDirectoryActionListener.class, CREATE_DIR_ACTION) ;
    addActionListener(CreateFileActionListener.class, CREATE_FILE_ACTION) ;
  }
  
  public String getCreateDirParams() { return CREATE_DIR_ACTION ; }
  public String getCreateFileParams() { return CREATE_FILE_ACTION ; }
  
  public void  onChange(UIExplorer uiExplorer, NodeDescriptor node)  {
    if(node.isLeafNode()) setRendered(false) ;
    else setRendered(true) ;
  }
  
  public void  onModify(UIExplorer uiExplorer, NodeDescriptor node)  {}
  
  public void  onAddChild(UIExplorer uiExplorer, NodeDescriptor node)  { }
  
  public void  onRemove(UIExplorer uiExplorer, NodeDescriptor node)  {  }
  
  static public class CreateDirectoryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIFileNodeEditor uicomp = (UIFileNodeEditor) event.getComponent() ;
      UIFileExplorer uiExplorer = 
        (UIFileExplorer) uicomp.getAncestorOfType(UIFileExplorer.class) ;
      NodeDescriptor node = uiExplorer.getSelectNode() ;
      String realPath = uiExplorer.getRealPathBaseDir() + node.getUri() + "/" + uicomp.dirInput.getValue() ;
      File file = new File(realPath) ;
      Information information = findInformationProvider(uiExplorer)  ;
      if(file.exists()) {
        information.addMessage(new ExoFacesMessage("#{UIFileNodeEditor.msg.name-exist}")) ;
      } else {
        file.mkdir() ;
        uiExplorer.broadcastOnAddChild(node) ;
        information.addMessage(new ExoFacesMessage("#{UIFileNodeEditor.msg.create-dir-ok}")) ;
      }
    }
  }
  
  static public class CreateFileActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIFileNodeEditor uicomp = (UIFileNodeEditor) event.getComponent() ;
      UIFileExplorer uiExplorer = 
        (UIFileExplorer) uicomp.getAncestorOfType(UIFileExplorer.class) ;
      NodeDescriptor node = uiExplorer.getSelectNode() ;
      String realPath = uiExplorer.getRealPathBaseDir() + node.getUri() + "/" + uicomp.fileInput.getValue() ;
      File file = new File(realPath) ;
      Information information = findInformationProvider(uiExplorer)  ;
      if(file.exists()) {
        information.addMessage(new ExoFacesMessage("#{UIFileNodeEditor.msg.name-exist}")) ;
      } else {
        file.createNewFile() ;
        uiExplorer.broadcastOnAddChild(node) ;
        information.addMessage(new ExoFacesMessage("#{UIFileNodeEditor.msg.create-file-ok}")) ;
      }
    }
  }
}