/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.organization.hibernate;


import java.util.Collection;
import java.util.List;

import org.exoplatform.commons.exception.UniqueObjectException;
import org.exoplatform.commons.utils.ListenerStack;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.GroupEventListener;
import org.exoplatform.services.organization.GroupHandler;
import org.exoplatform.services.organization.impl.GroupImpl;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Session;
/**
 * Created by The eXo Platform SARL
 * Author : Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Date: Aug 22, 2003
 * Time: 4:51:21 PM
 */
public class GroupHandlerImpl implements GroupHandler { 
  public static final String queryFindGroupByName =
    "from g in class org.exoplatform.services.organization.impl.GroupImpl " +
    "where g.groupName = ? ";
  
  public static final String queryFindGroupById =
    "from g in class org.exoplatform.services.organization.impl.GroupImpl " +
    "where g.id = ? ";
  
  public static final String queryFindGroupByParent =
    "from g in class org.exoplatform.services.organization.impl.GroupImpl " +
    "where g.parentId = ? ";

  private static final String queryFindRootGroups =
    "from g in class org.exoplatform.services.organization.impl.GroupImpl " +
    "where g.parentId is null";
  private static final String queryFindGroupsOfUser =
    "select distinct g " +
    "from g in class org.exoplatform.services.organization.impl.GroupImpl, " +
    "     m in class org.exoplatform.services.organization.impl.MembershipImpl " +
    "where m.groupId = g.id " +
    "  and m.userName = ?" ;
  
  private static final String queryFindGroupByMembership =
    "select g " +
    "from m in class org.exoplatform.services.organization.impl.MembershipImpl, " +
    "     g in class org.exoplatform.services.organization.impl.GroupImpl " +
    "where m.groupId = g.id " +
    "  and m.userName = ? " +
    "  and m.membershipType = ? ";
  
  private static final String queryGetAllGroups =
    "from g in class org.exoplatform.services.organization.impl.GroupImpl";
  
  private HibernateService service_ ;
  private List listeners_ ;

  public GroupHandlerImpl(HibernateService service) {
    service_ = service ; 
    listeners_ = new ListenerStack(5) ;
  }

  public void addGroupEventListener(GroupEventListener listener) {
    listeners_.add(listener) ;
  }

  final public Group createGroupInstance() {  return new GroupImpl(); }
  
  public void createGroup(Group group, boolean broadcast) throws Exception {
  	Session session = service_.openSession();
    String groupId =  "/" + group.getGroupName() ;
    if( group.getId() != null) groupId = group.getId();    
    Object o = session.get( GroupImpl.class, groupId) ;    
    if(o != null) {     
      Object[] args = {group.getGroupName()} ;
      throw new UniqueObjectException("OrganizationService.unique-group-exception",  args) ;
    }
  	if(broadcast) preSave(group, true)  ;    
  	session.save(group, groupId);//"/" + group.getGroupName());    
    if(broadcast) postSave(group, true)  ;
  	session.flush() ;
  }

  public void addChild(Group parent, Group child, boolean  broadcast) throws Exception {
    Session session = service_.openSession();
    Group parentGroup = (Group) session.get(GroupImpl.class, parent.getId()) ;
    String groupId = parentGroup.getId() + "/" + child.getGroupName() ;
    Object o = session.get(GroupImpl.class, groupId) ;
    if(o != null) {
      Object[] args = {child.getGroupName()} ;
      throw new UniqueObjectException("OrganizationService.unique-group-exception",  args) ;
    }
    GroupImpl childImpl = (GroupImpl) child ; 
    if(broadcast)  preSave(child, true)  ;
    childImpl.setParentId(parentGroup.getId()) ;
    session.save(childImpl, groupId);
    if(broadcast) postSave(child, true)  ;
    session.flush() ;
  }

  public void saveGroup(Group group, boolean broadcast) throws Exception {
  	Session session = service_.openSession();    
    if(broadcast) preSave(group, false)  ;    
  	session.update(group);
    if(broadcast) postSave(group, false)  ;   
    session.flush();    
  }

  public Group removeGroup(Group group, boolean broadcast) throws Exception {
    Session session = service_.openSession();    
    if(broadcast) preDelete( group) ;   
    session.delete( group);    
    List entries =  session.createQuery( queryFindGroupByParent).setString(0, group.getId()).list();      
    for(int i=0; i<entries.size(); i++) removeGroup( (Group) entries.get(i),  broadcast);   
    MembershipHandlerImpl.removeMembershipEntriesOfGroup(group, session) ;
    if(broadcast) postDelete(group)  ;
    session.flush();    
    return group ;
  }  
  
  static void removeGroupEntry(String groupName, Session session) throws Exception {
    List entries = session.createQuery(queryFindGroupByName).setString(0, groupName).list();
    for(int i=0; i<entries.size(); i++)	session.delete(entries.get(i));
  }

  public Collection findGroupByMembership(String userName, String membershipType) throws Exception {
  	Session session = service_.openSession();
    Query q  = session.createQuery(queryFindGroupByMembership).
                       setString(0, userName).setString(1, membershipType) ;
  	List groups =  q.list();
  	return groups ;
  }

  public Group findGroupByName(String groupName) throws Exception {
  	Session session = service_.openSession();
  	Group group =(Group)service_.findOne(session,queryFindGroupByName, groupName);
  	return group;
  }
  
  public Group findGroupById(String groupId) throws Exception {
    Session session = service_.openSession();
    Group group =(Group)service_.findOne(session, queryFindGroupById, groupId);    
    return group;
  }

  public Collection findGroups(Group parent) throws Exception {
  	Session session = service_.openSession();
  	if (parent == null) {
  		return session.createQuery(queryFindRootGroups).list();
//  		return  session.find( queryFindRootGroups );
  	}
//    return  session.find( queryFindGroupByParent, parent.getId(), Hibernate.STRING );
    return  session.createQuery( queryFindGroupByParent).setString(0, parent.getId()).list();

  }
  
  public Collection findGroupsOfUser(String user) throws Exception {
  	Session session = service_.openSession();
//  	return   session.find( queryFindGroupsOfUser, user, Hibernate.STRING );
  	return   session.createQuery( queryFindGroupsOfUser).setString(0, user).list();
  }
  
  public Collection getAllGroups() throws Exception {
    Session session = service_.openSession();
    Query q  = session.createQuery( queryGetAllGroups) ;
    List groups =  q.list();
    return groups ;
  }

  private void preSave(Group group , boolean isNew) throws Exception {
    for (int i = 0 ; i < listeners_.size(); i++) {
      GroupEventListener listener = (GroupEventListener) listeners_.get(i) ;
      listener.preSave(group, isNew) ;
    }
  }

  private void postSave(Group group , boolean isNew) throws Exception {
    for (int i = 0 ; i < listeners_.size(); i++) {
      GroupEventListener listener = (GroupEventListener) listeners_.get(i) ;
      listener.postSave(group, isNew) ;
    }
  }

  private void preDelete(Group group) throws Exception {
    for (int i = 0 ; i < listeners_.size(); i++) {
      GroupEventListener listener = (GroupEventListener) listeners_.get(i) ;
      listener.preDelete(group) ;
    }
  }

  private void postDelete(Group group) throws Exception {
    for (int i = 0 ; i < listeners_.size(); i++) {
      GroupEventListener listener = (GroupEventListener) listeners_.get(i) ;
      listener.postDelete(group) ;
    }
  }
}