/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.core.renderer.html;

import java.io.IOException;
import java.io.Writer;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.context.FacesContext;

import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.UIHiddenInput;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.model.Parameter;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Dec 27, 2005
 */
class  JSF {
  private static String START_ERROR = "<strike>" ;
  private static String END_ERROR = "</strike>" ;
  
  UIExoComponent uicomponent ;
  ResourceBundle res ;
  Writer w ;
  
  public JSF(UIExoComponent uicomp, ResourceBundle res, Writer w) {
    this.uicomponent  = uicomp ;
    this.res = res ;
    this.w  =  w ;
  }
  
  final public void link(String label , Parameter[] params) throws IOException {
    link(label , params, null) ;
  }
  
  final public void link(String label , Parameter[] params, String id) throws IOException {
    w.write("<a  href='");
    w.write(uicomponent.getBaseURL());
    for (int i = 0; i < params.length; i++) {
      w.write("&amp;"); w.write(params[i].getName()); w.write('='); w.write(params[i].getValue());
    }
    if(id != null) {
      w.write("&amp;"); w.write(UIExoComponent.OBJECTID); w.write("="); w.write(id);
    }
    w.write("'>");
    if(label == null) label = "??????" ;
    if(label.charAt(0) == '@') {
      label = res.getString(label) ;
    }
    w.write(label); w.write("</a>");
  }
  
  final public void resolve(String key) throws IOException {
    try {
       w.write(res.getString(key)) ;
    } catch(MissingResourceException ex) {
      w.write(START_ERROR);w.write(key) ; w.write(END_ERROR) ;
    }
  }
  
  final public void formBegin() throws IOException {
    UISimpleForm uiForm = (UISimpleForm) uicomponent;
    FacesContext facescontext = FacesContext.getCurrentInstance() ;
    String actionURL = uiForm.getBaseURL(facescontext) ;
    String formName = uiForm.getFormName() ;
    //String formId = uiForm.getFormId() ;
    String method = uiForm.getMethod() ;
    String clazz = uiForm.getClazz() ;
    if(clazz == null) clazz = "UIExoForm" ;
    w.write("<form id='"); w.write(formName); w.write("'") ;
    w.write(" action='"); w.write(actionURL); w.write("'") ;
    if(method != null){ 
      w.write(" method='"); w.write(method); w.write("'") ;
    }
    if(uiForm instanceof UISimpleMulltipartForm) {
      w.write(" enctype='multipart/form-data'");
    }
    //w.write(" class='UISimpleForm "); w.write(clazz) ;
    w.write(">\n");
    w.write("<div>\n");
    w.write("<input type='hidden' name='"); w.write(UIExoComponent.ACTION); w.write("'") ;
    w.write(" value=''/>\n") ;
    w.write("</div>\n");
    List hiddenInputs = uiForm.getHiddenInputs() ;
    if(hiddenInputs != null) {
      for (int i = 0; i < hiddenInputs.size(); i++) {
        UIHiddenInput uiInput = (UIHiddenInput) hiddenInputs.get(i) ;
        uiInput.encodeBegin(facescontext) ;
      }
    }
  }
  
  final public void formEnd() throws IOException {
    w.write("</form>") ;
  }
  
  final public void formInput(String name) throws IOException {
    UISimpleForm uiForm = (UISimpleForm) uicomponent;
    UIInput uiInput = uiForm.getUIInput(name); ;
    if (uiInput != null) {      
      FacesContext facescontext = FacesContext.getCurrentInstance();
      uiInput.encodeBegin(facescontext);
      uiInput.encodeChildren(facescontext);
      uiInput.encodeEnd(facescontext);      
    } else {
      w.write(START_ERROR  + "unknown component: " + name + END_ERROR);
    }    
  }
  
  final public void formButton(String label, String action) throws IOException {
    UISimpleForm uiForm = (UISimpleForm) uicomponent;   
    if(label.charAt(0) == '@') label = label.substring(1) ;
    try {
      label = res.getString(label) ;
   } catch(MissingResourceException ex) {
   }
    w.write("<a  href=\"javascript:submitForm('"); w.write(uiForm.getFormName()); w.write("','");
    w.write(action) ; w.write("');\">");
    w.write(label);
    w.write("</a>");
  }
  
  final public void renderChild(String id) throws IOException {
    UIExoComponent ui = uicomponent.findComponentById(id);
    if (ui != null) {      
      FacesContext facescontext = FacesContext.getCurrentInstance();
      ui.encodeBegin(facescontext);
      ui.encodeChildren(facescontext);
      ui.encodeEnd(facescontext);      
    } else {
      w.write(START_ERROR  + "unknown component: " + id + END_ERROR);
    }    
  }  
}