package org.exoplatform.portlets.backup.component;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ResourceBundle;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.exoplatform.commons.utils.MimeTypeResolver;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.backup.Metadata;
import org.exoplatform.xml.object.XMLObject;

public class UIUploadData extends UISimpleMulltipartForm { 

  protected ImportExportService service_ ;

  public UIUploadData( ImportExportService service) {
    super("uploadData", "post", null);

    setRendererType("SimpleFormVelocityRenderer");

    add(new UIUploadInput("upload"));

    addActionListener(SaveActionListener.class, SAVE_ACTION);
    
    service_ = service ;
  } 
  
  ImportExportService getBackupService() { return service_ ; } 

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {     
      UIUploadData uiForm = (UIUploadData) event.getSource();
      UIUploadInput input = (UIUploadInput) uiForm.getUIInput("upload");     
      
      String fileName = input.getFileName();
      MimeTypeResolver resolver = new MimeTypeResolver();
      String mimeType = resolver.getMimeType(fileName);
      if( !mimeType.trim().equals( "application/zip")){
        Information iprovider = findInformationProvider( uiForm) ;
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIBackupData.msg.file-type"))) ;
        return;      
      }
      byte[] content = input.getContent();      
      
      String dir = System.getProperty("java.io.tmpdir");
      File file = new File( dir,fileName);
      FileOutputStream output=new FileOutputStream(file);
      output.write( content);
      output.flush();
      output.close();     
      
      ZipFile zip = new ZipFile( file);
      ZipEntry meta = zip.getEntry( "metadata.xml");
      if( meta == null)
        meta = zip.getEntry( "organization/metadata.xml");
      if( meta == null){
        zip.close();
        Information iprovider = findInformationProvider( uiForm) ;
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIBackupData.msg.data-invalid"))) ;
        return;       
      }
      Metadata mdata = (Metadata) XMLObject.getObject( zip.getInputStream( meta)) ;
      String type = mdata.getType();
      String name = file.getName();    
      if( type.trim().equals("single-user") ) 
        copyFile( file, new File( uiForm.getBackupService().getUserDataDirectory(), name)); 
      else if( type.trim().equals("single-service"))
        copyFile( file, new File( uiForm.getBackupService().getServiceDataDirectory(), name) );
      else if( type.trim().equals("multi-users"))
        (new ZipFolder()).extractToFiles( file, uiForm.getBackupService().getUserDataDirectory());
      else if( type.trim().equals("multi-services"))
        (new ZipFolder()).extractToFiles( 
            file, uiForm.getBackupService().getServiceDataDirectory());    
      zip.close();            
      file.delete();      
    }   
   
    private void copyFile( File in, File out) throws Exception {
      FileInputStream input=new FileInputStream( in);      
      FileChannel cinput = input.getChannel();
      long fsize = cinput.size();       
      ByteBuffer buff = ByteBuffer.allocate((int)fsize);        
      cinput.read(buff);
      buff.rewind();
      
      FileOutputStream output = new FileOutputStream( out);
      FileChannel coutput = output.getChannel();      
      coutput.write(buff);        
      buff.clear();  
      coutput.close();
      output.close(); 
      cinput.close();        
      input.close();
    }  
  }

}
