/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portlets.management.registry.component;

import java.util.ResourceBundle;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.portletregistery.Portlet;
import org.exoplatform.services.portletregistery.PortletRegisteryService;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 19 juin 2004
 */
public class UIPortletForm extends UISimpleForm {
  protected UIStringInput displayName_;
  protected UITextArea description_;
  private Portlet portlet_ ;

  public UIPortletForm() {
    super("portletForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer") ;
    displayName_ = new UIStringInput("name", "");
    add(displayName_) ;
    description_ = new UITextArea("description", "");
    add(description_) ;
    addActionListener(UpdateActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public String getSaveAction() { return SAVE_ACTION ; }
  public String getCancelAction() { return CANCEL_ACTION ; }
  
  public void setPortletData(Portlet portlet) { 
  	portlet_ = portlet ;
  	displayName_.setValue(portlet.getDisplayName());
    description_.setValue(portlet.getDescription());
  }

  static public class UpdateActionListener extends ExoActionListener {
    public UpdateActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletForm uiForm = (UIPortletForm) event.getSource()  ;
      Information iprovider = findInformationProvider(event.getComponent()) ;
      String displayName = uiForm.displayName_.getValue();
      if (displayName == null || "".equals(displayName)) {
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIPortletForm.msg.portlet-name-null"))) ;
        return;
      }
      uiForm.portlet_.setDisplayName(displayName);
      uiForm.portlet_.setDescription(uiForm.description_.getValue());    
      PortletRegisteryService service =
        (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
      service.updatePortlet(uiForm.portlet_);
      uiForm.setRenderedSibling(UIPortletCategories.class);
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletForm uiForm = (UIPortletForm) event.getSource()  ;
      uiForm.setRenderedSibling(UIPortletCategories.class);
    }
  }
}