/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.web.servlet;

import java.io.InputStream;
import java.io.Writer;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import groovy.lang.GroovyClassLoader;
import groovy.lang.Writable;
import groovy.text.Template ;
import groovy.text.SimpleTemplateEngine ;
/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Dec 21, 2005
 */
public class ResourceManager {
  private ThreadLocal tlocal_ = new ThreadLocal() {
    protected synchronized Object initialValue() {
      return new  HashMap<String, Object> ();
    }
  };
  private ServletContext scontext_ ;
  private SimpleTemplateEngine engine_ ;
  private Map<String, Template> templates_ ;
  private GroovyClassLoader classloader_ ;
  
  public ResourceManager(GroovyClassLoader classloader, ServletContext context) {
    scontext_ = context ;
    classloader_  = classloader ;
    engine_ = new SimpleTemplateEngine() ;
    templates_ = new HashMap<String, Template>() ;
  }
  
  public Map<String, Object>  createDefaultBinding(HttpServletRequest request) {
    Map<String, Object> binding = (Map)tlocal_.get() ;
    binding.put("session", request.getSession()) ;
    binding.put("templates" , this) ;
    return binding ;
  }
  
  public void clearTemplateBinding() {
    Map binding = (Map)tlocal_.get() ;
    binding.clear() ;
  }
  
  public String include(String name) throws Exception {
    Map binding = (Map) tlocal_.get() ;
    Map localBinding = new HashMap(binding) ;
    Template template = getTemplate(name) ;
    String s = template.make(localBinding).toString() ;
    return s ;
  }
  
  public void merge(Writer writer, String tname, Map  binding) throws Exception {
    Template template = getTemplate(tname) ;
    Writable writable = template.make(binding) ;
    writable.writeTo(writer) ;
  }
  
  private Template  getTemplate(String id) throws Exception {
    Template template = templates_.get(id) ;
    if(template == null) {
      InputStream is = scontext_.getResourceAsStream("/groovy/" + id) ;
      byte[]  buf = new byte[is.available()] ;
      is.read(buf) ;
      String text =  new String(buf) ;
      template = engine_.createTemplate(text) ;
      //templates_.put(id, template) ;
    }
    return template ;
  }
  
  public Object getGroovyObject(String name) throws Exception {
    return  getGroovyObject(name, false);
  }
  
  public Object getGroovyObject(String name, boolean reload) throws Exception {
    System.out.println("load object " + name + " reload " + reload);
    GroovyClassLoader cl = classloader_ ;
    if(reload) {
      cl = new GroovyClassLoader(cl.getParent()) ;
      cl.addClasspath(scontext_.getRealPath("/groovy")) ;
      cl.addClasspath(scontext_.getRealPath("/WEB-INF/classes")) ;
    }
    try {
      Class clazz = cl.loadClass(name) ;
      return clazz.newInstance() ;
    } catch(Exception ex) {
      
    }
    name = name.replace(".", "/") ;
    name =  name + ".groovy" ;
    URL  classURL = scontext_.getResource("/groovy/" + name) ;
    if(classURL == null) {
      classURL = cl.getResource(name) ;
    }
    Class clazz = cl.parseClass(classURL.openStream()) ;
    return clazz.newInstance() ;
  }
}