/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.admin;

import condition.* ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Nov 15, 2005
 */
public class ResourceBundlePortletSuite extends WebUnitSuite {
  static String DESC = "Go to ResourceBundlePortletSuite page and test all the basic actions at the group tab" ;
  static String NAME = "ResourceBundlePortletSuite" ;
  
  public ResourceBundlePortletSuite() {    
    super(NAME, DESC) ;
    init("/Site Map/Internationalization") ;
  }  
  public ResourceBundlePortletSuite(String path) {
    super(NAME, DESC) ;
    init(path) ;
  }  
  public void  init(path) {    
		goToPage(path) ;
    //-----New Resource------
    addWebUnit(
	    new SubmitFormUnit("New Resource", "Click on New Resource link").
	    setFormId("searchForm").
    	setField("op","newResource").
    	addValidator(new ExpectFormIdValidator("resource"))
    ) ;  
    //Saving is not OK 
    addWebUnit(     
      new SubmitFormUnit("SaveIsNotOK", "Can't save resource").
      setFormId("resource").
      setField("text","a.key=a property").
      setField("name","@locale.users.exo.name").
      setField("op","save").
      addValidator(new ExpectTextValidator("Only alpha, digit, dot, dash and underscore characters allowed for the field name"))
    ) ;  
    //Saving is OK
    addWebUnit(    	
      new SubmitFormUnit("SaveNew", "Fill info and click on Save button").
    	setFormId("resource").
    	setField("text","a.key=a property").
    	setField("name","locale.users.exo.name").
    	setField("op","save").
    	addValidator(new ExpectTextValidator("Your Resource has been updated successfully."))
    ) ;      
    //Edit
    addWebUnit(
    	new SubmitFormUnit("EditNew", "Click on Edit button").
    	setFormId("resource").
    	setField("text","a.key=a property").
    	setField("name","locale.users.exo.name").
    	setField("op","edit")
    ) ;      
    //Preview
    addWebUnit(    	
      new SubmitFormUnit("PreviewNew", "Click on Preview button").
    	setFormId("resource").
    	setField("text","a.key=a property").
    	setField("name","locale.users.exo.name").
    	setField("op","preview").
    	addValidator(new ExpectTextValidator("a.key=a property"))
    ) ;  
    //Cancel
    addWebUnit(
    	new SubmitFormUnit("CancelNew", "Click on Cancel button").
    	setFormId("resource").
    	setField("text","a.key=a property").
    	setField("op","PhaseId[1].cancel").
			addValidator(new ExpectFormIdValidator("searchForm"))
    ) ; 
    //-----Search------
    addWebUnit(
      new SubmitFormUnit("SearchCeatedResource", "Click on Search link").
      setFormId("searchForm").
      setField("name","locale.users.exo.name").
      setField("op","searchResource").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIListResources&op=view&objectId=locale.users.exo.name"))      
    ) ;
		//-----View action------
    addWebUnit(
    	new ClickURLLinkUnit("View", "Link to View page").
    	setURL("&portal:isSecure=false&uicomponent=UIListResources&op=view&objectId=locale.users.exo.name").
      addValidator(new ExpectTextValidator("resource"))
    ) ;      
    //Edit
    addWebUnit(
    	new SubmitFormUnit("EditView", "Click on Edit button").
    	setFormId("resource").
    	setField("text","a.key=a property").
    	setField("name","locale.users.exo.name").
    	setField("op","edit").
    	addValidator(new ExpectTextValidator("a.key=a property"))
    ) ;  
    //Save
    addWebUnit(    	
      new SubmitFormUnit("SaveView", "Edit info and click on Save button").
    	setFormId("resource").
    	setField("text","a.key=a property(update)").
    	setField("name","locale.users.exo.name").
    	setField("op","save").
    	addValidator(new ExpectTextValidator("Your Resource has been updated successfully."))
    ) ;  
    //Preview
    addWebUnit(    	
      new SubmitFormUnit("PreviewView", "Click on Preview button").
    	setFormId("resource").
    	setField("text","a.key=a property(update)").
    	setField("name","locale.users.exo.name").
    	setField("op","preview")    	
    ) ;  
    //Cancel
    addWebUnit(
    	new SubmitFormUnit("CancelView", "Click on Cancel button").
    	setFormId("resource").
    	setField("text","a.key=a property(update)").
    	setField("name","locale.users.exo.name").
    	setField("op","PhaseId[1].cancel").
			addValidator(new ExpectFormIdValidator("searchForm"))
    ) ; 
    //-----Delete action-----
    addWebUnit(
	    new ClickURLLinkUnit("Delete", "Click on Delete button").
	    setURL("uicomponent=UIListResources&op=delete").
    	addValidator(new ExpectNoTextValidator("locale.users.exo.name(update)"))    	
    ) ;  
  }
  
  public java.util.List getRequiredSuites() { 
  	String[] roles = ["user", "admin"] ;
   	return [new UserLoginSuite("admin", "exo", roles)] ;
  }
  
}