/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;


/**
 * Test copy, cut and paste actions on newly created documents. This Suite
 * depends on the AddDocumentSuite one.      
 *     1) Go to production workspace
 *     2) Go to /test-folder
 *     3) Create a /test-folder2 directory
 *     4) Copy test-doc
 *     5) Go to /test-folder2
 *     6) Paste test-doc and check it is added
 *     7) Go back to /test-folder
 *     8) Create /test-folder3
 *     9) Cut test-doc
 *    10) Go to test-folder3
 *    11) Paste test-doc and check it is added
 *    12) Go back to test-folder
 *    13) Check test-doc was deleted
 *    14) Save Session
 *    15) Go back to workspaces selection page
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserCopyPasteAndCutSuite extends WebUnitSuite {
  static String DESC = "Go to ECM File Explorer page, click on test-folder and to do somethings" ;
  static String NAME = "UserCopyPasteAndCutSuite" ;
  String COMMENT = null;
  
  public UserCopyPasteAndCutSuite() {    
    super(NAME, DESC) ;
    addWebSuite(new UserAddFolderSuite());
    addWebSuite(new UserAddDocumentSuite());
    addWebSuite(new GotoFolderSuite());   
    //Create a /test-folder2 directory
    addWebUnit(
      new ClickURLLinkUnit("CreateTest-folder2", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;    
    //-----Save new folder
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder2").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=removeNode&objectId=test-folder2"))      
    ) ;
    //Copy test-doc
    addWebUnit(
      new ClickURLLinkUnit("CopyDocument", "click on copy button").
      setURL("&uicomponent=UIDocumentInfo&op=copy&objectId=test-doc").         
      addValidator(new ExpectTextValidator("The node /test-folder/test-doc has been copied."))         
    ) ;   
    //Go to /test-folder2
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder2", "click on test-folder2").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/test-folder/test-folder2").        
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("test-folder")).      
      addValidator(new ExpectTextValidator("test-folder2"))
    ) ;    
    
    COMMENT = "YOU CAN'T PASTE, THERE IS A MESSAGE : 'YOU CAN'T NOT LOG IN.'";
    //Paste test-doc and check it is added
    addWebUnit(
      new ClickURLLinkUnit("PasteDocument", "click on paste button").
      setURL("&uicomponent=UIToolbar&op=paste").        
      addValidator(new ExpectTextValidator("You can not log in."))      
    ) ;
    
    //Go back to /test-folder
    addWebUnit(
      new ClickURLLinkUnit("BackToTest-folder", "click on test-folder link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/test-folder").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/cms"))      
    ) ;
    //Create /test-folder3
    addWebUnit(
      new ClickURLLinkUnit("CreateTest-folder3", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;
    //-----Save new folder
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder3").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=removeNode&objectId=test-folder3"))      
    ) ;
    //Cut test-doc
    addWebUnit(
      new ClickURLLinkUnit("CutDocument", "click on cut button").
      setURL("&uicomponent=UIDocumentInfo&op=cut&objectId=test-doc").         
      addValidator(new ExpectTextValidator("The node /test-folder/test-doc has been cut."))         
    ) ;
    //Go to test-folder3
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder3", "click on test-folder3").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/test-folder/test-folder3").      
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("test-folder")).
      addValidator(new ExpectTextValidator("test-folder3"))
    ) ;
    
    COMMENT = "YOU CAN'T PASTE, THERE IS A MESSAGE : 'YOU CAN'T NOT LOG IN.'";
    //Paste test-doc and check it is added
    addWebUnit(
      new ClickURLLinkUnit("PasteDocument", "click on paste button").
      setURL("&uicomponent=UIToolbar&op=paste").        
      addValidator(new ExpectTextValidator("You can not log in."))      
    ) ;
    
    //Go back to test-folder and Check test-doc was deleted
    COMMENT = "BECAUSE YOU CAN'T PASTE, SO TEST-DOC WASN'T DELETED";
    addWebUnit(
      new ClickURLLinkUnit("BackToTest-folder", "click on test-folder link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/test-folder")        
      //addValidator(new ExpectNoTextValidator("test-doc"))      
    ) ;
    
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    //Return to workspace selection
    addWebUnit(
      new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
      addValidator(new ExpectTextValidator("Workspaces "))      
    ) ;    
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}