/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.user;

import condition.* ;
import suites.share.UserLoginSuite;
import suites.share.AccountUnit;
import suites.share.NewAccountSuite;
import suites.share.ViewUserProfileSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * May 21, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: OrganizationPortletSuite.java,v 1.3 2004/10/22 14:23:55 tuan08 Exp $
 **/
public class UsersManagementSuite extends WebUnitSuite {
  static String DESC = "Go to Users Management page and test all the basic actions at the users tab" ;
  static String NAME = "UsersManagementSuite" ;
  final static public String EXOPLATFORM_ACCOUNT = "exoplatform" ;
  final static public String ACCOUNT_TO_DELETE = "accountToDelete" ;
  
  public UsersManagementSuite() {    
    super(NAME, DESC) ;
    init("/Site Map/Management", "/Site Map/User Registration") ;
  }  
  public UsersManagementSuite(String path, String reg_path) {
    super(NAME, DESC) ;
    init(path, reg_path) ;
  }  
  public void  init(path, reg_path) {    
    goToPage(path) ;        
    addWebUnit(
       new ClickURLLinkUnit("DeleteIfExistedAccount", "Delete the test existed account").
       setURL("op=delete&objectId="+EXOPLATFORM_ACCOUNT).
       addCondition(new HaveLinkWithURLCondition("op=delete&objectId="+EXOPLATFORM_ACCOUNT))
    ) ;    
    addWebUnit(
       new ClickURLLinkUnit("DeleteIfExistedAccountToDelete", "Delete the test existed account").
       setURL("op=delete&objectId="+ACCOUNT_TO_DELETE).
       addCondition(new HaveLinkWithURLCondition("op=delete&objectId="+ACCOUNT_TO_DELETE))
    ) ;
    goToPage(reg_path) ;
    //create exoplatform account , the account may already existed , it is ok
    addWebUnit(AccountUnit.createNewAccountUnit(EXOPLATFORM_ACCOUNT, EXOPLATFORM_ACCOUNT)) ;
    //create accountToDelete account , the account may already existed , it is ok
    addWebUnit(AccountUnit.createNewAccountUnit(ACCOUNT_TO_DELETE, ACCOUNT_TO_DELETE)) ;
    
    goToPage(path);
    //======================users tab=============================================
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for user to delete").
      setFormId("quickSearchForm").
      setField("searchField","username").
      setField("term",ACCOUNT_TO_DELETE).
      setField("op","search")
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickDeleteAccount", "Delete Account").
      setURL("op=delete&objectId="+ACCOUNT_TO_DELETE).
      addCondition(new HaveLinkWithURLCondition("op=delete&objectId="+ACCOUNT_TO_DELETE))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for user to delete").
      setFormId("quickSearchForm").
      setField("searchField","username").
      setField("term",((String)ACCOUNT_TO_DELETE).substring(0,((String)ACCOUNT_TO_DELETE).length()-1)+"*" ).
      setField("op","search").
      addValidator(new ExpectNoTextValidator(ACCOUNT_TO_DELETE))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitSearcUserForm", "search for exoplatform user").
      setFormId("quickSearchForm").
      setField("searchField","username").
      setField("term",EXOPLATFORM_ACCOUNT).
      setField("op","search")
    ) ;
    
    addWebUnit(
      new ClickURLLinkUnit("ClickViewUserProfile", "Click the view user profile icon").
      setURL("op=viewProfile&objectId="+ EXOPLATFORM_ACCOUNT).
      addValidator(new ExpectTextValidator("Info"))
    ) ; 
    addWebUnit(
      new ClickLinkWithText("ClickBackLink", "Go back to the user list screen").
      setTextLink("back")
    ) ;
    //======================user info screen==========================================
    addWebUnit(
      new ClickURLLinkUnit("ClickViewUserInfo", "Click the view user info icon").
      setURL("op=viewUserInfo&objectId="+ EXOPLATFORM_ACCOUNT).
      addValidator(new ExpectFormIdValidator("membershipForm"))
    ) ;
    addWebUnit(
      new ClickLinkWithText("ClickBackLink", "Go back to the user list screen").
      setTextLink("back")
    ) ;
  }
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite("admin", "exo", roles)] ;
  }
}
