/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.maven2.plugin.exo;

import java.io.File;
import java.util.HashSet;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.exoplatform.maven2.plugin.Utils;
import org.apache.maven.archiver.MavenArchiveConfiguration;
import org.apache.maven.archiver.MavenArchiver;
import org.codehaus.plexus.archiver.ear.EarArchiver;
/** 
 * Created by The eXo Platform SARL
 * Author : Roman Pedchenko
 *          lautarul@gmail.com
 * Dec 20, 2005
 */
/**
 * @goal jcr
 * @requiresDependencyResolution runtime
 * @description mvn exo:jcr -Ddeploy=param for deploy JCR standalone
 */
public class ExoJcrDeploy extends AbstractMojo {   
  /**
   * @parameter
   * @required
   */
  private String workingDir;
  /**
   * The directory for the generated EAR.
   *
   * @parameter expression="${project.build.directory}"
   * @required
   */
  protected String outputDir;
  /**
   * @parameter
   * @required
   */
  private String sharedResourcesDir;
  /**
   * @parameter expression="${deploy}"
   */
  private String deploy ;
  /**
   * @parameter
   */
  private String excludeProjects ;
  /**
   * @parameter expression="${project}"
   * @required
   */
  protected MavenProject project;
  /**
   * @parameter
   */
  private String deployJonasDir;  
  /**
   * @parameter
   */
  private String deployJbossDir;
  /**
   * The Ear archiver.
   *
   * @parameter expression="${component.org.codehaus.plexus.archiver.Archiver#ear}"
   * @required
   */
  private EarArchiver earArchiver;
  /**
   * The maven archive configuration to use.
   *
   * @parameter
   */
  private MavenArchiveConfiguration archive = new MavenArchiveConfiguration();
  
  public void execute() throws MojoExecutionException{
    if(!"exo-jcr".equals(project.getPackaging())) printInfo() ;
    try {  
      if (deploy != null) {
        execDeploy() ;
        return;
      }
    }catch (Exception e) {
      e.printStackTrace() ;
    }
    printInfo() ;
  }   
  
  protected void makeEar(File earFile, File webappDir, String outDir) throws Exception {
    getLog().info( "Generating ear " + earFile.getAbsolutePath() );
    MavenArchiver archiver = new MavenArchiver();    
    archiver.setArchiver( earArchiver );
    archiver.setOutputFile( earFile );
    earArchiver.addDirectory(webappDir);
    earArchiver.setAppxml( new File(webappDir, "/META-INF/application.xml" ) );
    File manifest = new File(outDir + "/manifest.txt") ;
    Utils.createManifest(manifest, project, getIgnoreProjects()) ;
    earArchiver.setManifest(manifest) ;
    archiver.createArchive(project, archive);
//    project.getArtifact().setFile(earFile);
  }

  protected HashSet<String> getIgnoreProjects() {
    HashSet<String> ignoreProjects = new HashSet<String>() ;
    if(excludeProjects != null) {
      String[] pro =  excludeProjects.split(",") ;
      for(String s : pro) ignoreProjects.add(s.trim()) ;
    }
    return ignoreProjects ;
  }
  
  private void execDeploy() throws Exception {
    if (deploy.equals("tomcat")) {
      File deployTomcatDirFile = new File(workingDir + "/exo-tomcat");
      deployTomcat(deployTomcatDirFile, getIgnoreProjects());
    } else if(deploy.equals("jonas")) {
      File deployJonasDirFile  = new File(deployJonasDir);
      deployJonas(deployJonasDirFile, getIgnoreProjects());
    } else if(deploy.equals("jboss")) {
      File earDir = new File(deployJbossDir + "/jcr.sar");
      deployJboss(earDir, getIgnoreProjects());
    } else if(deploy.equals("jboss2")) {
      deployJboss2(deployJbossDir, getIgnoreProjects());
    } else {
      Utils.printMessage("info","The task 'exo:jcr -Ddeploy=" + deploy+ "' is invalid !\n");
      printInfo();
    }
  }
  
  protected void deployTomcat(File deployTomcatDir, HashSet<String> ignoreProjects) throws Exception  {
    File directoryJar = new File(deployTomcatDir + "/common/lib");
    File directoryWar = new File(deployTomcatDir + "/webapps");
    directoryJar.mkdirs();
    directoryWar.mkdirs();
    Utils.deployedDependency2(directoryJar, null, null, project, ignoreProjects) ;
    Utils.deployProject(directoryJar, directoryWar, project, false, ignoreProjects) ;
    Utils.patchConfig2(new File(sharedResourcesDir + "/patch-tomcat-jcr"), deployTomcatDir) ;
  }
  
  protected void deployJonas(File deployDir, HashSet<String> ignoreProjects)  throws Exception {
    File warDir = new File(deployDir + "/webapps/autoload");
    File rarDir = new File(deployDir + "/rars/autoload");
    File libDir = new File(deployDir + "/lib/apps");
    warDir.mkdirs();
    rarDir.mkdirs();
    libDir.mkdirs();
    Utils.deployedDependency2(libDir, null, rarDir, project, ignoreProjects) ;
    Utils.deployProject(libDir, warDir, project, false,  ignoreProjects);
    Utils.patchConfig2(new File(sharedResourcesDir + "/patch-jonas-jcr"), deployDir);
    /*File[] files = warDir.listFiles();
    for(int i = 0; i < files.length; i++ ) {
      if(files[i].getName().endsWith("war")) {
        Utils.removeManifestFromJar(files[i]);
      }
    }*/
  }
  
  protected void deployJboss(File earDir, HashSet<String> ignoreProjects) throws Exception {
    earDir.mkdirs();
    Utils.deployedDependency2(earDir, null, earDir, project, ignoreProjects);
    Utils.deployProject(earDir, earDir, project, false, ignoreProjects);
    Utils.createApplicationXml(earDir);
    Utils.patchConfig2(new File(sharedResourcesDir + "/jcr.sar"), earDir);
  }
  
  protected void deployJboss2(String earDir, HashSet<String> ignoreProjects) throws Exception {
    File earFile =  new File(earDir + "/jcr.ear");
    File earDir_ =  new File(earDir);
    earDir_.mkdirs();
    File outDir = new File(outputDir + "/ear");
    outDir.mkdirs();
    Utils.deployedDependency2(outDir, null, earDir_, project, ignoreProjects);
    Utils.deployProject(outDir, outDir, project, false, ignoreProjects);
    Utils.createApplicationXml(outDir);
    Utils.patchConfig2(new File(sharedResourcesDir + "/jcr.sar"), outDir);
    makeEar(earFile, outDir, outputDir + "/ear");
  }
  
  private void printInfo() throws MojoExecutionException {
    String info =  
      "The 'exo:jcr' maven2 plugin is used to assemble many exo modules into an application \n" +
      "and deploy the application to a server. To run the command successfully, you need to\n" +
      "run the command in an exo-jcr module and all the dependencies modules must be built\n" +
      "before running the command mvn exo:jcr -Ddeploy=value\n\n" +
      "The valid syntax is:\n" +
      "  mvn exo:jcr -Ddeploy=tomcat\n" +
      "  This command will copy will copy the jcr module and the dependency module to the tomcat server.\n" +
      "  mvn exo:jcr -Ddeploy=jonas\n" +
      "  This command will copy will copy the jcr module and the dependency module to the jonas server.\n" +
      "  mvn exo:jcr -Ddeploy=jboss\n" +
      "  This command will copy will copy the jcr module and the dependency module to the jboss server.\n" ;
    System.out.println(info) ;
    throw new MojoExecutionException("") ;
  }
}
